@@ -6,10 +6,10 @@ import time
 
 def test_notify_all():
     """A low-level variation on test_notify_all() in lib-python's
-    test_multiprocessing.py
+    _test_multiprocessing.py
     """
     N_THREADS = 1000
-    lock = SemLock(0, 1, 1)
+    lock = SemLock(0, 1, 1, "/test_notify_all", True)
     results = []
 
     def f(n):
