@@ -3,13 +3,10 @@ from pypy.interpreter.error import OperationError, get_cleared_operation_error
 from rpython.rlib.unroll import unrolling_iterable
 from rpython.rlib.objectmodel import specialize, not_rpython
 from rpython.rlib import jit, rgc, objectmodel
-from rpython.rlib.rarithmetic import r_uint
 
 TICK_COUNTER_STEP = 100
 
 def app_profile_call(space, w_callable, frame, event, w_arg):
-    # from here on, frame is just a normal w_object
-    frame = jit.hint(frame, access_directly=False)
     space.call_function(w_callable,
                         frame,
                         space.newtext(event), w_arg)
@@ -26,20 +23,24 @@ class ExecutionContext(object):
     # XXX [fijal] but they're not. is_being_profiled is guarded a bit all
     #     over the place as well as w_tracefunc
 
-    _immutable_fields_ = ['profilefunc?', 'w_tracefunc?']
+    _immutable_fields_ = ['profilefunc?', 'w_tracefunc?',
+                          'w_coroutine_wrapper_fn?']
 
     def __init__(self, space):
         self.space = space
         self.topframeref = jit.vref_None
+        # this is exposed to app-level as 'sys.exc_info()'.  At any point in
+        # time it is the exception caught by the topmost 'except ... as e:'
+        # app-level block.
+        self.sys_exc_operror = None
         self.w_tracefunc = None
         self.is_tracing = 0
         self.compiler = space.createcompiler()
         self.profilefunc = None
         self.w_profilefuncarg = None
         self.thread_disappeared = False   # might be set to True after os.fork()
-        # an instance of this will be raised the next time we switch to the
-        # thread that self represents
-        self.w_async_exception_type = None
+        self.w_coroutine_wrapper_fn = None
+        self.in_coroutine_wrapper = False
 
     @staticmethod
     def _mark_thread_disappeared(space):
@@ -172,7 +173,7 @@ class ExecutionContext(object):
 
     @jit.unroll_safe
     def run_trace_func(self, frame):
-        code = frame.getcode() # promote the frame!
+        code = frame.pycode
         d = frame.getorcreatedebug()
         if d.instr_lb <= frame.last_instr < d.instr_ub:
             if frame.last_instr < d.instr_prev_plus_one:
@@ -191,8 +192,10 @@ class ExecutionContext(object):
                 addr += c
                 if c:
                     d.instr_lb = addr
-
-                line += ord(lineno[p + 1])
+                line_offset = ord(lineno[p + 1])
+                if line_offset >= 0x80:
+                    line_offset -= 0x100
+                line += line_offset
                 p += 2
                 size -= 1
 
@@ -232,35 +235,17 @@ class ExecutionContext(object):
             self._trace(frame, 'exception', None, operationerr)
         #operationerr.print_detailed_traceback(self.space)
 
-    @jit.dont_look_inside
-    @specialize.arg(1)
-    def sys_exc_info(self, for_hidden=False):
+    def sys_exc_info(self):
         """Implements sys.exc_info().
         Return an OperationError instance or None.
 
-        Ignores exceptions within hidden frames unless for_hidden=True
-        is specified.
-
         # NOTE: the result is not the wrapped sys.exc_info() !!!
 
         """
-        return self.gettopframe()._exc_info_unroll(self.space, for_hidden)
+        return self.sys_exc_operror
 
     def set_sys_exc_info(self, operror):
-        frame = self.gettopframe_nohidden()
-        if frame:     # else, the exception goes nowhere and is lost
-            frame.last_exception = operror
-
-    def clear_sys_exc_info(self):
-        # Find the frame out of which sys_exc_info() would return its result,
-        # and hack this frame's last_exception to become the cleared
-        # OperationError (which is different from None!).
-        frame = self.gettopframe_nohidden()
-        while frame:
-            if frame.last_exception is not None:
-                frame.last_exception = get_cleared_operation_error(self.space)
-                break
-            frame = self.getnextframe_nohidden(frame)
+        self.sys_exc_operror = operror
 
     @jit.dont_look_inside
     def settrace(self, w_func):
@@ -333,6 +318,7 @@ class ExecutionContext(object):
 
         if w_callback is not None and event != "leaveframe":
             if operr is not None:
+                operr.normalize_exception(space)
                 w_value = operr.get_w_value(space)
                 w_arg = space.newtuple([operr.w_type, w_value,
                                         operr.get_w_traceback(space)])
@@ -349,8 +335,6 @@ class ExecutionContext(object):
                 if event == 'line':
                     d.is_in_line_tracing = True
                 try:
-                    # from here on, frame is just a normal w_object
-                    frame = jit.hint(frame, access_directly=False)
                     w_result = space.call_function(w_callback, frame, space.newtext(event), w_arg)
                     if space.is_w(w_result, space.w_None):
                         # bug-to-bug compatibility with CPython
@@ -377,7 +361,6 @@ class ExecutionContext(object):
                     event == 'c_exception'):
                 return
 
-            last_exception = frame.last_exception
             if event == 'leaveframe':
                 event = 'return'
 
@@ -393,7 +376,6 @@ class ExecutionContext(object):
                     raise
 
             finally:
-                frame.last_exception = last_exception
                 self.is_tracing -= 1
 
     def checksignals(self):
@@ -436,11 +418,8 @@ class AbstractActionFlag(object):
     def __init__(self):
         self._periodic_actions = []
         self._nonperiodic_actions = []
-        # a bitmask where the ith bit is set if self._nonperiodic_actions[i]
-        # has fired
-        self._fired_bitmask = r_uint(0)
-
         self.has_bytecode_counter = False
+        self._fired_actions_reset()
         # the default value is not 100, unlike CPython 2.7, but a much
         # larger value, because we use a technique that not only allows
         # but actually *forces* another thread to run whenever the counter
@@ -450,14 +429,30 @@ class AbstractActionFlag(object):
 
     def fire(self, action):
         """Request for the action to be run before the next opcode."""
-        assert action._action_index >= 0 # period actions must not call fire
-        mask = r_uint(1) << action._action_index
-        if not self._fired_bitmask & mask:
-            self._fired_bitmask |= mask
+        if not action._fired:
+            action._fired = True
+            self._fired_actions_append(action)
             # set the ticker to -1 in order to force action_dispatcher()
             # to run at the next possible bytecode
             self.reset_ticker(-1)
 
+    def _fired_actions_reset(self):
+        # linked list of actions. We cannot use a normal RPython list because
+        # we want AsyncAction.fire() to be marked as @rgc.collect: this way,
+        # we can call it from e.g. GcHooks or cpyext's dealloc_trigger.
+        self._fired_actions_first = None
+        self._fired_actions_last = None
+
+    @rgc.no_collect
+    def _fired_actions_append(self, action):
+        assert action._next is None
+        if self._fired_actions_first is None:
+            self._fired_actions_first = action
+            self._fired_actions_last = action
+        else:
+            self._fired_actions_last._next = action
+            self._fired_actions_last = action
+
     @not_rpython
     def register_periodic_action(self, action, use_bytecode_counter):
         """
@@ -478,13 +473,6 @@ class AbstractActionFlag(object):
             self._periodic_actions.insert(0, action)
         self._rebuild_action_dispatcher()
 
-    @not_rpython
-    def register_nonperiodic_action(self, action):
-        self._nonperiodic_actions.append(action)
-        assert len(self._nonperiodic_actions) < 32
-        self._rebuild_action_dispatcher()
-        return len(self._nonperiodic_actions) - 1
-
     def getcheckinterval(self):
         return self.checkinterval_scaled // TICK_COUNTER_STEP
 
@@ -509,25 +497,26 @@ class AbstractActionFlag(object):
                 action.perform(ec, frame)
 
             # nonperiodic actions
-            if self._fired_bitmask:
+            action = self._fired_actions_first
+            if action:
+                self._fired_actions_reset()
                 # NB. in case there are several actions, we reset each
-                # fired bit to 0  only when we're about to call
+                # 'action._fired' to false only when we're about to call
                 # 'action.perform()'.  This means that if
                 # 'action.fire()' happens to be called any time before
                 # the corresponding perform(), the fire() has no
                 # effect---which is the effect we want, because
-                # perform() will be called anyway.
-                for i in range(len(self._nonperiodic_actions)):
-                    mask = r_uint(1) << i
-                    if self._fired_bitmask & mask:
-                        action = self._nonperiodic_actions[i]
-                        self._fired_bitmask  &= ~mask
-                        action.perform(ec, frame)
-                # one of the actions with higher index re-triggered one of the
-                # earlier actions. don't run them in a loop here, execute some
-                # python code first.
-                if self._fired_bitmask:
-                    self.reset_ticker(-1)
+                # perform() will be called anyway.  All such pending
+                # actions with _fired == True are still inside the old
+                # chained list.  As soon as we reset _fired to False,
+                # we also reset _next to None and we are ready for
+                # another fire().
+                while action is not None:
+                    next_action = action._next
+                    action._next = None
+                    action._fired = False
+                    action.perform(ec, frame)
+                    action = next_action
 
         self.action_dispatcher = action_dispatcher
 
@@ -559,14 +548,11 @@ class AsyncAction(object):
     asynchronously with regular bytecode execution, but that still need
     to occur between two opcodes, not at a completely random time.
     """
+    _fired = False
+    _next = None
 
-    @not_rpython
     def __init__(self, space):
         self.space = space
-        if not isinstance(self, PeriodicAsyncAction):
-            self._action_index = self.space.actionflag.register_nonperiodic_action(self)
-        else:
-            self._action_index = -1
 
     @rgc.no_collect
     def fire(self):
@@ -643,9 +629,14 @@ class UserDelAction(AsyncAction):
             if self.gc_disabled(w_obj):
                 return
             try:
-                space.get_and_call_function(w_del, w_obj)
+                w_impl = space.get(w_del, w_obj)
             except Exception as e:
                 report_error(space, e, "method __del__ of ", w_obj)
+            else:
+                try:
+                    space.call_function(w_impl)
+                except Exception as e:
+                    report_error(space, e, '', w_impl)
 
         # Call the RPython-level _finalize_() method.
         try:
@@ -669,7 +660,7 @@ def report_error(space, e, where, w_obj):
 def make_finalizer_queue(W_Root, space):
     """Make a FinalizerQueue subclass which responds to GC finalizer
     events by 'firing' the UserDelAction class above.  It does not
-    directly fetches the objects to finalize at all; they stay in the 
+    directly fetches the objects to finalize at all; they stay in the
     GC-managed queue, and will only be fetched by UserDelAction
     (between bytecodes)."""
 
