@@ -35,25 +35,29 @@ class Function(W_Root):
                           'w_func_globals?',
                           'closure?[*]',
                           'defs_w?[*]',
-                          'name?']
+                          'name?',
+                          'qualname?',
+                          'w_kw_defs?']
 
-    _empty_defs = []
+    w_kw_defs = None
 
-    def __init__(self, space, code, w_globals=None, defs_w=[], closure=None,
-                 forcename=None):
+    def __init__(self, space, code, w_globals=None, defs_w=[], kw_defs_w=None,
+                 closure=None, w_ann=None, forcename=None, qualname=None):
         self.space = space
         self.name = forcename or code.co_name
+        self.qualname = qualname or self.name
+        assert isinstance(self.qualname, str)
         self.w_doc = None   # lazily read from code.getdocstring()
         self.code = code       # Code instance
         self.w_func_globals = w_globals  # the globals dictionary
         self.closure = closure    # normally, list of Cell instances or None
-        if not defs_w:
-            # in theory it would be cool to share *all* the empty lists in
-            # rpython where the identity doesn't matter
-            defs_w = Function._empty_defs
         self.defs_w = defs_w
         self.w_func_dict = None # filled out below if needed
         self.w_module = None
+        self.w_ann = w_ann
+        #
+        if kw_defs_w is not None:
+            self.init_kwdefaults_dict(kw_defs_w)
 
     def __repr__(self):
         # return "function %s.%s" % (self.space, self.name)
@@ -115,7 +119,7 @@ class Function(W_Root):
                 for i in funccallunrolling:
                     if i < nargs:
                         new_frame.locals_cells_stack_w[i] = args_w[i]
-                return new_frame.run()
+                return new_frame.run(self.name, self.qualname)
         elif nargs >= 1 and fast_natural_arity == Code.PASSTHROUGHARGS1:
             assert isinstance(code, gateway.BuiltinCodePassThroughArguments1)
             return code.funcrun_obj(self, args_w[0],
@@ -123,7 +127,7 @@ class Function(W_Root):
                                               list(args_w[1:])))
         return self.call_args(Arguments(self.space, list(args_w)))
 
-    def funccall_valuestack(self, nargs, frame, dropvalues, methodcall=False): # speed hack
+    def funccall_valuestack(self, nargs, frame, methodcall=False): # speed hack
         # methodcall is only for better error messages
         from pypy.interpreter import gateway
         from pypy.interpreter.pycode import PyCode
@@ -132,81 +136,67 @@ class Function(W_Root):
         #
         if (jit.we_are_jitted() and code is self.space._code_of_sys_exc_info
                                 and nargs == 0):
-            frame.dropvalues(dropvalues)
             from pypy.module.sys.vm import exc_info_direct
             return exc_info_direct(self.space, frame)
         #
         fast_natural_arity = code.fast_natural_arity
         if nargs == fast_natural_arity:
             if nargs == 0:
-                frame.dropvalues(dropvalues)
                 assert isinstance(code, gateway.BuiltinCode0)
                 return code.fastcall_0(self.space, self)
             elif nargs == 1:
-                f_0 = frame.peekvalue(0)
-                frame.dropvalues(dropvalues)
                 assert isinstance(code, gateway.BuiltinCode1)
-                return code.fastcall_1(self.space, self, f_0)
+                return code.fastcall_1(self.space, self, frame.peekvalue(0))
             elif nargs == 2:
-                f_0 = frame.peekvalue(0)
-                f_1 = frame.peekvalue(1)
-                frame.dropvalues(dropvalues)
                 assert isinstance(code, gateway.BuiltinCode2)
-                return code.fastcall_2(self.space, self, f_1, f_0)
+                return code.fastcall_2(self.space, self, frame.peekvalue(1),
+                                       frame.peekvalue(0))
             elif nargs == 3:
                 assert isinstance(code, gateway.BuiltinCode3)
-                f_0 = frame.peekvalue(0)
-                f_1 = frame.peekvalue(1)
-                f_2 = frame.peekvalue(2)
-                frame.dropvalues(dropvalues)
-                return code.fastcall_3(self.space, self, f_2,
-                                       f_1, f_0)
+                return code.fastcall_3(self.space, self, frame.peekvalue(2),
+                                       frame.peekvalue(1), frame.peekvalue(0))
             elif nargs == 4:
                 assert isinstance(code, gateway.BuiltinCode4)
-                f_0 = frame.peekvalue(0)
-                f_1 = frame.peekvalue(1)
-                f_2 = frame.peekvalue(2)
-                f_3 = frame.peekvalue(3)
-                frame.dropvalues(dropvalues)
-                return code.fastcall_4(self.space, self, f_3, f_2, f_1, f_0)
+                return code.fastcall_4(self.space, self, frame.peekvalue(3),
+                                       frame.peekvalue(2), frame.peekvalue(1),
+                                        frame.peekvalue(0))
         elif (nargs | Code.FLATPYCALL) == fast_natural_arity:
             assert isinstance(code, PyCode)
-            return self._flat_pycall(code, nargs, frame, dropvalues)
+            return self._flat_pycall(code, nargs, frame)
         elif fast_natural_arity & Code.FLATPYCALL:
             natural_arity = fast_natural_arity & 0xff
             if natural_arity > nargs >= natural_arity - len(self.defs_w):
                 assert isinstance(code, PyCode)
                 return self._flat_pycall_defaults(code, nargs, frame,
-                                                  natural_arity - nargs, dropvalues)
+                                                  natural_arity - nargs)
         elif fast_natural_arity == Code.PASSTHROUGHARGS1 and nargs >= 1:
             assert isinstance(code, gateway.BuiltinCodePassThroughArguments1)
             w_obj = frame.peekvalue(nargs-1)
-            args = frame.make_arguments(nargs-1)
-            frame.dropvalues(dropvalues)
+            args = frame.make_arguments(nargs-1, w_function=self)
             return code.funcrun_obj(self, w_obj, args)
 
-        args = frame.make_arguments(nargs, methodcall=methodcall)
-        frame.dropvalues(dropvalues)
+        args = frame.make_arguments(nargs, methodcall=methodcall, w_function=self)
         return self.call_args(args)
 
     @jit.unroll_safe
-    def _flat_pycall(self, code, nargs, frame, dropvalues):
+    def _flat_pycall(self, code, nargs, frame):
         # code is a PyCode
         new_frame = self.space.createframe(code, self.w_func_globals,
                                                    self)
         for i in xrange(nargs):
-            new_frame.locals_cells_stack_w[i] = frame.peekvalue(nargs-1-i)
+            w_arg = frame.peekvalue(nargs-1-i)
+            new_frame.locals_cells_stack_w[i] = w_arg
 
-        frame.dropvalues(dropvalues)
-        return new_frame.run()
+        return new_frame.run(self.name, self.qualname)
 
     @jit.unroll_safe
-    def _flat_pycall_defaults(self, code, nargs, frame, defs_to_load, dropvalues):
+    def _flat_pycall_defaults(self, code, nargs, frame, defs_to_load):
         # code is a PyCode
         new_frame = self.space.createframe(code, self.w_func_globals,
                                                    self)
         for i in xrange(nargs):
-            new_frame.locals_cells_stack_w[i] = frame.peekvalue(nargs-1-i)
+            w_arg = frame.peekvalue(nargs-1-i)
+            new_frame.locals_cells_stack_w[i] = w_arg
 
         ndefs = len(self.defs_w)
         start = ndefs - defs_to_load
@@ -214,8 +204,7 @@ class Function(W_Root):
         for j in xrange(start, ndefs):
             new_frame.locals_cells_stack_w[i] = self.defs_w[j]
             i += 1
-        frame.dropvalues(dropvalues)
-        return new_frame.run()
+        return new_frame.run(self.name, self.qualname)
 
     def getdict(self, space):
         if self.w_func_dict is None:
@@ -259,14 +248,15 @@ class Function(W_Root):
                 raise oefmt(space.w_ValueError, "closure is wrong size")
             closure = [space.interp_w(Cell, w_cell) for w_cell in closure_w]
         func = space.allocate_instance(Function, w_subtype)
-        Function.__init__(func, space, code, w_globals, defs_w, closure, name)
+        Function.__init__(func, space, code, w_globals, defs_w, None, closure,
+                          None,name)
         return func
 
     def descr_function_call(self, __args__):
         return self.call_args(__args__)
 
     def descr_function_repr(self):
-        return self.getrepr(self.space, 'function %s' % (self.name,))
+        return self.getrepr(self.space, 'function %s' % self.qualname)
 
 
     def _cleanup_(self):
@@ -291,6 +281,7 @@ class Function(W_Root):
     find = staticmethod(find)
 
     def descr_function__reduce__(self, space):
+        XXX   # This is not used any more
         from pypy.interpreter.gateway import BuiltinCode
         from pypy.interpreter.mixedmodule import MixedModule
         w_mod = space.getbuiltinmodule('_pickle_support')
@@ -298,8 +289,8 @@ class Function(W_Root):
         code = self.code
         if isinstance(code, BuiltinCode):
             new_inst = mod.get('builtin_function')
-            return space.newtuple2(new_inst,
-                                   space.newtuple([space.newtext(code.identifier)]))
+            return space.newtuple([new_inst,
+                                   space.newtuple([space.newtext(code.identifier)])])
 
         new_inst = mod.get('func_new')
         if self.closure is None:
@@ -323,6 +314,7 @@ class Function(W_Root):
         tup_base = []
         tup_state = [
             space.newtext(self.name),
+            space.newtext(self.qualname),
             w_doc,
             self.code,
             w_func_globals,
@@ -334,10 +326,11 @@ class Function(W_Root):
         return nt([new_inst, nt(tup_base), nt(tup_state)])
 
     def descr_function__setstate__(self, space, w_args):
+        XXX   # This is not used any more
         args_w = space.unpackiterable(w_args)
         try:
-            (w_name, w_doc, w_code, w_func_globals, w_closure, w_defs,
-             w_func_dict, w_module) = args_w
+            (w_name, w_qualname, w_doc, w_code, w_func_globals, w_closure,
+             w_defs, w_func_dict, w_module) = args_w
         except ValueError:
             # wrong args
             raise oefmt(space.w_ValueError,
@@ -345,6 +338,7 @@ class Function(W_Root):
 
         self.space = space
         self.name = space.text_w(w_name)
+        self.qualname = space.utf8_w(w_qualname)
         self.code = space.interp_w(Code, w_code)
         if not space.is_w(w_closure, space.w_None):
             from pypy.interpreter.nestedscope import Cell
@@ -364,11 +358,6 @@ class Function(W_Root):
         self.defs_w = space.fixedview(w_defs)
         self.w_module = w_module
 
-    def _check_code_mutable(self, attr):
-        if not self.can_change_code:
-            raise oefmt(self.space.w_TypeError,
-                        "Cannot change %s attribute of builtin functions", attr)
-
     def fget_func_defaults(self, space):
         values_w = self.defs_w
         # the `None in values_w` check here is to ensure that interp-level
@@ -378,8 +367,16 @@ class Function(W_Root):
             return space.w_None
         return space.newtuple(values_w)
 
+    def fget_defaults_count(self, space):
+        # PyPy extension.  This is mostly to make inspect.py happy about
+        # methods like dict.pop(), which take a default but is None.
+        # Previously, inspect.py would build a signature saying that all
+        # arguments are required, and then unittest.mock would complain
+        # if we call it with less arguments.  Now inspect.py looks up
+        # '__defaults_count__' too.
+        return space.newint(len(self.defs_w))
+
     def fset_func_defaults(self, space, w_defaults):
-        self._check_code_mutable("func_defaults")
         if space.is_w(w_defaults, space.w_None):
             self.defs_w = []
             return
@@ -389,33 +386,65 @@ class Function(W_Root):
         self.defs_w = space.fixedview(w_defaults)
 
     def fdel_func_defaults(self, space):
-        self._check_code_mutable("func_defaults")
         self.defs_w = []
 
+    def fget_func_kwdefaults(self, space):
+        if self.w_kw_defs is None:
+            return space.w_None
+        return self.w_kw_defs
+
+    def fset_func_kwdefaults(self, space, w_new):
+        if space.is_w(w_new, space.w_None):
+            self.w_kw_defs = None
+        else:
+            if not space.isinstance_w(w_new, space.w_dict):
+                raise oefmt(space.w_TypeError, "__kwdefaults__ must be a dict")
+            self.w_kw_defs = w_new
+
+    def fdel_func_kwdefaults(self, space):
+        self.w_kw_defs = None
+
+    def init_kwdefaults_dict(self, kw_defs_w):
+        # use the moduledict logic to get normally-constant entries
+        space = self.space
+        w_dict = space.newdict(module=True)
+        for w_name, w_value in kw_defs_w:
+            space.setitem(w_dict, w_name, w_value)
+        self.w_kw_defs = w_dict
+
     def fget_func_doc(self, space):
         if self.w_doc is None:
             self.w_doc = self.code.getdocstring(space)
         return self.w_doc
 
     def fset_func_doc(self, space, w_doc):
-        self._check_code_mutable("func_doc")
         self.w_doc = w_doc
 
-    def fdel_func_doc(self, space):
-        self._check_code_mutable("func_doc")
-        self.w_doc = space.w_None
-
     def fget_func_name(self, space):
         return space.newtext(self.name)
 
     def fset_func_name(self, space, w_name):
-        self._check_code_mutable("func_name")
         if space.isinstance_w(w_name, space.w_text):
             self.name = space.text_w(w_name)
         else:
             raise oefmt(space.w_TypeError,
                         "__name__ must be set to a string object")
 
+    def fget_func_qualname(self, space):
+        return space.newtext(self.qualname)
+
+    def fset_func_qualname(self, space, w_name):
+        try:
+            self.qualname = space.realutf8_w(w_name)
+        except OperationError as e:
+            if e.match(space, space.w_TypeError):
+                raise oefmt(space.w_TypeError,
+                            "__qualname__ must be set to a string object")
+            raise
+
+    def fdel_func_doc(self, space):
+        self.w_doc = space.w_None
+
     def fget___module__(self, space):
         if self.w_module is None:
             if self.w_func_globals is not None and not space.is_w(self.w_func_globals, space.w_None):
@@ -425,11 +454,9 @@ class Function(W_Root):
         return self.w_module
 
     def fset___module__(self, space, w_module):
-        self._check_code_mutable("func_name")
         self.w_module = w_module
 
     def fdel___module__(self, space):
-        self._check_code_mutable("func_name")
         self.w_module = space.w_None
 
     def fget_func_code(self, space):
@@ -458,112 +485,79 @@ class Function(W_Root):
             w_res = space.w_None
         return w_res
 
+    def fget_func_annotations(self, space):
+        if self.w_ann is None:
+            self.w_ann = space.newdict()
+        return self.w_ann
+
+    def fset_func_annotations(self, space, w_new):
+        if space.is_w(w_new, space.w_None):
+            w_new = None
+        elif not space.isinstance_w(w_new, space.w_dict):
+            raise oefmt(space.w_TypeError, "__annotations__ must be a dict")
+        self.w_ann = w_new
+
+    def fdel_func_annotations(self, space):
+        self.w_ann = None
+
 
 def descr_function_get(space, w_function, w_obj, w_cls=None):
     """functionobject.__get__(obj[, type]) -> method"""
     # this is not defined as a method on Function because it's generally
     # useful logic: w_function can be any callable.  It is used by Method too.
-    asking_for_bound = (space.is_none(w_cls) or
-                        not space.is_w(w_obj, space.w_None) or
-                        space.is_w(w_cls, space.type(space.w_None)))
-    if asking_for_bound:
-        return Method(space, w_function, w_obj, w_cls)
+    if w_obj is None or space.is_w(w_obj, space.w_None):
+        return w_function
     else:
-        return Method(space, w_function, None, w_cls)
+        return Method(space, w_function, w_obj)
 
 
 class Method(W_Root):
-    """A method is a function bound to a specific instance or class."""
-    _immutable_fields_ = ['w_function', 'w_instance', 'w_class']
+    """A method is a function bound to a specific instance."""
+    _immutable_fields_ = ['w_function', 'w_instance']
 
-    def __init__(self, space, w_function, w_instance, w_class):
+    def __init__(self, space, w_function, w_instance):
         self.space = space
+        assert w_instance is not None   # unbound methods only exist in Python 2
         self.w_function = w_function
-        self.w_instance = w_instance   # or None
-        if w_class is None:
-            w_class = space.w_None
-        self.w_class = w_class         # possibly space.w_None
+        self.w_instance = w_instance
 
-    def descr_method__new__(space, w_subtype, w_function, w_instance,
-                            w_class=None):
+    def descr_method__new__(space, w_subtype, w_function, w_instance):
         if space.is_w(w_instance, space.w_None):
             w_instance = None
-        if w_instance is None and space.is_none(w_class):
-            raise oefmt(space.w_TypeError, "unbound methods must have class")
+        if w_instance is None:
+            raise oefmt(space.w_TypeError, "self must not be None")
         method = space.allocate_instance(Method, w_subtype)
-        Method.__init__(method, space, w_function, w_instance, w_class)
+        Method.__init__(method, space, w_function, w_instance)
         return method
 
     def __repr__(self):
-        if self.w_instance:
-            pre = "bound"
-        else:
-            pre = "unbound"
-        return "%s method %s" % (pre, self.w_function.getname(self.space))
+        return u"bound method %s" % (self.w_function.getname(self.space),)
 
     def call_args(self, args):
         space = self.space
-        if self.w_instance is not None:
-            # bound method
-            return space.call_obj_args(self.w_function, self.w_instance, args)
-
-        # unbound method
-        w_firstarg = args.firstarg()
-        if w_firstarg is not None and (
-                space.abstract_isinstance_w(w_firstarg, self.w_class)):
-            pass  # ok
-        else:
-            clsdescr = self.w_class.getname(space)
-            if clsdescr and clsdescr != '?':
-                clsdescr += " instance"
-            else:
-                clsdescr = "instance"
-            if w_firstarg is None:
-                instdescr = "nothing"
-            else:
-                instname = space.abstract_getclass(w_firstarg).getname(space)
-                if instname and instname != '?':
-                    instdescr = instname + " instance"
-                else:
-                    instdescr = "instance"
-            raise oefmt(space.w_TypeError,
-                        "unbound method %N() must be called with %s as first "
-                        "argument (got %s instead)", self, clsdescr, instdescr)
-        return space.call_args(self.w_function, args)
+        return space.call_obj_args(self.w_function, self.w_instance, args)
 
     def descr_method_get(self, w_obj, w_cls=None):
-        space = self.space
-        if self.w_instance is not None:
-            return self    # already bound
-        else:
-            # only allow binding to a more specific class than before
-            if (w_cls is not None and
-                not space.is_w(w_cls, space.w_None) and
-                not space.abstract_issubclass_w(w_cls, self.w_class,
-                                                allow_override=True)):
-                return self    # subclass test failed
-            else:
-                return descr_function_get(space, self.w_function, w_obj, w_cls)
+        return self     # already bound
 
     def descr_method_call(self, __args__):
         return self.call_args(__args__)
 
     def descr_method_repr(self):
         space = self.space
-        name = self.w_function.getname(self.space)
-        # XXX do we handle all cases sanely here?
-        if space.is_w(self.w_class, space.w_None):
-            w_class = space.type(self.w_instance)
+        w_name = space.findattr(self.w_function, space.newtext('__qualname__'))
+        if w_name is None:
+            name = self.w_function.getname(self.space)
         else:
-            w_class = self.w_class
-        typename = w_class.getname(self.space)
-        if self.w_instance is None:
-            s = "<unbound method %s.%s>" % (typename, name)
-            return space.newtext(s)
-        else:
-            objrepr = space.text_w(space.repr(self.w_instance))
-            s = '<bound method %s.%s of %s>' % (typename, name, objrepr)
-            return space.newtext(s)
+            try:
+                name = space.utf8_w(w_name)
+            except OperationError as e:
+                if not e.match(space, space.w_TypeError):
+                    raise
+                name = '?'
+        objrepr = space.utf8_w(space.repr(self.w_instance))
+        s = b'<bound method %s of %s>' % (name, objrepr)
+        return space.newtext(s)
 
     def descr_method_getattribute(self, w_attr):
         space = self.space
@@ -581,60 +575,14 @@ class Method(W_Root):
         space = self.space
         if not isinstance(w_other, Method):
             return space.w_NotImplemented
-        return space.newbool(self._eq(w_other))
-
-    def descr_method_ne(self, w_other):
-        space = self.space
-        if not isinstance(w_other, Method):
-            return space.w_NotImplemented
-        return space.newbool(not self._eq(w_other))
-
-    def _eq(self, w_other):
-        space = self.space
-        if not space.eq_w(self.w_function, w_other.w_function):
-            return False
-        if self.w_instance is None:
-            if w_other.w_instance is not None:
-                return False
-            return True
-        else:
-            if w_other.w_instance is None:
-                return False
-            return space.eq_w(self.w_instance, w_other.w_instance)
-
-    def is_w(self, space, other):
-        if self.w_instance is not None:
-            return W_Root.is_w(self, space, other)
-        # The following special-case is only for *unbound* method objects.
-        # Motivation: in CPython, it seems that no strange internal type
-        # exists where the equivalent of ``x.method is x.method`` would
-        # return True.  This is unlike unbound methods, where e.g.
-        # ``list.append is list.append`` returns True.  The following code
-        # is here to emulate that behaviour.  Unlike CPython, we return
-        # True for all equal unbound methods, not just for built-in types.
-        if not isinstance(other, Method):
-            return False
-        return (other.w_instance is None and
-                self.w_function is other.w_function and
-                self.w_class is other.w_class)
-
-    def immutable_unique_id(self, space):
-        if self.w_instance is not None:
-            return W_Root.immutable_unique_id(self, space)
-        # the special-case is only for *unbound* method objects
-        #
-        from pypy.objspace.std.util import IDTAG_UNBOUND_METHOD as tag
-        from pypy.objspace.std.util import IDTAG_SHIFT
-        id = space.bigint_w(space.id(self.w_function))
-        id = id.lshift(LONG_BIT).or_(space.bigint_w(space.id(self.w_class)))
-        id = id.lshift(IDTAG_SHIFT).int_or_(tag)
-        return space.newlong_from_rbigint(id)
+        if not space.eq_w(self.w_instance, w_other.w_instance):
+            return space.w_False
+        return space.newbool(space.eq_w(self.w_function, w_other.w_function))
 
     def descr_method_hash(self):
         space = self.space
         w_result = space.hash(self.w_function)
-        if self.w_instance is not None:
-            w_result = space.xor(w_result, space.hash(self.w_instance))
+        w_result = space.xor(w_result, space.hash(self.w_instance))
         return w_result
 
     def descr_method__reduce__(self, space):
@@ -642,21 +590,17 @@ class Method(W_Root):
         from pypy.interpreter.gateway import BuiltinCode
         w_mod    = space.getbuiltinmodule('_pickle_support')
         mod      = space.interp_w(MixedModule, w_mod)
-        new_inst = mod.get('method_new')
-        w_instance = self.w_instance or space.w_None
+        w_instance = self.w_instance
         w_function = self.w_function
         if (isinstance(w_function, Function) and
                 isinstance(w_function.code, BuiltinCode)):
             new_inst = mod.get('builtin_method_new')
-            if space.is_w(w_instance, space.w_None):
-                tup = [self.w_class, space.newtext(w_function.name)]
-            else:
-                tup = [w_instance, space.newtext(w_function.name)]
-        elif space.is_w(self.w_class, space.w_None):
-            tup = [self.w_function, w_instance]
+            tup = [w_instance, space.newtext(w_function.name)]
         else:
-            tup = [self.w_function, w_instance, self.w_class]
-        return space.newtuple2(new_inst, space.newtuple(tup))
+            w_builtins = space.getbuiltinmodule('builtins')
+            new_inst = space.getattr(w_builtins, space.newtext('getattr'))
+            tup = [w_instance, space.newtext(w_function.getname(space))]
+        return space.newtuple([new_inst, space.newtuple(tup)])
 
 
 class StaticMethod(W_Root):
@@ -665,6 +609,19 @@ class StaticMethod(W_Root):
 
     def __init__(self, w_function):
         self.w_function = w_function
+        self.w_dict = None
+
+    def getdict(self, space):
+        if self.w_dict is None:
+            self.w_dict = space.newdict(instance=True)
+        return self.w_dict
+
+    def setdict(self, space, w_dict):
+        if not space.isinstance_w(w_dict, space.w_dict):
+            raise oefmt(space.w_TypeError,
+                        "__dict__ must be set to a dictionary, not a %T",
+                        w_dict)
+        self.w_dict = w_dict
 
     def descr_staticmethod_get(self, w_obj, w_cls=None):
         """staticmethod(x).__get__(obj[, type]) -> x"""
@@ -678,6 +635,9 @@ class StaticMethod(W_Root):
     def descr_init(self, space, w_function):
         self.w_function = w_function
 
+    def descr_isabstract(self, space):
+        return space.newbool(space.isabstractmethod_w(self.w_function))
+
 
 class ClassMethod(W_Root):
     """The classmethod objects."""
@@ -685,12 +645,24 @@ class ClassMethod(W_Root):
 
     def __init__(self, w_function):
         self.w_function = w_function
+        self.w_dict = None
+
+    def getdict(self, space):
+        if self.w_dict is None:
+            self.w_dict = space.newdict(instance=True)
+        return self.w_dict
+
+    def setdict(self, space, w_dict):
+        if not space.isinstance_w(w_dict, space.w_dict):
+            raise oefmt(space.w_TypeError,
+                        "__dict__ must be set to a dictionary, not a %T",
+                        w_dict)
+        self.w_dict = w_dict
 
     def descr_classmethod_get(self, space, w_obj, w_klass=None):
         if space.is_none(w_klass):
             w_klass = space.type(w_obj)
-        return Method(space, self.w_function, w_klass,
-                      space.type(w_klass))
+        return Method(space, self.w_function, w_klass)
 
     def descr_classmethod__new__(space, w_subtype, w_function):
         instance = space.allocate_instance(ClassMethod, w_subtype)
@@ -700,6 +672,10 @@ class ClassMethod(W_Root):
     def descr_init(self, space, w_function):
         self.w_function = w_function
 
+    def descr_isabstract(self, space):
+        return space.newbool(space.isabstractmethod_w(self.w_function))
+
+
 class FunctionWithFixedCode(Function):
     can_change_code = False
 
@@ -709,10 +685,12 @@ class BuiltinFunction(Function):
     def __init__(self, func):
         assert isinstance(func, Function)
         Function.__init__(self, func.space, func.code, func.w_func_globals,
-                          func.defs_w, func.closure, func.name)
+                          func.defs_w, None, func.closure,
+                          None, func.name)
         self.w_doc = func.w_doc
         self.w_func_dict = func.w_func_dict
         self.w_module = func.w_module
+        self.w_kw_defs = func.w_kw_defs
 
     def descr_builtinfunction__new__(space, w_subtype):
         raise oefmt(space.w_TypeError,
@@ -721,6 +699,9 @@ class BuiltinFunction(Function):
     def descr_function_repr(self):
         return self.space.newtext('<built-in function %s>' % (self.name,))
 
+    def descr__reduce__(self, space):
+        return space.newtext(self.qualname)
+
 def is_builtin_code(w_func):
     from pypy.interpreter.gateway import BuiltinCode
     if isinstance(w_func, Method):
