@@ -4,7 +4,7 @@ PyCode instances have the same co_xxx arguments as CPython code objects.
 The bytecode interpreter itself is implemented by the PyFrame class.
 """
 
-import dis, imp, struct, types, new, sys, os
+import imp, struct, types, sys, os
 
 from pypy.interpreter import eval
 from pypy.interpreter.signature import Signature
@@ -12,12 +12,13 @@ from pypy.interpreter.error import OperationError, oefmt
 from pypy.interpreter.gateway import unwrap_spec
 from pypy.interpreter.astcompiler.consts import (
     CO_OPTIMIZED, CO_NEWLOCALS, CO_VARARGS, CO_VARKEYWORDS, CO_NESTED,
-    CO_GENERATOR, CO_KILL_DOCSTRING, CO_YIELD_INSIDE_TRY)
+    CO_GENERATOR, CO_COROUTINE, CO_KILL_DOCSTRING, CO_YIELD_INSIDE_TRY,
+    CO_ITERABLE_COROUTINE, CO_ASYNC_GENERATOR)
+from pypy.tool import dis3
 from pypy.tool.stdlib_opcode import opcodedesc, HAVE_ARGUMENT
-from rpython.rlib.rarithmetic import intmask, r_longlong
+from rpython.rlib.rarithmetic import intmask
 from rpython.rlib.objectmodel import compute_hash, we_are_translated
-from rpython.rlib import jit
-from rpython.rlib.debug import debug_start, debug_stop, debug_print
+from rpython.rlib import jit, rstring
 
 
 class BytecodeCorruption(Exception):
@@ -32,22 +33,45 @@ def unpack_text_tuple(space, w_str_tuple):
 # Magic numbers for the bytecode version in code objects.
 # See comments in pypy/module/imp/importing.
 cpython_magic, = struct.unpack("<i", imp.get_magic())   # host magic number
-default_magic = (0xf303 + 7) | 0x0a0d0000               # this PyPy's magic
-                                                        # (from CPython 2.7.0)
 
-def make_signature(code):
+# we compute the magic number in a similar way to CPython, but we use a
+# different value for the highest 16 bits. Bump pypy_incremental_magic every
+# time you make pyc files incompatible.  This value ends up in the frozen
+# importlib, via MAGIC_NUMBER in module/_frozen_importlib/__init__.
+
+pypy_incremental_magic = 160 # bump it by 16
+assert pypy_incremental_magic % 16 == 0
+assert pypy_incremental_magic < 3000 # the magic number of Python 3. There are
+                                     # no known magic numbers below this value
+default_magic = pypy_incremental_magic | (ord('\r')<<16) | (ord('\n')<<24)
+
+# cpython_code_signature helper
+def cpython_code_signature(code):
     """Return a Signature instance."""
     argcount = code.co_argcount
     varnames = code.co_varnames
+    if we_are_translated():
+        kwonlyargcount = code.co_kwonlyargcount
+    else:
+        # for compatibility with CPython 2.7 code objects
+        kwonlyargcount = getattr(code, 'co_kwonlyargcount', 0)
     assert argcount >= 0     # annotator hint
+    assert kwonlyargcount >= 0
     argnames = list(varnames[:argcount])
+    if argcount < len(varnames):
+        kwonlyargs = list(varnames[argcount:argcount + kwonlyargcount])
+    else:
+        kwonlyargs = None
     if code.co_flags & CO_VARARGS:
         varargname = varnames[argcount]
         argcount += 1
     else:
         varargname = None
-    kwargname = varnames[argcount] if code.co_flags & CO_VARKEYWORDS else None
-    return Signature(argnames, varargname, kwargname)
+    if code.co_flags & CO_VARKEYWORDS:
+        kwargname = code.co_varnames[argcount + kwonlyargcount]
+    else:
+        kwargname = None
+    return Signature(argnames, varargname, kwargname, kwonlyargs)
 
 class CodeHookCache(object):
     def __init__(self, space):
@@ -55,16 +79,14 @@ class CodeHookCache(object):
 
 class PyCode(eval.Code):
     "CPython-style code objects."
-    _immutable_fields_ = ["_signature", "co_argcount", "co_cellvars[*]",
-                          "co_code", "co_consts_w[*]", "co_filename",
+    _immutable_fields_ = ["_signature", "co_argcount", "co_kwonlyargcount", "co_cellvars[*]",
+                          "co_code", "co_consts_w[*]", "co_filename", "w_filename",
                           "co_firstlineno", "co_flags", "co_freevars[*]",
                           "co_lnotab", "co_names_w[*]", "co_nlocals",
                           "co_stacksize", "co_varnames[*]",
-                          "_args_as_cellvars[*]",
-                          "w_globals?",
-                          "cell_families[*]"]
+                          "_args_as_cellvars[*]", "w_globals?"]
 
-    def __init__(self, space,  argcount, nlocals, stacksize, flags,
+    def __init__(self, space,  argcount, kwonlyargcount, nlocals, stacksize, flags,
                      code, consts, names, varnames, filename,
                      name, firstlineno, lnotab, freevars, cellvars,
                      hidden_applevel=False, magic=default_magic):
@@ -74,16 +96,22 @@ class PyCode(eval.Code):
         eval.Code.__init__(self, name)
         assert nlocals >= 0
         self.co_argcount = argcount
+        self.co_kwonlyargcount = kwonlyargcount
         self.co_nlocals = nlocals
         self.co_stacksize = stacksize
         self.co_flags = flags
         self.co_code = code
         self.co_consts_w = consts
-        self.co_names_w = [space.new_interned_str(aname) for aname in names]
+        self.co_names_w = [
+            space.new_interned_w_str(space.newtext(aname))
+            for aname in names]
         self.co_varnames = varnames
         self.co_freevars = freevars
         self.co_cellvars = cellvars
+        assert isinstance(filename, str)
+        rstring.check_str0(filename)
         self.co_filename = filename
+        self.w_filename = space.newfilename(filename)
         self.co_name = name
         self.co_firstlineno = firstlineno
         self.co_lnotab = lnotab
@@ -93,7 +121,7 @@ class PyCode(eval.Code):
         self.w_globals = None
         self.hidden_applevel = hidden_applevel
         self.magic = magic
-        self._signature = make_signature(self)
+        self._signature = cpython_code_signature(self)
         self._initialize()
         self._init_ready()
         self.new_code_hook()
@@ -116,27 +144,43 @@ class PyCode(eval.Code):
 
     def _initialize(self):
         from pypy.objspace.std.mapdict import init_mapdict_cache
-        from pypy.interpreter.nestedscope import CellFamily
         if self.co_cellvars:
             argcount = self.co_argcount
+            argcount += self.co_kwonlyargcount
             assert argcount >= 0     # annotator hint
             if self.co_flags & CO_VARARGS:
                 argcount += 1
             if self.co_flags & CO_VARKEYWORDS:
                 argcount += 1
+            # Cell vars could shadow already-set arguments.
+            # The compiler used to be clever about the order of
+            # the variables in both co_varnames and co_cellvars, but
+            # it no longer is for the sake of simplicity.  Moreover
+            # code objects loaded from CPython don't necessarily follow
+            # an order, which could lead to strange bugs if .pyc files
+            # produced by CPython are loaded by PyPy.  Note that CPython
+            # contains the following bad-looking nested loops at *every*
+            # function call!
+
+            # Precompute what arguments need to be copied into cellvars
+            args_as_cellvars = []
             argvars = self.co_varnames
             cellvars = self.co_cellvars
-            args_as_cellvars = _compute_args_as_cellvars(argvars, cellvars, argcount)
-            self._args_as_cellvars = args_as_cellvars
-            self.cell_families = [CellFamily(name) for name in cellvars]
+            for i in range(len(cellvars)):
+                cellname = cellvars[i]
+                for j in range(argcount):
+                    if cellname == argvars[j]:
+                        # argument j has the same name as the cell var i
+                        while len(args_as_cellvars) <= i:
+                            args_as_cellvars.append(-1)   # pad
+                        args_as_cellvars[i] = j
+            self._args_as_cellvars = args_as_cellvars[:]
         else:
             self._args_as_cellvars = []
-            self.cell_families = []
 
         self._compute_flatcall()
 
         init_mapdict_cache(self)
-        self._globals_caches = [None] * len(self.co_names_w)
 
     def _init_ready(self):
         "This is a hook for the vmprof module, which overrides this method."
@@ -149,7 +193,8 @@ class PyCode(eval.Code):
         #     <builtin>/lastdirname/basename.py
         # instead of freezing the complete translation-time path.
         filename = self.co_filename
-        if filename.startswith('<builtin>'):
+        if (filename.startswith('<builtin>') or
+            filename == '<frozen importlib._bootstrap>'):
             return
         filename = filename.lstrip('<').rstrip('>')
         if filename.lower().endswith('.pyc'):
@@ -159,8 +204,9 @@ class PyCode(eval.Code):
         if lastdirname:
             basename = '%s/%s' % (lastdirname, basename)
         self.co_filename = '<builtin>/%s' % (basename,)
+        self.w_filename = self.space.newfilename(self.co_filename)
 
-    co_names = property(lambda self: [self.space.unwrap(w_name) for w_name in self.co_names_w]) # for trace
+    co_names = property(lambda self: [self.space.text_w(w_name) for w_name in self.co_names_w]) # for trace
 
     def signature(self):
         return self._signature
@@ -170,6 +216,7 @@ class PyCode(eval.Code):
         """
         Hack to initialize the code object from a real (CPython) one.
         """
+        raise TypeError("assert reinterpretation for applevel tests is broken on PyPy3!")
         assert isinstance(code, types.CodeType)
         newconsts_w = [None] * len(code.co_consts)
         num = 0
@@ -183,6 +230,7 @@ class PyCode(eval.Code):
         # stick the underlying CPython magic value, if the code object
         # comes from there
         return cls(space, code.co_argcount,
+                      getattr(code, 'co_kwonlyargcount', 0),
                       code.co_nlocals,
                       code.co_stacksize,
                       code.co_flags,
@@ -205,6 +253,8 @@ class PyCode(eval.Code):
             return
         if len(self._args_as_cellvars) > 0:
             return
+        if self.co_kwonlyargcount > 0:
+            return
         if self.co_argcount > 0xff:
             return
 
@@ -218,9 +268,9 @@ class PyCode(eval.Code):
         fresh_frame = jit.hint(frame, access_directly=True,
                                       fresh_virtualizable=True)
         args.parse_into_scope(None, fresh_frame.locals_cells_stack_w, func.name,
-                              sig, func.defs_w)
+                              sig, func.defs_w, func.w_kw_defs)
         fresh_frame.init_cells()
-        return frame.run()
+        return frame.run(func.name, func.qualname)
 
     def funcrun_obj(self, func, w_obj, args):
         frame = self.space.createframe(self, func.w_func_globals,
@@ -230,9 +280,9 @@ class PyCode(eval.Code):
         fresh_frame = jit.hint(frame, access_directly=True,
                                       fresh_virtualizable=True)
         args.parse_into_scope(w_obj, fresh_frame.locals_cells_stack_w, func.name,
-                              sig, func.defs_w)
+                              sig, func.defs_w, func.w_kw_defs)
         fresh_frame.init_cells()
-        return frame.run()
+        return frame.run(func.name, func.qualname)
 
     def getvarnames(self):
         return self.co_varnames
@@ -240,7 +290,7 @@ class PyCode(eval.Code):
     def getdocstring(self, space):
         if self.co_consts_w:   # it is probably never empty
             w_first = self.co_consts_w[0]
-            if space.isinstance_w(w_first, space.w_basestring):
+            if space.isinstance_w(w_first, space.w_unicode):
                 return w_first
         return space.w_None
 
@@ -251,42 +301,15 @@ class PyCode(eval.Code):
             if isinstance(w_co, PyCode):
                 w_co.remove_docstrings(space)
 
-    def _to_code(self):
-        """For debugging only."""
-        consts = [None] * len(self.co_consts_w)
-        num = 0
-        for w in self.co_consts_w:
-            if isinstance(w, PyCode):
-                consts[num] = w._to_code()
-            else:
-                consts[num] = self.space.unwrap(w)
-            num += 1
-        return new.code(self.co_argcount,
-                        self.co_nlocals,
-                        self.co_stacksize,
-                        self.co_flags,
-                        self.co_code,
-                        tuple(consts),
-                        tuple(self.co_names),
-                        tuple(self.co_varnames),
-                        self.co_filename,
-                        self.co_name,
-                        self.co_firstlineno,
-                        self.co_lnotab,
-                        tuple(self.co_freevars),
-                        tuple(self.co_cellvars))
-
     def exec_host_bytecode(self, w_globals, w_locals):
-        if sys.version_info < (2, 7):
-            raise Exception("PyPy no longer supports Python 2.6 or lower")
-        frame = self.space.FrameClass(self.space, self, w_globals, None)
-        frame.setdictscope(w_locals)
-        return frame.run()
+        raise Exception("no longer supported after the switch to wordcode!")
 
     def dump(self):
-        """A dis.dis() dump of the code object."""
-        co = self._to_code()
-        dis.dis(co)
+        """NOT_RPYTHON: A dis.dis() dump of the code object."""
+        if not hasattr(self, 'co_consts'):
+            self.co_consts = [w if isinstance(w, PyCode) else self.space.unwrap(w)
+                              for w in self.co_consts_w]
+        dis3.dis(self)
 
     def fget_co_consts(self, space):
         return space.newtuple(self.co_consts_w)
@@ -306,9 +329,10 @@ class PyCode(eval.Code):
     def descr_code__eq__(self, w_other):
         space = self.space
         if not isinstance(w_other, PyCode):
-            return space.w_NotImplemented
+            return space.w_False
         areEqual = (self.co_name == w_other.co_name and
                     self.co_argcount == w_other.co_argcount and
+                    self.co_kwonlyargcount == w_other.co_kwonlyargcount and
                     self.co_nlocals == w_other.co_nlocals and
                     self.co_flags == w_other.co_flags and
                     self.co_firstlineno == w_other.co_firstlineno and
@@ -331,16 +355,11 @@ class PyCode(eval.Code):
 
         return space.w_True
 
-    def descr_code__ne__(self, w_other):
-        space = self.space
-        if not isinstance(w_other, PyCode):
-            return space.w_NotImplemented
-        return space.not_(self.descr_code__eq__(w_other))
-
     def descr_code__hash__(self):
         space = self.space
         result =  compute_hash(self.co_name)
         result ^= self.co_argcount
+        result ^= self.co_kwonlyargcount
         result ^= self.co_nlocals
         result ^= self.co_flags
         result ^= self.co_firstlineno
@@ -352,20 +371,15 @@ class PyCode(eval.Code):
         for w_name in self.co_names_w:
             w_result = space.xor(w_result, space.hash(w_name))
         for w_const in self.co_consts_w:
-            w_key = self.const_comparison_key(space, w_const)
-            w_result = space.xor(w_result, space.hash(w_key))
+            w_result = space.xor(w_result, space.hash(w_const))
         return w_result
 
-    @staticmethod
-    def const_comparison_key(space, w_obj):
-        return _convert_const(space, w_obj)
-
-    @unwrap_spec(argcount=int, nlocals=int, stacksize=int, flags=int,
+    @unwrap_spec(argcount=int, kwonlyargcount=int, nlocals=int, stacksize=int, flags=int,
                  codestring='bytes',
-                 filename='text', name='text', firstlineno=int,
+                 filename='fsencode', name='text', firstlineno=int,
                  lnotab='bytes', magic=int)
     def descr_code__new__(space, w_subtype,
-                          argcount, nlocals, stacksize, flags,
+                          argcount, kwonlyargcount, nlocals, stacksize, flags,
                           codestring, w_constants, w_names,
                           w_varnames, filename, name, firstlineno,
                           lnotab, w_freevars=None, w_cellvars=None,
@@ -373,6 +387,9 @@ class PyCode(eval.Code):
         if argcount < 0:
             raise oefmt(space.w_ValueError,
                         "code: argcount must not be negative")
+        if kwonlyargcount < 0:
+            raise oefmt(space.w_ValueError,
+                        "code: kwonlyargcount must not be negative")
         if nlocals < 0:
             raise oefmt(space.w_ValueError,
                         "code: nlocals must not be negative")
@@ -390,7 +407,7 @@ class PyCode(eval.Code):
         else:
             cellvars = []
         code = space.allocate_instance(PyCode, w_subtype)
-        PyCode.__init__(code, space, argcount, nlocals, stacksize, flags, codestring, consts_w[:], names,
+        PyCode.__init__(code, space, argcount, kwonlyargcount, nlocals, stacksize, flags, codestring, consts_w[:], names,
                       varnames, filename, name, firstlineno, lnotab, freevars, cellvars, magic=magic)
         return code
 
@@ -401,6 +418,7 @@ class PyCode(eval.Code):
         new_inst = mod.get('code_new')
         tup      = [
             space.newint(self.co_argcount),
+            space.newint(self.co_kwonlyargcount),
             space.newint(self.co_nlocals),
             space.newint(self.co_stacksize),
             space.newint(self.co_flags),
@@ -408,7 +426,7 @@ class PyCode(eval.Code):
             space.newtuple(self.co_consts_w),
             space.newtuple(self.co_names_w),
             space.newtuple([space.newtext(v) for v in self.co_varnames]),
-            space.newtext(self.co_filename),
+            self.w_filename,
             space.newtext(self.co_name),
             space.newint(self.co_firstlineno),
             space.newbytes(self.co_lnotab),
@@ -416,45 +434,26 @@ class PyCode(eval.Code):
             space.newtuple([space.newtext(v) for v in self.co_cellvars]),
             space.newint(self.magic),
         ]
-        return space.newtuple2(new_inst, space.newtuple(tup))
+        return space.newtuple([new_inst, space.newtuple(tup)])
 
     def get_repr(self):
-        return "<code object %s, file '%s', line %d>" % (
-            self.co_name, self.co_filename, self.co_firstlineno)
-
-    def iterator_greenkey_printable(self):
-        return self.get_repr()
+        # This is called by the default get_printable_location so it
+        # must avoid doing too much (that might release the gil)
+        return '<code object %s, file "%s", line %d>' % (
+            self.co_name, self.co_filename,
+            -1 if self.co_firstlineno == 0 else self.co_firstlineno)
 
     def __repr__(self):
         return self.get_repr()
 
     def repr(self, space):
-        return space.newtext(self.get_repr())
-
-def _compute_args_as_cellvars(varnames, cellvars, argcount):
-    # Cell vars could shadow already-set arguments.
-    # The compiler used to be clever about the order of
-    # the variables in both co_varnames and co_cellvars, but
-    # it no longer is for the sake of simplicity.  Moreover
-    # code objects loaded from CPython don't necessarily follow
-    # an order, which could lead to strange bugs if .pyc files
-    # produced by CPython are loaded by PyPy.  Note that CPython
-    # contains the following bad-looking nested loops at *every*
-    # function call!
-
-    # Precompute what arguments need to be copied into cellvars
-    args_as_cellvars = []
-    for i in range(len(cellvars)):
-        cellname = cellvars[i]
-        for j in range(argcount):
-            if cellname == varnames[j]:
-                # argument j has the same name as the cell var i
-                while len(args_as_cellvars) < i:
-                    args_as_cellvars.append(-1)   # pad
-                args_as_cellvars.append(j)
-                last_arg_cellarg = i
-    return args_as_cellvars[:]
-
+        space = self.space
+        # co_name should be an identifier
+        name = self.co_name
+        fn = space.utf8_w(self.w_filename)
+        return space.newtext(b'<code object %s at 0x%s, file "%s", line %d>' % (
+            name, self.getaddrstring(space), fn,
+            -1 if self.co_firstlineno == 0 else self.co_firstlineno))
 
 def _code_const_eq(space, w_a, w_b):
     # this is a mess! CPython has complicated logic for this. essentially this
@@ -470,12 +469,13 @@ def _convert_const(space, w_a):
     # frozensets of converted contents.
     w_type = space.type(w_a)
     if space.is_w(w_type, space.w_unicode):
-        # unicodes are supposed to compare by value, but not equal to bytes
-        return space.newtuple2(w_type, w_a)
+        # unicodes are supposed to compare by value
+        return w_a
     if space.is_w(w_type, space.w_bytes):
-        # and vice versa
-        return space.newtuple2(w_type, w_a)
-    if type(w_a) is PyCode:
+        # bytes too
+        return w_a
+    if isinstance(w_a, PyCode):
+        # for code objects we use the logic recursively
         return w_a
     # for tuples and frozensets convert recursively
     if space.is_w(w_type, space.w_tuple):
