@@ -2,7 +2,7 @@
 """
 
 import sys
-from rpython.rlib import jit
+from rpython.rlib import jit, rweakref
 from rpython.rlib.debug import make_sure_not_resized, check_nonneg
 from rpython.rlib.debug import ll_assert_not_none
 from rpython.rlib.jit import hint
@@ -23,9 +23,8 @@ from pypy.tool import stdlib_opcode
 
 # Define some opcodes used
 for op in '''DUP_TOP POP_TOP SETUP_LOOP SETUP_EXCEPT SETUP_FINALLY SETUP_WITH
-POP_BLOCK END_FINALLY YIELD_VALUE WITH_CLEANUP'''.split():
+SETUP_ASYNC_WITH POP_BLOCK END_FINALLY WITH_CLEANUP_START'''.split():
     globals()[op] = stdlib_opcode.opmap[op]
-HAVE_ARGUMENT = stdlib_opcode.HAVE_ARGUMENT
 
 class FrameDebugData(object):
     """ A small object that holds debug data for tracing
@@ -38,6 +37,7 @@ class FrameDebugData(object):
     is_being_profiled        = False
     is_in_line_tracing       = False
     w_locals                 = None
+    hidden_operationerr      = None
 
     def __init__(self, pycode):
         self.f_lineno = pycode.co_firstlineno
@@ -66,10 +66,11 @@ class PyFrame(W_Root):
     __metaclass__ = extendabletype
 
     frame_finished_execution = False
+    f_generator_wref         = rweakref.dead_ref  # for generators/coroutines
+    f_generator_nowref       = None               # (only one of the two attrs)
     last_instr               = -1
-    last_exception           = None
     f_backref                = jit.vref_None
-    
+
     escaped                  = False  # see mark_as_escaped()
     debugdata                = None
 
@@ -79,7 +80,7 @@ class PyFrame(W_Root):
     lastblock = None
 
     # other fields:
-    
+
     # builtin - builtin cache, only if honor__builtins__ is True
     # defaults to False
 
@@ -237,27 +238,94 @@ class PyFrame(W_Root):
                                  "an unexpected number of free variables")
         index = code.co_nlocals
         for i in range(ncellvars):
-            self.locals_cells_stack_w[index] = Cell(
-                    None, self.pycode.cell_families[i])
+            self.locals_cells_stack_w[index] = Cell()
             index += 1
         for i in range(nfreevars):
             self.locals_cells_stack_w[index] = outer_func.closure[i]
             index += 1
 
-    def run(self):
-        """Start this frame's execution."""
-        if self.getcode().co_flags & pycode.CO_GENERATOR:
-            from pypy.interpreter.generator import GeneratorIterator
-            return GeneratorIterator(self)
-        else:
-            return self.execute_frame()
+    def _is_generator_or_coroutine(self):
+        return (self.getcode().co_flags & (pycode.CO_COROUTINE |
+                                           pycode.CO_GENERATOR |
+                                           pycode.CO_ASYNC_GENERATOR)) != 0
 
-    def execute_frame(self, w_inputvalue=None, operr=None):
+    def run(self, name=None, qualname=None):
+        """Start this frame's execution."""
+        if self._is_generator_or_coroutine():
+            return self.initialize_as_generator(name, qualname)
+        else:
+            # untranslated: check that sys_exc_info is exactly
+            # restored after running any Python function.
+            # Translated: actually save and restore it, as an attempt to
+            # work around rare cases that can occur if RecursionError or
+            # MemoryError is raised at just the wrong place
+            executioncontext = self.space.getexecutioncontext()
+            exc_on_enter = executioncontext.sys_exc_info()
+            if we_are_translated():
+                try:
+                    return self.execute_frame()
+                finally:
+                    executioncontext.set_sys_exc_info(exc_on_enter)
+            else:
+                # untranslated, we check consistency, but not in case of
+                # interp-level exceptions different than OperationError
+                # (e.g. a random failing test, or a pytest Skipped exc.)
+                try:
+                    w_res = self.execute_frame()
+                    assert exc_on_enter is executioncontext.sys_exc_info()
+                except OperationError:
+                    assert exc_on_enter is executioncontext.sys_exc_info()
+                    raise
+                return w_res
+    run._always_inline_ = True
+
+    def initialize_as_generator(self, name, qualname):
+        space = self.space
+        flags = self.getcode().co_flags
+        if flags & pycode.CO_COROUTINE:
+            from pypy.interpreter.generator import Coroutine
+            gen = Coroutine(self, name, qualname)
+            ec = space.getexecutioncontext()
+            w_wrapper = ec.w_coroutine_wrapper_fn
+        elif flags & pycode.CO_ASYNC_GENERATOR:
+            from pypy.interpreter.generator import AsyncGenerator
+            gen = AsyncGenerator(self, name, qualname)
+            ec = None
+            w_wrapper = None
+        elif flags & pycode.CO_GENERATOR:
+            from pypy.interpreter.generator import GeneratorIterator
+            gen = GeneratorIterator(self, name, qualname)
+            ec = None
+            w_wrapper = None
+        else:
+            raise AssertionError("bad co_flags")
+
+        if space.config.translation.rweakref:
+            self.f_generator_wref = rweakref.ref(gen)
+        else:
+            self.f_generator_nowref = gen
+        w_gen = gen
+
+        if w_wrapper is not None:
+            if ec.in_coroutine_wrapper:
+                raise oefmt(space.w_RuntimeError,
+                            "coroutine wrapper %R attempted "
+                            "to recursively wrap %R",
+                            w_wrapper, self.getcode())
+            ec.in_coroutine_wrapper = True
+            try:
+                w_gen = space.call_function(w_wrapper, w_gen)
+            finally:
+                ec.in_coroutine_wrapper = False
+        return w_gen
+
+    def execute_frame(self, in_generator=None, w_arg_or_err=None):
         """Execute this frame.  Main entry point to the interpreter.
-        The optional arguments are there to handle a generator's frame:
-        w_inputvalue is for generator.send() and operr is for
-        generator.throw().
+        'in_generator' is non-None iff we are starting or resuming
+        a generator or coroutine frame; in that case, w_arg_or_err
+        is the input argument -or- an SApplicationException instance.
         """
+        from pypy.interpreter import pyopcode
         # the following 'assert' is an annotation hint: it hides from
         # the annotator all methods that are defined in PyFrame but
         # overridden in the {,Host}FrameClass subclasses of PyFrame.
@@ -270,30 +338,28 @@ class PyFrame(W_Root):
         try:
             executioncontext.call_trace(self)
             #
+            # Execution starts just after the last_instr.  Initially,
+            # last_instr is -1.  After a generator suspends it points to
+            # the YIELD_VALUE/YIELD_FROM instruction.
             try:
-                if operr is not None:
-                    next_instr = self.handle_operation_error(
-                            executioncontext, operr)
-                    self.last_instr = intmask(next_instr - 1)
+                try:
+                    if in_generator is None:
+                        assert self.last_instr == -1
+                        next_instr = r_uint(0)
+                    else:
+                        next_instr = in_generator.resume_execute_frame(
+                                                        self, w_arg_or_err)
+                except pyopcode.Yield:
+                    w_exitvalue = self.popvalue()
                 else:
-                    # Execution starts just after the last_instr.  Initially,
-                    # last_instr is -1.  After a generator suspends it points to
-                    # the YIELD_VALUE instruction.
-                    next_instr = r_uint(self.last_instr + 1)
-                    if next_instr != 0:
-                        self.pushvalue(w_inputvalue)
-                w_exitvalue = self.dispatch(self.pycode, next_instr,
-                                            executioncontext)
+                    w_exitvalue = self.dispatch(self.pycode, next_instr,
+                                                executioncontext)
             except OperationError:
                 raise
             except Exception as e:      # general fall-back
                 raise self._convert_unexpected_exception(e)
             finally:
                 executioncontext.return_trace(self, w_exitvalue)
-            # it used to say self.last_exception = None
-            # this is now done by the code in pypyjit module
-            # since we don't want to invalidate the virtualizable
-            # for no good reason
             got_exception = False
         finally:
             executioncontext.leave(self, w_exitvalue, got_exception)
@@ -312,12 +378,9 @@ class PyFrame(W_Root):
         assert self.locals_cells_stack_w[depth] is None
         self.valuestackdepth = depth + 1
 
-    def assert_stack_index(self, index):
-        if we_are_translated():
-            return
-        assert self._check_stack_index(index)
-
     def _check_stack_index(self, index):
+        # will be completely removed by the optimizer if only used in an assert
+        # and if asserts are disabled
         code = self.pycode
         ncellvars = len(code.co_cellvars)
         nfreevars = len(code.co_freevars)
@@ -329,7 +392,7 @@ class PyFrame(W_Root):
 
     def popvalue_maybe_none(self):
         depth = self.valuestackdepth - 1
-        self.assert_stack_index(depth)
+        assert self._check_stack_index(depth)
         assert depth >= 0
         w_object = self.locals_cells_stack_w[depth]
         self.locals_cells_stack_w[depth] = None
@@ -358,7 +421,7 @@ class PyFrame(W_Root):
     def peekvalues(self, n):
         values_w = [None] * n
         base = self.valuestackdepth - n
-        self.assert_stack_index(base)
+        assert self._check_stack_index(base)
         assert base >= 0
         while True:
             n -= 1
@@ -371,7 +434,7 @@ class PyFrame(W_Root):
     def dropvalues(self, n):
         n = hint(n, promote=True)
         finaldepth = self.valuestackdepth - n
-        self.assert_stack_index(finaldepth)
+        assert self._check_stack_index(finaldepth)
         assert finaldepth >= 0
         while True:
             n -= 1
@@ -407,14 +470,14 @@ class PyFrame(W_Root):
     def peekvalue_maybe_none(self, index_from_top=0):
         index_from_top = hint(index_from_top, promote=True)
         index = self.valuestackdepth + ~index_from_top
-        self.assert_stack_index(index)
+        assert self._check_stack_index(index)
         assert index >= 0
         return self.locals_cells_stack_w[index]
 
     def settopvalue(self, w_object, index_from_top=0):
         index_from_top = hint(index_from_top, promote=True)
         index = self.valuestackdepth + ~index_from_top
-        self.assert_stack_index(index)
+        assert self._check_stack_index(index)
         assert index >= 0
         self.locals_cells_stack_w[index] = ll_assert_not_none(w_object)
 
@@ -428,138 +491,32 @@ class PyFrame(W_Root):
             depth -= 1
         self.valuestackdepth = finaldepth
 
-    def make_arguments(self, nargs, methodcall=False):
-        return Arguments(
-                self.space, self.peekvalues(nargs), methodcall=methodcall)
+    def _guess_function_name_parens(self, fnname=None, w_function=None):
+        """ Returns 'funcname()' from either a function name fnname or a
+        wrapped callable w_function. If it's not a function or a method, returns
+        'Classname object'"""
+        # CPython has a similar function, PyEval_GetFuncName
+        from pypy.interpreter.function import Function, Method
+        if fnname is not None:
+            return fnname + '()'
+        if w_function is None:
+            return None
+        if isinstance(w_function, Function):
+            return w_function.name + '()'
+        if isinstance(w_function, Method):
+            return self._guess_function_name_parens(None, w_function.w_function)
+        return w_function.getname(self.space) + ' object'
 
-    def argument_factory(self, arguments, keywords, keywords_w, w_star, w_starstar, methodcall=False):
+    def make_arguments(self, nargs, methodcall=False, w_function=None, fnname=None):
+        fnname_parens = self._guess_function_name_parens(fnname, w_function)
+        return Arguments(
+                self.space, self.peekvalues(nargs), methodcall=methodcall, fnname_parens=fnname_parens)
+
+    def argument_factory(self, arguments, keywords, keywords_w, w_star, w_starstar, methodcall=False, w_function=None, fnname=None):
+        fnname_parens = self._guess_function_name_parens(fnname, w_function)
         return Arguments(
                 self.space, arguments, keywords, keywords_w, w_star,
-                w_starstar, methodcall=methodcall)
-
-    @jit.dont_look_inside
-    def descr__reduce__(self, space):
-        from pypy.interpreter.mixedmodule import MixedModule
-        w_mod    = space.getbuiltinmodule('_pickle_support')
-        mod      = space.interp_w(MixedModule, w_mod)
-        new_inst = mod.get('frame_new')
-        w_tup_state = self._reduce_state(space)
-        nt = space.newtuple
-        return nt([new_inst, nt([]), w_tup_state])
-
-    @jit.dont_look_inside
-    def _reduce_state(self, space):
-        from pypy.module._pickle_support import maker # helper fns
-        nt = space.newtuple
-
-        if self.get_w_f_trace() is None:
-            f_lineno = self.get_last_lineno()
-        else:
-            f_lineno = self.getorcreatedebug().f_lineno
-
-        nlocals = self.pycode.co_nlocals
-        values_w = self.locals_cells_stack_w
-        w_locals_cells_stack = maker.slp_into_tuple_with_nulls(space, values_w)
-
-        w_blockstack = nt([block._get_state_(space) for block in self.get_blocklist()])
-        if self.last_exception is None:
-            w_exc_value = space.w_None
-            w_tb = space.w_None
-        else:
-            w_exc_value = self.last_exception.get_w_value(space)
-            w_tb = self.last_exception.get_w_traceback(space)
-
-        d = self.getorcreatedebug()
-        w_backref = self.f_backref()
-        if w_backref is None:
-            w_backref = space.w_None
-        tup_state = [
-            w_backref,
-            self.get_builtin(),
-            self.pycode,
-            w_locals_cells_stack,
-            w_blockstack,
-            w_exc_value, # last_exception
-            w_tb,        #
-            self.get_w_globals(),
-            space.newint(self.last_instr),
-            space.newbool(self.frame_finished_execution),
-            space.newint(f_lineno),
-            space.w_None,           #XXX placeholder for f_locals
-
-            #f_restricted requires no additional data!
-            space.w_None,
-
-            space.newint(d.instr_lb),
-            space.newint(d.instr_ub),
-            space.newint(d.instr_prev_plus_one),
-            space.newint(self.valuestackdepth),
-            ]
-        return nt(tup_state)
-
-    @jit.dont_look_inside
-    def descr__setstate__(self, space, w_args):
-        from pypy.module._pickle_support import maker # helper fns
-        from pypy.interpreter.pycode import PyCode
-        from pypy.interpreter.module import Module
-        args_w = space.unpackiterable(w_args, 17)
-        w_f_back, w_builtin, w_pycode, w_locals_cells_stack, w_blockstack, w_exc_value, w_tb,\
-            w_globals, w_last_instr, w_finished, w_f_lineno, w_f_locals, \
-            w_f_trace, w_instr_lb, w_instr_ub, w_instr_prev_plus_one, w_stackdepth = args_w
-
-        new_frame = self
-        pycode = space.interp_w(PyCode, w_pycode)
-
-        values_w = maker.slp_from_tuple_with_nulls(space, w_locals_cells_stack)
-        nfreevars = len(pycode.co_freevars)
-        closure = None
-        if nfreevars:
-            base = pycode.co_nlocals + len(pycode.co_cellvars)
-            closure = values_w[base: base + nfreevars]
-
-        # do not use the instance's __init__ but the base's, because we set
-        # everything like cells from here
-        # XXX hack
-        from pypy.interpreter.function import Function
-        outer_func = Function(space, None, closure=closure,
-                             forcename="fake")
-        PyFrame.__init__(self, space, pycode, w_globals, outer_func)
-        f_back = space.interp_w(PyFrame, w_f_back, can_be_None=True)
-        new_frame.f_backref = jit.non_virtual_ref(f_back)
-
-        if space.config.objspace.honor__builtins__:
-            new_frame.builtin = space.interp_w(Module, w_builtin)
-        else:
-            assert space.interp_w(Module, w_builtin) is space.builtin
-        new_frame.set_blocklist([unpickle_block(space, w_blk)
-                                 for w_blk in space.unpackiterable(w_blockstack)])
-        self.locals_cells_stack_w = values_w[:]
-        valuestackdepth = space.int_w(w_stackdepth)
-        if not self._check_stack_index(valuestackdepth):
-            raise oefmt(space.w_ValueError, "invalid stackdepth")
-        assert valuestackdepth >= 0
-        self.valuestackdepth = valuestackdepth
-        if space.is_w(w_exc_value, space.w_None):
-            new_frame.last_exception = None
-        else:
-            from pypy.interpreter.pytraceback import PyTraceback
-            tb = space.interp_w(PyTraceback, w_tb)
-            new_frame.last_exception = OperationError(space.type(w_exc_value),
-                                                      w_exc_value, tb
-                                                      )
-        new_frame.last_instr = space.int_w(w_last_instr)
-        new_frame.frame_finished_execution = space.is_true(w_finished)
-        d = new_frame.getorcreatedebug()
-        d.f_lineno = space.int_w(w_f_lineno)
-
-        if space.is_w(w_f_trace, space.w_None):
-            d.w_f_trace = None
-        else:
-            d.w_f_trace = w_f_trace
-
-        d.instr_lb = space.int_w(w_instr_lb)   #the three for tracing
-        d.instr_ub = space.int_w(w_instr_ub)
-        d.instr_prev_plus_one = space.int_w(w_instr_prev_plus_one)
+                w_starstar, methodcall=methodcall, fnname_parens=fnname_parens)
 
     def hide(self):
         return self.pycode.hidden_applevel
@@ -598,21 +555,18 @@ class PyFrame(W_Root):
     def fast2locals(self):
         # Copy values from the fastlocals to self.w_locals
         d = self.getorcreatedebug()
-        w_locals = d.w_locals
-        write = False
-        if w_locals is None:
-            w_locals = self.space.newdict(instance=True)
-            write = True
+        if d.w_locals is None:
+            d.w_locals = self.space.newdict()
         varnames = self.getcode().getvarnames()
         for i in range(min(len(varnames), self.getcode().co_nlocals)):
             name = varnames[i]
             w_value = self.locals_cells_stack_w[i]
             if w_value is not None:
-                self.space.setitem_str(w_locals, name, w_value)
+                self.space.setitem_str(d.w_locals, name, w_value)
             else:
                 w_name = self.space.newtext(name)
                 try:
-                    self.space.delitem(w_locals, w_name)
+                    self.space.delitem(d.w_locals, w_name)
                 except OperationError as e:
                     if not e.match(self.space, self.space.w_KeyError):
                         raise
@@ -631,9 +585,7 @@ class PyFrame(W_Root):
             except ValueError:
                 pass
             else:
-                self.space.setitem_str(w_locals, name, w_value)
-        if write:
-            d.w_locals = w_locals
+                self.space.setitem_str(d.w_locals, name, w_value)
 
 
     @jit.unroll_safe
@@ -710,7 +662,7 @@ class PyFrame(W_Root):
             return space.newint(self.getorcreatedebug().f_lineno)
 
     def fset_f_lineno(self, space, w_new_lineno):
-        "Returns the line number of the instruction currently being executed."
+        "Change the line number of the instruction currently being executed."
         try:
             new_lineno = space.int_w(w_new_lineno)
         except OperationError:
@@ -723,12 +675,7 @@ class PyFrame(W_Root):
                         "can't jump from the 'call' trace event of a new frame")
         if self.get_w_f_trace() is None:
             raise oefmt(space.w_ValueError,
-                        "f_lineno can only be set by a trace function.")
-
-        code = self.pycode.co_code
-        if ord(code[self.last_instr]) == YIELD_VALUE:
-            raise oefmt(space.w_ValueError,
-                        "can't jump from a yield statement")
+                        "f_lineno can only be set by a trace function")
 
         # Only allow jumps when we're tracing a line event.
         d = self.getorcreatedebug()
@@ -739,123 +686,175 @@ class PyFrame(W_Root):
         line = self.pycode.co_firstlineno
         if new_lineno < line:
             raise oefmt(space.w_ValueError,
-                        "line %d comes before the current code.", new_lineno)
+                        "line %d comes before the current code block", new_lineno)
         elif new_lineno == line:
             new_lasti = 0
         else:
-            new_lasti = -1
-            addr = 0
+            # Find the bytecode offset for the start of the given
+            # line, or the first code-owning line after it.
             lnotab = self.pycode.co_lnotab
+            addr = 0
+            new_lasti = -1
             for offset in xrange(0, len(lnotab), 2):
                 addr += ord(lnotab[offset])
-                line += ord(lnotab[offset + 1])
+                line_offset = ord(lnotab[offset + 1])
+                if line_offset >= 0x80:
+                    line_offset -= 0x100
+                line += line_offset
                 if line >= new_lineno:
                     new_lasti = addr
                     new_lineno = line
                     break
 
+        # If we didn't reach the requested line, return an error.
         if new_lasti == -1:
             raise oefmt(space.w_ValueError,
-                        "line %d comes after the current code.", new_lineno)
+                        "line %d comes after the current code block", new_lineno)
 
-        # Don't jump to a line with an except in it.
+        min_addr = min(new_lasti, self.last_instr)
+        max_addr = max(new_lasti, self.last_instr)
+
+        # You can't jump onto a line with an 'except' statement on it -
+        # they expect to have an exception on the top of the stack, which
+        # won't be true if you jump to them.  They always start with code
+        # that either pops the exception using POP_TOP (plain 'except:'
+        # lines do this) or duplicates the exception on the stack using
+        # DUP_TOP (if there's an exception type specified).  See compile.c,
+        # 'com_try_except' for the full details.  There aren't any other
+        # cases (AFAIK) where a line's code can start with DUP_TOP or
+        # POP_TOP, but if any ever appear, they'll be subject to the same
+        # restriction (but with a different error message).
+        code = self.pycode.co_code
         if ord(code[new_lasti]) in (DUP_TOP, POP_TOP):
             raise oefmt(space.w_ValueError,
                         "can't jump to 'except' line as there's no exception")
 
-        # Don't jump into or out of a finally block.
+        # You can't jump into or out of a 'finally' block because the 'try'
+        # block leaves something on the stack for the END_FINALLY to clean
+        # up.      So we walk the bytecode, maintaining a simulated blockstack.
+        # When we reach the old or new address and it's in a 'finally' block
+        # we note the address of the corresponding SETUP_FINALLY.  The jump
+        # is only legal if neither address is in a 'finally' block or
+        # they're both in the same one.  'blockstack' is a stack of the
+        # bytecode addresses of the SETUP_X opcodes, and 'in_finally' tracks
+        # whether we're in a 'finally' block at each blockstack level.
+
+        # PYPY NOTE: CPython (at least 3.5.2+) doesn't check except blocks,
+        # but that results in crashes.  But for now I'm going to follow the
+        # logic of CPython 3.6.9 exactly anyway, which is quite messy already.
+
         f_lasti_setup_addr = -1
         new_lasti_setup_addr = -1
-        blockstack = []
+        blockstack = []     # current blockstack (addresses of SETUP_*)
+        in_finally = []     # list of flags "is this a finally block?"
         addr = 0
         while addr < len(code):
+            assert addr & 1 == 0
             op = ord(code[addr])
-            if op in (SETUP_LOOP, SETUP_EXCEPT, SETUP_FINALLY, SETUP_WITH):
-                blockstack.append([addr, False])
+            if op in (SETUP_LOOP, SETUP_EXCEPT, SETUP_FINALLY, SETUP_WITH,
+                      SETUP_ASYNC_WITH):
+                blockstack.append(addr)
+                in_finally.append(False)
             elif op == POP_BLOCK:
-                setup_op = ord(code[blockstack[-1][0]])
-                if setup_op == SETUP_FINALLY or setup_op == SETUP_WITH:
-                    blockstack[-1][1] = True
+                if len(blockstack) == 0:
+                    raise oefmt(space.w_SystemError,
+                           "POP_BLOCK not properly nested in this bytecode")
+                setup_op = ord(code[blockstack[-1]])
+                if setup_op in (SETUP_FINALLY, SETUP_WITH, SETUP_ASYNC_WITH):
+                    in_finally[-1] = True
                 else:
-                    blockstack.pop()
+                    del blockstack[-1]
             elif op == END_FINALLY:
+                # Ignore END_FINALLYs for SETUP_EXCEPTs - they exist
+                # in the bytecode but don't correspond to an actual
+                # 'finally' block.  (If len(blockstack) is 0, we must
+                # be seeing such an END_FINALLY.)
                 if len(blockstack) > 0:
-                    setup_op = ord(code[blockstack[-1][0]])
-                    if setup_op == SETUP_FINALLY or setup_op == SETUP_WITH:
-                        blockstack.pop()
+                    setup_op = ord(code[blockstack[-1]])
+                    if setup_op in (SETUP_FINALLY, SETUP_WITH, SETUP_ASYNC_WITH):
+                        del blockstack[-1]
 
+            # For the addresses we're interested in, see whether they're
+            # within a 'finally' block and if so, remember the address
+            # of the SETUP_FINALLY.
             if addr == new_lasti or addr == self.last_instr:
-                for ii in range(len(blockstack)):
-                    setup_addr, in_finally = blockstack[~ii]
-                    if in_finally:
-                        if addr == new_lasti:
-                            new_lasti_setup_addr = setup_addr
-                        if addr == self.last_instr:
-                            f_lasti_setup_addr = setup_addr
+                setup_addr = -1
+                for i in xrange(len(blockstack) - 1, -1, -1):
+                    if in_finally[i]:
+                        setup_addr = blockstack[i]
                         break
 
-            if op >= HAVE_ARGUMENT:
-                addr += 3
-            else:
-                addr += 1
+                if setup_addr != -1:
+                    if addr == new_lasti:
+                        new_lasti_setup_addr = setup_addr
+                    if addr == self.last_instr:
+                        f_lasti_setup_addr = setup_addr
+            addr += 2
 
-        assert len(blockstack) == 0
+        # Verify that the blockstack tracking code didn't get lost.
+        if len(blockstack) != 0:
+            raise oefmt(space.w_SystemError,
+                        "blocks not properly nested in this bytecode")
 
+        # After all that, are we jumping into / out of a 'finally' block?
         if new_lasti_setup_addr != f_lasti_setup_addr:
             raise oefmt(space.w_ValueError,
-                        "can't jump into or out of a 'finally' block %d -> %d",
+                        "can't jump into or out of a 'finally' block "
+                        "(%d -> %d)",
                         f_lasti_setup_addr, new_lasti_setup_addr)
 
-        if new_lasti < self.last_instr:
-            min_addr = new_lasti
-            max_addr = self.last_instr
-        else:
-            min_addr = self.last_instr
-            max_addr = new_lasti
+        # now we know we're not jumping into or out of a place which
+        # needs a SysExcInfoRestorer.  Check that we're not jumping
+        # *into* a block, but only (potentially) out of some blocks.
+
+        # Police block-jumping (you can't jump into the middle of a block)
+        # and ensure that the blockstack finishes up in a sensible state (by
+        # popping any blocks we're jumping out of).  We look at all the
+        # blockstack operations between the current position and the new
+        # one, and keep track of how many blocks we drop out of on the way.
+        # By also keeping track of the lowest blockstack position we see, we
+        # can tell whether the jump goes into any blocks without coming out
+        # again - in that case we raise an exception below.
 
         delta_iblock = min_delta_iblock = 0
         addr = min_addr
         while addr < max_addr:
+            assert addr & 1 == 0
             op = ord(code[addr])
 
-            if op in (SETUP_LOOP, SETUP_EXCEPT, SETUP_FINALLY, SETUP_WITH):
+            if op in (SETUP_LOOP, SETUP_EXCEPT, SETUP_FINALLY, SETUP_WITH,
+                      SETUP_ASYNC_WITH):
                 delta_iblock += 1
             elif op == POP_BLOCK:
                 delta_iblock -= 1
-                if delta_iblock < min_delta_iblock:
-                    min_delta_iblock = delta_iblock
 
-            if op >= stdlib_opcode.HAVE_ARGUMENT:
-                addr += 3
-            else:
-                addr += 1
+            min_delta_iblock = min(min_delta_iblock, delta_iblock)
+            addr += 2
 
-        f_iblock = 0
-        block = self.lastblock
-        while block:
-            f_iblock += 1
-            block = block.previous
-        min_iblock = f_iblock + min_delta_iblock
+        # 'min_delta_iblock' is <= 0; its absolute value is the number of
+        # blocks we exit.  'go_iblock' is the delta number of blocks
+        # between the last_instr and the new_lasti, in this order.
         if new_lasti > self.last_instr:
-            new_iblock = f_iblock + delta_iblock
+            go_iblock = delta_iblock        # Forwards jump.
         else:
-            new_iblock = f_iblock - delta_iblock
+            go_iblock = -delta_iblock       # Backwards jump.
 
-        if new_iblock > min_iblock:
+        if go_iblock > min_delta_iblock:
             raise oefmt(space.w_ValueError,
                         "can't jump into the middle of a block")
+        assert go_iblock <= 0
 
         # Pop any blocks that we're jumping out of.
         from pypy.interpreter.pyopcode import FinallyBlock
-        while f_iblock > new_iblock:
+        for ii in range(-go_iblock):
             block = self.pop_block()
-            block.cleanup(self)
-            f_iblock -= 1
-            if (isinstance(block, FinallyBlock)
-                    and ord(code[block.handlerposition]) == WITH_CLEANUP):
-                self.popvalue()  # Pop the exit function.
+            block.cleanupstack(self)
+            if (isinstance(block, FinallyBlock) and
+                ord(code[block.handlerposition]) == WITH_CLEANUP_START):
+                self.popvalue()     # Pop the exit function.
 
         d.f_lineno = new_lineno
+        assert new_lasti & 1 == 0
         self.last_instr = new_lasti
 
     def get_last_lineno(self):
@@ -888,54 +887,52 @@ class PyFrame(W_Root):
     def fdel_f_trace(self, space):
         self.getorcreatedebug().w_f_trace = None
 
-    def fget_f_exc_type(self, space):
-        if self.last_exception is not None:
-            f = self.f_backref()
-            while f is not None and f.last_exception is None:
-                f = f.f_backref()
-            if f is not None:
-                return f.last_exception.w_type
-        return space.w_None
-
-    def fget_f_exc_value(self, space):
-        if self.last_exception is not None:
-            f = self.f_backref()
-            while f is not None and f.last_exception is None:
-                f = f.f_backref()
-            if f is not None:
-                return f.last_exception.get_w_value(space)
-        return space.w_None
-
-    def fget_f_exc_traceback(self, space):
-        if self.last_exception is not None:
-            f = self.f_backref()
-            while f is not None and f.last_exception is None:
-                f = f.f_backref()
-            if f is not None:
-                return f.last_exception.get_w_traceback(space)
-        return space.w_None
-
     def fget_f_restricted(self, space):
         if space.config.objspace.honor__builtins__:
             return space.newbool(self.builtin is not space.builtin)
         return space.w_False
 
-    @jit.unroll_safe
-    @specialize.arg(2)
-    def _exc_info_unroll(self, space, for_hidden=False):
-        """Return the most recent OperationError being handled in the
-        call stack
-        """
-        frame = self
-        while frame:
-            last = frame.last_exception
-            if last is not None:
-                if last is get_cleared_operation_error(self.space):
-                    break
-                if for_hidden or not frame.hide():
-                    return last
-            frame = frame.f_backref()
-        return None
+    def get_generator(self):
+        if self.space.config.translation.rweakref:
+            return self.f_generator_wref()
+        else:
+            return self.f_generator_nowref
+
+    def descr_clear(self, space):
+        """F.clear(): clear most references held by the frame"""
+        # Clears a random subset of the attributes: the local variables
+        # and the w_locals.  Note that CPython doesn't clear f_locals
+        # (which can create leaks) but it's hard to notice because
+        # the next Python-level read of 'frame.f_locals' will clear it.
+        if not self.frame_finished_execution:
+            if not self._is_generator_or_coroutine():
+                raise oefmt(space.w_RuntimeError,
+                            "cannot clear an executing frame")
+            gen = self.get_generator()
+            if gen is not None:
+                if gen.running:
+                    raise oefmt(space.w_RuntimeError,
+                                "cannot clear an executing frame")
+                # xxx CPython raises the RuntimeWarning "coroutine was never
+                # awaited" in this case too.  Does it make any sense?
+                gen.descr_close()
+
+        debug = self.getdebug()
+        if debug is not None:
+            debug.w_f_trace = None
+            if debug.w_locals is not None:
+                debug.w_locals = space.newdict()
+
+        # clear the locals, including the cell/free vars, and the stack
+        for i in range(len(self.locals_cells_stack_w)):
+            w_oldvalue = self.locals_cells_stack_w[i]
+            if isinstance(w_oldvalue, Cell):
+                w_newvalue = Cell()
+            else:
+                w_newvalue = None
+            self.locals_cells_stack_w[i] = w_newvalue
+        self.valuestackdepth = 0
+        self.lastblock = None    # the FrameBlock chained list
 
     def _convert_unexpected_exception(self, e):
         from pypy.interpreter import error
