@@ -13,20 +13,20 @@ def test_f_globals():
     import sys
     f = sys._getframe()
     assert f.f_globals is globals()
-    with pytest.raises(TypeError):
+    with pytest.raises(AttributeError):
         f.f_globals = globals()
 
 def test_f_builtins():
-    import sys, __builtin__
+    import sys, builtins
     f = sys._getframe()
-    assert f.f_builtins is __builtin__.__dict__
+    assert f.f_builtins is builtins.__dict__
 
 def test_f_code():
     def g():
         import sys
         f = sys._getframe()
         return f.f_code
-    assert g() is g.func_code
+    assert g() is g.__code__
 
 def test_f_trace_del():
     import sys
@@ -42,9 +42,54 @@ def test_f_lineno():
         y = f.f_lineno
         z = f.f_lineno
         return [x, y, z]
-    origin = g.func_code.co_firstlineno
+    origin = g.__code__.co_firstlineno
     assert g() == [origin+3, origin+4, origin+5]
 
+def test_f_lineno_huge_jump():
+    code = """def g():
+        import sys
+        f = sys._getframe()
+        x = f.f_lineno
+        %s
+        y = f.f_lineno
+        %s
+        z = f.f_lineno
+        return [x, y, z]""" % ("\n" * 127, "\n" * 1000)
+    d = {}
+    exec(code, d)
+    g = d['g']
+    origin = g.__code__.co_firstlineno
+    print(repr(g.__code__.co_lnotab))
+    assert g() == [origin+3, origin+5+127, origin+7+127+1000]
+
+class JumpTracer:
+    """Defines a trace function that jumps from one place to another."""
+
+    def __init__(self, function, jumpFrom, jumpTo, event='line',
+                 decorated=False):
+        self.code = function.__code__
+        self.jumpFrom = jumpFrom
+        self.jumpTo = jumpTo
+        self.event = event
+        self.firstLine = None if decorated else self.code.co_firstlineno
+        self.done = False
+
+    def trace(self, frame, event, arg):
+        if self.done:
+            return
+        # frame.f_code.co_firstlineno is the first line of the decorator when
+        # 'function' is decorated and the decorator may be written using
+        # multiple physical lines when it is too long. Use the first line
+        # trace event in 'function' to find the first line of 'function'.
+        if (self.firstLine is None and frame.f_code == self.code and
+                event == 'line'):
+            self.firstLine = frame.f_lineno - 1
+        if (event == self.event and self.firstLine and
+                frame.f_lineno == self.firstLine + self.jumpFrom):
+            frame.f_lineno = self.firstLine + self.jumpTo
+            self.done = True
+        return self.trace
+
 def test_f_lineno_set(tempfile):
     def tracer(f, *args):
         def y(f, *args):
@@ -55,11 +100,13 @@ def test_f_lineno_set(tempfile):
                         # test_local_trace_function_returning_None_ignored
         return x
 
-    open    # force fetching of this name now
-
-    def function():
+    # obscure: call open beforehand, py3k's open invokes some app
+    # level code that confuses our tracing (likely due to the
+    # testing env, otherwise it's not a problem)
+    f = open(tempfile, 'w')
+    def function(f=f):
         xyz
-        with open(tempfile, 'w') as f:
+        with f as f:
             pass
         return 3
 
@@ -69,9 +116,87 @@ def test_f_lineno_set(tempfile):
     sys.settrace(None)
     # assert did not crash
 
+def test_f_lineno_set_2():
+    skip("this test is known to crash CPython (verified in 3.6.9).  "
+         "Now it crashes PyPy too. Too bad?")
+    counter = [0]
+    errors = []
+
+    def tracer(f, event, *args):
+        if event == 'line':
+            counter[0] += 1
+            if counter[0] == 2:
+                try:
+                    f.f_lineno += 2
+                except ValueError as e:
+                    errors.append(e)
+        return tracer
+
+    def function():
+        try:
+            raise ValueError
+        except ValueError:
+            x = 42
+        return x
+
+    import sys
+    sys.settrace(tracer)
+    x = function()
+    sys.settrace(None)
+    assert x == 42
+    assert len(errors) == 1
+    assert str(errors[0]).startswith(
+        "can't jump into or out of an 'expect' or 'finally' block")
+
+def test_f_lineno_set_3():
+    def jump_in_nested_finally(output):
+        try:
+            output.append(2)
+        finally:
+            output.append(4)
+            try:
+                output.append(6)
+            finally:
+                output.append(8)
+            output.append(9)
+    output = []
+    tracer = JumpTracer(jump_in_nested_finally, 4, 9)
+
+    import sys
+    sys.settrace(tracer.trace)
+    jump_in_nested_finally(output)
+    sys.settrace(None)
+    assert output == [2, 9]
+
+def test_f_lineno_set_4():
+    def jump_in_nested_finally(output):
+        try:
+            output.append(2)
+            1/0
+            return
+        finally:
+            output.append(6)
+            output.append(7)
+        output.append(8)
+    output = []
+    tracer = JumpTracer(jump_in_nested_finally, 6, 7)
+
+    import sys
+    sys.settrace(tracer.trace)
+    try:
+        jump_in_nested_finally(output)
+    except ZeroDivisionError:
+        sys.settrace(None)
+    else:
+        sys.settrace(None)
+        assert False, 'did not raise'
+    assert output == [2, 7]
+
 def test_f_lineno_set_firstline():
     seen = []
     def tracer(f, event, *args):
+        if f.f_code.co_name == "decode":
+            return tracer
         seen.append((event, f.f_lineno))
         if len(seen) == 5:
             f.f_lineno = 1       # bug shown only when setting lineno to 1
@@ -79,8 +204,11 @@ def test_f_lineno_set_firstline():
 
     def g():
         import sys
+        source = "x=1\ny=x+1\nz=y+1\nt=z+1\ns=t+1\n"
+        # compile first to ensure that no spurious events appear in the trace
+        code = compile(source, '<string>', 'exec')
         sys.settrace(tracer)
-        exec "x=1\ny=x+1\nz=y+1\nt=z+1\ns=t+1\n" in {}
+        exec(code, {})
         sys.settrace(None)
 
     g()
@@ -112,30 +240,6 @@ def test_f_back_virtualref():
     frame = f()
     assert frame.f_back.f_code.co_name == 'f'
 
-def test_f_exc_xxx():
-    import sys
-
-    class OuterException(Exception):
-        pass
-    class InnerException(Exception):
-        pass
-
-    def g(exc_info):
-        f = sys._getframe()
-        assert f.f_exc_type is None
-        assert f.f_exc_value is None
-        assert f.f_exc_traceback is None
-        try:
-            raise InnerException
-        except:
-            assert f.f_exc_type is exc_info[0]
-            assert f.f_exc_value is exc_info[1]
-            assert f.f_exc_traceback is exc_info[2]
-    try:
-        raise OuterException
-    except:
-        g(sys.exc_info())
-
 def test_virtualref_through_traceback():
     import sys
     def g():
@@ -179,7 +283,7 @@ def test_trace_basic():
     x = f(4)
     sys.settrace(None)
     assert x == 42
-    print l
+    print(l)
     assert l == [(0, 'f', 'call', None),
                     (1, 'f', 'line', None),
                         (0, 'g', 'call', None),
@@ -219,7 +323,11 @@ def test_trace_exc():
 
 def test_trace_ignore_hidden():
     import sys
-    import _testing
+    try:
+        import _testing
+    except ImportError:
+        skip('PyPy only test')
+    _testing.Hidden  # avoid module lazy-loading weirdness when untranslated
 
     l = []
     def trace(a,b,c):
@@ -237,7 +345,7 @@ def test_trace_ignore_hidden():
     assert l[0][1] == 'call'
     assert res == 'hidden' # sanity
 
-def test_trace_hidden_prints(tempfile):
+def test_trace_hidden_applevel_builtins():
     import sys
 
     l = []
@@ -245,19 +353,17 @@ def test_trace_hidden_prints(tempfile):
         l.append((a,b,c))
         return trace
 
-    outputf = open(tempfile, 'w')
     def f():
-        print >> outputf, 1
-        print >> outputf, 2
-        print >> outputf, 3
+        sum([])
+        sum([])
+        sum([])
         return "that's the return value"
 
     sys.settrace(trace)
     f()
     sys.settrace(None)
-    outputf.close()
     # should get 1 "call", 3 "line" and 1 "return" events, and no call
-    # or return for the internal app-level implementation of 'print'
+    # or return for the internal app-level implementation of sum
     assert len(l) == 6
     assert [what for (frame, what, arg) in l] == [
         'call', 'line', 'line', 'line', 'line', 'return']
@@ -299,7 +405,7 @@ def test_trace_raises_on_return():
 
     def f(): return 1
 
-    for i in xrange(sys.getrecursionlimit() + 1):
+    for i in range(sys.getrecursionlimit() + 1):
         sys.settrace(trace)
         try:
             f()
@@ -333,34 +439,6 @@ def test_trace_try_finally():
     assert issubclass(l[0][0], Exception)
     assert issubclass(l[1][0], Exception)
 
-def test_trace_raise_three_arg():
-    import sys
-    l = []
-    def trace(frame, event, arg):
-        if event == 'exception':
-            l.append(arg)
-        return trace
-
-    def g():
-        try:
-            raise Exception
-        except Exception as e:
-            import sys
-            raise Exception, e, sys.exc_info()[2]
-
-    def f():
-        try:
-            g()
-        except:
-            pass
-
-    sys.settrace(trace)
-    f()
-    sys.settrace(None)
-    assert len(l) == 2
-    assert issubclass(l[0][0], Exception)
-    assert issubclass(l[1][0], Exception)
-
 def test_trace_generator_finalisation():
     import sys
     l = []
@@ -372,7 +450,7 @@ def test_trace_generator_finalisation():
         return trace
 
     d = {}
-    exec """if 1:
+    exec("""if 1:
     def g():
         try:
             yield True
@@ -382,11 +460,11 @@ def test_trace_generator_finalisation():
     def f():
         try:
             gen = g()
-            gen.next()
+            next(gen)
             gen.close()
         except:
             pass
-    """ in d
+    """, d)
     f = d['f']
 
     sys.settrace(trace)
@@ -431,31 +509,6 @@ def test_dont_trace_on_reraise():
     assert len(l) == 1
     assert issubclass(l[0][0], Exception)
 
-def test_dont_trace_on_raise_with_tb():
-    import sys
-    l = []
-    def ltrace(a,b,c):
-        if b == 'exception':
-            l.append(c)
-        return ltrace
-    def trace(a,b,c): return ltrace
-    def f():
-        try:
-            raise Exception
-        except:
-            return sys.exc_info()
-    def g():
-        exc, val, tb = f()
-        try:
-            raise exc, val, tb
-        except:
-            pass
-    sys.settrace(trace)
-    g()
-    sys.settrace(None)
-    assert len(l) == 1
-    assert isinstance(l[0][1], Exception)
-
 def test_trace_changes_locals():
     import sys
     def trace(frame, what, arg):
@@ -468,6 +521,25 @@ def test_trace_changes_locals():
     sys.settrace(None)
     assert res == 42
 
+def test_trace_onliner_if():
+    import sys
+    l = []
+    def trace(frame, event, arg):
+        l.append((frame.f_lineno, event))
+        return trace
+    def onliners():
+        if True: False
+        else: True
+        return 0
+    sys.settrace(trace)
+    onliners()
+    sys.settrace(None)
+    firstlineno = onliners.__code__.co_firstlineno
+    assert l == [(firstlineno + 0, 'call'),
+                    (firstlineno + 1, 'line'),
+                    (firstlineno + 3, 'line'),
+                    (firstlineno + 3, 'return')]
+
 def test_set_unset_f_trace():
     import sys
     seen = []
@@ -489,7 +561,7 @@ def test_set_unset_f_trace():
     len(seen)     # take one line
     del f.f_trace
     len(seen)     # take one line
-    firstline = set_the_trace.func_code.co_firstlineno
+    firstline = set_the_trace.__code__.co_firstlineno
     assert seen == [(1, f, firstline + 6, 'line', None),
                     (1, f, firstline + 7, 'line', None),
                     (1, f, firstline + 8, 'line', None)]
@@ -512,6 +584,66 @@ def test_locals2fast_freevar_bug():
     sys.settrace(None)
     assert res == 10
 
+def test_preserve_exc_state_in_generators():
+    import sys
+    def yield_raise():
+        try:
+            raise KeyError("caught")
+        except KeyError:
+            yield sys.exc_info()[0]
+            yield sys.exc_info()[0]
+
+    it = yield_raise()
+    assert next(it) is KeyError
+    assert next(it) is KeyError
+
+def test_frame_clear():
+    import sys, gc, weakref
+    #
+    raises(RuntimeError, sys._getframe().clear)
+    def g():
+        yield 5
+        raises(RuntimeError, sys._getframe().clear)
+        yield 6
+    assert list(g()) == [5, 6]
+    #
+    class A:
+        pass
+    a1 = A(); a1ref = weakref.ref(a1)
+    a2 = A(); a2ref = weakref.ref(a2)
+    seen = []
+    def f():
+        local_a1 = a1
+        for loc in [5, 6, a2]:
+            try:
+                yield sys._getframe()
+            finally:
+                seen.append(42)
+            seen.append(43)
+    gen = f()
+    frame = next(gen)
+    a1 = a2 = None
+    gc.collect(); gc.collect()
+    assert a1ref() is not None
+    assert a2ref() is not None
+    assert seen == []
+    frame.clear()
+    assert seen == [42]
+    gc.collect(); gc.collect()
+    assert a1ref() is None, "locals not cleared"
+    assert a2ref() is None, "stack not cleared"
+    #
+    raises(StopIteration, next, gen)
+
+def test_frame_clear_really():
+    import sys
+    def f(x):
+        return sys._getframe()
+    frame = f(42)
+    assert frame.f_locals['x'] == 42
+    frame.clear()
+    assert frame.f_locals == {}
+
 def test_throw_trace_bug():
     import sys
     def f():
@@ -537,9 +669,11 @@ def test_generator_trace_stopiteration():
     gen = f()
     assert next(gen) == 5
     seen = []
+    frames = []
     def trace_func(frame, event, *args):
         print('TRACE:', frame, event, args)
         seen.append(event)
+        frames.append(frame)
         return trace_func
     def g():
         for x in gen:
@@ -547,10 +681,111 @@ def test_generator_trace_stopiteration():
     sys.settrace(trace_func)
     g()
     sys.settrace(None)
-    print 'seen:', seen
+    print('seen:', seen)
     # on Python 3 we get an extra 'exception' when 'for' catches
-    # StopIteration
-    assert seen == ['call', 'line', 'call', 'return', 'return']
+    # StopIteration (but not always! mess)
+    assert seen == ['call', 'line', 'call', 'return', 'exception', 'return']
+    assert frames[-2].f_code.co_name == 'g'
+
+def test_nongenerator_trace_stopiteration():
+    import sys
+    gen = iter([5])
+    assert next(gen) == 5
+    seen = []
+    frames = []
+    def trace_func(frame, event, *args):
+        print('TRACE:', frame, event, args)
+        seen.append(event)
+        frames.append(frame)
+        return trace_func
+    def g():
+        for x in gen:
+            never_entered
+    sys.settrace(trace_func)
+    g()
+    sys.settrace(None)
+    print('seen:', seen)
+    # hack: don't report the StopIteration for some "simple"
+    # iterators.
+    assert seen == ['call', 'line', 'return']
+    assert frames[-2].f_code.co_name == 'g'
+
+def test_yieldfrom_trace_stopiteration():
+    import sys
+    def f2():
+        yield 5
+    def f():
+        yield from f2()
+    gen = f()
+    assert next(gen) == 5
+    seen = []
+    frames = []
+    def trace_func(frame, event, *args):
+        print('TRACE:', frame, event, args)
+        seen.append(event)
+        frames.append(frame)
+        return trace_func
+    def g():
+        for x in gen:
+            never_entered
+    sys.settrace(trace_func)
+    g()      # invokes next_yield_from() from resume_execute_frame()
+    sys.settrace(None)
+    print('seen:', seen)
+    assert seen == ['call', 'line', 'call', 'call', 'return',
+                    'exception', 'return', 'exception', 'return']
+    assert frames[-4].f_code.co_name == 'f'
+    assert frames[-2].f_code.co_name == 'g'
+
+def test_yieldfrom_trace_stopiteration_2():
+    import sys
+    def f2():
+        if False:
+            yield 5
+    def f():
+        yield from f2()
+    gen = f()
+    seen = []
+    frames = []
+    def trace_func(frame, event, *args):
+        print('TRACE:', frame, event, args)
+        seen.append(event)
+        frames.append(frame)
+        return trace_func
+    def g():
+        for x in gen:
+            never_entered
+    sys.settrace(trace_func)
+    g()      # invokes next_yield_from() from YIELD_FROM()
+    sys.settrace(None)
+    print('seen:', seen)
+    assert seen == ['call', 'line', 'call', 'line', 'call', 'line',
+                    'return', 'exception', 'return', 'exception', 'return']
+    assert frames[-4].f_code.co_name == 'f'
+    assert frames[-2].f_code.co_name == 'g'
+
+def test_yieldfrom_trace_stopiteration_3():
+    import sys
+    def f():
+        yield from []
+    gen = f()
+    seen = []
+    frames = []
+    def trace_func(frame, event, *args):
+        print('TRACE:', frame, event, args)
+        seen.append(event)
+        frames.append(frame)
+        return trace_func
+    def g():
+        for x in gen:
+            never_entered
+    sys.settrace(trace_func)
+    g()      # invokes next_yield_from() from YIELD_FROM()
+    sys.settrace(None)
+    print('seen:', seen)
+    assert seen == ['call', 'line', 'call', 'line',
+                    'return', 'exception', 'return']
+    assert frames[-4].f_code.co_name == 'f'
 
 def test_local_trace_function_returning_None_ignored():
     # behave the same as CPython does, and in contradiction with
@@ -574,3 +809,29 @@ def test_local_trace_function_returning_None_ignored():
     function()
     sys.settrace(None)
     assert seen == ["line", "line", "line", "return"]
+
+def test_clear_locals():
+    def make_frames():
+        def outer():
+            x = 5
+            y = 6
+            def inner():
+                z = x + 2
+                1/0
+                t = 9
+            return inner()
+        try:
+            outer()
+        except ZeroDivisionError as e:
+            tb = e.__traceback__
+            frames = []
+            while tb:
+                frames.append(tb.tb_frame)
+                tb = tb.tb_next
+        return frames
+
+    f, outer, inner = make_frames()
+    outer.clear()
+    inner.clear()
+    assert not outer.f_locals
+    assert not inner.f_locals
