@@ -1,5 +1,7 @@
 from __future__ import with_statement
 import py
+import commands
+import pypy.conftest
 
 def splitcases(s):
     lines = [line.rstrip() for line in s.split('\n')]
@@ -16,224 +18,138 @@ VALID = splitcases("""
     def f():
         def g():
             global x
-            exec "hi"
-            x
-
-    def f():
-        def g():
-            global x
-            from a import *
+            exec("hi")
             x
 
     def f(x):
         def g():
             global x
-            exec "hi"
-            x
-
-    def f(x):
-        def g():
-            global x
-            from a import *
+            exec("hi")
             x
 
     def f():
         def g():
-            from a import *
+            exec("hi")
 
     def f():
-        def g():
-            exec "hi"
+        exec("hi")
 
     def f():
-        from a import *
-
-    def f():
-        exec "hi"
-
-    def f():
-        from a import *
+        exec("hi")
         def g():
             global x
             x
 
     def f():
-        exec "hi"
-        def g():
-            global x
-            x
-
-    def f():
-        from a import *
+        exec("hi")
         def g(x):
             x
 
     def f():
-        exec "hi"
-        def g(x):
-            x
-
-    def f():
-        from a import *
+        exec("hi")
         lambda x: x
 
     def f():
-        exec "hi"
-        lambda x: x
-
-    def f():
-        from a import *
+        exec("hi")
         x
 
     def f():
-        exec "hi"
-        x
-
-    def f():
-        from a import *
-        (i for i in x)
-
-    def f():
-        exec "hi"
+        exec("hi")
         (i for i in x)
 
     def f():
         class g:
-            exec "hi"
-            x
-
-    def f():
-        class g:
-            from a import *
+            exec("hi")
             x
 
 """)
 
-##    --- the following ones are valid in CPython, but not sensibly so:
+##    --- the following one is valid in CPython, but not sensibly so:
 ##    --- if x is rebound, then it is even rebound in the parent scope!
 ##    def f(x):
 ##        class g:
-##            from a import *
-##            x
-##    def f(x):
-##        class g:
 ##            exec "x=41"
 ##            x
 
 INVALID = splitcases("""
 
     def f():
-        def g():
-            exec "hi"
-            x
-    # NB. the above one is invalid in CPython, but there is no real reason
-
-    def f(x):
-        def g():
-            exec "hi"
-            x
-
-    def f():
-        def g():
-            from a import *
-            x
-    # NB. the above one is invalid in CPython, but there is no real reason
-
-    def f(x):
-        def g():
-            from a import *
-            x
-
-    def f():
-        exec "hi"
-        def g():
-            x
-
-    def f():
-        exec "hi"
-        lambda x: y
-
-    def f():
-        from a import *
-        x
-        def g():
-            x
-
-    def f():
-        from a import *
-        lambda x: y
-
-    def f():
-        exec "hi"
-        class g:
-            x
-
-    def f():
-        from a import *
-        x
-        class g:
-            x
-
-    def f():
-        exec "hi"
-        x = 5
-        class g:
-            def h():
-                x
-
-    def f():
-        from a import *
-        x = 4
-        class g:
-            def h():
-                x
-
-    def f(x):
-        exec "hi"
-        x = 4
-        class g:
-            x
-
-    def f(x):
-        from a import *
-        class g:
-            x
-
-    def f(x):
-        exec "hi"
-        x = 5
-        class g:
-            def h():
-                x
-
-    def f(x):
-        from a import *
-        x = 5
-        class g:
-            def h():
-                x
+        from x import *
 
     def f():
         (i for i in x) = 10
 
-    def f(x):
-        def g():
-            from a import *
-            def k():
-                return x
+    async def foo(a=await something()):
+        pass
+
+    async def foo():
+        await
+
+    def foo():
+        await something()
+
+    async def foo():
+        yield from []
+
+    async def foo():
+        await await fut
+
+    async def foo():
+        yield
+        return 42
 
 """)
 
 
 for i in range(len(VALID)):
-    exec """def test_valid_%d(space):
+    exec """def test_valid_%d(space, tmpdir):
+                checkvalid_cpython(tmpdir, %d, %r)
                 checkvalid(space, %r)
-""" % (i, VALID[i])
+""" % (i, i, VALID[i], VALID[i])
 
 for i in range(len(INVALID)):
-    exec """def test_invalid_%d(space):
+    exec """def test_invalid_%d(space, tmpdir):
+                checkinvalid_cpython(tmpdir, %d, %r)
                 checkinvalid(space, %r)
-""" % (i, INVALID[i])
+""" % (i, i, INVALID[i], INVALID[i])
+
+
+def checksyntax_cpython(tmpdir, i, s):
+    python3 = pypy.conftest.option.python
+    if python3 is None:
+        print 'Warning: cannot run python3 to check syntax'
+        return
+
+    src = '''
+try:
+    exec("""%s
+""")
+except SyntaxError as e:
+    print(e)
+    raise SystemExit(1)
+else:
+    print('OK')
+''' % s
+    pyfile = tmpdir.join('checkvalid_%d.py' % i)
+    pyfile.write(src)
+    res = commands.getoutput('"%s" "%s"' % (python3, pyfile))
+    return res
+
+def checkvalid_cpython(tmpdir, i, s):
+    res = checksyntax_cpython(tmpdir, i, s)
+    if res is not None and res != 'OK':
+        print s
+        print
+        print res
+        assert False, 'checkvalid_cpython failed'
+
+def checkinvalid_cpython(tmpdir, i, s):
+    res = checksyntax_cpython(tmpdir, i, s)
+    if res is not None and res == 'OK':
+        print s
+        print
+        print res
+        assert False, 'checkinvalid_cpython failed, did not raise SyntaxError'
 
 
 def checkvalid(space, s):
@@ -268,39 +184,38 @@ class AppTestCondExpr:
     def test_condexpr(self):
         for s, expected in [("x = 1 if True else 2", 1),
                             ("x = 1 if False else 2", 2)]:
-            exec s
-            assert x == expected
-
-    def test_condexpr_no_warning(self):
-        import warnings
-
-        warnings.simplefilter('error', SyntaxWarning)
-        exec "1 if True else 2"
-        warnings.simplefilter('default', SyntaxWarning)
+            ns = {}
+            exec(s, ns)
+            assert ns['x'] == expected
 
 class AppTestYield:
     def test_bare_yield(self):
         s = "def f():\n    yield"
 
-        exec s
+        exec(s)
 
 
 class AppTestDecorators:
 
     def test_function_decorators(self):
+        '''
         def other():
             return 4
         def dec(f):
             return other
         ns = {}
         ns["dec"] = dec
-        exec """@dec
-def g():
-    pass""" in ns
+        exec("""if 1:
+                    @dec
+                    def g():
+                        pass
+             """, ns)
         assert ns["g"] is other
         assert ns["g"]() == 4
+        '''
 
     def test_application_order(self):
+        '''
         def dec1(f):
             record.append(1)
             return f
@@ -309,62 +224,87 @@ def g():
             return f
         record = []
         ns = {"dec1" : dec1, "dec2" : dec2}
-        exec """@dec1
-@dec2
-def g(): pass""" in ns
+        exec("""if 1:
+                    @dec1
+                    @dec2
+                    def g():
+                        pass
+             """, ns)
         assert record == [2, 1]
         del record[:]
-        exec """@dec1
-@dec2
-class x: pass""" in ns
+        exec("""if 1:
+                    @dec1
+                    @dec2
+                    class x:
+                        pass
+             """, ns)
         assert record == [2, 1]
+        '''
 
     def test_class_decorators(self):
         s = """@func
 class x: pass"""
         ns = {"func" : lambda cls: 4}
-        exec s in ns
+        exec(s, ns)
         assert ns["x"] == 4
 
 
 class AppTestPrintFunction:
 
     def test_simple_print(self):
-        import __builtin__
-        s = """from __future__ import print_function
-x = print
-"""
+        """
+        import builtins
+        s = "x = print"
         ns = {}
-        exec s in ns
-        assert ns["x"] is getattr(__builtin__, "print")
+        exec(s, ns)
+        assert ns["x"] is builtins.print
+        """
 
     def test_print(self):
-        s = """from __future__ import print_function
-import StringIO
-s = StringIO.StringIO()
+        s = """
+from io import StringIO
+s = StringIO()
 print("Hello,", "person", file=s)
 """
         ns = {}
-        exec s in ns
+        exec(s, ns)
         assert ns["s"].getvalue() == "Hello, person\n"
 
 
 class AppTestUnicodeLiterals:
 
     def test_simple(self):
-        s = """from __future__ import unicode_literals
+        s = """
 x = 'u'
 y = r'u'
-z = u'u'
 b = b'u'
-c = br'u'"""
+c = br'u'
+d = rb'u'
+"""
         ns = {}
-        exec s in ns
-        assert isinstance(ns["x"], unicode)
-        assert isinstance(ns["y"], unicode)
-        assert isinstance(ns["z"], unicode)
-        assert isinstance(ns["b"], str)
-        assert isinstance(ns["c"], str)
+        exec(s, ns)
+        assert isinstance(ns["x"], str)
+        assert isinstance(ns["y"], str)
+        assert isinstance(ns["b"], bytes)
+        assert isinstance(ns["c"], bytes)
+        assert isinstance(ns["d"], bytes)
+
+    def test_triple_quotes(self):
+        s = '''
+x = """u"""
+y = r"""u"""
+b = b"""u"""
+c = br"""u"""
+d = rb"""u"""
+'''
+
+        ns = {}
+        exec(s, ns)
+        assert isinstance(ns["x"], str)
+        assert isinstance(ns["y"], str)
+        assert isinstance(ns["b"], bytes)
+        assert isinstance(ns["c"], bytes)
+        assert isinstance(ns["d"], bytes)
 
     def test_both_futures_with_semicolon(self):
         # Issue #2526: a corner case which crashes only if the file
@@ -373,7 +313,7 @@ c = br'u'"""
         s = """
 from __future__ import unicode_literals; from __future__ import print_function
 """
-        exec s in {}
+        exec(s, {})
 
 
 class AppTestComprehensions:
@@ -404,6 +344,9 @@ class AppTestComprehensions:
 
 class AppTestWith:
     def test_with_simple(self):
+
+        s = """
+if 1:
         class Context:
             def __init__(self):
                 self.calls = list()
@@ -417,28 +360,78 @@ class AppTestWith:
         acontext = Context()
         with acontext:
             pass
+        """
+        ns = {}
+        exec(s, ns)
+        acontext = ns['acontext']
         assert acontext.calls == '__enter__ __exit__'.split()
 
     def test_compound_with(self):
+        s = """class Context:
+    def __init__(self, var):
+        self.record = []
+        self.var = var
+    def __enter__(self):
+        self.record.append(("__enter__", self.var))
+        return self.var
+    def __exit__(self, tp, value, tb):
+        self.record.append(("__exit__", self.var))
+c1 = Context("blah")
+c2 = Context("bling")
+with c1 as v1, c2 as v2:
+    pass
+    """
+        ns = {}
+        exec(s, ns)
+        assert ns["v1"] == "blah"
+        assert ns["v2"] == "bling"
+        assert ns["c1"].record == [("__enter__", "blah"), ("__exit__", "blah")]
+        assert ns["c2"].record == [("__enter__", "bling"),
+                                   ("__exit__", "bling")]
+
+
+    def test_start_with_blank_line(self):
+        s = """
+if 1:
         class Context:
-            def __init__(self, var):
-                self.record = []
-                self.var = var
+            def __init__(self):
+                self.calls = list()
+
             def __enter__(self):
-                self.record.append(("__enter__", self.var))
-                return self.var
-            def __exit__(self, tp, value, tb):
-                self.record.append(("__exit__", self.var))
-        c1 = Context("blah")
-        c2 = Context("bling")
-        with c1 as v1, c2 as v2:
+                self.calls.append('__enter__')
+
+            def __exit__(self, exc_type, exc_value, exc_tb):
+                self.calls.append('__exit__')
+
+        acontext = Context()
+        with acontext:
             pass
-        assert v1 == "blah"
-        assert v2 == "bling"
-        assert c1.record == [("__enter__", "blah"), ("__exit__", "blah")]
-        assert c2.record == [("__enter__", "bling"), ("__exit__", "bling")]
+"""
+        ns = {}
+        exec(s, ns)
+        acontext = ns['acontext']
+        assert acontext.calls == '__enter__ __exit__'.split()
+
+    def test_raw_doc_string(self):
+        s = """r'doc'
+class Context(object):
+    def __enter__(self):
+        global enter
+        enter = True
+    def __exit__(self, *exc):
+        global exit
+        exit = True
+with Context() as w:
+    pass"""
+        ns = {}
+        exec(s, ns)
+        assert ns['enter']
+        assert ns['exit']
 
     def test_with_as_var(self):
+
+        s = """
+if 1:
         class Context:
             def __init__(self):
                 self.calls = list()
@@ -455,10 +448,17 @@ class AppTestWith:
         with acontextfact as avar:
             avar.append('__body__')
             pass
+        """
+        ns = {}
+        exec(s, ns)
+        acontextfact = ns['acontextfact']
         assert acontextfact.exit_params == (None, None, None)
         assert acontextfact.calls == '__enter__ __body__ __exit__'.split()
 
     def test_with_raise_exception(self):
+
+        s = """
+if 1:
         class Context:
             def __init__(self):
                 self.calls = list()
@@ -482,12 +482,20 @@ class AppTestWith:
             pass
         else:
             raise AssertionError('With did not raise RuntimeError')
+        """
+        ns = {}
+        exec(s, ns)
+        acontextfact = ns['acontextfact']
+        error = ns['error']
         assert acontextfact.calls == '__enter__ __body__ __exit__'.split()
         assert acontextfact.exit_params[0:2] == (RuntimeError, error)
         import types
         assert isinstance(acontextfact.exit_params[2], types.TracebackType)
 
     def test_with_swallow_exception(self):
+
+        s = """
+if 1:
         class Context:
             def __init__(self):
                 self.calls = list()
@@ -507,6 +515,11 @@ class AppTestWith:
             avar.append('__body__')
             raise error
             avar.append('__after_raise__')
+        """
+        ns = {}
+        exec(s, ns)
+        acontextfact = ns['acontextfact']
+        error = ns['error']
         assert acontextfact.calls == '__enter__ __body__ __exit__'.split()
         assert acontextfact.exit_params[0:2] == (RuntimeError, error)
         import types
@@ -531,6 +544,9 @@ class AppTestWith:
         assert c.calls == ['exit']
 
     def test_with_break(self):
+
+        s = """
+if 1:
         class Context:
             def __init__(self):
                 self.calls = list()
@@ -552,10 +568,17 @@ class AppTestWith:
                 avar.append('__after_break__')
         else:
             raise AssertionError('Break failed with With, reached else clause')
+        """
+        ns = {}
+        exec(s, ns)
+        acontextfact = ns['acontextfact']
         assert acontextfact.calls == '__enter__ __body__ __exit__'.split()
         assert acontextfact.exit_params == (None, None, None)
 
     def test_with_continue(self):
+
+        s = """
+if 1:
         class Context:
             def __init__(self):
                 self.calls = list()
@@ -577,10 +600,16 @@ class AppTestWith:
                 avar.append('__after_continue__')
         else:
             avar.append('__continue__')
+        """
+        ns = {}
+        exec(s, ns)
+        acontextfact = ns['acontextfact']
         assert acontextfact.calls == '__enter__ __body__ __exit__ __continue__'.split()
         assert acontextfact.exit_params == (None, None, None)
 
     def test_with_return(self):
+        s = """
+if 1:
         class Context:
             def __init__(self):
                 self.calls = list()
@@ -601,16 +630,28 @@ class AppTestWith:
                 return '__return__'
                 avar.append('__after_return__')
         acontextfact.calls.append(g(acontextfact))
+        """
+        ns = {}
+        exec(s, ns)
+        acontextfact = ns['acontextfact']
         assert acontextfact.calls == '__enter__ __body__ __exit__ __return__'.split()
         assert acontextfact.exit_params == (None, None, None)
 
     def test_with_as_keyword(self):
-        with raises(SyntaxError):
-            exec "with = 9"
+        try:
+            exec("with = 9")
+        except SyntaxError:
+            pass
+        else:
+            assert False, 'Assignment to with did not raise SyntaxError'
 
     def test_with_as_keyword_compound(self):
-        with raises(SyntaxError):
-            exec "from __future__ import generators, with_statement\nwith = 9"
+        try:
+            exec("from __future__ import generators, with_statement\nwith = 9")
+        except SyntaxError:
+            pass
+        else:
+            assert False, 'Assignment to with did not raise SyntaxError'
 
     def test_missing_as_SyntaxError(self):
         snippets = [
@@ -621,16 +662,58 @@ with foo a bar:
     pass
 """]
         for snippet in snippets:
-            with raises(SyntaxError):
-                exec snippet
+            try:
+                exec(snippet)
+            except SyntaxError:
+                pass
+            else:
+                assert False, "%s: did not raise SyntaxError" % snippet
 
 
+    def test_with_propagate_compileflag(self):
+        s = """
+if 1:
+        compile('''with x:
+        pass''', '', 'exec')
+        """
+        exec(s)
+
+class AppTestFunctionAnnotations:
+
+    def test_simple(self):
+        """
+        def f(e:3=4): pass
+        assert f.__annotations__ == {"e" : 3}
+        def f(a : 1, b : 2, *var : 3, hi : 4, bye : 5=0, **kw : 6) -> 42: pass
+        assert f.__annotations__ == {"a" : 1, "b" : 2, "var" : 3, "hi" : 4,
+                                    "bye" : 5, "kw" : 6, "return" : 42}
+        """
+
+    def test_bug_annotations_lambda(self):
+        """
+        # those used to crash
+        def broken(*a: lambda x: None):
+            pass
+
+        def broken(**a: lambda x: None):
+            pass
+        """
+
+    def test_bug_annotation_inside_nested_function(self):
+        """
+        # this used to crash
+        def f1():
+            def f2(*args: int):
+                pass
+        f1()
+        """
+
 class AppTestSyntaxError:
 
     def test_tokenizer_error_location(self):
         line4 = "if ?: pass\n"
         try:
-            exec "print\nprint\nprint\n" + line4
+            exec("print\nprint\nprint\n" + line4)
         except SyntaxError as e:
             assert e.lineno == 4
             assert e.text == line4
@@ -640,12 +723,12 @@ class AppTestSyntaxError:
 
     def test_grammar_error_location(self):
         try:
-            exec """if 1:
+            exec("""if 1:
                 class Foo:
                     bla
                     a b c d e
                     bar
-            """
+            """)
         except SyntaxError as e:
             assert e.lineno == 4
             assert e.text.endswith('a b c d e\n')
@@ -656,17 +739,83 @@ class AppTestSyntaxError:
     def test_astbuilder_error_location(self):
         program = "(1, 2) += (3, 4)\n"
         try:
-            exec program
+            exec(program)
         except SyntaxError as e:
             assert e.lineno == 1
-            assert e.text == program
+            assert e.text is None
         else:
             raise Exception("no SyntaxError??")
 
     def test_bad_encoding(self):
+        '''
         program = """
 # -*- coding: uft-8 -*-
 pass
 """
-        with raises(SyntaxError):
-            exec program
+        raises(SyntaxError, exec, program)
+        '''
+
+    def test_exception_target_in_nested_scope(self):
+        # issue 4617: This used to raise a SyntaxError
+        # "can not delete variable 'e' referenced in nested scope"
+        def print_error():
+            e
+        try:
+            something
+        except Exception as e:
+            print_error()
+            # implicit "del e" here
+
+    def test_cpython_issue2382(self):
+        code = 'Python = "\u1e54\xfd\u0163\u0125\xf2\xf1" +'
+        exc = raises(SyntaxError, compile, code, 'foo', 'exec')
+        assert exc.value.offset in (19, 20) # pypy, cpython
+
+    def test_empty_tuple_target(self): """
+        def f(n):
+            () = ()
+            ((), ()) = [[], []]
+            del ()
+            del ((), ())
+            [] = {}
+            ([], ()) = [[], ()]
+            [[], ()] = ((), [])
+            del []
+            del [[], ()]
+            for () in [(), []]: pass
+            for [] in ([],): pass
+            class Zen:
+                def __enter__(self): return ()
+                def __exit__(self, *args): pass
+            with Zen() as (): pass
+            () = [2, 3] * n
+        f(0)
+        try:
+            f(5)
+        except ValueError:
+            pass
+        else:
+            raise AssertionError("should have raised")
+        """
+
+
+if __name__ == '__main__':
+    # only to check on top of CPython (you need 2.4)
+    from py.test import raises
+    for s in VALID:
+        try:
+            compile(s, '?', 'exec')
+        except Exception as e:
+            print '-'*20, 'FAILED TO COMPILE:', '-'*20
+            print s
+            print '%s: %s' % (e.__class__, e)
+            print '-'*60
+    for s in INVALID:
+        try:
+            raises(SyntaxError, compile, s, '?', 'exec')
+        except Exception as e:
+            print '-'*20, 'UNEXPECTEDLY COMPILED:', '-'*20
+            print s
+            print '%s: %s' % (e.__class__, e)
+            print '-'*60
+
