@@ -8,16 +8,15 @@ from pypy.interpreter.gateway import (interp2app, BuiltinCode, unwrap_spec,
 
 from rpython.rlib.jit import promote
 from rpython.rlib.objectmodel import compute_identity_hash, specialize
-from rpython.rlib.objectmodel import instantiate, not_rpython, try_inline, dont_inline
+from rpython.rlib.objectmodel import instantiate, not_rpython
 from rpython.tool.sourcetools import compile2, func_with_new_name
 
 
 class TypeDef(object):
     @not_rpython
     def __init__(self, __name, __base=None, __total_ordering__=None,
-                 __buffer=None,
-                 __rpython_level_class__=None,
-                 **rawdict):
+                 __buffer=None, __confirm_applevel_del__=False,
+                 _text_signature_=None, variable_sized=False, **rawdict):
         "initialization-time only"
         self.name = __name
         if __base is None:
@@ -29,13 +28,18 @@ class TypeDef(object):
         self.bases = bases
         # Used in cpyext to fill tp_as_buffer slots
         assert __buffer in {None, 'read-write', 'read'}, "Unknown value for __buffer"
+        for base in bases:
+            if __buffer is None:
+                __buffer = base.buffer
         self.buffer = __buffer
         self.heaptype = False
         self.hasdict = '__dict__' in rawdict
         # no __del__: use an RPython _finalize_() method and register_finalizer
-        assert '__del__' not in rawdict
+        if not __confirm_applevel_del__:
+            assert '__del__' not in rawdict
         self.weakrefable = '__weakref__' in rawdict
-        self.doc = rawdict.pop('__doc__', None)
+        self.doc = rawdict.get('__doc__', None)
+        self.text_signature = _text_signature_
         for base in bases:
             self.hasdict |= base.hasdict
             self.weakrefable |= base.weakrefable
@@ -43,9 +47,10 @@ class TypeDef(object):
         self.acceptable_as_base_class = '__new__' in rawdict
         self.applevel_subclasses_base = None
         self.add_entries(**rawdict)
-        assert __total_ordering__ in (None, ), "__total_ordering__ was buggy, mostly unused, and has been removed"
-        self.rpy_cls = __rpython_level_class__
-        self._install_shortcuts()
+        assert __total_ordering__ in (None, 'auto'), "Unknown value for __total_ordering"
+        if __total_ordering__ == 'auto':
+            self.auto_total_ordering()
+        self.variable_sized = variable_sized
 
     def add_entries(self, **rawdict):
         # xxx fix the names of the methods to match what app-level expects
@@ -54,6 +59,14 @@ class TypeDef(object):
                 value.name = key
         self.rawdict.update(rawdict)
 
+    def auto_total_ordering(self):
+        assert '__lt__' in self.rawdict, "__total_ordering='auto' requires __lt__"
+        assert '__eq__' in self.rawdict, "__total_ordering='auto' requires __eq__"
+        self.add_entries(__le__ = auto__le__,
+                         __gt__ = auto__gt__,
+                         __ge__ = auto__ge__,
+                         __ne__ = auto__ne__)
+
     def _freeze_(self):
         # hint for the annotator: track individual constant instances of TypeDef
         return True
@@ -61,55 +74,25 @@ class TypeDef(object):
     def __repr__(self):
         return "<%s name=%r>" % (self.__class__.__name__, self.name)
 
-    def _install_shortcuts(self):
-        rawdict = self.rawdict
-        # guess the class, if not given explicitly
-        rpy_cls = self.rpy_cls
-        for key, val in rawdict.iteritems():
-            ncls = None
-            if isinstance(val, interp2app):
-                ncls = val.self_type
-                if ncls:
-                    if rpy_cls is None:
-                        rpy_cls = ncls
-                    else:
-                        if issubclass(ncls, rpy_cls):
-                            rpy_cls = ncls # use most specific class
-                        else:
-                            assert issubclass(rpy_cls, ncls)
-        if rpy_cls is None:
-            # safety check: if rpy_cls is unknown, and one of the SHORTCUTS is
-            # in rawdict, the shortcut could be wrong (if we have an
-            # intermediate class that's not W_Root).
-            names = [name for name, shortcut_name, fallback, checkerfunc in SHORTCUTS
-                if name in rawdict]
-            assert not names
-            return
-        if 'micronumpy' in rpy_cls.__module__:
-            return
-        if '_descroperation_shortcuts_installed' in rpy_cls.__dict__:
-            return
-        rpy_cls._descroperation_shortcuts_installed = True
-        for name, shortcut_name, fallback, checkerfunc in SHORTCUTS:
-            if name not in rawdict or rawdict[name]._staticdefs:
-                if W_Root not in rpy_cls.__bases__:
 
-                    shortcut = getattr(rpy_cls, shortcut_name).im_func
-                    if shortcut is not fallback:
-                        assert shortcut.source_typedef in self.all_bases(), \
-                                "getting a wrong shortcut %s for class %s from some base class that is not W_Root" % (name, rpy_cls)
-                continue
-            shortcut_func = rawdict[name]._make_descroperation_shortcut(
-                    name, rpy_cls, checkerfunc)
-            shortcut_func.source_typedef = self
-            setattr(rpy_cls, shortcut_name, shortcut_func)
+# generic special cmp methods defined on top of __lt__ and __eq__, used by
+# automatic total ordering
 
-    def all_bases(self):
-        for base in self.bases:
-            yield base
-            for up in base.all_bases():
-                yield up
+@interp2app
+def auto__le__(space, w_self, w_other):
+    return space.not_(space.lt(w_other, w_self))
 
+@interp2app
+def auto__gt__(space, w_self, w_other):
+    return space.lt(w_other, w_self)
+
+@interp2app
+def auto__ge__(space, w_self, w_other):
+    return space.not_(space.lt(w_self, w_other))
+
+@interp2app
+def auto__ne__(space, w_self, w_other):
+    return space.not_(space.eq(w_self, w_other))
 
 
 # ____________________________________________________________
@@ -151,102 +134,43 @@ def get_unique_interplevel_subclass(space, cls):
         return subcls
 _unique_subclass_cache = {}
 
-def _getusercls(cls, reallywantdict=False):
+def _getusercls(cls):
     from rpython.rlib import objectmodel
     from pypy.objspace.std.objectobject import W_ObjectObject
-    from pypy.module.__builtin__.interp_classobj import W_InstanceObject
     from pypy.objspace.std.mapdict import (BaseUserClassMapdict,
             MapdictDictSupport, MapdictWeakrefSupport,
             _make_storage_mixin_size_n, MapdictStorageMixin)
-    # some subtleties here: We want w_obj.getclass to be a small func
-    # set, ie less than 5 different implementations. That way, it can be
-    # inlined into its callers. This means we cannot give every single
-    # user-defined subclass its own getclass, instead we use the same function
-    # for all of them. This has the effect that the call to
-    # w_obj._get_mapdict_map() in BaseUserClassMapdict.getclass is an
-    # *indirect* call. That's fine, however, we want the object subclasses to
-    # work somewhat better than the rest, so W_ObjectObjectUserDictWeakrefable
-    # should have a *copy* of getclass. This is all achieved by using
-    # _share_methods (which shares functions, does not create copies) instead of
-    # import_from_mixin, which *does* copy functions. There is a test for all
-    # of this in test_mapdict.py, test_correct_method_sharing
     typedef = cls.typedef
     name = cls.__name__ + "User"
-    isobjectsubclass = cls is W_ObjectObject
 
-    if isobjectsubclass or cls is W_InstanceObject:
+    if cls is W_ObjectObject:
         base_mixin = _make_storage_mixin_size_n()
     else:
         base_mixin = MapdictStorageMixin
-    if not isobjectsubclass:
-        shared_methods = [BaseUserClassMapdict]
-    else:
-        shared_methods = []
-    if reallywantdict or not typedef.hasdict:
+    copy_methods = [BaseUserClassMapdict]
+    if not typedef.hasdict:
         # the type has no dict, mapdict to provide the dict
-        shared_methods.append(MapdictDictSupport)
+        copy_methods.append(MapdictDictSupport)
         name += "Dict"
     if not typedef.weakrefable:
         # the type does not support weakrefs yet, mapdict to provide weakref
         # support
-        shared_methods.append(MapdictWeakrefSupport)
+        copy_methods.append(MapdictWeakrefSupport)
         name += "Weakrefable"
 
     class subcls(cls):
         user_overridden_class = True
         objectmodel.import_from_mixin(base_mixin)
-        if isobjectsubclass:
-            objectmodel.import_from_mixin(BaseUserClassMapdict)
-
-    for _, shortcut_name, meth, _ in SHORTCUTS:
-        setattr(subcls, shortcut_name, meth)
-
-    for copycls in shared_methods:
-        _share_methods(copycls, subcls)
+    for copycls in copy_methods:
+        _copy_methods(copycls, subcls)
     subcls.__name__ = name
     return subcls
 
-def _share_methods(copycls, subcls):
+def _copy_methods(copycls, subcls):
     for key, value in copycls.__dict__.items():
         if (not key.startswith('__') or key == '__del__'):
             setattr(subcls, key, value)
 
-# ____________________________________________________________
-# descroperation shortcuts
-
-SHORTCUTS = []
-
-def use_special_method_shortcut(name, checkerfunc=None):
-    """
-    use a shortcut for implementations of the special method 'name' for
-    built-in types in the decorated descroperation function. The behaviour for
-    builtin types will be equivalent to:
-
-        w_descr = space.lookup(w_obj, name)
-        return space.get_and_call_function(w_descr, w_obj)
-
-    but only if the special method name exists in the type. Note that this
-    means if the descroperation method contains extra logic after the
-    get_and_call_function it will be ignored (which is often safe for built-in
-    types).
-
-    checkerfunc is a non-translation only safety: it's called with the space
-    and the result of the get_and_call_function call and must return True.
-    """
-    def wrapper(func):
-        @dont_inline
-        def shortcut_fallback(self, space, *args_w):
-            return func(space, self, *args_w)
-        shortcut_fallback.func_name = "shortcut_fallback_%s" % name
-        shortcut_name = "shortcut_%s" % name
-        SHORTCUTS.append((name, shortcut_name, shortcut_fallback, checkerfunc))
-        @try_inline
-        def call_shortcut(space, self, *args_w):
-            return getattr(self, shortcut_name)(space, *args_w)
-        setattr(W_Root, shortcut_name, shortcut_fallback)
-        return call_shortcut
-    return wrapper
-
 
 # ____________________________________________________________
 
@@ -259,9 +183,9 @@ def make_descr_typecheck_wrapper(tag, func, extraargs=(), cls=None,
 
 @specialize.memo()
 def _make_descr_typecheck_wrapper(tag, func, extraargs, cls, use_closure):
-    from rpython.flowspace.bytecode import cpython_code_signature
     # - if cls is None, the wrapped object is passed to the function
     # - if cls is a class, an unwrapped instance is passed
+    # - if cls is a string, XXX unused?
     if cls is None and use_closure:
         return func
     if hasattr(func, 'im_func'):
@@ -289,7 +213,8 @@ def _make_descr_typecheck_wrapper(tag, func, extraargs, cls, use_closure):
 
     name = func.__name__
     extra = ', '.join(extraargs)
-    sig = cpython_code_signature(func.func_code)
+    from pypy.interpreter import pycode
+    sig = pycode.cpython_code_signature(func.func_code)
     argnames = sig.argnames
     if use_closure:
         if argnames[1] == 'space':
@@ -305,6 +230,33 @@ def _make_descr_typecheck_wrapper(tag, func, extraargs, cls, use_closure):
     exec source.compile() in miniglobals
     return miniglobals['descr_typecheck_%s' % func.__name__]
 
+@specialize.arg(0)
+def make_objclass_getter(tag, func, cls):
+    if func and hasattr(func, 'im_func'):
+        assert not cls or cls is func.im_class
+        cls = func.im_class
+    return _make_objclass_getter(cls)
+
+@specialize.memo()
+def _make_objclass_getter(cls):
+    if not cls:
+        return None, cls
+    miniglobals = {}
+    if isinstance(cls, str):
+        assert cls.startswith('<'), "pythontype typecheck should begin with <"
+        cls_name = cls[1:]
+        typeexpr = "space.w_%s" % cls_name
+    else:
+        miniglobals['cls'] = cls
+        typeexpr = "space.gettypeobject(cls.typedef)"
+    source = """if 1:
+        def objclass_getter(space):
+            return %s
+        \n""" % (typeexpr,)
+    exec compile2(source) in miniglobals
+    res = miniglobals['objclass_getter'], cls
+    return res
+
 class GetSetProperty(W_Root):
     _immutable_fields_ = ["fget", "fset", "fdel"]
     w_objclass = None
@@ -312,33 +264,33 @@ class GetSetProperty(W_Root):
     @specialize.arg(7)
     def __init__(self, fget, fset=None, fdel=None, doc=None,
                  cls=None, use_closure=False, tag=None, name=None):
-        if fget and hasattr(fget, 'im_func'):
-            assert not cls or cls is fget.im_class
-            cls = fget.im_class
+        objclass_getter, cls = make_objclass_getter(tag, fget, cls)
         fget = make_descr_typecheck_wrapper((tag, 0), fget,
                                             cls=cls, use_closure=use_closure)
         fset = make_descr_typecheck_wrapper((tag, 1), fset, ('w_value',),
                                             cls=cls, use_closure=use_closure)
         fdel = make_descr_typecheck_wrapper((tag, 2), fdel,
                                             cls=cls, use_closure=use_closure)
-        self._init(fget, fset, fdel, doc, cls, use_closure,
+        self._init(fget, fset, fdel, doc, cls, objclass_getter, use_closure,
                    name)
 
-    def _init(self, fget, fset, fdel, doc, cls, use_closure,
+    def _init(self, fget, fset, fdel, doc, cls, objclass_getter, use_closure,
               name):
         self.fget = fget
         self.fset = fset
         self.fdel = fdel
         self.doc = doc
         self.reqcls = cls
+        self.w_qualname = None
+        self.objclass_getter = objclass_getter
         self.use_closure = use_closure
         self.name = name if name is not None else '<generic property>'
 
     def copy_for_type(self, w_objclass):
-        if self.reqcls is None:
+        if self.objclass_getter is None:
             new = instantiate(GetSetProperty)
             new._init(self.fget, self.fset, self.fdel, self.doc, self.reqcls,
-                      self.use_closure, self.name)
+                      None, self.use_closure, self.name)
             new.w_objclass = w_objclass
             return new
         else:
@@ -366,9 +318,9 @@ class GetSetProperty(W_Root):
 
     def readonly_attribute(self, space):   # overwritten in cpyext
         if self.name == '<generic property>':
-            raise oefmt(space.w_TypeError, "readonly attribute")
+            raise oefmt(space.w_AttributeError, "readonly attribute")
         else:
-            raise oefmt(space.w_TypeError, "readonly attribute '%s'", self.name)
+            raise oefmt(space.w_AttributeError, "readonly attribute '%s'", self.name)
 
     def descr_property_set(self, space, w_obj, w_value):
         """property.__set__(obj, value)
@@ -399,14 +351,29 @@ class GetSetProperty(W_Root):
                 self.reqcls, Arguments(space, [w_obj,
                                                space.newtext(self.name)]))
 
-    def descr_get_objclass(self, space):
-        if self.w_objclass is not None:
-            return self.w_objclass
-        if self.reqcls is not None:
-            return space.gettypeobject(self.reqcls.typedef)
+    def descr_get_qualname(self, space):
+        if self.w_qualname is None:
+            self.w_qualname = self._calculate_qualname(space)
+        return self.w_qualname
+
+    def _calculate_qualname(self, space):
+        if self.reqcls is None:
+            type_qualname = '?'
+        else:
+            w_type = space.gettypeobject(self.reqcls.typedef)
+            type_qualname = space.text_w(
+                space.getattr(w_type, space.newtext('__qualname__')))
+        qualname = "%s.%s" % (type_qualname, self.name)
+        return space.newtext(qualname)
+
+    def descr_get_objclass(space, property):
+        if property.w_objclass is not None:
+            return property.w_objclass
+        if property.objclass_getter is not None:
+            return property.objclass_getter(space)
         # NB. this is an AttributeError to make inspect.py happy
         raise oefmt(space.w_AttributeError,
-                    "generic self has no __objclass__")
+                    "generic property has no __objclass__")
 
     def spacebind(self, space):
         if hasattr(space, '_see_getsetproperty'):
@@ -440,6 +407,7 @@ GetSetProperty.typedef = TypeDef(
     __set__ = interp2app(GetSetProperty.descr_property_set),
     __delete__ = interp2app(GetSetProperty.descr_property_del),
     __name__ = interp_attrproperty('name', cls=GetSetProperty, wrapfn="newtext_or_none"),
+    __qualname__ = GetSetProperty(GetSetProperty.descr_get_qualname),
     __objclass__ = GetSetProperty(GetSetProperty.descr_get_objclass),
     __doc__ = interp_attrproperty('doc', cls=GetSetProperty, wrapfn="newtext_or_none"),
     )
@@ -534,6 +502,10 @@ from pypy.interpreter.module import Module
 from pypy.interpreter.function import (Function, Method, StaticMethod,
     ClassMethod, BuiltinFunction, descr_function_get)
 from pypy.interpreter.pytraceback import PyTraceback
+from pypy.interpreter.generator import GeneratorIterator, Coroutine
+from pypy.interpreter.generator import CoroutineWrapper, AIterWrapper
+from pypy.interpreter.generator import AsyncGenerator
+from pypy.interpreter.generator import AsyncGenASend, AsyncGenAThrow
 from pypy.interpreter.nestedscope import Cell
 from pypy.interpreter.special import NotImplemented, Ellipsis
 
@@ -577,6 +549,12 @@ def fget_co_varnames(space, code): # unwrapping through unwrap_spec
 def fget_co_argcount(space, code): # unwrapping through unwrap_spec
     return space.newint(code.signature().num_argnames())
 
+def fget_co_kwonlyargcount(space, code): # unwrapping through unwrap_spec
+    return space.newint(code.signature().num_kwonlyargnames())
+
+def fget_zero(space, code):
+    return space.newint(0)
+
 def fget_co_flags(space, code): # unwrapping through unwrap_spec
     sig = code.signature()
     flags = 0
@@ -622,6 +600,7 @@ Code.typedef = TypeDef('internal-code',
     co_name = interp_attrproperty('co_name', cls=Code, wrapfn="newtext_or_none"),
     co_varnames = GetSetProperty(fget_co_varnames, cls=Code),
     co_argcount = GetSetProperty(fget_co_argcount, cls=Code),
+    co_kwonlyargcount = GetSetProperty(fget_zero, cls=Code),
     co_flags = GetSetProperty(fget_co_flags, cls=Code),
     co_consts = GetSetProperty(fget_co_consts, cls=Code),
     )
@@ -632,6 +611,7 @@ BuiltinCode.typedef = TypeDef('builtin-code',
     co_name = interp_attrproperty('co_name', cls=BuiltinCode, wrapfn="newtext_or_none"),
     co_varnames = GetSetProperty(fget_co_varnames, cls=BuiltinCode),
     co_argcount = GetSetProperty(fget_co_argcount, cls=BuiltinCode),
+    co_kwonlyargcount = GetSetProperty(fget_co_kwonlyargcount, cls=BuiltinCode),
     co_flags = GetSetProperty(fget_co_flags, cls=BuiltinCode),
     co_consts = GetSetProperty(fget_co_consts, cls=BuiltinCode),
     )
@@ -641,11 +621,12 @@ assert not BuiltinCode.typedef.acceptable_as_base_class  # no __new__
 PyCode.typedef = TypeDef('code',
     __new__ = interp2app(PyCode.descr_code__new__.im_func),
     __eq__ = interp2app(PyCode.descr_code__eq__),
-    __ne__ = interp2app(PyCode.descr_code__ne__),
+    __ne__ = descr_generic_ne,
     __hash__ = interp2app(PyCode.descr_code__hash__),
     __reduce__ = interp2app(PyCode.descr__reduce__),
     __repr__ = interp2app(PyCode.repr),
     co_argcount = interp_attrproperty('co_argcount', cls=PyCode, wrapfn="newint"),
+    co_kwonlyargcount = interp_attrproperty('co_kwonlyargcount', cls=PyCode, wrapfn="newint"),
     co_nlocals = interp_attrproperty('co_nlocals', cls=PyCode, wrapfn="newint"),
     co_stacksize = interp_attrproperty('co_stacksize', cls=PyCode, wrapfn="newint"),
     co_flags = interp_attrproperty('co_flags', cls=PyCode, wrapfn="newint"),
@@ -655,7 +636,7 @@ PyCode.typedef = TypeDef('code',
     co_varnames = GetSetProperty(PyCode.fget_co_varnames),
     co_freevars = GetSetProperty(PyCode.fget_co_freevars),
     co_cellvars = GetSetProperty(PyCode.fget_co_cellvars),
-    co_filename = interp_attrproperty('co_filename', cls=PyCode, wrapfn="newtext"),
+    co_filename = interp_attrproperty_w('w_filename', cls=PyCode),
     co_name = interp_attrproperty('co_name', cls=PyCode, wrapfn="newtext"),
     co_firstlineno = interp_attrproperty('co_firstlineno', cls=PyCode, wrapfn="newint"),
     co_lnotab = interp_attrproperty('co_lnotab', cls=PyCode, wrapfn="newbytes"),
@@ -664,17 +645,15 @@ PyCode.typedef = TypeDef('code',
 PyCode.typedef.acceptable_as_base_class = False
 
 PyFrame.typedef = TypeDef('frame',
-    __reduce__ = interp2app(PyFrame.descr__reduce__),
-    __setstate__ = interp2app(PyFrame.descr__setstate__),
+    #__reduce__ = interp2app(PyFrame.descr__reduce__),  --- logic not updated
+    #__setstate__ = interp2app(PyFrame.descr__setstate__),
+    clear = interp2app(PyFrame.descr_clear),
     f_builtins = GetSetProperty(PyFrame.fget_f_builtins),
     f_lineno = GetSetProperty(PyFrame.fget_f_lineno, PyFrame.fset_f_lineno),
     f_back = GetSetProperty(PyFrame.fget_f_back),
     f_lasti = GetSetProperty(PyFrame.fget_f_lasti),
     f_trace = GetSetProperty(PyFrame.fget_f_trace, PyFrame.fset_f_trace,
                              PyFrame.fdel_f_trace),
-    f_exc_type = GetSetProperty(PyFrame.fget_f_exc_type),
-    f_exc_value = GetSetProperty(PyFrame.fget_f_exc_value),
-    f_exc_traceback = GetSetProperty(PyFrame.fget_f_exc_traceback),
     f_restricted = GetSetProperty(PyFrame.fget_f_restricted),
     f_code = GetSetProperty(PyFrame.fget_code),
     f_locals = GetSetProperty(PyFrame.fget_getdictscope),
@@ -686,9 +665,12 @@ Module.typedef = TypeDef("module",
     __new__ = interp2app(Module.descr_module__new__.im_func),
     __init__ = interp2app(Module.descr_module__init__),
     __repr__ = interp2app(Module.descr_module__repr__),
+    __getattribute__=interp2app(Module.descr_getattribute),
+    __dir__ = interp2app(Module.descr_module__dir__),
     __reduce__ = interp2app(Module.descr__reduce__),
     __dict__ = GetSetProperty(descr_get_dict, cls=Module), # module dictionaries are readonly attributes
-    __doc__ = 'module(name[, doc])\n\nCreate a module object.\nThe name must be a string; the optional doc argument can have any type.'
+    __doc__ = 'module(name[, doc])\n\nCreate a module object.\nThe name must be a string; the optional doc argument can have any type.',
+    __weakref__ = make_weakref_descr(Module),
     )
 
 getset_func_doc = GetSetProperty(Function.fget_func_doc,
@@ -707,10 +689,18 @@ getset___module__ = GetSetProperty(Function.fget___module__,
 getset_func_defaults = GetSetProperty(Function.fget_func_defaults,
                                       Function.fset_func_defaults,
                                       Function.fdel_func_defaults)
+getset_func_kwdefaults = GetSetProperty(Function.fget_func_kwdefaults,
+                                        Function.fset_func_kwdefaults,
+                                        Function.fdel_func_kwdefaults)
 getset_func_code = GetSetProperty(Function.fget_func_code,
                                   Function.fset_func_code)
 getset_func_name = GetSetProperty(Function.fget_func_name,
                                   Function.fset_func_name)
+getset_func_qualname = GetSetProperty(Function.fget_func_qualname,
+                                      Function.fset_func_qualname)
+getset_func_annotations = GetSetProperty(Function.fget_func_annotations,
+                                        Function.fset_func_annotations,
+                                        Function.fdel_func_annotations)
 
 getset_func_dict = GetSetProperty(descr_get_dict, descr_set_dict, cls=Function)
 
@@ -720,20 +710,17 @@ Function.typedef = TypeDef("function",
                           descrmismatch='__call__'),
     __get__ = interp2app(descr_function_get),
     __repr__ = interp2app(Function.descr_function_repr, descrmismatch='__repr__'),
-    __reduce__ = interp2app(Function.descr_function__reduce__),
-    __setstate__ = interp2app(Function.descr_function__setstate__),
-    func_code = getset_func_code,
-    func_doc = getset_func_doc,
-    func_name = getset_func_name,
-    func_dict = getset_func_dict,
-    func_defaults = getset_func_defaults,
-    func_globals = interp_attrproperty_w('w_func_globals', cls=Function),
-    func_closure = GetSetProperty(Function.fget_func_closure),
+    #__reduce__ = interp2app(Function.descr_function__reduce__),
+    #__setstate__ = interp2app(Function.descr_function__setstate__),
     __code__ = getset_func_code,
     __doc__ = getset_func_doc,
     __name__ = getset_func_name,
+    __qualname__ = getset_func_qualname,
     __dict__ = getset_func_dict,
     __defaults__ = getset_func_defaults,
+    __defaults_count__ = GetSetProperty(Function.fget_defaults_count),
+    __kwdefaults__ = getset_func_kwdefaults,
+    __annotations__ = getset_func_annotations,
     __globals__ = interp_attrproperty_w('w_func_globals', cls=Function),
     __closure__ = GetSetProperty(Function.fget_func_closure),
     __module__ = getset___module__,
@@ -742,21 +729,18 @@ Function.typedef = TypeDef("function",
 Function.typedef.acceptable_as_base_class = False
 
 Method.typedef = TypeDef(
-    "instancemethod",
+    "method",
     __doc__ = """instancemethod(function, instance, class)
 
 Create an instance method object.""",
     __new__ = interp2app(Method.descr_method__new__.im_func),
     __call__ = interp2app(Method.descr_method_call),
     __get__ = interp2app(Method.descr_method_get),
-    im_func = interp_attrproperty_w('w_function', cls=Method),
     __func__ = interp_attrproperty_w('w_function', cls=Method),
-    im_self = interp_attrproperty_w('w_instance', cls=Method),
     __self__ = interp_attrproperty_w('w_instance', cls=Method),
-    im_class = interp_attrproperty_w('w_class', cls=Method),
     __getattribute__ = interp2app(Method.descr_method_getattribute),
     __eq__ = interp2app(Method.descr_method_eq),
-    __ne__ = interp2app(Method.descr_method_ne),
+    __ne__ = descr_generic_ne,
     __hash__ = interp2app(Method.descr_method_hash),
     __repr__ = interp2app(Method.descr_method_repr),
     __reduce__ = interp2app(Method.descr_method__reduce__),
@@ -782,6 +766,9 @@ It can be called either on the class (e.g. C.f()) or on an instance
     __new__ = interp2app(StaticMethod.descr_staticmethod__new__.im_func),
     __init__=interp2app(StaticMethod.descr_init),
     __func__= interp_attrproperty_w('w_function', cls=StaticMethod),
+    __isabstractmethod__ = GetSetProperty(StaticMethod.descr_isabstract),
+    __dict__ = GetSetProperty(descr_get_dict, descr_set_dict,
+                              cls=StaticMethod),
     )
 
 ClassMethod.typedef = TypeDef(
@@ -790,6 +777,8 @@ ClassMethod.typedef = TypeDef(
     __init__=interp2app(ClassMethod.descr_init),
     __get__=interp2app(ClassMethod.descr_classmethod_get),
     __func__=interp_attrproperty_w('w_function', cls=ClassMethod),
+    __isabstractmethod__=GetSetProperty(ClassMethod.descr_isabstract),
+    __dict__=GetSetProperty(descr_get_dict, descr_set_dict, cls=ClassMethod),
     __doc__="""classmethod(function) -> class method
 
 Convert a function to be a class method.
@@ -816,6 +805,7 @@ BuiltinFunction.typedef.rawdict.update({
     '__self__': GetSetProperty(always_none, cls=BuiltinFunction),
     '__repr__': interp2app(BuiltinFunction.descr_function_repr),
     '__doc__': getset_func_doc,
+    '__reduce__': interp2app(BuiltinFunction.descr__reduce__),
     })
 del BuiltinFunction.typedef.rawdict['__get__']
 BuiltinFunction.typedef.acceptable_as_base_class = False
@@ -823,6 +813,7 @@ BuiltinFunction.typedef.acceptable_as_base_class = False
 PyTraceback.typedef = TypeDef("traceback",
     __reduce__ = interp2app(PyTraceback.descr__reduce__),
     __setstate__ = interp2app(PyTraceback.descr__setstate__),
+    __dir__ = interp2app(PyTraceback.descr__dir__),
     tb_frame = interp_attrproperty_w('frame', cls=PyTraceback),
     tb_lasti = interp_attrproperty('lasti', cls=PyTraceback, wrapfn="newint"),
     tb_lineno = GetSetProperty(PyTraceback.descr_tb_lineno),
@@ -830,8 +821,123 @@ PyTraceback.typedef = TypeDef("traceback",
     )
 assert not PyTraceback.typedef.acceptable_as_base_class  # no __new__
 
+GeneratorIterator.typedef = TypeDef("generator",
+    __repr__   = interp2app(GeneratorIterator.descr__repr__),
+    #__reduce__   = interp2app(GeneratorIterator.descr__reduce__),
+    #__setstate__ = interp2app(GeneratorIterator.descr__setstate__),
+    __next__   = interp2app(GeneratorIterator.descr_next,
+                            descrmismatch='__next__'),
+    send       = interp2app(GeneratorIterator.descr_send,
+                            descrmismatch='send'),
+    throw      = interp2app(GeneratorIterator.descr_throw,
+                            descrmismatch='throw'),
+    close      = interp2app(GeneratorIterator.descr_close,
+                            descrmismatch='close'),
+    __iter__   = interp2app(GeneratorIterator.descr__iter__,
+                            descrmismatch='__iter__'),
+    gi_running = interp_attrproperty('running', cls=GeneratorIterator, wrapfn="newbool"),
+    gi_frame   = GetSetProperty(GeneratorIterator.descr_gicr_frame),
+    gi_code    = interp_attrproperty_w('pycode', cls=GeneratorIterator),
+    gi_yieldfrom=interp_attrproperty_w('w_yielded_from', cls=GeneratorIterator),
+    __name__   = GetSetProperty(GeneratorIterator.descr__name__,
+                                GeneratorIterator.descr_set__name__),
+    __qualname__ = GetSetProperty(GeneratorIterator.descr__qualname__,
+                                  GeneratorIterator.descr_set__qualname__),
+    __weakref__ = make_weakref_descr(GeneratorIterator),
+)
+assert not GeneratorIterator.typedef.acceptable_as_base_class  # no __new__
+
+Coroutine.typedef = TypeDef("coroutine",
+    __repr__   = interp2app(Coroutine.descr__repr__),
+    #__reduce__   = interp2app(Coroutine.descr__reduce__),
+    #__setstate__ = interp2app(Coroutine.descr__setstate__),
+    send       = interp2app(Coroutine.descr_send,
+                            descrmismatch='send'),
+    throw      = interp2app(Coroutine.descr_throw,
+                            descrmismatch='throw'),
+    close      = interp2app(Coroutine.descr_close,
+                            descrmismatch='close'),
+    __await__  = interp2app(Coroutine.descr__await__,
+                            descrmismatch='__await__'),
+    cr_running = interp_attrproperty('running', cls=Coroutine, wrapfn="newbool"),
+    cr_frame   = GetSetProperty(Coroutine.descr_gicr_frame),
+    cr_code    = interp_attrproperty_w('pycode', cls=Coroutine),
+    cr_await   = interp_attrproperty_w('w_yielded_from', cls=Coroutine),
+    __name__   = GetSetProperty(Coroutine.descr__name__,
+                                Coroutine.descr_set__name__,
+                                doc="name of the coroutine"),
+    __qualname__ = GetSetProperty(Coroutine.descr__qualname__,
+                                  Coroutine.descr_set__qualname__,
+                                  doc="qualified name of the coroutine"),
+    __weakref__ = make_weakref_descr(Coroutine),
+)
+assert not Coroutine.typedef.acceptable_as_base_class  # no __new__
+
+AsyncGenerator.typedef = TypeDef("async_generator",
+    __repr__   = interp2app(AsyncGenerator.descr__repr__),
+    #__reduce__   = interp2app(Coroutine.descr__reduce__),
+    #__setstate__ = interp2app(Coroutine.descr__setstate__),
+    asend      = interp2app(AsyncGenerator.descr_asend,
+                            descrmismatch='asend'),
+    athrow     = interp2app(AsyncGenerator.descr_athrow,
+                            descrmismatch='athrow'),
+    aclose     = interp2app(AsyncGenerator.descr_aclose,
+                            descrmismatch='aclose'),
+    __aiter__  = interp2app(AsyncGenerator.descr__aiter__,
+                            descrmismatch='__aiter__'),
+    __anext__  = interp2app(AsyncGenerator.descr__anext__,
+                            descrmismatch='__anext__'),
+    ag_running = interp_attrproperty('running', cls=AsyncGenerator, wrapfn="newbool"),
+    ag_frame   = GetSetProperty(AsyncGenerator.descr_gicr_frame),
+    ag_code    = interp_attrproperty_w('pycode', cls=AsyncGenerator),
+    ag_await   = interp_attrproperty_w('w_yielded_from', cls=AsyncGenerator),
+    __name__   = GetSetProperty(AsyncGenerator.descr__name__,
+                                AsyncGenerator.descr_set__name__,
+                                doc="name of the async generator"),
+    __qualname__ = GetSetProperty(AsyncGenerator.descr__qualname__,
+                                  AsyncGenerator.descr_set__qualname__,
+                                  doc="qualified name of the async generator"),
+    __weakref__ = make_weakref_descr(AsyncGenerator),
+)
+assert not AsyncGenerator.typedef.acceptable_as_base_class  # no __new__
+
+CoroutineWrapper.typedef = TypeDef("coroutine_wrapper",
+    __iter__     = interp2app(CoroutineWrapper.descr__iter__),
+    __next__     = interp2app(CoroutineWrapper.descr__next__),
+    send         = interp2app(CoroutineWrapper.descr_send),
+    throw        = interp2app(CoroutineWrapper.descr_throw),
+    close        = interp2app(CoroutineWrapper.descr_close),
+)
+assert not CoroutineWrapper.typedef.acceptable_as_base_class  # no __new__
+
+AIterWrapper.typedef = TypeDef("aiter_wrapper",
+    __await__    = interp2app(AIterWrapper.descr__await__),
+    __iter__     = interp2app(AIterWrapper.descr__iter__),
+    __next__     = interp2app(AIterWrapper.descr__next__),
+)
+assert not AIterWrapper.typedef.acceptable_as_base_class  # no __new__
+
+AsyncGenASend.typedef = TypeDef("async_generator_asend",
+    __await__    = interp2app(AsyncGenASend.descr__iter__),
+    __iter__     = interp2app(AsyncGenASend.descr__iter__),
+    __next__     = interp2app(AsyncGenASend.descr__next__),
+    close        = interp2app(AsyncGenASend.descr_close),
+    send         = interp2app(AsyncGenASend.descr_send),
+    throw        = interp2app(AsyncGenASend.descr_throw),
+)
+AsyncGenAThrow.typedef = TypeDef("async_generator_athrow",
+    __await__    = interp2app(AsyncGenAThrow.descr__iter__),
+    __iter__     = interp2app(AsyncGenAThrow.descr__iter__),
+    __next__     = interp2app(AsyncGenAThrow.descr__next__),
+    close        = interp2app(AsyncGenAThrow.descr_close),
+    send         = interp2app(AsyncGenAThrow.descr_send),
+    throw        = interp2app(AsyncGenAThrow.descr_throw),
+)
+
 Cell.typedef = TypeDef("cell",
-    __cmp__      = interp2app(Cell.descr__cmp__),
+    __total_ordering__ = 'auto',
+    __lt__       = interp2app(Cell.descr__lt__),
+    __eq__       = interp2app(Cell.descr__eq__),
     __hash__     = None,
     __reduce__   = interp2app(Cell.descr__reduce__),
     __repr__     = interp2app(Cell.descr__repr__),
@@ -841,14 +947,24 @@ Cell.typedef = TypeDef("cell",
 assert not Cell.typedef.acceptable_as_base_class  # no __new__
 
 Ellipsis.typedef = TypeDef("ellipsis",
+    __new__ = interp2app(Ellipsis.descr_new_ellipsis),
     __repr__ = interp2app(Ellipsis.descr__repr__),
+    __reduce__ = interp2app(Ellipsis.descr__reduce__),
 )
-assert not Ellipsis.typedef.acceptable_as_base_class  # no __new__
+Ellipsis.typedef.acceptable_as_base_class = False
 
 NotImplemented.typedef = TypeDef("NotImplementedType",
+    __new__ = interp2app(NotImplemented.descr_new_notimplemented),
     __repr__ = interp2app(NotImplemented.descr__repr__),
+    __reduce__ = interp2app(NotImplemented.descr__reduce__),
 )
-assert not NotImplemented.typedef.acceptable_as_base_class  # no __new__
+NotImplemented.typedef.acceptable_as_base_class = False
 
 SuspendedUnroller.typedef = TypeDef("SuspendedUnroller")
-assert not SuspendedUnroller.typedef.acceptable_as_base_class  # no __new__
+SuspendedUnroller.typedef.acceptable_as_base_class = False
+
+## W_OperationError.typedef = TypeDef("OperationError",
+##     __reduce__ = interp2app(W_OperationError.descr_reduce),
+##     __setstate__ = interp2app(W_OperationError.descr_setstate),
+## )
+## W_OperationError.typedef.acceptable_as_base_class = False
