@@ -69,7 +69,7 @@ class W_CData(W_Root):
         return self.space.newtext("<cdata '%s%s' %s>" % (
             self.ctype.name, extra1, extra2))
 
-    def nonzero(self):
+    def bool(self):
         with self as ptr:
             nonzero = self.ctype.nonzero(ptr)
         return self.space.newbool(nonzero)
@@ -79,13 +79,6 @@ class W_CData(W_Root):
             w_result = self.ctype.cast_to_int(ptr)
         return w_result
 
-    def long(self, space):
-        w_result = self.int(space)
-        space = self.space
-        if space.is_w(space.type(w_result), space.w_int):
-            w_result = space.newlong(space.int_w(w_result))
-        return w_result
-
     def float(self):
         with self as ptr:
             w_result = self.ctype.float(ptr)
@@ -244,7 +237,7 @@ class W_CData(W_Root):
                 w_value.get_array_length() == length):
                 # fast path: copying from exactly the correct type
                 with w_value as source:
-                    source = rffi.cast(rffi.CONST_VOIDP, source)
+                    source = rffi.cast(rffi.VOIDP, source)
                     target = rffi.cast(rffi.VOIDP, target)
                     size = rffi.cast(rffi.SIZE_T, ctitemsize * length)
                     rffi.c_memcpy(target, source, size)
@@ -496,9 +489,6 @@ class W_CData(W_Root):
     def descr_exit(self, args_w):
         self.enter_exit(True)
 
-    def get_maximum_buffer_size(self):
-        return -1
-
 
 class W_CDataMem(W_CData):
     """This is used only by the results of cffi.cast('int', x)
@@ -564,28 +554,21 @@ class W_CDataNewOwning(W_CData):
 
 class W_CDataNewStd(W_CDataNewOwning):
     """Subclass using the standard allocator, lltype.malloc()/lltype.free()"""
-    _attrs_ = ['datasize']    # changed to -1 after being explicitly freed
-
-    def __init__(self, space, cdata, ctype, length, datasize):
-        W_CDataNewOwning.__init__(self, space, cdata, ctype, length)
-        assert datasize >= 0
-        self.datasize = datasize
+    _attrs_ = ['explicitly_freed']
+    explicitly_freed = False
 
     @rgc.must_be_light_finalizer
     def __del__(self):
-        if self.datasize >= 0:
+        if not self.explicitly_freed:
             lltype.free(self._ptr, flavor='raw')
 
     def _do_exit(self):
-        if self.datasize >= 0:
-            rgc.add_memory_pressure(-self.datasize, self)
-            self.datasize = -1
+        if not self.explicitly_freed:
+            rgc.add_memory_pressure(-self._sizeof(), self)
+            self.explicitly_freed = True
             rgc.may_ignore_finalizer(self)
             lltype.free(self._ptr, flavor='raw')
 
-    def get_maximum_buffer_size(self):
-        return self.datasize
-
 
 class W_CDataNewNonStd(W_CDataNewOwning):
     """Subclass using a non-standard allocator"""
@@ -637,9 +620,6 @@ class W_CDataPtrToStructOrUnion(W_CData):
             if isinstance(structobj, W_CDataNewOwning):
                 structobj._do_exit()
 
-    def get_maximum_buffer_size(self):
-        return self.structobj._sizeof()
-
 
 class W_CDataSliced(W_CData):
     """Subclass with an explicit length, for slices."""
@@ -689,14 +669,6 @@ class W_CDataFromBuffer(W_CData):
     def get_array_length(self):
         return self.length
 
-    def _sizeof(self):
-        from pypy.module._cffi_backend import ctypearray
-        ctype = self.ctype
-        if isinstance(ctype, ctypearray.W_CTypeArray):
-            return self.length * ctype.ctitem.size
-        else:
-            return W_CData._sizeof(self)
-
     def _repr_extra(self):
         from pypy.module._cffi_backend import ctypearray
         if self.w_keepalive is None:
@@ -751,9 +723,8 @@ W_CData.typedef = TypeDef(
     __module__ = '_cffi_backend',   # attribute also visible on instances
     __name__ = '<cdata>',           # attribute also visible on instances
     __repr__ = interp2app(W_CData.repr),
-    __nonzero__ = interp2app(W_CData.nonzero),
+    __bool__ = interp2app(W_CData.bool),
     __int__ = interp2app(W_CData.int),
-    __long__ = interp2app(W_CData.long),
     __float__ = interp2app(W_CData.float),
     __complex__ = interp2app(W_CData.complex),
     __len__ = interp2app(W_CData.len),
