@@ -49,7 +49,7 @@ class W_CTypePrimitive(W_CType):
         return rutf8.codepoint_at_pos(w_u._utf8, 0)
 
     def cast(self, w_ob):
-        from pypy.module._cffi_backend import ctypeptr, wrapper
+        from pypy.module._cffi_backend import ctypeptr
         space = self.space
         if (isinstance(w_ob, cdataobj.W_CData) and
                isinstance(w_ob.ctype, ctypeptr.W_CTypePtrOrArray)):
@@ -62,10 +62,6 @@ class W_CTypePrimitive(W_CType):
         elif space.isinstance_w(w_ob, space.w_unicode):
             value = self.cast_unicode(w_ob)
             value = self._cast_result(value)
-        elif (isinstance(w_ob, wrapper.W_FunctionWrapper)
-                and w_ob.directfnptr):
-            value = rffi.cast(lltype.Signed, w_ob.directfnptr)
-            value = self._cast_result(value)
         else:
             value = self._cast_generic(w_ob)
         w_cdata = cdataobj.W_CDataMem(space, self)
@@ -226,6 +222,7 @@ class W_CTypePrimitiveUniChar(W_CTypePrimitiveCharOrUniChar):
                 raise oefmt(self.space.w_ValueError,
                             "%s out of range for conversion to unicode: %s",
                             self.name, hex(e.ordinal))
+        assert lgt >= 0
         return self.space.newutf8(utf8, lgt)
 
 
@@ -279,10 +276,10 @@ class W_CTypePrimitiveSigned(W_CTypePrimitive):
         w_cdata.write_raw_signed_data(value)
 
     def unpack_list_of_int_items(self, ptr, length):
-        if self.size == rffi.sizeof(rffi.SIGNED):
+        if self.size == rffi.sizeof(rffi.LONG):
             from rpython.rlib.rrawarray import populate_list_from_raw_array
             res = []
-            buf = rffi.cast(rffi.SIGNEDP, ptr)
+            buf = rffi.cast(rffi.LONGP, ptr)
             populate_list_from_raw_array(res, buf, length)
             return res
         elif self.value_smaller_than_long:
@@ -295,9 +292,9 @@ class W_CTypePrimitiveSigned(W_CTypePrimitive):
         int_list = self.space.listview_int(w_ob)
         if (int_list is not None and
                 self._within_bounds(len(int_list), expected_length)):
-            if self.size == rffi.sizeof(rffi.SIGNED): # fastest path
+            if self.size == rffi.sizeof(rffi.LONG): # fastest path
                 from rpython.rlib.rrawarray import copy_list_to_raw_array
-                cdata = rffi.cast(rffi.SIGNEDP, cdata)
+                cdata = rffi.cast(rffi.LONGP, cdata)
                 copy_list_to_raw_array(int_list, cdata)
             else:
                 overflowed = misc.pack_list_to_raw_array_bounds_signed(
