@@ -2,6 +2,8 @@
 Pointers.
 """
 
+import os
+
 from rpython.rlib import rposix
 from rpython.rlib.rarithmetic import ovfcheck
 from rpython.rtyper.annlowlevel import llstr
@@ -75,7 +77,7 @@ class W_CTypePtrOrArray(W_CType):
                 self._convert_array_from_listview(cdata, space.listview(w_ob))
         elif self.accept_str:
             if not space.isinstance_w(w_ob, space.w_bytes):
-                raise self._convert_error("str or list or tuple", w_ob)
+                raise self._convert_error("bytes or list or tuple", w_ob)
             s = space.bytes_w(w_ob)
             n = len(s)
             if self.length >= 0 and n > self.length:
@@ -297,14 +299,22 @@ class W_CTypePointer(W_CTypePtrBase):
         return W_CTypePtrBase.cast(self, w_ob)
 
     def prepare_file(self, w_ob):
-        from pypy.module._file.interp_file import W_File
-        if isinstance(w_ob, W_File):
+        from pypy.module._io.interp_iobase import W_IOBase
+        if isinstance(w_ob, W_IOBase):
             return prepare_file_argument(self.space, w_ob)
         else:
             return lltype.nullptr(rffi.CCHARP.TO)
 
-    def _prepare_pointer_call_argument(self, w_init, cdata):
+    def _prepare_pointer_call_argument(self, w_init, cdata, keepalives, i):
         space = self.space
+        if self.accept_str and space.isinstance_w(w_init, space.w_bytes):
+            # special case to optimize strings passed to a "char *" argument
+            value = space.bytes_w(w_init)
+            if isinstance(self.ctitem, ctypeprim.W_CTypePrimitiveBool):
+                self._must_be_string_of_zero_or_one(value)
+            keepalives[i] = value
+            return misc.write_string_as_charp(cdata, value)
+        #
         if (space.isinstance_w(w_init, space.w_list) or
             space.isinstance_w(w_init, space.w_tuple)):
             length = space.int_w(space.len(w_init))
@@ -350,27 +360,14 @@ class W_CTypePointer(W_CTypePtrBase):
         return 1
 
     def convert_argument_from_object(self, cdata, w_ob, keepalives, i):
-        # writes the pointer to cdata[0], writes the must-free flag in
-        # the byte just before cdata[0], and returns True if something
-        # must be done later to free.
         from pypy.module._cffi_backend.ctypefunc import set_mustfree_flag
-        if isinstance(w_ob, cdataobj.W_CData):
-            result = 0
-        else:
-            space = self.space
-            if self.accept_str and space.isinstance_w(w_ob, space.w_bytes):
-                # special case to optimize strings passed to a "char *" argument
-                value = space.bytes_w(w_ob)
-                if isinstance(self.ctitem, ctypeprim.W_CTypePrimitiveBool):
-                    self._must_be_string_of_zero_or_one(value)
-                keepalives[i] = misc.write_string_as_charp(cdata, value)
-                return True
-            result = self._prepare_pointer_call_argument(w_ob, cdata)
-
+        result = (not isinstance(w_ob, cdataobj.W_CData) and
+                  self._prepare_pointer_call_argument(w_ob, cdata,
+                                                      keepalives, i))
         if result == 0:
             self.convert_from_object(cdata, w_ob)
         set_mustfree_flag(cdata, result)
-        return result == 1      # 0 or 2 => False, nothing to do later
+        return result
 
     def getcfield(self, attr):
         from pypy.module._cffi_backend.ctypestruct import W_CTypeStructOrUnion
@@ -436,13 +433,17 @@ class CffiFileObj(object):
 
 
 def prepare_file_argument(space, w_fileobj):
-    w_fileobj.direct_flush()
+    from pypy.module._io.interp_iobase import W_IOBase
+    assert isinstance(w_fileobj, W_IOBase)
+    space.call_method(w_fileobj, "flush")
     if w_fileobj.cffi_fileobj is None:
-        fd = w_fileobj.direct_fileno()
+        fd = space.int_w(space.call_method(w_fileobj, "fileno"))
         if fd < 0:
             raise oefmt(space.w_ValueError, "file has no OS file descriptor")
+        fd = os.dup(fd)
+        mode = space.text_w(space.getattr(w_fileobj, space.newtext("mode")))
         try:
-            w_fileobj.cffi_fileobj = CffiFileObj(fd, w_fileobj.mode)
+            w_fileobj.cffi_fileobj = CffiFileObj(fd, mode)
         except OSError as e:
             raise wrap_oserror(space, e)
     return rffi.cast(rffi.CCHARP, w_fileobj.cffi_fileobj.llf)
