@@ -151,7 +151,7 @@ class W_FFIObject(W_Root):
     def ffi_type(self, w_x, accept):
         space = self.space
         if (accept & ACCEPT_STRING) and (
-                space.isinstance_w(w_x, space.w_basestring)):
+                space.isinstance_w(w_x, space.w_text)):
             string = space.text_w(w_x)
             consider_fn_as_fnptr = (accept & CONSIDER_FN_AS_FNPTR) != 0
             if jit.isconstant(string):
@@ -761,7 +761,7 @@ if sys.platform == 'win32':
 
 
 W_FFIObject.typedef = TypeDef(
-        'CompiledFFI',
+        '_cffi_backend.FFI',
         __new__     = interp2app(W_FFIObject___new__),
         __init__    = interp2app(W_FFIObject.descr_init),
         CData       = ClassAttr(make_CData),
