@@ -1,9 +1,21 @@
-import os, py
+import os, pytest
 
 from rpython.tool.udir import udir
 from pypy.interpreter.gateway import unwrap_spec, interp2app
 from pypy.module._cffi_backend.newtype import _clean_cache
 import pypy.module.cpyext.api     # side-effect of pre-importing it
+from sysconfig import get_config_var
+
+
+def get_ext_suffix():
+    # soabi is None on cpython < 3.7 (incl 2.7), 'pypy-73' on pypy2 v7.3.1
+    # and something like 'cpython-38-x86_64-linux-gnu' on cpython 3.8
+    soabi = get_config_var('SOABI') or ''
+    ret = soabi + get_config_var('SO')
+    # either '.so' or 'pypy-73.so'
+    if ret[0] == '.':
+        return ret[1:]
+    return ret
 
 
 @unwrap_spec(cdef='text', module_name='text', source='text', packed=int)
@@ -16,13 +28,13 @@ def prepare(space, cdef, module_name, source, w_includes=None,
         from cffi import recompiler     # needs to be at least cffi 1.0.4
         from cffi import ffiplatform
     except ImportError:
-        py.test.skip("system cffi module not found or older than 1.0.0")
+        pytest.skip("system cffi module not found or older than 1.0.0")
     if w_min_version is None:
         min_version = (1, 4, 0)
     else:
         min_version = tuple(space.unwrap(w_min_version))
     if cffi.__version_info__ < min_version:
-        py.test.skip("system cffi module needs to be at least %s, got %s" % (
+        pytest.skip("system cffi module needs to be at least %s, got %s" % (
             min_version, cffi.__version_info__))
     space.appexec([], """():
         import _cffi_backend     # force it to be initialized
@@ -55,7 +67,7 @@ def prepare(space, cdef, module_name, source, w_includes=None,
     base_module_name = module_name.split('.')[-1]
     sources = []
     if w_extra_source is not None:
-        sources.append(space.str_w(w_extra_source))
+        sources.append(space.text_w(w_extra_source))
     kwargs = {}
     if w_extra_compile_args is not None:
         kwargs['extra_compile_args'] = space.unwrap(w_extra_compile_args)
@@ -66,7 +78,7 @@ def prepare(space, cdef, module_name, source, w_includes=None,
             **kwargs)
     ffiplatform.compile(str(rdir), ext)
 
-    for extension in ['so', 'pyd', 'dylib']:
+    for extension in [get_ext_suffix(), 'so', 'pyd', 'dylib']:
         so_file = str(rdir.join('%s.%s' % (path, extension)))
         if os.path.exists(so_file):
             break
@@ -75,8 +87,12 @@ def prepare(space, cdef, module_name, source, w_includes=None,
 
     args_w = [space.wrap(module_name), space.wrap(so_file)]
     w_res = space.appexec(args_w, """(modulename, filename):
-        import imp
-        mod = imp.load_dynamic(modulename, filename)
+        import _imp
+        class Spec: pass
+        spec = Spec()
+        spec.name = modulename
+        spec.origin = filename
+        mod = _imp.create_dynamic(spec)
         assert mod.__name__ == modulename
         return (mod.ffi, mod.lib)
     """)
@@ -89,17 +105,18 @@ def prepare(space, cdef, module_name, source, w_includes=None,
 
 
 class AppTestRecompiler:
-    spaceconfig = dict(usemodules=['_cffi_backend', 'imp'])
+    spaceconfig = dict(usemodules=['_cffi_backend', 'imp', 'cpyext', 'struct'])
 
     def setup_class(cls):
         if cls.runappdirect:
-            py.test.skip("not a test for -A")
+            pytest.skip("not a test for -A")
         cls.w_prepare = cls.space.wrap(interp2app(prepare))
         cls.w_udir = cls.space.wrap(str(udir))
         cls.w_os_sep = cls.space.wrap(os.sep)
 
     def setup_method(self, meth):
         self._w_modules = self.space.appexec([], """():
+            import cpyext      # ignore stuff there in the leakfinder
             import sys
             return set(sys.modules)
         """)
@@ -111,7 +128,7 @@ class AppTestRecompiler:
             del self.space._cleanup_ffi
         self.space.appexec([self._w_modules], """(old_modules):
             import sys
-            for key in sys.modules.keys():
+            for key in list(sys.modules.keys()):
                 if key not in old_modules:
                     del sys.modules[key]
         """)
@@ -343,9 +360,9 @@ class AppTestRecompiler:
             'test_verify_exact_field_offset',
             """struct foo_s { short a; int b; };""")
         e = raises(ffi.error, ffi.new, "struct foo_s *", [])    # lazily
-        assert str(e.value) == ("struct foo_s: wrong offset for field 'b' (cdef "
-                           'says 0, but C compiler says 4). fix it or use "...;" '
-                           "in the cdef for struct foo_s to make it flexible")
+        assert str(e.value).startswith(
+            "struct foo_s: wrong offset for field 'b' (cdef "
+            'says 0, but C compiler says 4). fix it or use "...;" ')
 
     def test_type_caching(self):
         ffi1, lib1 = self.prepare(
@@ -928,13 +945,13 @@ class AppTestRecompiler:
         # but we can get its address
         p = ffi.addressof(lib, 'globvar')
         assert ffi.typeof(p) == ffi.typeof('opaque_t *')
-        assert ffi.string(ffi.cast("char *", p), 8) == "hello"
+        assert ffi.string(ffi.cast("char *", p), 8) == b"hello"
 
     def test_constant_of_value_unknown_to_the_compiler(self):
         extra_c_source = self.udir + self.os_sep + (
             'extra_test_constant_of_value_unknown_to_the_compiler.c')
-        with open(extra_c_source, 'w') as f:
-            f.write('const int external_foo = 42;\n')
+        with open(extra_c_source, 'wb') as f:
+            f.write(b'const int external_foo = 42;\n')
         ffi, lib = self.prepare(
             "const int external_foo;",
             'test_constant_of_value_unknown_to_the_compiler',
@@ -1085,7 +1102,7 @@ class AppTestRecompiler:
         #
         @ffi.callback("int *(*)(void)")
         def get_my_value():
-            return values + it.next()
+            return values + next(it)
         lib.get_my_value = get_my_value
         #
         values[0] = 41
@@ -1429,7 +1446,7 @@ class AppTestRecompiler:
                 def getvalue(self):
                     if self._result is None:
                         os.close(self._wr)
-                        self._result = os.read(self._rd, 4096)
+                        self._result = os.read(self._rd, 4096).decode()
                         os.close(self._rd)
                         # xxx hack away these lines
                         while self._result.startswith('[platform:execute]'):
@@ -1500,11 +1517,11 @@ class AppTestRecompiler:
         baz1 = ffi.def_extern()(baz)
         assert baz1 is baz
         seen = []
-        baz(40L, 4L)
-        res = lib.baz(50L, 8L)
+        baz(40, 4)
+        res = lib.baz(50, 8)
         assert res is None
-        assert seen == [("Baz", 40L, 4L), ("Baz", 50, 8)]
-        assert type(seen[0][1]) is type(seen[0][2]) is long
+        assert seen == [("Baz", 40, 4), ("Baz", 50, 8)]
+        assert type(seen[0][1]) is type(seen[0][2]) is int
         assert type(seen[1][1]) is type(seen[1][2]) is int
 
         @ffi.def_extern(name="bok")
@@ -1671,6 +1688,7 @@ class AppTestRecompiler:
         raises(TypeError, ffi.def_extern(name='bar', onerror=42),
                        lambda x: x)
 
+    @pytest.mark.skipif("not config.option.runappdirect")
     def test_extern_python_stdcall(self):
         ffi, lib = self.prepare("""
             extern "Python" int __stdcall foo(int);
@@ -2050,7 +2068,7 @@ class AppTestRecompiler:
                 return s;
             }
         """, packed=True, min_version=(1, 8, 3))
-        assert lib.f().y == chr(40)
+        assert ord(lib.f().y) == 40
         assert lib.f().x == 200
         e = raises(NotImplementedError, lib.g, 0)
         assert str(e.value) == (
