@@ -1,11 +1,14 @@
+import sys
 from rpython.rlib import jit, rutf8
 from rpython.rlib.objectmodel import we_are_translated, not_rpython
 from rpython.rlib.rstring import StringBuilder, UnicodeBuilder
 from rpython.rlib.rutf8 import MAXUNICODE
+from rpython.rlib import runicode
 
-from pypy.interpreter.error import OperationError, oefmt
+from pypy.interpreter.error import OperationError, oefmt, wrap_oserror
 from pypy.interpreter.gateway import interp2app, unwrap_spec, WrappedDefault
 from pypy.interpreter import unicodehelper
+from pypy.module.unicodedata.interp_ucd import unicodedb
 
 
 class VersionTag(object):
@@ -35,18 +38,20 @@ class CodecState(object):
             the unicode characters, not into the position of utf8 bytes,
             so it needs to be converted by the codec
 
-            Returns (unicode_or_none, str_or_none, newpos) as error
-            handlers may return unicode or on Python 3, bytes.
+            Returns (str_or_none, newpos) as error
+            handlers return utf8 so we add whether they used unicode or bytes
             """
             w_errorhandler = lookup_error(space, errors)
             if decode:
                 w_cls = space.w_UnicodeDecodeError
+                assert isinstance(input, str)
                 w_input = space.newbytes(input)
                 length = len(input)
             else:
                 w_cls = space.w_UnicodeEncodeError
-                length = rutf8.check_utf8(input, allow_surrogates=True)
-                w_input = space.newutf8(input, length)
+                assert isinstance(input, str)
+                length = rutf8.codepoints_in_utf8(input)
+                w_input = space.newtext(input, length)
             w_exc =  space.call_function(
                 w_cls,
                 space.newtext(encoding),
@@ -56,24 +61,52 @@ class CodecState(object):
                 space.newtext(reason))
             w_res = space.call_function(w_errorhandler, w_exc)
             if (not space.isinstance_w(w_res, space.w_tuple)
-                or space.len_w(w_res) != 2
-                or not space.isinstance_w(
-                                 space.getitem(w_res, space.newint(0)),
-                                 space.w_unicode)):
-                raise oefmt(space.w_TypeError,
-                            "%s error handler must return (unicode, int) "
-                            "tuple, not %R",
-                            "decoding" if decode else "encoding", w_res)
+                or space.len_w(w_res) != 2):
+                if decode:
+                    msg = ("decoding error handler must return "
+                           "(str, int) tuple")
+                else:
+                    msg = ("encoding error handler must return "
+                           "(str/bytes, int) tuple")
+                raise OperationError(space.w_TypeError, space.newtext(msg))
+
             w_replace, w_newpos = space.fixedview(w_res, 2)
-            newpos = space.int_w(w_newpos)
-            if newpos < 0:
-                newpos = length + newpos
+            if space.isinstance_w(w_replace, space.w_unicode):
+                rettype = 'u'
+            elif not decode and space.isinstance_w(w_replace, space.w_bytes):
+                rettype = 'b'
+            else:
+                if decode:
+                    msg = ("decoding error handler must return "
+                           "(str, int) tuple")
+                else:
+                    msg = ("encoding error handler must return "
+                           "(str/bytes, int) tuple")
+                raise OperationError(space.w_TypeError, space.newtext(msg))
+            try:
+                newpos = space.int_w(w_newpos)
+            except OperationError as e:
+                if not e.match(space, space.w_OverflowError):
+                    raise
+                newpos = -1
+            else:
+                if newpos < 0:
+                    newpos = length + newpos
             if newpos < 0 or newpos > length:
                 raise oefmt(space.w_IndexError,
                             "position %d from error handler out of bounds",
                             newpos)
-            w_replace = space.convert_to_w_unicode(w_replace)
-            return w_replace._utf8, newpos
+            w_obj = space.getattr(w_exc, space.newtext('object'))
+            if  decode:
+                if not space.isinstance_w(w_obj, space.w_bytes):
+                    raise oefmt(space.w_ValueError,
+                            "error handler modified exc.object must be bytes")
+            else:
+                if not space.isinstance_w(w_obj, space.w_unicode):
+                    raise oefmt(space.w_ValueError,
+                            "error handler modified exc.object must be str")
+            obj = space.utf8_w(w_obj) 
+            return space.utf8_w(w_replace), newpos, rettype, obj
         return call_errorhandler
 
     def make_decode_errorhandler(self, space):
@@ -141,9 +174,16 @@ def lookup_codec(space, encoding):
 def _lookup_codec_loop(space, encoding, normalized_encoding):
     state = space.fromcache(CodecState)
     if state.codec_need_encodings:
+        # registers new codecs.
+        # This import uses the "builtin" import method, and is needed
+        # to bootstrap the full importlib module.
         w_import = space.getattr(space.builtin, space.newtext("__import__"))
-        # registers new codecs
         space.call_function(w_import, space.newtext("encodings"))
+        from pypy.module.sys.interp_encoding import base_encoding
+        # May be 'utf-8'
+        normalized_base = base_encoding.replace("-", "_").lower()
+        space.call_function(w_import, space.newtext("encodings." +
+                                                    normalized_base))
         state.codec_need_encodings = False
         if len(state.codec_search_path) == 0:
             raise oefmt(space.w_LookupError,
@@ -200,6 +240,8 @@ def replace_errors(space, w_exc):
     w_start = space.getattr(w_exc, space.newtext('start'))
     w_end = space.getattr(w_exc, space.newtext('end'))
     size = space.int_w(w_end) - space.int_w(w_start)
+    if size < 0:
+        size = 0
     if space.isinstance_w(w_exc, space.w_UnicodeEncodeError):
         text = '?' * size
         return space.newtuple([space.newutf8(text, size), w_end])
@@ -214,10 +256,11 @@ def replace_errors(space, w_exc):
                     "don't know how to handle %T in error callback", w_exc)
 
 def xmlcharrefreplace_errors(space, w_exc):
+
     check_exception(space, w_exc)
     if space.isinstance_w(w_exc, space.w_UnicodeEncodeError):
         w_obj = space.getattr(w_exc, space.newtext('object'))
-        space.realutf8_w(w_obj) # weeoes
+        space.realutf8_w(w_obj) # for errors
         w_obj = space.convert_arg_to_w_unicode(w_obj)
         start = space.int_w(space.getattr(w_exc, space.newtext('start')))
         w_end = space.getattr(w_exc, space.newtext('end'))
@@ -241,6 +284,43 @@ def xmlcharrefreplace_errors(space, w_exc):
                     "don't know how to handle %T in error callback", w_exc)
 
 def backslashreplace_errors(space, w_exc):
+
+    check_exception(space, w_exc)
+    if (space.isinstance_w(w_exc, space.w_UnicodeEncodeError) or
+            space.isinstance_w(w_exc, space.w_UnicodeTranslateError)):
+        w_obj = space.getattr(w_exc, space.newtext('object'))
+        space.realutf8_w(w_obj) # for errors
+        w_obj = space.convert_arg_to_w_unicode(w_obj)
+        start = space.int_w(space.getattr(w_exc, space.newtext('start')))
+        w_end = space.getattr(w_exc, space.newtext('end'))
+        end = space.int_w(w_end)
+        start = w_obj._index_to_byte(start)
+        end = w_obj._index_to_byte(end)
+        builder = StringBuilder()
+        pos = start
+        obj = w_obj._utf8
+        while pos < end:
+            code = rutf8.codepoint_at_pos(obj, pos)
+            unicodehelper.raw_unicode_escape_helper(builder, code)
+            pos = rutf8.next_codepoint_pos(obj, pos)
+        return space.newtuple([space.newtext(builder.build()), w_end])
+    elif space.isinstance_w(w_exc, space.w_UnicodeDecodeError):
+        obj = space.bytes_w(space.getattr(w_exc, space.newtext('object')))
+        start = space.int_w(space.getattr(w_exc, space.newtext('start')))
+        w_end = space.getattr(w_exc, space.newtext('end'))
+        end = space.int_w(w_end)
+        builder = StringBuilder()
+        pos = start
+        while pos < end:
+            oc = ord(obj[pos])
+            unicodehelper.raw_unicode_escape_helper(builder, oc)
+            pos += 1
+        return space.newtuple([space.newtext(builder.build()), w_end])
+    else:
+        raise oefmt(space.w_TypeError,
+                    "don't know how to handle %T in error callback", w_exc)
+
+def namereplace_errors(space, w_exc):
     check_exception(space, w_exc)
     if space.isinstance_w(w_exc, space.w_UnicodeEncodeError):
         w_obj = space.getattr(w_exc, space.newtext('object'))
@@ -256,21 +336,14 @@ def backslashreplace_errors(space, w_exc):
         pos = start
         while pos < end:
             oc = rutf8.codepoint_at_pos(obj, pos)
-            num = hex(oc)
-            if (oc >= 0x10000):
-                builder.append("\\U")
-                zeros = 8
-            elif (oc >= 0x100):
-                builder.append("\\u")
-                zeros = 4
+            try:
+                name = unicodedb.name(oc)
+            except KeyError:
+                unicodehelper.raw_unicode_escape_helper(builder, oc)
             else:
-                builder.append("\\x")
-                zeros = 2
-            lnum = len(num)
-            nb = zeros + 2 - lnum # num starts with '0x'
-            if nb > 0:
-                builder.append_multiple_char('0', nb)
-            builder.append_slice(num, 2, lnum)
+                builder.append('\\N{')
+                builder.append(name)
+                builder.append('}')
             pos = rutf8.next_codepoint_pos(obj, pos)
         r = builder.build()
         lgt = rutf8.check_utf8(r, True)
@@ -279,16 +352,208 @@ def backslashreplace_errors(space, w_exc):
         raise oefmt(space.w_TypeError,
                     "don't know how to handle %T in error callback", w_exc)
 
+
+(ENC_UNKNOWN, ENC_UTF8,
+ ENC_UTF16BE, ENC_UTF16LE,
+ ENC_UTF32BE, ENC_UTF32LE) = range(-1, 5)
+BIG_ENDIAN = sys.byteorder == 'big'
+
+STANDARD_ENCODINGS = {
+    'utf8':      (3, ENC_UTF8),
+    'utf_8':     (3, ENC_UTF8),
+    'cp_utf8':   (3, ENC_UTF8),
+    'utf16':     (2, ENC_UTF16BE) if BIG_ENDIAN else (2, ENC_UTF16LE),
+    'utf_16':    (2, ENC_UTF16BE) if BIG_ENDIAN else (2, ENC_UTF16LE),
+    'utf16be':   (2, ENC_UTF16BE),
+    'utf_16be':  (2, ENC_UTF16BE),
+    'utf16_be':  (2, ENC_UTF16BE),
+    'utf_16_be': (2, ENC_UTF16BE),
+    'utf16le':   (2, ENC_UTF16LE),
+    'utf_16le':  (2, ENC_UTF16LE),
+    'utf16_le':  (2, ENC_UTF16LE),
+    'utf_16_le': (2, ENC_UTF16LE),
+    'utf32':     (4, ENC_UTF32BE) if BIG_ENDIAN else (4, ENC_UTF32LE),
+    'utf_32':    (4, ENC_UTF32BE) if BIG_ENDIAN else (4, ENC_UTF32LE),
+    'utf32be':   (4, ENC_UTF32BE),
+    'utf_32be':  (4, ENC_UTF32BE),
+    'utf32_be':  (4, ENC_UTF32BE),
+    'utf_32_be': (4, ENC_UTF32BE),
+    'utf32le':   (4, ENC_UTF32LE),
+    'utf_32le':  (4, ENC_UTF32LE),
+    'utf32_le':  (4, ENC_UTF32LE),
+    'utf_32_le': (4, ENC_UTF32LE),
+}
+
+def get_standard_encoding(encoding):
+    encoding = encoding.lower().replace('-', '_')
+    return STANDARD_ENCODINGS.get(encoding, (0, ENC_UNKNOWN))
+
+def surrogatepass_errors(space, w_exc):
+    check_exception(space, w_exc)
+    if space.isinstance_w(w_exc, space.w_UnicodeEncodeError):
+        w_obj = space.getattr(w_exc, space.newtext('object'))
+        w_obj = space.convert_arg_to_w_unicode(w_obj)
+        start = space.int_w(space.getattr(w_exc, space.newtext('start')))
+        w_end = space.getattr(w_exc, space.newtext('end'))
+        encoding = space.text_w(space.getattr(w_exc, space.newtext('encoding')))
+        bytelength, code = get_standard_encoding(encoding)
+        if code == ENC_UNKNOWN:
+            # Not supported, fail with original exception
+            raise OperationError(space.type(w_exc), w_exc)
+        end = space.int_w(w_end)
+        builder = StringBuilder()
+        start = w_obj._index_to_byte(start)
+        end = w_obj._index_to_byte(end)
+        obj = w_obj._utf8
+        pos = start
+        while pos < end:
+            ch = rutf8.codepoint_at_pos(obj, pos)
+            pos = rutf8.next_codepoint_pos(obj, pos)
+            if ch < 0xd800 or ch > 0xdfff:
+                # Not a surrogate, fail with original exception
+                raise OperationError(space.type(w_exc), w_exc)
+            if code == ENC_UTF8:
+                builder.append(chr(0xe0 | (ch >> 12)))
+                builder.append(chr(0x80 | ((ch >> 6) & 0x3f)))
+                builder.append(chr(0x80 | (ch & 0x3f)))
+            elif code == ENC_UTF16LE:
+                builder.append(chr(ch & 0xff))
+                builder.append(chr(ch >> 8))
+            elif code == ENC_UTF16BE:
+                builder.append(chr(ch >> 8))
+                builder.append(chr(ch & 0xff))
+            elif code == ENC_UTF32LE:
+                builder.append(chr(ch & 0xff))
+                builder.append(chr(ch >> 8))
+                builder.append(chr(0))
+                builder.append(chr(0))
+            elif code == ENC_UTF32BE:
+                builder.append(chr(0))
+                builder.append(chr(0))
+                builder.append(chr(ch >> 8))
+                builder.append(chr(ch & 0xff))
+        return space.newtuple([space.newbytes(builder.build()), w_end])
+    elif space.isinstance_w(w_exc, space.w_UnicodeDecodeError):
+        start = space.int_w(space.getattr(w_exc, space.newtext('start')))
+        obj = space.bytes_w(space.getattr(w_exc, space.newtext('object')))
+        encoding = space.text_w(space.getattr(w_exc, space.newtext('encoding')))
+        bytelength, code = get_standard_encoding(encoding)
+        ch = 0
+        # Try decoding a single surrogate character. If there are more,
+        # let the codec call us again
+        ch0 = ord(obj[start + 0]) if len(obj) > start + 0 else -1
+        ch1 = ord(obj[start + 1]) if len(obj) > start + 1 else -1
+        ch2 = ord(obj[start + 2]) if len(obj) > start + 2 else -1
+        ch3 = ord(obj[start + 3]) if len(obj) > start + 3 else -1
+        if code == ENC_UTF8:
+            if (ch1 != -1 and ch2 != -1 and
+                ch0 & 0xf0 == 0xe0 and
+                ch1 & 0xc0 == 0x80 and
+                ch2 & 0xc0 == 0x80):
+                # it's a three-byte code
+                ch = ((ch0 & 0x0f) << 12) + ((ch1 & 0x3f) << 6) + (ch2 & 0x3f)
+        elif code == ENC_UTF16LE:
+            ch = (ch1 << 8) | ch0
+        elif code == ENC_UTF16BE:
+            ch = (ch0 << 8) | ch1
+        elif code == ENC_UTF32LE:
+            ch = (ch3 << 24) | (ch2 << 16) | (ch1 << 8) | ch0
+        elif code == ENC_UTF32BE:
+            ch = (ch0 << 24) | (ch1 << 16) | (ch2 << 8) | ch3
+        if ch < 0xd800 or ch > 0xdfff:
+            # it's not a surrogate - fail
+            ch = 0
+        if ch == 0:
+            raise OperationError(space.type(w_exc), w_exc)
+        ch_utf8 = rutf8.unichr_as_utf8(ch, allow_surrogates=True)
+        return space.newtuple([space.newtext(ch_utf8, 1),
+                               space.newint(start + bytelength)])
+    else:
+        raise oefmt(space.w_TypeError,
+                    "don't know how to handle %T in error callback", w_exc)
+
+def surrogateescape_errors(space, w_exc):
+    check_exception(space, w_exc)
+    if space.isinstance_w(w_exc, space.w_UnicodeEncodeError):
+        w_obj = space.getattr(w_exc, space.newtext('object'))
+        w_obj = space.convert_arg_to_w_unicode(w_obj)
+        start = space.int_w(space.getattr(w_exc, space.newtext('start')))
+        w_end = space.getattr(w_exc, space.newtext('end'))
+        end = space.int_w(w_end)
+        res = ''
+        start = w_obj._index_to_byte(start)
+        end = w_obj._index_to_byte(end)
+        obj = w_obj._utf8
+        pos = start
+        while pos < end:
+            code = rutf8.codepoint_at_pos(obj, pos)
+            if code < 0xdc80 or code > 0xdcff:
+                # Not a UTF-8b surrogate, fail with original exception
+                raise OperationError(space.type(w_exc), w_exc)
+            res += chr(code - 0xdc00)
+            pos = rutf8.next_codepoint_pos(obj, pos)
+        return space.newtuple([space.newbytes(res), w_end])
+    elif space.isinstance_w(w_exc, space.w_UnicodeDecodeError):
+        consumed = 0
+        start = space.int_w(space.getattr(w_exc, space.newtext('start')))
+        end = space.int_w(space.getattr(w_exc, space.newtext('end')))
+        obj = space.bytes_w(space.getattr(w_exc, space.newtext('object')))
+        replace = u''
+        while consumed < 4 and consumed < end - start:
+            c = ord(obj[start+consumed])
+            if c < 128:
+                # Refuse to escape ASCII bytes.
+                break
+            replace += unichr(0xdc00 + c)
+            consumed += 1
+        if not consumed:
+            # codec complained about ASCII byte.
+            raise OperationError(space.type(w_exc), w_exc)
+        replace_utf8 = runicode.unicode_encode_utf_8(replace, len(replace),
+                                         'strict', allow_surrogates=True)
+        return space.newtuple([space.newtext(replace_utf8, len(replace)),
+                               space.newint(start + consumed)])
+    else:
+        raise oefmt(space.w_TypeError,
+                    "don't know how to handle %T in error callback", w_exc)
+
 @not_rpython
 def register_builtin_error_handlers(space):
     state = space.fromcache(CodecState)
     for error in ("strict", "ignore", "replace", "xmlcharrefreplace",
-                  "backslashreplace"):
+                  "backslashreplace", "surrogateescape", "surrogatepass",
+                  "namereplace"):
         name = error + "_errors"
         state.codec_error_registry[error] = interp2app(
                 globals()[name]).spacebind(space)
 
 
+def _wrap_codec_error(space, operr, action, encoding):
+    # Note that UnicodeErrors are not wrapped and returned as is,
+    # "thanks to" a limitation of try_set_from_cause.
+    message = "%s with '%s' codec failed" % (action, encoding)
+    return operr.try_set_from_cause(space, message)
+
+def _call_codec(space, w_coder, w_obj, action, encoding, errors):
+    try:
+        if errors:
+            w_res = space.call_function(w_coder, w_obj, space.newtext(errors))
+        else:
+            w_res = space.call_function(w_coder, w_obj)
+    except OperationError as operr:
+        raise _wrap_codec_error(space, operr, action, encoding)
+    if (not space.isinstance_w(w_res, space.w_tuple) or space.len_w(w_res) != 2):
+        if action[:2] == 'en':
+            raise oefmt(space.w_TypeError,
+                    "encoder must return a tuple (object, integer)")
+        elif action[:2] == 'de':
+            raise oefmt(space.w_TypeError,
+                    "decoder must return a tuple (object, integer)")
+        else:
+            raise oefmt(space.w_TypeError,
+                    "%s must return a tuple (object, integer)", action)
+    return space.getitem(w_res, space.newint(0))
+
 @unwrap_spec(errors='text')
 def lookup_error(space, errors):
     """lookup_error(errors) -> handler
@@ -320,29 +585,16 @@ def encode(space, w_obj, encoding=None, errors=None):
     if encoding is None:
         encoding = space.sys.defaultencoding
     w_encoder = space.getitem(lookup_codec(space, encoding), space.newint(0))
-    if errors:
-        w_res = space.call_function(w_encoder, w_obj, space.newtext(errors))
-    else:
-        w_res = space.call_function(w_encoder, w_obj)
-    w_retval = space.getitem(w_res, space.newint(0))
-    if not space.isinstance_w(w_retval, space.w_bytes):
-        raise oefmt(space.w_TypeError,
-                    "encoder did not return an string object (type '%T')",
-                    w_retval)
-    return w_retval
+    return _call_codec(space, w_encoder, w_obj, "encoding", encoding, errors)
 
 @unwrap_spec(errors='text_or_none')
 def readbuffer_encode(space, w_data, errors='strict'):
     s = space.getarg_w('s#', w_data)
     return space.newtuple([space.newbytes(s), space.newint(len(s))])
 
-@unwrap_spec(errors='text_or_none')
-def charbuffer_encode(space, w_data, errors='strict'):
-    s = space.getarg_w('t#', w_data)
-    return space.newtuple([space.newbytes(s), space.newint(len(s))])
-
 @unwrap_spec(encoding='text_or_none', errors='text_or_none')
 def decode(space, w_obj, encoding=None, errors=None):
+    from pypy.objspace.std.unicodeobject import W_UnicodeObject
     """decode(obj, [encoding[,errors]]) -> object
 
     Decodes obj using the codec registered for encoding. encoding defaults
@@ -355,14 +607,7 @@ def decode(space, w_obj, encoding=None, errors=None):
     if encoding is None:
         encoding = space.sys.defaultencoding
     w_decoder = space.getitem(lookup_codec(space, encoding), space.newint(1))
-    if errors:
-        w_res = space.call_function(w_decoder, w_obj, space.newtext(errors))
-    else:
-        w_res = space.call_function(w_decoder, w_obj)
-    if (not space.isinstance_w(w_res, space.w_tuple) or space.len_w(w_res) != 2):
-        raise oefmt(space.w_TypeError,
-                    "encoder must return a tuple (object, integer)")
-    return space.getitem(w_res, space.newint(0))
+    return _call_codec(space, w_decoder, w_obj, "decoding", encoding, errors)
 
 @unwrap_spec(errors='text')
 def register_error(space, errors, w_handler):
@@ -381,7 +626,24 @@ def register_error(space, errors, w_handler):
         raise oefmt(space.w_TypeError, "handler must be callable")
 
 # ____________________________________________________________
-# delegation to unicodehelper
+# Helpers for unicode.encode() and bytes.decode()
+def lookup_text_codec(space, action, encoding):
+    w_codec_info = lookup_codec(space, encoding)
+    try:
+        is_text_encoding = space.is_true(
+                space.getattr(w_codec_info, space.newtext('_is_text_encoding')))
+    except OperationError as e:
+        if e.match(space, space.w_AttributeError):
+            is_text_encoding = True
+        else:
+            raise
+    if not is_text_encoding:
+        raise oefmt(space.w_LookupError,
+                    "'%s' is not a text encoding; "
+                    "use codecs.%s() to handle arbitrary codecs", encoding, action)
+    return w_codec_info
+
+# ____________________________________________________________
 
 def _find_implementation(impl_name):
     func = getattr(unicodehelper, impl_name)
@@ -392,15 +654,19 @@ def make_encoder_wrapper(name):
     func = _find_implementation(rname)
     @unwrap_spec(errors='text_or_none')
     def wrap_encoder(space, w_arg, errors="strict"):
-        w_arg = space.convert_arg_to_w_unicode(w_arg)
+        # w_arg is a W_Unicode or W_Bytes?
+        w_arg = space.convert_arg_to_w_unicode(w_arg, errors)
         if errors is None:
             errors = 'strict'
+        allow_surrogates = False
+        if errors in ('surrogatepass',):
+            allow_surrogates = True
         state = space.fromcache(CodecState)
-        utf8len = w_arg._length
-        # XXX deal with func() returning length or not
-        result = func(w_arg._utf8, errors, state.encode_error_handler)
-        return space.newtuple([space.newbytes(result), space.newint(utf8len)])
-    wrap_encoder.func_name = rname
+        ulen = w_arg._length
+        result = func(w_arg._utf8, errors, state.encode_error_handler,
+                      allow_surrogates=allow_surrogates)
+        return space.newtuple([space.newbytes(result), space.newint(ulen)])
+    wrap_encoder.__name__ = func.__name__
     globals()[name] = wrap_encoder
 
 def make_decoder_wrapper(name):
@@ -413,11 +679,10 @@ def make_decoder_wrapper(name):
             errors = 'strict'
         final = space.is_true(w_final)
         state = space.fromcache(CodecState)
-        result, consumed, length = func(string, errors,
-                                              final, state.decode_error_handler)
-        return space.newtuple([space.newutf8(result, length),
-                               space.newint(consumed)])
-    wrap_decoder.func_name = rname
+        result, length, pos = func(string, errors, final, state.decode_error_handler)
+        # must return bytes, pos
+        return space.newtuple([space.newutf8(result, length), space.newint(pos)])
+    wrap_decoder.__name__ = func.__name__
     globals()[name] = wrap_decoder
 
 for encoder in [
@@ -450,40 +715,87 @@ for decoder in [
          ]:
     make_decoder_wrapper(decoder)
 
-from rpython.rlib import runicode
-if hasattr(runicode, 'str_decode_mbcs'):
+if getattr(unicodehelper, '_WIN32', False):
     make_encoder_wrapper('mbcs_encode')
     make_decoder_wrapper('mbcs_decode')
+    make_encoder_wrapper('oem_encode')
+    make_decoder_wrapper('oem_decode')
+
+    # need to add the code_page argument
+
+    @unwrap_spec(code_page=int, errors='text_or_none')
+    def code_page_encode(space, code_page, w_arg, errors="strict"):
+        # w_arg is a W_Unicode or W_Bytes?
+        if code_page < 0:
+            raise oefmt(space.w_ValueError, "invalid code page number %d",
+                        code_page)
+        w_arg = space.convert_arg_to_w_unicode(w_arg, errors)
+        if errors is None:
+            errors = 'strict'
+        allow_surrogates = False
+        if errors in ('surrogatepass',):
+            allow_surrogates = True
+        state = space.fromcache(CodecState)
+        ulen = w_arg._length
+        try:
+            result = unicodehelper.utf8_encode_code_page(code_page, w_arg._utf8,
+                      errors, state.encode_error_handler,
+                      allow_surrogates=allow_surrogates)
+        except OSError as e:
+            raise wrap_oserror(space, e)
+        return space.newtuple([space.newbytes(result), space.newint(ulen)])
+
+    @unwrap_spec(code_page=int, string='bufferstr', errors='text_or_none',
+                 w_final=WrappedDefault(False))
+    def code_page_decode(space, code_page, string, errors="strict", w_final=None):
+        if code_page < 0:
+            raise oefmt(space.w_ValueError, "invalid code page number %d",
+                        code_page)
+        if errors is None:
+            errors = 'strict'
+        final = space.is_true(w_final)
+        state = space.fromcache(CodecState)
+        try:
+            result, length, pos = unicodehelper.str_decode_code_page(code_page,
+                                   string, errors, final,
+                                   state.decode_error_handler)
+        except OSError as e:
+            raise wrap_oserror(space, e)
+        # must return bytes, pos
+        return space.newtuple([space.newutf8(result, length), space.newint(pos)])
+
 
 # utf-8 functions are not regular, because we have to pass
-# "allow_surrogates=True"
+# "allow_surrogates=False"
 @unwrap_spec(errors='text_or_none')
 def utf_8_encode(space, w_obj, errors="strict"):
     utf8, lgt = space.utf8_len_w(w_obj)
-    if rutf8.has_surrogates(utf8):
-        utf8 = rutf8.reencode_utf8_with_surrogates(utf8)
-    return space.newtuple([space.newbytes(utf8), space.newint(lgt)])
+    if lgt == len(utf8): # ascii
+        return space.newtuple([space.newbytes(utf8), space.newint(lgt)])
+    if errors is None:
+        errors = 'strict'
+    state = space.fromcache(CodecState)
+    try:
+        result = unicodehelper.utf8_encode_utf_8(utf8, errors,
+                     state.encode_error_handler, allow_surrogates=False)
+    except unicodehelper.ErrorHandlerError as e:
+        raise oefmt(space.w_IndexError,
+                   "position %d from error handler invalid, already encoded %d",
+                    e.new,e.old)
+
+    return space.newtuple([space.newbytes(result), space.newint(lgt)])
 
 @unwrap_spec(string='bufferstr', errors='text_or_none',
              w_final = WrappedDefault(False))
 def utf_8_decode(space, string, errors="strict", w_final=None):
-    from pypy.interpreter import unicodehelper
-
     if errors is None:
         errors = 'strict'
     final = space.is_true(w_final)
     state = space.fromcache(CodecState)
-    # call the fast version for checking
-    try:
-        lgt = rutf8.check_utf8(string, allow_surrogates=True)
-    except rutf8.CheckError:
-        res, consumed, lgt = unicodehelper.str_decode_utf8(string,
-            errors, final, state.decode_error_handler)
-        return space.newtuple([space.newutf8(res, lgt),
-                               space.newint(consumed)])
-    else:
-        return space.newtuple([space.newutf8(string, lgt),
-                               space.newint(len(string))])
+    res, lgt, pos = unicodehelper.str_decode_utf8(string,
+        errors, final, state.decode_error_handler)
+    return space.newtuple([space.newutf8(res, lgt),
+                           space.newint(pos)])
 
 @unwrap_spec(data='bufferstr', errors='text_or_none', byteorder=int,
              w_final=WrappedDefault(False))
@@ -495,20 +807,17 @@ def utf_16_ex_decode(space, data, errors='strict', byteorder=0, w_final=None):
     final = space.is_true(w_final)
     state = space.fromcache(CodecState)
     if byteorder == 0:
-        byteorder = 'native'
+        _byteorder = 'native'
     elif byteorder == -1:
-        byteorder = 'little'
+        _byteorder = 'little'
     else:
-        byteorder = 'big'
-    consumed = len(data)
-    if final:
-        consumed = 0
-    res, consumed, lgt, byteorder = str_decode_utf_16_helper(
+        _byteorder = 'big'
+    res, lgt, pos, bo = str_decode_utf_16_helper(
         data, errors, final,
-        state.decode_error_handler, byteorder)
+        state.decode_error_handler, _byteorder)
+    # return result, consumed, byteorder for buffered incremental encoders
     return space.newtuple([space.newutf8(res, lgt),
-                           space.newint(consumed),
-                           space.newint(byteorder)])
+                           space.newint(pos), space.newint(bo)])
 
 @unwrap_spec(data='bufferstr', errors='text_or_none', byteorder=int,
              w_final=WrappedDefault(False))
@@ -518,20 +827,17 @@ def utf_32_ex_decode(space, data, errors='strict', byteorder=0, w_final=None):
     final = space.is_true(w_final)
     state = space.fromcache(CodecState)
     if byteorder == 0:
-        byteorder = 'native'
+        _byteorder = 'native'
     elif byteorder == -1:
-        byteorder = 'little'
+        _byteorder = 'little'
     else:
-        byteorder = 'big'
-    consumed = len(data)
-    if final:
-        consumed = 0
-    res, consumed, lgt, byteorder = str_decode_utf_32_helper(
+        _byteorder = 'big'
+    res, lgt, pos, bo = str_decode_utf_32_helper(
         data, errors, final,
-        state.decode_error_handler, byteorder)
+        state.decode_error_handler, _byteorder)
+    # return result, consumed, byteorder for buffered incremental encoders
     return space.newtuple([space.newutf8(res, lgt),
-                           space.newint(consumed),
-                           space.newint(byteorder)])
+                           space.newint(pos), space.newint(bo)])
 
 # ____________________________________________________________
 # Charmap
@@ -570,13 +876,13 @@ class Charmap_Decode:
             if not 0 <= x <= 0x10FFFF:
                 raise oefmt(space.w_TypeError,
                     "character mapping must be in range(0x110000)")
-            return rutf8.unichr_as_utf8(x)
+            return rutf8.unichr_as_utf8(x, allow_surrogates=True)
         elif space.is_w(w_ch, space.w_None):
             # Charmap may return None
             return errorchar
 
         raise oefmt(space.w_TypeError,
-            "character mapping must return integer, None or unicode")
+            "character mapping must return integer, None or str")
 
 class Charmap_Encode:
     def __init__(self, space, w_mapping):
@@ -609,13 +915,11 @@ class Charmap_Encode:
             return errorchar
 
         raise oefmt(space.w_TypeError,
-            "character mapping must return integer, None or str")
+            "character mapping must return integer, bytes or None, not str")
 
 
 @unwrap_spec(string='bufferstr', errors='text_or_none')
 def charmap_decode(space, string, errors="strict", w_mapping=None):
-    from pypy.interpreter import unicodehelper
-
     if errors is None:
         errors = 'strict'
     if len(string) == 0:
@@ -629,15 +933,13 @@ def charmap_decode(space, string, errors="strict", w_mapping=None):
 
     final = True
     state = space.fromcache(CodecState)
-    result, consumed, lgt = unicodehelper.str_decode_charmap(
+    result, lgt, pos = unicodehelper.str_decode_charmap(
         string, errors, final, state.decode_error_handler, mapping)
     return space.newtuple([space.newutf8(result, lgt),
-                           space.newint(consumed)])
+                           space.newint(len(string))])
 
 @unwrap_spec(errors='text_or_none')
 def charmap_encode(space, w_unicode, errors="strict", w_mapping=None):
-    from pypy.interpreter import unicodehelper
-
     if errors is None:
         errors = 'strict'
     if space.is_none(w_mapping):
@@ -683,10 +985,10 @@ class UnicodeData_Handler:
             return -1
         return space.int_w(w_code)
 
-@unwrap_spec(string='bufferstr', errors='text_or_none',
-             w_final=WrappedDefault(False))
-def unicode_escape_decode(space, string, errors="strict", w_final=None):
-    from pypy.interpreter import unicodehelper
+@unwrap_spec(errors='text_or_none', w_final=WrappedDefault(False))
+def unicode_escape_decode(space, w_string, errors="strict", w_final=None):
+    string = space.getarg_w('s*', w_string).as_str()
+
 
     if errors is None:
         errors = 'strict'
@@ -695,27 +997,55 @@ def unicode_escape_decode(space, string, errors="strict", w_final=None):
 
     unicode_name_handler = state.get_unicodedata_handler(space)
 
-    result, consumed, lgt = unicodehelper.str_decode_unicode_escape(
+    result, u_len, lgt, first_escape_error_char = unicodehelper.str_decode_unicode_escape(
         string, errors,
         final, state.decode_error_handler,
         unicode_name_handler)
 
-    return space.newtuple([space.newutf8(result, lgt), space.newint(consumed)])
+    if first_escape_error_char is not None:
+        # Here, 'first_escape_error_char' is a single string character.
+        # Careful, it might be >= '\x80'.  If it is, it would made an
+        # invalid utf-8 string when pasted directory in it.
+        if ' ' <= first_escape_error_char < '\x7f':
+            msg = "invalid escape sequence '\\%s'" % (first_escape_error_char,)
+        else:
+            msg = "invalid escape sequence: '\\' followed by %s" % (
+                space.text_w(space.repr(
+                    space.newbytes(first_escape_error_char))),)
+        space.warn(
+            space.newtext(msg),
+            space.w_DeprecationWarning
+        )
+    return space.newtuple([space.newutf8(result, u_len), space.newint(lgt)])
+
+# ____________________________________________________________
+# Raw Unicode escape (accepts bytes or str)
+
+@unwrap_spec(errors='text_or_none', w_final=WrappedDefault(False))
+def raw_unicode_escape_decode(space, w_string, errors="strict", w_final=None):
+    string = space.getarg_w('s*', w_string).as_str()
+    if errors is None:
+        errors = 'strict'
+    final = space.is_true(w_final)
+    state = space.fromcache(CodecState)
+    result, u_len, lgt = unicodehelper.str_decode_raw_unicode_escape(
+        string, errors, final, state.decode_error_handler)
+    return space.newtuple([space.newtext(result), space.newint(lgt)])
 
 # ____________________________________________________________
 # Unicode-internal
 
 @unwrap_spec(errors='text_or_none')
 def unicode_internal_decode(space, w_string, errors="strict"):
-    from pypy.interpreter import unicodehelper
-
     if errors is None:
         errors = 'strict'
     # special case for this codec: unicodes are returned as is
     if space.isinstance_w(w_string, space.w_unicode):
         return space.newtuple([w_string, space.len(w_string)])
 
-    string = space.readbuf_w(w_string).as_str()
+    string = space.charbuf_w(w_string)
+    space.warn(space.newtext("unicode_internal codec has been deprecated"),
+               space.w_DeprecationWarning)
 
     if len(string) == 0:
         return space.newtuple([space.newutf8('', 0),
@@ -723,28 +1053,44 @@ def unicode_internal_decode(space, w_string, errors="strict"):
 
     final = True
     state = space.fromcache(CodecState)
-    result, consumed, lgt = unicodehelper.str_decode_unicode_internal(
+    result, lgt = unicodehelper.str_decode_unicode_internal(
         string, errors,
         final, state.decode_error_handler)
     return space.newtuple([space.newutf8(result, lgt),
-                           space.newint(consumed)])
+                           space.newint(len(string))])
+
+@unwrap_spec(errors='text_or_none')
+def unicode_internal_encode(space, w_uni, errors="strict"):
+    space.warn(space.newtext("unicode_internal codec has been deprecated"),
+               space.w_DeprecationWarning)
+    if errors is None:
+        errors = 'strict'
+    if space.isinstance_w(w_uni, space.w_unicode):
+        utf8 = space.utf8_w(w_uni)
+        state = space.fromcache(CodecState)
+        result = unicodehelper.utf8_encode_unicode_internal(
+            utf8, errors, state.encode_error_handler)
+        w_lgt = space.newint(space.len_w(w_uni))
+        return space.newtuple([space.newbytes(result), w_lgt])
+    else:
+        # special case for this codec: bytes are returned as is
+        string = space.charbuf_w(w_uni)
+        return space.newtuple([space.newbytes(string), space.newint(len(string))])
 
 # ____________________________________________________________
-# support for the "string escape" codec
+# support for the "string escape" translation
 # This is a bytes-to bytes transformation
 
 @unwrap_spec(data='bytes', errors='text_or_none')
 def escape_encode(space, data, errors='strict'):
     from pypy.objspace.std.bytesobject import string_escape_encode
-    result = string_escape_encode(data, quote="'")
-    start = 1
-    end = len(result) - 1
-    assert end >= 0
-    w_result = space.newbytes(result[start:end])
-    return space.newtuple([w_result, space.newint(len(data))])
+    result = string_escape_encode(data, False)
+    return space.newtuple([space.newbytes(result), space.newint(len(data))])
 
-@unwrap_spec(data='bytes', errors='text_or_none')
-def escape_decode(space, data, errors='strict'):
+@unwrap_spec(errors='text_or_none')
+def escape_decode(space, w_data, errors='strict'):
+    data = space.getarg_w('s#', w_data)
     from pypy.interpreter.pyparser.parsestring import PyString_DecodeEscape
-    result = PyString_DecodeEscape(space, data, errors, None)
+    result, _ = PyString_DecodeEscape(space, data, errors, None)
+
     return space.newtuple([space.newbytes(result), space.newint(len(data))])
