@@ -5,7 +5,7 @@ from rpython.rtyper.lltypesystem import rffi, lltype
 from rpython.rlib.rarithmetic import r_singlefloat, r_longfloat
 from rpython.rlib import rfloat, rawrefcount
 from pypy.module._rawffi.interp_rawffi import letter2tp
-from pypy.module._rawffi.interp_array import W_ArrayInstance
+from pypy.module._rawffi.array import W_ArrayInstance
 from pypy.module._cppyy import helper, capi, ffitypes, lowlevelviews
 
 # Converter objects are used to translate between RPython and C++. They are
@@ -993,7 +993,7 @@ _converters["void**"]                   = VoidPtrPtrConverter
 _converters["void*&"]                   = VoidPtrRefConverter
 
 # special cases (note: 'string' aliases added below)
-_converters["std::string"]                       = StdStringConverter
+_converters["std::basic_string<char>"]           = StdStringConverter
 _converters["const std::basic_string<char>&"]    = StdStringConverter     # TODO: shouldn't copy
 _converters["std::basic_string<char>&"]          = StdStringRefConverter
 _converters["std::basic_string<char>&&"]         = StdStringMoveConverter
@@ -1126,15 +1126,14 @@ def _add_aliased_converters():
         ("char",                            "signed char"),   # TODO: check
         ("const char*",                     "char*"),
 
-        ("std::string",                     "string"),
-        ("std::string",                     "std::basic_string<char>"),
+        ("std::basic_string<char>",         "string"),
         ("const std::basic_string<char>&",  "const string&"),
         ("std::basic_string<char>&",        "string&"),
         ("std::basic_string<char>&&",       "string&&"),
 
         ("PyObject*",                       "_object*"),
     )
-
+ 
     for c_type, alias in aliases:
         _converters[alias] = _converters[c_type]
 _add_aliased_converters()
