@@ -72,7 +72,7 @@ def get_nullptr(space):
     state = space.fromcache(State)
     if state.w_nullptr is None:
         from pypy.module._rawffi.interp_rawffi import unpack_simple_shape
-        from pypy.module._rawffi.interp_array import W_Array, W_ArrayInstance
+        from pypy.module._rawffi.array import W_Array, W_ArrayInstance
         arr = space.interp_w(W_Array, unpack_simple_shape(space, space.newtext('P')))
         # TODO: fix this hack; fromaddress() will allocate memory if address
         # is null and there seems to be no way around it (ll_buffer can not
@@ -509,11 +509,8 @@ Create an instance method object.""",
     __new__          = interp2app(MethodWithProps.descr_method__new__.im_func),
     __call__         = interp2app(MethodWithProps.descr_method_call),
     __get__          = interp2app(MethodWithProps.descr_method_get),
-    im_func          = interp_attrproperty_w('w_function', cls=MethodWithProps),
     __func__         = interp_attrproperty_w('w_function', cls=MethodWithProps),
-    im_self          = interp_attrproperty_w('w_instance', cls=MethodWithProps),
     __self__         = interp_attrproperty_w('w_instance', cls=MethodWithProps),
-    im_class         = interp_attrproperty_w('w_class', cls=MethodWithProps),
     __getattribute__ = interp2app(MethodWithProps.descr_method_getattribute),
     __eq__           = interp2app(MethodWithProps.descr_method_eq),
     __ne__           = descr_generic_ne,
@@ -554,9 +551,9 @@ class W_CPPOverload(W_Root):
                             not space.is_w(w_obj, space.w_None) or
                             space.is_w(w_cls, space.type(space.w_None)))
         if asking_for_bound:
-            return MethodWithProps(space, self, w_obj, w_cls)
+            return MethodWithProps(space, self, w_obj)
         else:
-            return MethodWithProps(space, self, None, w_cls)
+            return self   # unbound methods don't exist in Python 3
 
     @unwrap_spec(args_w='args_w')
     def call_args(self, args_w):
@@ -685,7 +682,7 @@ class W_CPPOverload(W_Root):
     def fget_doc(self, space):
         return self.prototype()
 
-    def getname(self, space):
+    def getname(self, space): 
         # for the benefit of Method/instancemethod
         return capi.c_method_name(space, self.functions[0].cppmethod)
 
@@ -717,7 +714,7 @@ class W_CPPStaticOverload(W_CPPOverload):
             # onto a class and w_this should be set
             cppinstance = self.space.interp_w(W_CPPInstance, w_obj)
             if cppinstance.clsdecl.handle != self.scope.handle:
-                return MethodWithProps(self.space, self, w_obj, w_cls)    # bound
+                return MethodWithProps(self.space, self, w_obj)    # bound
         return self      # unbound
 
     @unwrap_spec(args_w='args_w')
@@ -1325,7 +1322,7 @@ class W_CPPNamespaceDecl(W_CPPScopeDecl):
         for name in alldir:
             w_alldir.append(self.space.newtext(name))
         return w_alldir
-
+        
     def missing_attribute_error(self, name):
         return oefmt(self.space.w_AttributeError,
             "namespace '%s' has no attribute %s", self.name, name)
@@ -1619,7 +1616,7 @@ class W_CPPInstance(W_Root):
         raise oefmt(self.space.w_TypeError,
                     "cannot instantiate abstract class '%s'",
                     self.clsdecl.name)
-
+ 
     def instance__eq__(self, w_other):
         # special case: if other is None, compare pointer-style
         if self.space.is_w(w_other, self.space.w_None):
