@@ -17,7 +17,6 @@ from rpython.tool.sourcetools import func_renamer
 from pypy.module._io.interp_iobase import (
     W_IOBase, DEFAULT_BUFFER_SIZE, convert_size, trap_eintr,
     check_readable_w, check_writable_w, check_seekable_w)
-from pypy.module._io.interp_io import W_BlockingIOError
 from rpython.rlib import rthread
 
 STATE_ZERO, STATE_OK, STATE_DETACHED = range(3)
@@ -30,13 +29,12 @@ def make_write_blocking_error(space, written):
     # In that case what we get is a potentially nonsense errno.  But
     # we'll use get_saved_errno() anyway, and hope (like CPython does)
     # that we're getting a reasonable value at this point.
-    w_type = space.gettypeobject(W_BlockingIOError.typedef)
     w_value = space.call_function(
-        w_type,
+        space.w_BlockingIOError,
         space.newint(rposix.get_saved_errno()),
         space.newtext("write could not complete without blocking"),
         space.newint(written))
-    return OperationError(w_type, w_value)
+    return OperationError(space.w_BlockingIOError, w_value)
 
 
 class TryLock(object):
@@ -65,53 +63,100 @@ class BlockingIOError(Exception):
     pass
 
 class W_BufferedIOBase(W_IOBase):
-    def _unsupportedoperation(self, space, message):
-        w_exc = space.getattr(space.getbuiltinmodule('_io'),
-                              space.newtext('UnsupportedOperation'))
-        raise OperationError(w_exc, space.newtext(message))
-
     def _check_init(self, space):
         raise NotImplementedError
 
-    def _deprecated_max_buffer_size(self, space):
-        space.warn(space.newtext("max_buffer_size is deprecated"),
-                   space.w_DeprecationWarning)
-
     def read_w(self, space, w_size=None):
+        """Read and return up to n bytes.
+
+If the argument is omitted, None, or negative, reads and
+returns all data until EOF.
+
+If the argument is positive, and the underlying raw stream is
+not 'interactive', multiple raw reads may be issued to satisfy
+the byte count (unless EOF is reached first).  But for
+interactive raw streams (as well as sockets and pipes), at most
+one raw read will be issued, and a short result does not imply
+that EOF is imminent.
+
+Returns an empty bytes object on EOF.
+
+Returns None if the underlying raw stream was open in non-blocking
+mode and no data is available at the moment."""
         self._unsupportedoperation(space, "read")
 
     def read1_w(self, space, w_size):
+        """Read and return up to n bytes, with at most one read() call
+to the underlying raw stream. A short result does not imply
+that EOF is imminent.
+
+Returns an empty bytes object on EOF."""
         self._unsupportedoperation(space, "read1")
 
     def write_w(self, space, w_data):
+        """Write the given buffer to the IO stream.
+
+Returns the number of bytes written, which is always the length of b
+in bytes.
+
+Raises BlockingIOError if the buffer is full and the
+underlying raw stream cannot accept more data at the moment."""
         self._unsupportedoperation(space, "write")
 
     def detach_w(self, space):
+        """Disconnect this buffer from its underlying raw stream and return it.
+
+After the raw stream has been detached, the buffer is in an unusable
+state."""
         self._unsupportedoperation(space, "detach")
 
     def readinto_w(self, space, w_buffer):
-        return self._readinto(space, w_buffer, "read")
+        return self._readinto(space, w_buffer, read_once=False)
 
-    def _readinto(self, space, w_buffer, methodname):
+    def readinto1_w(self, space, w_buffer):
+        return self._readinto(space, w_buffer, read_once=True)
+
+    def _readinto(self, space, w_buffer, read_once):
         rwbuffer = space.writebuf_w(w_buffer)
         length = rwbuffer.getlength()
+        methodname = "read1" if read_once else "read"
         w_data = space.call_method(self, methodname, space.newint(length))
 
         if not space.isinstance_w(w_data, space.w_bytes):
             raise oefmt(space.w_TypeError, "%s() should return bytes",
                         methodname)
         data = space.bytes_w(w_data)
-        self.output_slice(space, rwbuffer, 0, data)
+        if len(data) > length:
+            raise oefmt(space.w_ValueError,
+                        "%s() returned too much data: "
+                        "%d bytes requested, %d returned",
+                        methodname, length, len(data))
+        rwbuffer.setslice(0, data)
         return space.newint(len(data))
 
 W_BufferedIOBase.typedef = TypeDef(
     '_io._BufferedIOBase', W_IOBase.typedef,
+    __doc__="""Base class for buffered IO objects.
+
+The main difference with RawIOBase is that the read() method
+supports omitting the size argument, and does not have a default
+implementation that defers to readinto().
+
+In addition, read(), readinto() and write() may raise
+BlockingIOError if the underlying raw stream is in non-blocking
+mode and not ready; unlike their raw counterparts, they will never
+return None.
+
+A typical implementation should not inherit from a RawIOBase
+implementation, but wrap one.
+""",
     __new__ = generic_new_descr(W_BufferedIOBase),
     read = interp2app(W_BufferedIOBase.read_w),
     read1 = interp2app(W_BufferedIOBase.read1_w),
     write = interp2app(W_BufferedIOBase.write_w),
     detach = interp2app(W_BufferedIOBase.detach_w),
     readinto = interp2app(W_BufferedIOBase.readinto_w),
+    readinto1 = interp2app(W_BufferedIOBase.readinto1_w),
 )
 
 class BufferedMixin:
@@ -246,11 +291,11 @@ class BufferedMixin:
 
     @unwrap_spec(pos=r_longlong, whence=int)
     def seek_w(self, space, pos, whence=0):
-        self._check_init(space)
+        self._check_closed(space, "seek of closed file")
         if whence not in (0, 1, 2):
             raise oefmt(space.w_ValueError,
                         "whence must be between 0 and 2, not %d", whence)
-        self._check_closed(space, "seek of closed file")
+        check_seekable_w(space, self.w_raw)
         if whence != 2 and self.readable:
             # Check if seeking leaves us inside the current buffer, so as to
             # return quickly if possible. Also, we needn't take the lock in
@@ -302,11 +347,23 @@ class BufferedMixin:
         with self.lock:
             if self._closed(space):
                 return
+        flush_operr = None
         try:
             space.call_method(self, "flush")
+        except OperationError as e:
+            flush_operr = e
+            raise
         finally:
             with self.lock:
-                space.call_method(self.w_raw, "close")
+                try:
+                    space.call_method(self.w_raw, "close")
+                except OperationError as e:
+                    if flush_operr:
+                        e.chain_exceptions(space, flush_operr)
+                    raise
+
+    def _dealloc_warn_w(self, space, w_source):
+        space.call_method(self.w_raw, "_dealloc_warn", w_source)
 
     def simple_flush_w(self, space):
         self._check_init(space)
@@ -390,7 +447,6 @@ class BufferedMixin:
     # Read methods
 
     def read_w(self, space, w_size=None):
-        self._check_init(space)
         self._check_closed(space, "read of closed file")
         size = convert_size(space, w_size)
 
@@ -410,7 +466,6 @@ class BufferedMixin:
 
     @unwrap_spec(size=int)
     def peek_w(self, space, size=0):
-        self._check_init(space)
         self._check_closed(space, "peek of closed file")
         with self.lock:
             if self.writable:
@@ -438,7 +493,6 @@ class BufferedMixin:
 
     @unwrap_spec(size=int)
     def read1_w(self, space, size):
-        self._check_init(space)
         self._check_closed(space, "read of closed file")
 
         if size < 0:
@@ -555,7 +609,7 @@ class BufferedMixin:
         remaining = n
         written = 0
         if current_size:
-            self.output_slice(space, result_buffer,
+            result_buffer.setslice(
                 written, self.buffer[self.pos:self.pos + current_size])
             remaining -= current_size
             written += current_size
@@ -600,7 +654,7 @@ class BufferedMixin:
             if remaining > 0:
                 if size > remaining:
                     size = remaining
-                self.output_slice(space, result_buffer,
+                result_buffer.setslice(
                     written, self.buffer[self.pos:self.pos + size])
                 self.pos += size
                 written += size
@@ -620,7 +674,6 @@ class BufferedMixin:
         return None
 
     def readline_w(self, space, w_limit=None):
-        self._check_init(space)
         self._check_closed(space, "readline of closed file")
 
         limit = convert_size(space, w_limit)
@@ -696,9 +749,8 @@ class BufferedMixin:
             self.read_end = self.pos
 
     def write_w(self, space, w_data):
-        self._check_init(space)
         self._check_closed(space, "write to closed file")
-        data = space.getarg_w('s*', w_data).as_str()
+        data = space.charbuf_w(w_data)
         size = len(data)
 
         with self.lock:
@@ -722,11 +774,8 @@ class BufferedMixin:
             try:
                 self._writer_flush_unlocked(space)
             except OperationError as e:
-                if not e.match(space, space.gettypeobject(
-                    W_BlockingIOError.typedef)):
+                if not e.match(space, space.w_BlockingIOError):
                     raise
-                w_exc = e.get_w_value(space)
-                assert isinstance(w_exc, W_BlockingIOError)
                 if self.readable:
                     self._reader_reset_buf()
                 # Make some place by shifting the buffer
@@ -806,7 +855,6 @@ class BufferedMixin:
         return space.newint(written)
 
     def flush_w(self, space):
-        self._check_init(space)
         self._check_closed(space, "flush of closed file")
         with self.lock:
             self._flush_and_rewind_unlocked(space)
@@ -821,7 +869,63 @@ class BufferedMixin:
             finally:
                 self._reader_reset_buf()
 
-class W_BufferedReader(BufferedMixin, W_BufferedIOBase):
+class BufferedReaderMixin(BufferedMixin):
+    _mixin_ = True
+
+    def readinto_w(self, space, w_buffer):
+        return self._readinto(space, w_buffer, read_once=False)
+
+    def readinto1_w(self, space, w_buffer):
+        return self._readinto(space, w_buffer, read_once=True)
+
+    def _readinto(self, space, w_buffer, read_once):
+        rwbuffer = space.writebuf_w(w_buffer)
+        length = rwbuffer.getlength()
+        with self.lock:
+            have = self._readahead()
+            if have >= length:
+                rwbuffer.setslice(0, self.buffer[self.pos:self.pos + length])
+                self.pos += length
+                return space.newint(length)
+            written = 0
+            if have > 0:
+                rwbuffer.setslice(0, self.buffer[self.pos:self.read_end])
+                written = have
+
+            while written < length:
+                if self.writable:
+                    self._flush_and_rewind_unlocked(space)
+                self._reader_reset_buf()
+                self.pos = 0
+                if written + len(self.buffer) < length:
+                    try:
+                        got = self._raw_read(
+                            space, rwbuffer, written, length - written)
+                        written += got
+                    except BlockingIOError:
+                        got = 0
+                    if got == 0:
+                        break
+                elif read_once and written:
+                    break
+                else:
+                    try:
+                        have = self._fill_buffer(space)
+                    except BlockingIOError:
+                        have = 0
+                    if have == 0:
+                        break
+                    endpos = min(have, length - written)
+                    assert endpos >= 0
+                    rwbuffer.setslice(written, self.buffer[0:endpos])
+                    written += endpos
+                    self.pos = endpos
+                if read_once:
+                    break
+            return space.newint(written)
+
+
+class W_BufferedReader(BufferedReaderMixin, W_BufferedIOBase):
     @unwrap_spec(buffer_size=int)
     def descr_init(self, space, w_raw, buffer_size=DEFAULT_BUFFER_SIZE):
         self.state = STATE_ZERO
@@ -839,17 +943,19 @@ W_BufferedReader.typedef = TypeDef(
     '_io.BufferedReader', W_BufferedIOBase.typedef,
     __new__ = generic_new_descr(W_BufferedReader),
     __init__  = interp2app(W_BufferedReader.descr_init),
+    __getstate__ = interp2app(W_BufferedReader.getstate_w),
 
     read = interp2app(W_BufferedReader.read_w),
     peek = interp2app(W_BufferedReader.peek_w),
     read1 = interp2app(W_BufferedReader.read1_w),
+    readinto = interp2app(W_BufferedReader.readinto_w),
+    readinto1 = interp2app(W_BufferedReader.readinto1_w),
     raw = interp_attrproperty_w("w_raw", cls=W_BufferedReader),
     readline = interp2app(W_BufferedReader.readline_w),
 
     # from the mixin class
     __repr__ = interp2app(W_BufferedReader.repr_w),
     readable = interp2app(W_BufferedReader.readable_w),
-    writable = interp2app(W_BufferedReader.writable_w),
     seekable = interp2app(W_BufferedReader.seekable_w),
     seek = interp2app(W_BufferedReader.seek_w),
     tell = interp2app(W_BufferedReader.tell_w),
@@ -859,18 +965,15 @@ W_BufferedReader.typedef = TypeDef(
     truncate = interp2app(W_BufferedReader.truncate_w),
     fileno = interp2app(W_BufferedReader.fileno_w),
     isatty = interp2app(W_BufferedReader.isatty_w),
+    _dealloc_warn = interp2app(W_BufferedReader._dealloc_warn_w),
     closed = GetSetProperty(W_BufferedReader.closed_get_w),
     name = GetSetProperty(W_BufferedReader.name_get_w),
     mode = GetSetProperty(W_BufferedReader.mode_get_w),
 )
 
 class W_BufferedWriter(BufferedMixin, W_BufferedIOBase):
-    @unwrap_spec(buffer_size=int, max_buffer_size=int)
-    def descr_init(self, space, w_raw, buffer_size=DEFAULT_BUFFER_SIZE,
-                   max_buffer_size=-234):
-        if max_buffer_size != -234:
-            self._deprecated_max_buffer_size(space)
-
+    @unwrap_spec(buffer_size=int)
+    def descr_init(self, space, w_raw, buffer_size=DEFAULT_BUFFER_SIZE):
         self.state = STATE_ZERO
         check_writable_w(space, w_raw)
 
@@ -886,6 +989,7 @@ W_BufferedWriter.typedef = TypeDef(
     '_io.BufferedWriter', W_BufferedIOBase.typedef,
     __new__ = generic_new_descr(W_BufferedWriter),
     __init__  = interp2app(W_BufferedWriter.descr_init),
+    __getstate__ = interp2app(W_BufferedWriter.getstate_w),
 
     write = interp2app(W_BufferedWriter.write_w),
     flush = interp2app(W_BufferedWriter.flush_w),
@@ -893,7 +997,6 @@ W_BufferedWriter.typedef = TypeDef(
 
     # from the mixin class
     __repr__ = interp2app(W_BufferedWriter.repr_w),
-    readable = interp2app(W_BufferedWriter.readable_w),
     writable = interp2app(W_BufferedWriter.writable_w),
     seekable = interp2app(W_BufferedWriter.seekable_w),
     seek = interp2app(W_BufferedWriter.seek_w),
@@ -903,6 +1006,7 @@ W_BufferedWriter.typedef = TypeDef(
     isatty = interp2app(W_BufferedWriter.isatty_w),
     detach = interp2app(W_BufferedWriter.detach_w),
     truncate = interp2app(W_BufferedWriter.truncate_w),
+    _dealloc_warn = interp2app(W_BufferedWriter._dealloc_warn_w),
     closed = GetSetProperty(W_BufferedWriter.closed_get_w),
     name = GetSetProperty(W_BufferedWriter.name_get_w),
     mode = GetSetProperty(W_BufferedWriter.mode_get_w),
@@ -930,12 +1034,9 @@ class W_BufferedRWPair(W_BufferedIOBase):
     w_reader = None
     w_writer = None
 
-    @unwrap_spec(buffer_size=int, max_buffer_size=int)
+    @unwrap_spec(buffer_size=int)
     def descr_init(self, space, w_reader, w_writer,
-                   buffer_size=DEFAULT_BUFFER_SIZE, max_buffer_size=-234):
-        if max_buffer_size != -234:
-            self._deprecated_max_buffer_size(space)
-
+                   buffer_size=DEFAULT_BUFFER_SIZE):
         try:
             self.w_reader = W_BufferedReader(space)
             self.w_reader.descr_init(space, w_reader, buffer_size)
@@ -946,16 +1047,6 @@ class W_BufferedRWPair(W_BufferedIOBase):
             self.w_writer = None
             raise
 
-    def _finalize_(self):
-        # Don't call the base __del__: do not close the files!
-        # Usually the _finalize_() method is not called at all because
-        # we set 'needs_to_finalize = False' in this class, so
-        # W_IOBase.__init__() won't call register_finalizer().
-        # However, this method might still be called: if the user
-        # makes an app-level subclass and adds a custom __del__.
-        pass
-    needs_to_finalize = False
-
     # forward to reader
     for method in ['read', 'peek', 'read1', 'readinto', 'readable']:
         locals()[method + '_w'] = make_forwarding_method(
@@ -983,11 +1074,20 @@ class W_BufferedRWPair(W_BufferedIOBase):
             raise oefmt(space.w_ValueError,
                         "I/O operation on uninitialized object")
         w_meth = space.getattr(self.w_reader, space.newtext("close"))
-        space.call_args(w_meth, __args__)
+        try:
+            space.call_args(w_meth, __args__)
+        except OperationError as e2:
+            if e:
+                e2.chain_exceptions(space, e)
+            e = e2
 
         if e:
             raise e
 
+    def needs_finalizer(self):
+        # self.w_writer and self.w_reader have their own finalizer
+        return type(self) is not W_BufferedRWPair
+
     def isatty_w(self, space):
         if space.is_true(space.call_method(self.w_writer, "isatty")):
             return space.w_True
@@ -1006,17 +1106,14 @@ W_BufferedRWPair.typedef = TypeDef(
     '_io.BufferedRWPair', W_BufferedIOBase.typedef,
     __new__ = generic_new_descr(W_BufferedRWPair),
     __init__  = interp2app(W_BufferedRWPair.descr_init),
+    __getstate__ = interp2app(W_BufferedRWPair.getstate_w),
     closed = GetSetProperty(W_BufferedRWPair.closed_get_w),
     **methods
 )
 
-class W_BufferedRandom(BufferedMixin, W_BufferedIOBase):
-    @unwrap_spec(buffer_size=int, max_buffer_size=int)
-    def descr_init(self, space, w_raw, buffer_size=DEFAULT_BUFFER_SIZE,
-                   max_buffer_size = -234):
-        if max_buffer_size != -234:
-            self._deprecated_max_buffer_size(space)
-
+class W_BufferedRandom(BufferedReaderMixin, W_BufferedIOBase):
+    @unwrap_spec(buffer_size=int)
+    def descr_init(self, space, w_raw, buffer_size=DEFAULT_BUFFER_SIZE):
         self.state = STATE_ZERO
         check_readable_w(space, w_raw)
         check_writable_w(space, w_raw)
@@ -1036,11 +1133,14 @@ W_BufferedRandom.typedef = TypeDef(
     '_io.BufferedRandom', W_BufferedIOBase.typedef,
     __new__ = generic_new_descr(W_BufferedRandom),
     __init__ = interp2app(W_BufferedRandom.descr_init),
+    __getstate__ = interp2app(W_BufferedRandom.getstate_w),
 
     read = interp2app(W_BufferedRandom.read_w),
     peek = interp2app(W_BufferedRandom.peek_w),
     read1 = interp2app(W_BufferedRandom.read1_w),
     readline = interp2app(W_BufferedRandom.readline_w),
+    readinto = interp2app(W_BufferedRandom.readinto_w),
+    readinto1 = interp2app(W_BufferedRandom.readinto1_w),
 
     write = interp2app(W_BufferedRandom.write_w),
     flush = interp2app(W_BufferedRandom.flush_w),
@@ -1058,6 +1158,7 @@ W_BufferedRandom.typedef = TypeDef(
     truncate = interp2app(W_BufferedRandom.truncate_w),
     fileno = interp2app(W_BufferedRandom.fileno_w),
     isatty = interp2app(W_BufferedRandom.isatty_w),
+    _dealloc_warn = interp2app(W_BufferedRandom._dealloc_warn_w),
     closed = GetSetProperty(W_BufferedRandom.closed_get_w),
     name = GetSetProperty(W_BufferedRandom.name_get_w),
     mode = GetSetProperty(W_BufferedRandom.mode_get_w),
