@@ -1,6 +1,6 @@
 import sys
 
-from pypy.interpreter.baseobjspace import W_Root
+from pypy.interpreter.baseobjspace import W_Root, BufferInterfaceNotFound
 from pypy.interpreter.error import OperationError, oefmt
 from pypy.interpreter.gateway import WrappedDefault, interp2app, unwrap_spec
 from pypy.interpreter.typedef import (
@@ -12,7 +12,7 @@ from rpython.rlib.rarithmetic import intmask, r_uint, r_ulonglong
 from rpython.rlib.rbigint import rbigint
 from rpython.rlib.rstring import StringBuilder
 from rpython.rlib.rutf8 import (check_utf8, next_codepoint_pos,
-                                codepoints_in_utf8, get_utf8_length,
+                                codepoints_in_utf8, codepoints_in_utf8,
                                 Utf8StringBuilder)
 
 
@@ -105,16 +105,10 @@ class W_IncrementalNewlineDecoder(W_Root):
         # desired, all in one pass.
         seennl = self.seennl
 
-        # If, up to now, newlines are consistently \n, do a quick check
-        # for the \r
-        only_lf = False
-        if seennl == SEEN_LF or seennl == 0:
-            only_lf = (output.find('\r') < 0)
-
-        if only_lf:
-            # If not already seen, quick scan for a possible "\n" character.
+        if output.find('\r') < 0:
+            # If no \r, quick scan for a possible "\n" character.
             # (there's nothing else to be done, even when in translation mode)
-            if seennl == 0 and output.find('\n') >= 0:
+            if output.find('\n') >= 0:
                 seennl |= SEEN_LF
                 # Finished: we have scanned for newlines, and none of them
                 # need translating.
@@ -237,14 +231,29 @@ W_TextIOBase.typedef = TypeDef(
 )
 
 
-def _determine_encoding(space, encoding):
+def _determine_encoding(space, encoding, w_buffer):
     if encoding is not None:
         return space.newtext(encoding)
 
+    # Try os.device_encoding(fileno)
+    try:
+        w_fileno = space.call_method(w_buffer, 'fileno')
+    except OperationError as e:
+        from pypy.module._io.interp_io import Cache
+        if not (e.match(space, space.w_AttributeError) or
+                e.match(space, space.fromcache(Cache).w_unsupportedoperation)):
+            raise
+    else:
+        w_os = space.call_method(space.builtin, '__import__', space.newtext('os'))
+        w_encoding = space.call_method(w_os, 'device_encoding', w_fileno)
+        if space.isinstance_w(w_encoding, space.w_unicode):
+            return w_encoding
+
     try:
         w_locale = space.call_method(space.builtin, '__import__',
-                                     space.newtext('locale'))
-        w_encoding = space.call_method(w_locale, 'getpreferredencoding')
+                                     space.newtext('_bootlocale'))
+        w_encoding = space.call_method(w_locale, 'getpreferredencoding',
+                                       space.w_False)
     except OperationError as e:
         # getpreferredencoding() may also raise ImportError
         if not e.match(space, space.w_ImportError):
@@ -299,13 +308,18 @@ class PositionSnapshot:
 
 
 class DecodeBuffer(object):
-    def __init__(self, text=None):
+    def __init__(self, text=None, ulen=-1):
+        # self.text is a valid utf-8 string
+        if text is not None:
+            assert ulen >= 0
         self.text = text
         self.pos = 0
         self.upos = 0
+        self.ulen = ulen
 
     def set(self, space, w_decoded):
         check_decoded(space, w_decoded)
+        self.ulen = space.len_w(w_decoded)
         self.text = space.utf8_w(w_decoded)
         self.pos = 0
         self.upos = 0
@@ -314,12 +328,14 @@ class DecodeBuffer(object):
         self.text = None
         self.pos = 0
         self.upos = 0
+        self.ulen = -1
 
     def get_chars(self, size):
+        """ returns a tuple (utf8, lgt) """
         if self.text is None or size == 0:
-            return ""
+            return "", 0
 
-        lgt = codepoints_in_utf8(self.text)
+        lgt = self.ulen
         available = lgt - self.upos
         if size < 0 or size > available:
             size = available
@@ -327,9 +343,8 @@ class DecodeBuffer(object):
 
         if self.pos > 0 or size < available:
             start = self.pos
-            ret = []
             pos = start
-            for  i in range(size):
+            for i in range(size):
                 pos = next_codepoint_pos(self.text, pos)
                 self.upos += 1
             assert start >= 0
@@ -340,8 +355,9 @@ class DecodeBuffer(object):
             chars = self.text
             self.pos = len(self.text)
             self.upos = lgt
+            size = lgt
 
-        return chars
+        return chars, size
 
     def has_data(self):
         return (self.text is not None and not self.exhausted())
@@ -378,22 +394,22 @@ class DecodeBuffer(object):
             limit = sys.maxint
         scanned = 0
         while scanned < limit:
-            try:
-                ch = self.next_char()
-                scanned += 1
-            except StopIteration:
+            if self.exhausted():
                 return False
+            ch = self.text[self.pos]
+            self._advance_codepoint()
+            scanned += 1
             if ch == '\n':
                 return True
             if ch == '\r':
                 if scanned >= limit:
                     return False
-                try:
-                    ch = self.peek_char()
-                except StopIteration:
+                if self.exhausted():
+                    # don't split potential \r\n
                     return False
+                ch = self.text[self.pos]
                 if ch == '\n':
-                    self.next_char()
+                    self._advance_codepoint()
                     return True
                 else:
                     return True
@@ -404,39 +420,66 @@ class DecodeBuffer(object):
             limit = sys.maxint
         scanned = 0
         while scanned < limit:
-            try:
-                ch = self.next_char()
-            except StopIteration:
+            if self.exhausted():
                 return False
+            ch = self.text[self.pos]
+            self._advance_codepoint()
             scanned += 1
             if ch == '\r':
                 if scanned >= limit:
                     return False
-                try:
-                    if self.peek_char() == '\n':
-                        self.next_char()
-                        return True
-                except StopIteration:
-                    # This is the tricky case: we found a \r right at the end
+                if self.exhausted():
+                    # This is the tricky case: we found a \r right at the end,
+                    # un-consume it
                     self.pos -= 1
                     self.upos -= 1
                     return False
+                if self.text[self.pos] == '\n':
+                    self._advance_codepoint()
+                    return True
         return False
 
     def find_char(self, marker, limit):
+        # only works for ascii markers!
+        assert 0 <= ord(marker) < 128
+        # ascii fast path
+        if self.ulen == len(self.text):
+            end = len(self.text)
+            if limit >= 0:
+                end = min(end, self.pos + limit)
+            pos = self.pos
+            assert pos >= 0
+            assert end >= 0
+            pos = self.text.find(marker, pos, end)
+            if pos >= 0:
+                self.pos = self.upos = pos + 1
+                return True
+            else:
+                self.pos = self.upos = end
+                return False
+
         if limit < 0:
             limit = sys.maxint
+        # XXX it might be better to search for the marker quickly, then compute
+        # the new upos afterwards.
         scanned = 0
         while scanned < limit:
-            try:
-                ch = self.next_char()
-            except StopIteration:
+            # don't use next_char here, since that computes a slice etc
+            if self.exhausted():
                 return False
-            if ch == marker:
+            # this is never true if self.text[pos] is part of a larger char
+            found = self.text[self.pos] == marker
+            self._advance_codepoint()
+            if found:
                 return True
             scanned += 1
         return False
 
+    def _advance_codepoint(self):
+        # must only be called after checking self.exhausted()!
+        self.pos = next_codepoint_pos(self.text, self.pos)
+        self.upos += 1
+
 
 def check_decoded(space, w_decoded):
     if not space.isinstance_w(w_decoded, space.w_unicode):
@@ -466,12 +509,13 @@ class W_TextIOWrapper(W_TextIOBase):
                                               # of the stream
         self.snapshot = None
 
-    @unwrap_spec(encoding="text_or_none", line_buffering=int)
+    @unwrap_spec(encoding="text_or_none", line_buffering=int, write_through=int)
     def descr_init(self, space, w_buffer, encoding=None,
-                   w_errors=None, w_newline=None, line_buffering=0):
+                   w_errors=None, w_newline=None, line_buffering=0,
+                   write_through=0):
         self.state = STATE_ZERO
         self.w_buffer = w_buffer
-        self.w_encoding = _determine_encoding(space, encoding)
+        self.w_encoding = _determine_encoding(space, encoding, w_buffer)
 
         if space.is_none(w_errors):
             w_errors = space.newtext("strict")
@@ -486,6 +530,7 @@ class W_TextIOWrapper(W_TextIOBase):
                         "illegal newline value: %R", w_newline)
 
         self.line_buffering = line_buffering
+        self.write_through = write_through
 
         self.readuniversal = not newline # null or empty
         self.readtranslate = newline is None
@@ -501,10 +546,16 @@ class W_TextIOWrapper(W_TextIOBase):
         else:
             self.writenl = None
 
+        w_codec = interp_codecs.lookup_codec(space,
+                                             space.text_w(self.w_encoding))
+        if not space.is_true(space.getattr(w_codec,
+                                           space.newtext('_is_text_encoding'))):
+            msg = ("%R is not a text encoding; "
+                   "use codecs.open() to handle arbitrary codecs")
+            raise oefmt(space.w_LookupError, msg, self.w_encoding)
+
         # build the decoder object
         if space.is_true(space.call_method(w_buffer, "readable")):
-            w_codec = interp_codecs.lookup_codec(space,
-                                                 space.text_w(self.w_encoding))
             self.w_decoder = space.call_method(w_codec,
                                                "incrementaldecoder", w_errors)
             if self.readuniversal:
@@ -514,14 +565,14 @@ class W_TextIOWrapper(W_TextIOBase):
 
         # build the encoder object
         if space.is_true(space.call_method(w_buffer, "writable")):
-            w_codec = interp_codecs.lookup_codec(space,
-                                                 space.text_w(self.w_encoding))
             self.w_encoder = space.call_method(w_codec,
                                                "incrementalencoder", w_errors)
 
         self.seekable = space.is_true(space.call_method(w_buffer, "seekable"))
         self.telling = self.seekable
 
+        self.has_read1 = space.findattr(w_buffer, space.newtext("read1"))
+
         self.encoding_start_of_stream = False
         if self.seekable and self.w_encoder:
             self.encoding_start_of_stream = True
@@ -547,16 +598,19 @@ class W_TextIOWrapper(W_TextIOBase):
         self._check_init(space)
         W_TextIOBase._check_closed(self, space, message)
 
+    def __w_attr_repr(self, space, name):
+        w_attr = space.findattr(self, space.newtext(name))
+        if w_attr is None:
+            return space.newtext("")
+        return space.mod(space.newtext("%s=%%r " % name), w_attr)
+
     def descr_repr(self, space):
         self._check_init(space)
-        w_name = space.findattr(self, space.newtext("name"))
-        if w_name is None:
-            w_name_str = space.newtext("")
-        else:
-            w_name_str = space.mod(space.newtext("name=%r "), w_name)
-        w_args = space.newtuple([w_name_str, self.w_encoding])
+        w_args = space.newtuple([self.__w_attr_repr(space, 'name'),
+                                 self.__w_attr_repr(space, 'mode'),
+                                 self.w_encoding])
         return space.mod(
-            space.newtext("<_io.TextIOWrapper %sencoding=%r>"), w_args
+            space.newtext("<_io.TextIOWrapper %s%sencoding=%r>"), w_args
         )
 
     def readable_w(self, space):
@@ -609,13 +663,23 @@ class W_TextIOWrapper(W_TextIOBase):
 
     def close_w(self, space):
         self._check_attached(space)
-        if not space.is_true(space.getattr(self.w_buffer,
-                                           space.newtext("closed"))):
+        if space.is_true(space.getattr(self.w_buffer,
+                                       space.newtext("closed"))):
+            return
+        try:
+            space.call_method(self, "flush")
+        except OperationError as e:
             try:
-                space.call_method(self, "flush")
-            finally:
                 ret = space.call_method(self.w_buffer, "close")
-            return ret
+            except OperationError as e2:
+                e2.chain_exceptions(space, e)
+            raise
+        else:
+            ret = space.call_method(self.w_buffer, "close")
+        return ret
+
+    def _dealloc_warn_w(self, space, w_source):
+        space.call_method(self.w_buffer, "_dealloc_warn", w_source)
 
     # _____________________________________________________________
     # read methods
@@ -628,15 +692,22 @@ class W_TextIOWrapper(W_TextIOBase):
         remain buffered in the decoder, yet to be converted."""
 
         if not self.w_decoder:
-            raise oefmt(space.w_IOError, "not readable")
+            self._unsupportedoperation(space, "not readable")
 
         if self.telling:
             # To prepare for tell(), we need to snapshot a point in the file
             # where the decoder's input buffer is empty.
             w_state = space.call_method(self.w_decoder, "getstate")
+            if (not space.isinstance_w(w_state, space.w_tuple)
+                    or space.len_w(w_state) != 2):
+                raise oefmt(space.w_TypeError, "illegal decoder state")
             # Given this, we know there was a valid snapshot point
             # len(dec_buffer) bytes ago with decoder state (b'', dec_flags).
             w_dec_buffer, w_dec_flags = space.unpackiterable(w_state, 2)
+            if not space.isinstance_w(w_dec_buffer, space.w_bytes):
+                msg = ("illegal decoder state: the first value should be a "
+                    "bytes object not '%T'")
+                raise oefmt(space.w_TypeError, msg, w_dec_buffer)
             dec_buffer = space.bytes_w(w_dec_buffer)
             dec_flags = space.int_w(w_dec_flags)
         else:
@@ -644,15 +715,18 @@ class W_TextIOWrapper(W_TextIOBase):
             dec_flags = 0
 
         # Read a chunk, decode it, and put the result in self.decoded
-        w_input = space.call_method(self.w_buffer, "read1",
+        func_name = "read1" if self.has_read1 else "read"
+        w_input = space.call_method(self.w_buffer, func_name,
                                     space.newint(self.chunk_size))
 
-        if not space.isinstance_w(w_input, space.w_bytes):
-            msg = "decoder getstate() should have returned a bytes " \
-                  "object not '%T'"
-            raise oefmt(space.w_TypeError, msg, w_input)
+        try:
+            input_buf = w_input.buffer_w(space, space.BUF_SIMPLE)
+        except BufferInterfaceNotFound:
+            msg = ("underlying %s() should have returned a bytes-like "
+                   "object, not '%T'")
+            raise oefmt(space.w_TypeError, msg, func_name, w_input)
 
-        eof = space.len_w(w_input) == 0
+        eof = input_buf.getlength() == 0
         w_decoded = space.call_method(self.w_decoder, "decode",
                                       w_input, space.newbool(eof))
         self.decoded.set(space, w_decoded)
@@ -662,7 +736,7 @@ class W_TextIOWrapper(W_TextIOBase):
         if self.telling:
             # At the snapshot point, len(dec_buffer) bytes before the read,
             # the next input to be decoded is dec_buffer + input_chunk.
-            next_input = dec_buffer + space.bytes_w(w_input)
+            next_input = dec_buffer + input_buf.as_str()
             self.snapshot = PositionSnapshot(dec_flags, next_input)
 
         return not eof
@@ -694,23 +768,27 @@ class W_TextIOWrapper(W_TextIOBase):
         self._check_attached(space)
         self._check_closed(space)
         if not self.w_decoder:
-            raise oefmt(space.w_IOError, "not readable")
+            self._unsupportedoperation(space, "not readable")
 
         size = convert_size(space, w_size)
         self._writeflush(space)
 
         if size < 0:
-            # Read everything
-            w_bytes = space.call_method(self.w_buffer, "read")
-            w_decoded = space.call_method(self.w_decoder, "decode", w_bytes, space.w_True)
-            check_decoded(space, w_decoded)
-            chars = self.decoded.get_chars(-1)
-            lgt = get_utf8_length(chars)
-            w_result = space.newutf8(chars, lgt)
-            w_final = space.add(w_result, w_decoded)
-            self.snapshot = None
-            return w_final
+            return self._read_all(space)
+        else:
+            return self._read(space, size)
 
+    def _read_all(self, space):
+        w_bytes = space.call_method(self.w_buffer, "read")
+        w_decoded = space.call_method(self.w_decoder, "decode", w_bytes, space.w_True)
+        check_decoded(space, w_decoded)
+        w_result = space.newutf8(*self.decoded.get_chars(-1))
+        w_final = space.add(w_result, w_decoded)
+        self.decoded.reset()
+        self.snapshot = None
+        return w_final
+
+    def _read(self, space, size):
         remaining = size
         builder = Utf8StringBuilder(size)
 
@@ -718,9 +796,9 @@ class W_TextIOWrapper(W_TextIOBase):
         while remaining > 0:
             if not self._ensure_data(space):
                 break
-            data = self.decoded.get_chars(remaining)
-            builder.append(data)
-            remaining -= len(data)
+            data, size = self.decoded.get_chars(remaining)
+            builder.append_utf8(data, size)
+            remaining -= size
 
         return space.newutf8(builder.build(), builder.getlength())
 
@@ -743,31 +821,38 @@ class W_TextIOWrapper(W_TextIOBase):
         self._check_attached(space)
         self._check_closed(space)
         self._writeflush(space)
-
         limit = convert_size(space, w_limit)
+        text, lgt = self._readline(space, limit)
+        return space.newutf8(text, lgt)
+
+    def _readline(self, space, limit):
+        # This is a separate function so that readline_w() can be jitted.
         remnant = None
-        builder = StringBuilder()
-        # XXX maybe use Utf8StringBuilder instead?
+        remnant_ulen = -1
+        builder = Utf8StringBuilder()
         while True:
             # First, get some data if necessary
             has_data = self._ensure_data(space)
             if not has_data:
                 # end of file
                 if remnant:
-                    builder.append(remnant)
+                    builder.append_utf8(remnant, remnant_ulen)
                 break
 
             if remnant:
                 assert not self.readtranslate and self.readnl == '\r\n'
                 assert self.decoded.pos == 0
                 if remnant == '\r' and self.decoded.text[0] == '\n':
-                    builder.append('\r\n')
+                    builder.append_utf8('\r\n', 2)
                     self.decoded.pos = 1
+                    self.decoded.upos = 1
                     remnant = None
+                    remnant_ulen = -1
                     break
                 else:
-                    builder.append(remnant)
+                    builder.append_utf8(remnant, remnant_ulen)
                     remnant = None
+                    remnant_ulen = -1
                     continue
 
             if limit >= 0:
@@ -776,12 +861,13 @@ class W_TextIOWrapper(W_TextIOBase):
             else:
                 remaining = -1
             start = self.decoded.pos
+            ustart = self.decoded.upos
             assert start >= 0
             found = self._scan_line_ending(remaining)
             end_scan = self.decoded.pos
+            uend_scan = self.decoded.upos
             if end_scan > start:
-                s = self.decoded.text[start:end_scan]
-                builder.append(s)
+                builder.append_utf8_slice(self.decoded.text, start, end_scan, uend_scan - ustart)
 
             if found or (limit >= 0 and builder.getlength() >= limit):
                 break
@@ -789,13 +875,13 @@ class W_TextIOWrapper(W_TextIOBase):
             # There may be some remaining chars we'll have to prepend to the
             # next chunk of data
             if not self.decoded.exhausted():
-                remnant = self.decoded.get_chars(-1)
+                remnant, remnant_ulen = self.decoded.get_chars(-1)
             # We have consumed the buffer
             self.decoded.reset()
 
         result = builder.build()
-        lgt = get_utf8_length(result)
-        return space.newutf8(result, lgt)
+        lgt = builder.getlength()
+        return (result, lgt)
 
     # _____________________________________________________________
     # write methods
@@ -805,7 +891,7 @@ class W_TextIOWrapper(W_TextIOBase):
         self._check_closed(space)
 
         if not self.w_encoder:
-            raise oefmt(space.w_IOError, "not writable")
+            self._unsupportedoperation(space, "not writable")
 
         if not space.isinstance_w(w_text, space.w_unicode):
             raise oefmt(space.w_TypeError,
@@ -819,10 +905,13 @@ class W_TextIOWrapper(W_TextIOBase):
                 haslf = True
         if haslf and self.writetranslate and self.writenl:
             w_text = space.call_method(w_text, "replace", space.newutf8('\n', 1),
-                                       space.newutf8(self.writenl, get_utf8_length(self.writenl)))
+                               space.newutf8(self.writenl, codepoints_in_utf8(self.writenl)))
             text = space.utf8_w(w_text)
 
         needflush = False
+        text_needflush = False
+        if self.write_through:
+            text_needflush = True
         if self.line_buffering and (haslf or text.find('\r') >= 0):
             needflush = True
 
@@ -840,12 +929,14 @@ class W_TextIOWrapper(W_TextIOBase):
         self.pending_bytes.append(b)
         self.pending_bytes_count += len(b)
 
-        if self.pending_bytes_count > self.chunk_size or needflush:
+        if (self.pending_bytes_count > self.chunk_size or
+            needflush or text_needflush):
             self._writeflush(space)
 
         if needflush:
             space.call_method(self.w_buffer, "flush")
 
+        self.decoded.reset()
         self.snapshot = None
 
         if self.w_decoder:
@@ -854,9 +945,13 @@ class W_TextIOWrapper(W_TextIOBase):
         return space.newint(textlen)
 
     def _writeflush(self, space):
+        # jit inlinable fast path
         if not self.pending_bytes:
             return
 
+        self._really_flush(space)
+
+    def _really_flush(self, space):
         pending_bytes = ''.join(self.pending_bytes)
         self.pending_bytes = None
         self.pending_bytes_count = 0
@@ -896,26 +991,31 @@ class W_TextIOWrapper(W_TextIOBase):
                               space.newtuple([space.newbytes(""),
                                               space.newint(cookie.dec_flags)]))
 
-    def _encoder_setstate(self, space, cookie):
-        if cookie.start_pos == 0 and cookie.dec_flags == 0:
+    def _encoder_reset(self, space, start_of_stream):
+        if start_of_stream:
             space.call_method(self.w_encoder, "reset")
             self.encoding_start_of_stream = True
         else:
             space.call_method(self.w_encoder, "setstate", space.newint(0))
             self.encoding_start_of_stream = False
 
+    def _encoder_setstate(self, space, cookie):
+        self._encoder_reset(space,
+                            cookie.start_pos == 0 and cookie.dec_flags == 0)
+
     @unwrap_spec(whence=int)
     def seek_w(self, space, w_pos, whence=0):
         self._check_attached(space)
 
         if not self.seekable:
-            raise oefmt(space.w_IOError, "underlying stream is not seekable")
+            self._unsupportedoperation(space,
+                                       "underlying stream is not seekable")
 
         if whence == 1:
             # seek relative to current position
             if not space.eq_w(w_pos, space.newint(0)):
-                raise oefmt(space.w_IOError,
-                            "can't do nonzero cur-relative seeks")
+                self._unsupportedoperation(
+                    space, "can't do nonzero cur-relative seeks")
             # Seeking to the current position should attempt to sync the
             # underlying buffer with the current position.
             w_pos = space.call_method(self, "tell")
@@ -923,15 +1023,20 @@ class W_TextIOWrapper(W_TextIOBase):
         elif whence == 2:
             # seek relative to end of file
             if not space.eq_w(w_pos, space.newint(0)):
-                raise oefmt(space.w_IOError,
-                            "can't do nonzero end-relative seeks")
+                self._unsupportedoperation(
+                    space, "can't do nonzero end-relative seeks")
             space.call_method(self, "flush")
             self.decoded.reset()
             self.snapshot = None
             if self.w_decoder:
                 space.call_method(self.w_decoder, "reset")
-            return space.call_method(self.w_buffer, "seek",
-                                     w_pos, space.newint(whence))
+            w_res = space.call_method(self.w_buffer, "seek",
+                                      w_pos, space.newint(whence))
+            if self.w_encoder:
+                # If seek() == 0, we are at the start of stream
+                start_of_stream = space.eq_w(w_res, space.newint(0))
+                self._encoder_reset(space, start_of_stream)
+            return w_res
 
         elif whence != 0:
             raise oefmt(space.w_ValueError,
@@ -992,7 +1097,8 @@ class W_TextIOWrapper(W_TextIOBase):
     def tell_w(self, space):
         self._check_closed(space)
         if not self.seekable:
-            raise oefmt(space.w_IOError, "underlying stream is not seekable")
+            self._unsupportedoperation(space,
+                                       "underlying stream is not seekable")
         if not self.telling:
             raise oefmt(space.w_IOError,
                         "telling position disabled by next() call")
@@ -1095,8 +1201,9 @@ W_TextIOWrapper.typedef = TypeDef(
     __new__ = generic_new_descr(W_TextIOWrapper),
     __init__  = interp2app(W_TextIOWrapper.descr_init),
     __repr__ = interp2app(W_TextIOWrapper.descr_repr),
+    __next__ = interp2app(W_TextIOWrapper.next_w),
+    __getstate__ = interp2app(W_TextIOWrapper.getstate_w),
 
-    next = interp2app(W_TextIOWrapper.next_w),
     read = interp2app(W_TextIOWrapper.read_w),
     readline = interp2app(W_TextIOWrapper.readline_w),
     write = interp2app(W_TextIOWrapper.write_w),
@@ -1114,6 +1221,7 @@ W_TextIOWrapper.typedef = TypeDef(
     seekable = interp2app(W_TextIOWrapper.seekable_w),
     isatty = interp2app(W_TextIOWrapper.isatty_w),
     fileno = interp2app(W_TextIOWrapper.fileno_w),
+    _dealloc_warn = interp2app(W_TextIOWrapper._dealloc_warn_w),
     name = GetSetProperty(W_TextIOWrapper.name_get_w),
     buffer = interp_attrproperty_w("w_buffer", cls=W_TextIOWrapper),
     closed = GetSetProperty(W_TextIOWrapper.closed_get_w),
