@@ -4,7 +4,7 @@ import sys
 import time
 
 from rpython.rlib import jit, rgc, rthread
-from rpython.rlib.rarithmetic import intmask, r_uint
+from rpython.rlib.rarithmetic import r_uint
 from rpython.rtyper.lltypesystem import rffi, lltype
 from rpython.rtyper.tool import rffi_platform as platform
 from rpython.translator.tool.cbuild import ExternalCompilationInfo
@@ -13,16 +13,16 @@ from pypy.interpreter.baseobjspace import W_Root
 from pypy.interpreter.error import oefmt, wrap_oserror
 from pypy.interpreter.gateway import interp2app, unwrap_spec
 from pypy.interpreter.typedef import GetSetProperty, TypeDef
-from pypy.module._multiprocessing.interp_connection import w_handle
 
 RECURSIVE_MUTEX, SEMAPHORE = range(2)
+sys_platform = sys.platform
 
 if sys.platform == 'win32':
     from rpython.rlib import rwin32
-    from pypy.module._multiprocessing.interp_win32 import (
+    from pypy.module._multiprocessing.interp_win32_py3 import (
         _GetTickCount, handle_w)
 
-    SEM_VALUE_MAX = int(2**31-1) # max rffi.LONG
+    SEM_VALUE_MAX = sys.maxint
 
     _CreateSemaphore = rwin32.winexternal(
         'CreateSemaphoreA', [rffi.VOIDP, rffi.LONG, rffi.LONG, rwin32.LPCSTR],
@@ -33,8 +33,10 @@ if sys.platform == 'win32':
     _ReleaseSemaphore = rwin32.winexternal(
         'ReleaseSemaphore', [rwin32.HANDLE, rffi.LONG, rffi.LONGP],
         rwin32.BOOL,
-        save_err=rffi.RFFI_SAVE_LASTERROR, releasegil=False)
+        save_err=rffi.RFFI_SAVE_LASTERROR, releasegil=True)
 
+    def sem_unlink(name):
+        return None
 else:
     from rpython.rlib import rposix
 
@@ -57,10 +59,6 @@ else:
                                                      ('tv_usec', rffi.LONG)])
         TIMESPEC = platform.Struct('struct timespec', [('tv_sec', rffi.TIME_T),
                                                        ('tv_nsec', rffi.LONG)])
-        TIMEZONE = platform.Struct('struct timezone', [
-                                            ('tz_minuteswest', rffi.INT_real),
-                                            ('tz_dsttime', rffi.INT_real),
-                                  ])
         SEM_FAILED = platform.ConstantInteger('SEM_FAILED')
         SEM_VALUE_MAX = platform.DefinedConstantInteger('SEM_VALUE_MAX')
         SEM_TIMED_WAIT = platform.Has('sem_timedwait')
@@ -69,10 +67,8 @@ else:
     config = platform.configure(CConfig)
     TIMEVAL        = config['TIMEVAL']
     TIMESPEC       = config['TIMESPEC']
-    TIMEZONE       = config['TIMEZONE']
     TIMEVALP       = rffi.CArrayPtr(TIMEVAL)
     TIMESPECP      = rffi.CArrayPtr(TIMESPEC)
-    TIMEZONEP      = rffi.CArrayPtr(TIMEZONE)
     SEM_T          = rffi.COpaquePtr('sem_t', compilation_info=eci)
     #                rffi.cast(SEM_T, config['SEM_FAILED'])
     SEM_FAILED     = config['SEM_FAILED']
@@ -106,7 +102,7 @@ else:
                             save_err=rffi.RFFI_SAVE_ERRNO)
     _sem_post = external('sem_post', [SEM_T], rffi.INT, releasegil=False,
                          save_err=rffi.RFFI_SAVE_ERRNO)
-    _sem_getvalue = external('sem_getvalue', [SEM_T, rffi.INT_realP], rffi.INT,
+    _sem_getvalue = external('sem_getvalue', [SEM_T, rffi.INTP], rffi.INT,
                              save_err=rffi.RFFI_SAVE_ERRNO)
 
     _gettimeofday = external('gettimeofday', [TIMEVALP, rffi.VOIDP], rffi.INT,
@@ -197,7 +193,7 @@ else:
             raise OSError(rposix.get_saved_errno(), "sem_post failed")
 
     def sem_getvalue(sem):
-        sval_ptr = lltype.malloc(rffi.INT_realP.TO, 1, flavor='raw')
+        sval_ptr = lltype.malloc(rffi.INTP.TO, 1, flavor='raw')
         try:
             res = _sem_getvalue(sem, sval_ptr)
             if res < 0:
@@ -208,9 +204,8 @@ else:
 
     def gettimeofday():
         now = lltype.malloc(TIMEVALP.TO, 1, flavor='raw')
-        void = lltype.nullptr(rffi.VOIDP.TO)
         try:
-            res = _gettimeofday(now, void)
+            res = _gettimeofday(now, None)
             if res < 0:
                 raise OSError(rposix.get_saved_errno(), "gettimeofday failed")
             return (rffi.getintfield(now[0], 'c_tv_sec'),
@@ -221,6 +216,14 @@ else:
     def handle_w(space, w_handle):
         return rffi.cast(SEM_T, space.int_w(w_handle))
 
+# utilized by POSIX and win32
+def semaphore_unlink(space, w_name):
+    name = space.text_w(w_name)
+    try:
+        sem_unlink(name)
+    except OSError as e:
+        raise wrap_oserror(space, e)
+
 class CounterState:
     def __init__(self, space):
         self.counter = 0
@@ -323,7 +326,7 @@ if sys.platform == 'win32':
         try:
             if not _ReleaseSemaphore(self.handle, 1, previous_ptr):
                 raise rwin32.lastSavedWindowsError("ReleaseSemaphore")
-            return intmask(previous_ptr[0]) + 1
+            return previous_ptr[0] + 1
         finally:
             lltype.free(previous_ptr, flavor='raw')
 
@@ -333,12 +336,12 @@ if sys.platform == 'win32':
 else:
     def create_semaphore(space, name, val, max):
         sem = sem_open(name, os.O_CREAT | os.O_EXCL, 0600, val)
-        try:
-            sem_unlink(name)
-        except OSError:
-            pass
-        else:
-            rgc.add_memory_pressure(SEM_T_SIZE)
+        rgc.add_memory_pressure(SEM_T_SIZE)
+        return sem
+
+    def reopen_semaphore(name):
+        sem = sem_open(name, 0, 0600, 0)
+        rgc.add_memory_pressure(SEM_T_SIZE)
         return sem
 
     def delete_semaphore(handle):
@@ -446,20 +449,29 @@ else:
 
 
 class W_SemLock(W_Root):
-    def __init__(self, space, handle, kind, maxvalue):
+    def __init__(self, space, handle, kind, maxvalue, name):
         self.handle = handle
         self.kind = kind
         self.count = 0
         self.maxvalue = maxvalue
         self.register_finalizer(space)
         self.last_tid = -1
+        self.name = name
+
+    def name_get(self, space):
+        if self.name is None:
+            return space.w_None
+        return space.newtext(self.name)
 
     def kind_get(self, space):
         return space.newint(self.kind)
+
     def maxvalue_get(self, space):
         return space.newint(self.maxvalue)
+
     def handle_get(self, space):
-        return w_handle(space, self.handle)
+        h = rffi.cast(rffi.INTPTR_T, self.handle)
+        return space.newint(h)
 
     def get_count(self, space):
         return space.newint(self.count)
@@ -525,10 +537,20 @@ class W_SemLock(W_Root):
     def after_fork(self):
         self.count = 0
 
-    @unwrap_spec(kind=int, maxvalue=int)
-    def rebuild(space, w_cls, w_handle, kind, maxvalue):
+    @unwrap_spec(kind=int, maxvalue=int, name='text_or_none')
+    def rebuild(space, w_cls, w_handle, kind, maxvalue, name):
+        #
+        if sys_platform != 'win32' and name is not None:
+            # like CPython, in this case ignore 'w_handle'
+            try:
+                handle = reopen_semaphore(name)
+            except OSError as e:
+                raise wrap_oserror(space, e)
+        else:
+            handle = handle_w(space, w_handle)
+        #
         self = space.allocate_instance(W_SemLock, w_cls)
-        self.__init__(space, handle_w(space, w_handle), kind, maxvalue)
+        self.__init__(space, handle, kind, maxvalue, name)
         return self
 
     def enter(self, space):
@@ -540,21 +562,24 @@ class W_SemLock(W_Root):
     def _finalize_(self):
         delete_semaphore(self.handle)
 
-@unwrap_spec(kind=int, value=int, maxvalue=int)
-def descr_new(space, w_subtype, kind, value, maxvalue):
+@unwrap_spec(kind=int, value=int, maxvalue=int, name='text', unlink=int)
+def descr_new(space, w_subtype, kind, value, maxvalue, name, unlink):
     if kind != RECURSIVE_MUTEX and kind != SEMAPHORE:
         raise oefmt(space.w_ValueError, "unrecognized kind")
 
     counter = space.fromcache(CounterState).getCount()
-    name = "/mp%d-%d" % (os.getpid(), counter)
 
     try:
         handle = create_semaphore(space, name, value, maxvalue)
+        if unlink:
+            sem_unlink(name)
+            name = None
     except OSError as e:
         raise wrap_oserror(space, e)
 
+
     self = space.allocate_instance(W_SemLock, w_subtype)
-    self.__init__(space, handle, kind, maxvalue)
+    self.__init__(space, handle, kind, maxvalue, name)
 
     return self
 
@@ -564,6 +589,7 @@ W_SemLock.typedef = TypeDef(
     kind = GetSetProperty(W_SemLock.kind_get),
     maxvalue = GetSetProperty(W_SemLock.maxvalue_get),
     handle = GetSetProperty(W_SemLock.handle_get),
+    name = GetSetProperty(W_SemLock.name_get),
     _count = interp2app(W_SemLock.get_count),
     _is_mine = interp2app(W_SemLock.is_mine),
     _is_zero = interp2app(W_SemLock.is_zero),
