@@ -1,10 +1,10 @@
 import sys
 from rpython.rlib.rstring import StringBuilder
 from rpython.rlib.objectmodel import specialize, always_inline
-from rpython.rlib import rfloat, jit, objectmodel, rutf8
+from rpython.rlib import rfloat, runicode, jit, objectmodel, rutf8
 from rpython.rtyper.lltypesystem import lltype, rffi
+from pypy.interpreter.error import oefmt, OperationError
 from rpython.rlib.rarithmetic import r_uint
-from pypy.interpreter.error import oefmt
 from pypy.interpreter import unicodehelper
 from pypy.interpreter.baseobjspace import W_Root
 from pypy.module._pypyjson import simd
@@ -47,6 +47,10 @@ class IntCache(object):
             return self.cache[intval - self.START]
         return self.space.newint(intval)
 
+class DecoderError(Exception):
+    def __init__(self, msg, pos):
+        self.msg = msg
+        self.pos = pos
 
 class JSONDecoder(W_Root):
 
@@ -165,13 +169,11 @@ class JSONDecoder(W_Root):
         elif ch.isdigit():
             return self.decode_numeric(i)
         else:
-            self._raise("No JSON object could be decoded: unexpected '%s' at char %d",
-                        ch, i)
+            raise DecoderError("Unexpected '%s'" % ch, i)
 
 
-    @specialize.arg(1)
-    def _raise(self, msg, *args):
-        raise oefmt(self.space.w_ValueError, msg, *args)
+    def _raise(self, msg, pos):
+        raise DecoderError(msg, pos)
 
     def decode_null(self, i):
         if (self.ll_chars[i]   == 'u' and
@@ -179,7 +181,7 @@ class JSONDecoder(W_Root):
             self.ll_chars[i+2] == 'l'):
             self.pos = i+3
             return self.space.w_None
-        self._raise("Error when decoding null at char %d", i)
+        raise DecoderError("Error when decoding null", i)
 
     def decode_true(self, i):
         if (self.ll_chars[i]   == 'r' and
@@ -187,7 +189,7 @@ class JSONDecoder(W_Root):
             self.ll_chars[i+2] == 'e'):
             self.pos = i+3
             return self.space.w_True
-        self._raise("Error when decoding true at char %d", i)
+        raise DecoderError("Error when decoding true", i)
 
     def decode_false(self, i):
         if (self.ll_chars[i]   == 'a' and
@@ -196,7 +198,7 @@ class JSONDecoder(W_Root):
             self.ll_chars[i+3] == 'e'):
             self.pos = i+4
             return self.space.w_False
-        self._raise("Error when decoding false at char %d", i)
+        raise DecoderError("Error when decoding false", i)
 
     def decode_infinity(self, i, sign=1):
         if (self.ll_chars[i]   == 'n' and
@@ -208,14 +210,14 @@ class JSONDecoder(W_Root):
             self.ll_chars[i+6] == 'y'):
             self.pos = i+7
             return self.space.newfloat(rfloat.INFINITY * sign)
-        self._raise("Error when decoding Infinity at char %d", i)
+        raise DecoderError("Error when decoding Infinity", i)
 
     def decode_nan(self, i):
         if (self.ll_chars[i]   == 'a' and
             self.ll_chars[i+1] == 'N'):
             self.pos = i+2
             return self.space.newfloat(rfloat.NAN)
-        self._raise("Error when decoding NaN at char %d", i)
+        raise DecoderError("Error when decoding NaN", i)
 
     def decode_numeric(self, i):
         start = i
@@ -225,7 +227,7 @@ class JSONDecoder(W_Root):
         ch = self.ll_chars[i]
         if ch == '.':
             if not self.ll_chars[i+1].isdigit():
-                self._raise("Expected digit at char %d", i+1)
+                raise DecoderError("Expected digit", i+1)
             return self.decode_float(start)
         elif ch == 'e' or ch == 'E':
             return self.decode_float(start)
@@ -238,8 +240,8 @@ class JSONDecoder(W_Root):
     def decode_float(self, i):
         from rpython.rlib import rdtoa
         start = rffi.ptradd(self.ll_chars, i)
-        floatval = rdtoa.dg_strtod(rffi.cast(rffi.CONST_CCHARP, start), self.end_ptr)
-        diff = rffi.cast(rffi.SIGNED, self.end_ptr[0]) - rffi.cast(rffi.SIGNED, start)
+        floatval = rdtoa.dg_strtod(start, self.end_ptr)
+        diff = rffi.cast(rffi.LONG, self.end_ptr[0]) - rffi.cast(rffi.LONG, start)
         self.pos = i + diff
         return self.space.newfloat(floatval)
 
@@ -279,7 +281,7 @@ class JSONDecoder(W_Root):
                 break
         count = i - start
         if count == 0:
-            self._raise("Expected digit at char %d", i)
+            raise DecoderError("Expected digit", i)
         # if the number has more digits than OVF_DIGITS, it might have
         # overflowed
         ovf_maybe = (count >= OVF_DIGITS)
@@ -287,17 +289,16 @@ class JSONDecoder(W_Root):
 
     def _raise_control_char_in_string(self, ch, startindex, currindex):
         if ch == '\0':
-            self._raise("Unterminated string starting at char %d",
+            self._raise("Unterminated string starting at",
                         startindex - 1)
         else:
-            self._raise("Invalid control character at char %d", currindex-1)
+            self._raise("Invalid control character at", currindex-1)
 
     def _raise_object_error(self, ch, start, i):
         if ch == '\0':
-            self._raise("Unterminated object starting at char %d", start)
+            self._raise("Unterminated object starting at", start)
         else:
-            self._raise("Unexpected '%s' when decoding object (char %d)",
-                        ch, i)
+            self._raise("Unexpected '%s' when decoding object" % ch, i)
 
     def decode_array(self, i):
         """ Decode a list. i must be after the opening '[' """
@@ -321,10 +322,10 @@ class JSONDecoder(W_Root):
             elif ch == ',':
                 pass
             elif ch == '\0':
-                self._raise("Unterminated array starting at char %d", start)
+                raise DecoderError("Unterminated array starting at", start)
             else:
-                self._raise("Unexpected '%s' when decoding array (char %d)",
-                            ch, i-1)
+                raise DecoderError("Unexpected '%s' when decoding array" % ch,
+                                   i-1)
 
     def decode_object(self, i):
         start = i
@@ -342,18 +343,11 @@ class JSONDecoder(W_Root):
         currmap = self.startmap
         while True:
             # parse a key: value
-            newmap = self.decode_key_map(i, currmap)
-            if newmap is None:
-                # We've seen a repeated value, switch to dict-based storage.
-                dict_w = self._switch_to_dict(currmap, values_w, nextindex)
-                # We re-parse the last key, to get the correct overwriting
-                # effect. Pointless to care for performance here.
-                return self.decode_object_dict(i, start, dict_w)
-            currmap = newmap
+            currmap = self.decode_key_map(i, currmap)
             i = self.skip_whitespace(self.pos)
             ch = self.ll_chars[i]
             if ch != ':':
-                self._raise("No ':' found at char %d", i)
+                raise DecoderError("No ':' found at", i)
             i += 1
 
             w_value = self.decode_any(i, currmap)
@@ -410,7 +404,7 @@ class JSONDecoder(W_Root):
             i = self.skip_whitespace(self.pos)
             ch = self.ll_chars[i]
             if ch != ':':
-                self._raise("No ':' found at char %d", i)
+                self._raise("No ':' found at", i)
             i += 1
 
             w_value = self.decode_any(i)
@@ -498,10 +492,7 @@ class JSONDecoder(W_Root):
             # may be a surrogate pair
             return self.decode_escape_sequence_unicode(i, stringbuilder)
         else:
-            if ch <= ' ':
-                self._raise("Invalid \\escape: (char %d)", i-2)
-            else:
-                self._raise("Invalid \\escape: %s (char %d)", ch, i-2)
+            raise DecoderError("Invalid \\escape: %s" % ch, i-1)
         return i
 
     def _get_int_val_from_hex4(self, i):
@@ -535,7 +526,7 @@ class JSONDecoder(W_Root):
                                      (lowsurr - 0xdc00))
                     i += 6
         except ValueError:
-            self._raise("Invalid \uXXXX escape (char %d)", i-1)
+            raise DecoderError("Invalid \uXXXX escape (char %d)", i-1)
             return # help the annotator to know that we'll never go beyond
                    # this point
         #
@@ -620,8 +611,6 @@ class JSONDecoder(W_Root):
         """ Given the current map currmap of an object, decode the next key at
         position i. This returns the new map of the object. """
         newmap = self._decode_key_map(i, currmap)
-        if newmap is None:
-            return None
         currmap.observe_transition(newmap, self.startmap)
         return newmap
 
@@ -635,7 +624,7 @@ class JSONDecoder(W_Root):
         start = i
         ch = ll_chars[i]
         if ch != '"':
-            self._raise("Key name must be string at char %d", i)
+            raise DecoderError("Key name must be string at char", i)
         i += 1
         w_key = self._decode_key_string(i)
         return currmap.get_next(w_key, self.s, start, self.pos, self.startmap)
@@ -801,11 +790,6 @@ class MapBase(object):
             self.nextmap_first._check_invariants()
 
     def get_next(self, w_key, string, start, stop, terminator):
-        """ Returns the next map, given a wrapped key w_key, the json input
-        string with positions start and stop, as well as a terminator.
-
-        Returns None if the key already appears somewhere in the map chain.
-        """
         from pypy.objspace.std.dictmultiobject import unicode_hash, unicode_eq
         if isinstance(self, JSONMap):
             assert not self.state == MapBase.BLOCKED
@@ -820,8 +804,6 @@ class MapBase(object):
         if nextmap_first is None:
             # first transition ever seen, don't initialize nextmap_all
             next = self._make_next_map(w_key, string[start:stop])
-            if next is None:
-                return None
             self.nextmap_first = next
         else:
             if self.nextmap_all is None:
@@ -836,8 +818,6 @@ class MapBase(object):
             # if we are at this point we didn't find the transition yet, so
             # create a new one
             next = self._make_next_map(w_key, string[start:stop])
-            if next is None:
-                return None
             self.nextmap_all[w_key] = next
 
             # one new leaf has been created
@@ -880,14 +860,6 @@ class MapBase(object):
                 self.mark_useful(terminator)
 
     def _make_next_map(self, w_key, key_repr):
-        # Check whether w_key is already part of the self.prev chain
-        # to prevent strangeness in the json dict implementation.
-        # This is slow, but it should be rare to call this function.
-        check = self
-        while isinstance(check, JSONMap):
-            if check.w_key._utf8 == w_key._utf8:
-                return None
-            check = check.prev
         return JSONMap(self.space, self, w_key, key_repr)
 
     def fill_dict(self, dict_w, values_w):
@@ -1176,22 +1148,22 @@ class JSONMap(MapBase):
             res += ", fillcolor=lightslategray"
         return res
 
-@jit.dont_look_inside
-def loads(space, w_s):
-    if space.isinstance_w(w_s, space.w_unicode):
-        raise oefmt(space.w_TypeError,
-                    "Expected utf8-encoded str, got unicode")
-    s = space.bytes_w(w_s)
+def loads(space, w_s, w_errorcls=None):
+    s = space.text_w(w_s)
     decoder = JSONDecoder(space, s)
     try:
         w_res = decoder.decode_any(0)
         i = decoder.skip_whitespace(decoder.pos)
         if i < len(s):
             start = i
-            end = len(s) - 1
-            raise oefmt(space.w_ValueError,
-                        "Extra data: char %d - %d", start, end)
+            raise DecoderError('Extra data', start)
         return w_res
+    except DecoderError as e:
+        if w_errorcls is None:
+            w_errorcls = space.w_ValueError
+        w_e = space.call_function(w_errorcls, space.newtext(e.msg), w_s,
+                                  space.newint(e.pos))
+        raise OperationError(w_errorcls, w_e)
     finally:
         decoder.close()
 
