@@ -9,11 +9,12 @@ from pypy.interpreter.unicodehelper import wcharpsize2utf8
 from pypy.interpreter.unicodehelper import wrap_unicode_out_of_range_error
 
 from rpython.rlib.clibffi import *
+from rpython.rlib.objectmodel import we_are_translated
 from rpython.rtyper.lltypesystem import lltype, rffi
 from rpython.rtyper.tool import rffi_platform
 from rpython.rlib.unroll import unrolling_iterable
 from rpython.rlib import rutf8
-from rpython.rlib.objectmodel import specialize, dict_to_switch
+from rpython.rlib.objectmodel import specialize
 import rpython.rlib.rposix as rposix
 
 _MS_WINDOWS = os.name == "nt"
@@ -63,8 +64,6 @@ if _MS_WINDOWS:
     TYPEMAP_PTR_LETTERS += 'X'
     TYPEMAP_NUMBER_LETTERS += 'v'
 
-TYPEMAP_FUNCTION = dict_to_switch(TYPEMAP)
-
 def size_alignment(ffi_type):
     return intmask(ffi_type.c_size), intmask(ffi_type.c_alignment)
 
@@ -151,9 +150,14 @@ def got_libffi_error(space):
     raise oefmt(space.w_SystemError, "not supported by libffi")
 
 def wrap_dlopenerror(space, e, filename):
-    return oefmt(space.w_OSError,
-                 "Cannot load library %s: %s",
-                 filename, e.msg if e.msg else "unspecified error")
+    if e.msg:
+        # dlerror can return garbage messages under ll2ctypes (not
+        # we_are_translated()), so repr it to avoid potential problems
+        # converting to unicode later
+        msg = e.msg if we_are_translated() else repr(e.msg)
+    else:
+        msg = 'unspecified error'
+    return oefmt(space.w_OSError, 'Cannot load library %s: %8', filename, msg)
 
 
 class W_CDLL(W_Root):
@@ -163,8 +167,8 @@ class W_CDLL(W_Root):
         self.w_cache = space.newdict()
         self.space = space
 
-    @unwrap_spec(flags=int, variadic_args=int)
-    def ptr(self, space, w_name, w_argtypes, w_restype, flags=FUNCFLAG_CDECL, variadic_args=0):
+    @unwrap_spec(flags=int)
+    def ptr(self, space, w_name, w_argtypes, w_restype, flags=FUNCFLAG_CDECL):
         """ Get a pointer for function name with provided argtypes
         and restype
         """
@@ -197,7 +201,7 @@ class W_CDLL(W_Root):
 
             try:
                 ptr = self.cdll.getrawpointer(name, ffi_argtypes, ffi_restype,
-                                              flags, variadic_args)
+                                              flags)
             except KeyError:
                 raise oefmt(space.w_AttributeError,
                             "No symbol %s found in library %s",
@@ -209,7 +213,7 @@ class W_CDLL(W_Root):
             ordinal = space.int_w(w_name)
             try:
                 ptr = self.cdll.getrawpointer_byordinal(ordinal, ffi_argtypes,
-                                                        ffi_restype, flags, variadic_args)
+                                                        ffi_restype, flags)
             except KeyError:
                 raise oefmt(space.w_AttributeError,
                             "No symbol %d found in library %s",
@@ -287,7 +291,7 @@ def read_ptr(ptr, ofs, TP):
             with lltype.scoped_alloc(T.TO, 1) as t_array:
                 rffi.c_memcpy(
                     rffi.cast(rffi.VOIDP, t_array),
-                    rffi.cast(rffi.CONST_VOIDP, ptr),
+                    rffi.cast(rffi.VOIDP, ptr),
                     rffi.sizeof(TP))
                 ptr_val = t_array[0]
                 return ptr_val
@@ -310,7 +314,7 @@ def write_ptr(ptr, ofs, value):
                 s_array[0] = value
                 rffi.c_memcpy(
                     rffi.cast(rffi.VOIDP, ptr),
-                    rffi.cast(rffi.CONST_VOIDP, s_array),
+                    rffi.cast(rffi.VOIDP, s_array),
                     rffi.sizeof(TP))
                 return
     else:
@@ -339,11 +343,13 @@ class W_DataShape(W_Root):
 
     @unwrap_spec(n=int)
     def descr_size_alignment(self, space, n=1):
-        return space.newtuple2(space.newint(self.size * n),
-                               space.newint(self.alignment))
+        return space.newtuple([space.newint(self.size * n),
+                               space.newint(self.alignment)])
 
 
 class W_DataInstance(W_Root):
+    fmt = 'B'
+    itemsize = 1
     def __init__(self, space, size, address=r_uint(0)):
         if address:
             self.ll_buffer = rffi.cast(rffi.VOIDP, address)
@@ -383,21 +389,12 @@ class W_DataInstance(W_Root):
     def buffer_w(self, space, flags):
         return SimpleView(RawFFIBuffer(self))
 
-    def readbuf_w(self, space):
-        return RawFFIBuffer(self)
-
-    def writebuf_w(self, space):
-        return RawFFIBuffer(self)
-
     def getrawsize(self):
         raise NotImplementedError("abstract base class")
 
 @specialize.arg(0)
 def unwrap_truncate_int(TP, space, w_arg):
-    if space.isinstance_w(w_arg, space.w_int):
-        return rffi.cast(TP, space.int_w(w_arg))
-    else:
-        return rffi.cast(TP, space.bigint_w(w_arg).ulonglongmask())
+    return rffi.cast(TP, space.bigint_w(w_arg).ulonglongmask())
 
 
 @specialize.arg(1)
@@ -418,11 +415,14 @@ def unwrap_value(space, push_func, add_arg, argdesc, letter, w_arg):
         push_func(add_arg, argdesc, rffi.cast(rffi.LONGDOUBLE,
                                               space.float_w(w_arg)))
     elif letter == "c":
-        s = space.bytes_w(w_arg)
-        if len(s) != 1:
-            raise oefmt(space.w_TypeError,
-                        "Expected string of length one as character")
-        val = s[0]
+        if space.isinstance_w(w_arg, space.w_int):
+            val = space.byte_w(w_arg)
+        else:
+            s = space.bytes_w(w_arg)
+            if len(s) != 1:
+                raise oefmt(space.w_TypeError,
+                            "Expected bytes of length one as character")
+            val = s[0]
         push_func(add_arg, argdesc, val)
     elif letter == 'u':
         s, lgt = space.utf8_len_w(w_arg)
@@ -584,7 +584,7 @@ def _create_new_accessor(func_name, name):
             raise oefmt(space.w_ValueError, "Expecting string of length one")
         tp_letter = tp_letter[0] # fool annotator
         try:
-            return space.newint(intmask(getattr(TYPEMAP_FUNCTION(tp_letter), name)))
+            return space.newint(intmask(getattr(TYPEMAP[tp_letter], name)))
         except KeyError:
             raise oefmt(space.w_ValueError, "Unknown type specification %s",
                         tp_letter)
@@ -629,20 +629,24 @@ def charp2rawstring(space, address, maxlength=-1):
 def wcharp2rawunicode(space, address, maxlength=-1):
     if maxlength == -1:
         return wcharp2unicode(space, address)
+    elif maxlength < 0:
+        maxlength = 0
     s = wcharpsize2utf8(space, rffi.cast(rffi.CWCHARP, address), maxlength)
     return space.newutf8(s, maxlength)
 
-@unwrap_spec(address=r_uint, newcontent='bufferstr')
-def rawstring2charp(space, address, newcontent):
+@unwrap_spec(address=r_uint, newcontent='bufferstr', offset=int, size=int)
+def rawstring2charp(space, address, newcontent, offset=0, size=-1):
     from rpython.rtyper.annlowlevel import llstr
     from rpython.rtyper.lltypesystem.rstr import copy_string_to_raw
     array = rffi.cast(rffi.CCHARP, address)
-    copy_string_to_raw(llstr(newcontent), array, 0, len(newcontent))
+    if size < 0:
+        size = len(newcontent) - offset
+    copy_string_to_raw(llstr(newcontent), array, offset, size)
 
 if _MS_WINDOWS:
     @unwrap_spec(code=int)
     def FormatError(space, code):
-        return space.newtext(rwin32.FormatError(code))
+        return space.newtext(*rwin32.FormatErrorW(code))
 
     @unwrap_spec(hresult=int)
     def check_HRESULT(space, hresult):
@@ -663,7 +667,7 @@ def set_errno(space, w_errno):
 
 if sys.platform == 'win32':
     # see also
-    # issue #1944 ctypes-on-windows-getlasterror
+    # https://bitbucket.org/pypy/pypy/issue/1944/ctypes-on-windows-getlasterror
     def get_last_error(space):
         return space.newint(rwin32.GetLastError_alt_saved())
     @unwrap_spec(error=int)
@@ -671,7 +675,7 @@ if sys.platform == 'win32':
         rwin32.SetLastError_alt_saved(error)
 else:
     # always have at least a dummy version of these functions
-    # issue 1242
+    # (https://bugs.pypy.org/issue1242)
     def get_last_error(space):
         return space.newint(0)
     @unwrap_spec(error=int)
