@@ -255,24 +255,21 @@ class AppTestFfi:
         try:
             _rawffi.CDLL("xxxxx_this_name_does_not_exist_xxxxx")
         except OSError as e:
-            print e
+            print(e)
             assert str(e).startswith(
                 "Cannot load library xxxxx_this_name_does_not_exist_xxxxx: ")
         else:
             raise AssertionError("did not fail??")
 
     def test_libload_None(self):
+        if self.iswin32:
+            skip("unix specific")
         import _rawffi
         # this should return *all* loaded libs, dlopen(NULL)
-        try:
-            dll = _rawffi.CDLL(None)
-        except TypeError:
-            if not self.iswin32:
-                raise
-        else:
-            func = dll.ptr('rand', [], 'i')
-            res = func()
-            assert res[0] != 0
+        dll = _rawffi.CDLL(None)
+        func = dll.ptr('rand', [], 'i')
+        res = func()
+        assert res[0] != 0
 
     def test_libc_load(self):
         import _rawffi
@@ -303,13 +300,14 @@ class AppTestFfi:
         get_char = lib.ptr('get_char', ['P', 'H'], 'c')
         A = _rawffi.Array('c')
         B = _rawffi.Array('H')
-        dupa = A(5, 'dupa')
+        dupa = A(5, b'dupa')
         dupaptr = dupa.byptr()
         for i in range(4):
             intptr = B(1)
             intptr[0] = i
             res = get_char(dupaptr, intptr)
-            assert res[0] == 'dupa'[i]
+            char = b'dupa'[i:i+1]
+            assert res[0] == char
             intptr.free()
         dupaptr.free()
         dupa.free()
@@ -319,11 +317,11 @@ class AppTestFfi:
         import _rawffi
         A = _rawffi.Array('c')
         buf = A(10, autofree=True)
-        buf[0] = '*'
-        assert buf[1:5] == '\x00' * 4
-        buf[7:] = 'abc'
-        assert buf[9] == 'c'
-        assert buf[:8] == '*' + '\x00'*6 + 'a'
+        buf[0] = b'*'
+        assert buf[1:5] == b'\x00' * 4
+        buf[7:] = b'abc'
+        assert buf[9] == b'c'
+        assert buf[:8] == b'*' + b'\x00'*6 + b'a'
 
     def test_returning_str(self):
         import _rawffi
@@ -332,17 +330,17 @@ class AppTestFfi:
         A = _rawffi.Array('c')
         arg1 = A(1)
         arg2 = A(1)
-        arg1[0] = 'y'
-        arg2[0] = 'x'
+        arg1[0] = b'y'
+        arg2[0] = b'x'
         res = char_check(arg1, arg2)
-        assert _rawffi.charp2string(res[0]) == 'xxxxxx'
-        assert _rawffi.charp2rawstring(res[0]) == 'xxxxxx'
-        assert _rawffi.charp2rawstring(res[0], 3) == 'xxx'
-        a = A(6, 'xx\x00\x00xx')
-        assert _rawffi.charp2string(a.buffer) == 'xx'
-        assert _rawffi.charp2rawstring(a.buffer, 4) == 'xx\x00\x00'
-        arg1[0] = 'x'
-        arg2[0] = 'y'
+        assert _rawffi.charp2string(res[0]) == b'xxxxxx'
+        assert _rawffi.charp2rawstring(res[0]) == b'xxxxxx'
+        assert _rawffi.charp2rawstring(res[0], 3) == b'xxx'
+        a = A(6, b'xx\x00\x00xx')
+        assert _rawffi.charp2string(a.buffer) == b'xx'
+        assert _rawffi.charp2rawstring(a.buffer, 4) == b'xx\x00\x00'
+        arg1[0] = b'x'
+        arg2[0] = b'y'
         res = char_check(arg1, arg2)
         assert res[0] == 0
         assert _rawffi.charp2string(res[0]) is None
@@ -350,39 +348,28 @@ class AppTestFfi:
         arg2.free()
         a.free()
 
-    def test_unicode_array(self):
-        import _rawffi
-        A = _rawffi.Array('u')
-        a = A(6, u'\u1234')
-        assert a[0] == u'\u1234'
-        a[0] = u'\U00012345'
-        assert a[0] == u'\U00012345'
-        a[0] = u'\ud800'
-        assert a[0] == u'\ud800'
-        B = _rawffi.Array('i')
-        b = B.fromaddress(a.itemaddress(0), 1)
-        b[0] = 0xffffffff
-        raises(ValueError, "a[0]")
-        a.free()
-
     def test_returning_unicode(self):
         import _rawffi
         A = _rawffi.Array('u')
-        a = A(6, u'xx\x00\x00xx')
+        a = A(6, 'xx\x00\x00xx')
         for i in (-1, 6):
             res = _rawffi.wcharp2unicode(a.buffer, i)
-            assert isinstance(res, unicode)
+            assert isinstance(res, str)
             assert res == u'xx'
         a.free()
 
     def test_rawstring2charp(self):
         import _rawffi
         A = _rawffi.Array('c')
-        a = A(10, 'x'*10)
-        _rawffi.rawstring2charp(a.buffer, "foobar")
-        assert ''.join([a[i] for i in range(10)]) == "foobarxxxx"
-        _rawffi.rawstring2charp(a.buffer, buffer("baz"))
-        assert ''.join([a[i] for i in range(10)]) == "bazbarxxxx"
+        a = A(10, b'x'*10)
+        _rawffi.rawstring2charp(a.buffer, b"foobar")
+        assert b''.join([a[i] for i in range(10)]) == b"foobarxxxx"
+        _rawffi.rawstring2charp(a.buffer, memoryview(b"baz"))
+        assert b''.join([a[i] for i in range(10)]) == b"bazbarxxxx"
+        _rawffi.rawstring2charp(a.buffer, memoryview(b"ABCDEF"), 2)
+        assert b''.join([a[i] for i in range(10)]) == b"CDEFarxxxx"
+        _rawffi.rawstring2charp(a.buffer, memoryview(b"ZYXWVU"), 2, 3)
+        assert b''.join([a[i] for i in range(10)]) == b"XWVFarxxxx"
         a.free()
 
     def test_raw_callable(self):
@@ -517,13 +504,13 @@ class AppTestFfi:
         X = _rawffi.Structure([('x1', 'i'), ('x2', 'h'), ('x3', 'c'), ('next', 'P')])
         next = X()
         next.next = 0
-        next.x3 = 'x'
+        next.x3 = b'x'
         x = X()
         x.next = next
         x.x1 = 1
         x.x2 = 2
-        x.x3 = 'x'
-        assert X.fromaddress(x.next).x3 == 'x'
+        x.x3 = b'x'
+        assert X.fromaddress(x.next).x3 == b'x'
         x.free()
         next.free()
         create_double_struct = lib.ptr("create_double_struct", [], 'P')
@@ -692,7 +679,6 @@ class AppTestFfi:
         raises(ValueError, "lib.ptr('get_char', [], 'x')")
         raises(ValueError, "_rawffi.Structure(['x1', 'xx'])")
         raises(ValueError, _rawffi.Structure, [('x1', 'xx')])
-        raises(TypeError, _rawffi.Structure, [(u'\xe9', 'l')])
         raises(ValueError, "_rawffi.Array('xx')")
 
     def test_longs_ulongs(self):
@@ -778,9 +764,9 @@ class AppTestFfi:
 
     def test_raising_callback(self):
         import _rawffi, sys
-        import StringIO
+        from io import StringIO
         lib = _rawffi.CDLL(self.lib_name)
-        err = StringIO.StringIO()
+        err = StringIO()
         orig = sys.stderr
         sys.stderr = err
         try:
@@ -796,7 +782,7 @@ class AppTestFfi:
             val = err.getvalue()
             assert 'ZeroDivisionError' in val
             assert 'callback' in val
-            assert res[0] == 0L
+            assert res[0] == 0
         finally:
             sys.stderr = orig
 
@@ -816,8 +802,7 @@ class AppTestFfi:
 
     def test_sizes_and_alignments(self):
         import _rawffi
-        for k, (s, a) in self.sizes_and_alignments.iteritems():
-            print k,s,a
+        for k, (s, a) in self.sizes_and_alignments.items():
             assert _rawffi.sizeof(k) == s
             assert _rawffi.alignment(k) == a
 
@@ -907,33 +892,20 @@ class AppTestFfi:
         import _rawffi, sys
         A = _rawffi.Array('u')
         a = A(3)
-        a[0] = u'x'
-        a[1] = u'y'
-        a[2] = u'z'
-        assert a[0] == u'x'
+        a[0] = 'x'
+        a[1] = 'y'
+        a[2] = 'z'
+        assert a[0] == 'x'
         b = _rawffi.Array('c').fromaddress(a.buffer, 38)
         if sys.maxunicode > 65535:
             # UCS4 build
             if sys.byteorder == 'big':
-                assert b[0] == '\x00'
-                assert b[1] == '\x00'
-                assert b[2] == '\x00'
-                assert b[3] == 'x'
-                assert b[4] == '\x00'
-                assert b[5] == '\x00'
-                assert b[6] == '\x00'
-                assert b[7] == 'y'
+                assert b[0:8] == b'\x00\x00\x00x\x00\x00\x00y'
             else:
-                assert b[0] == 'x'
-                assert b[1] == '\x00'
-                assert b[2] == '\x00'
-                assert b[3] == '\x00'
-                assert b[4] == 'y'
+                assert b[0:5] == b'x\x00\x00\x00y'
         else:
             # UCS2 build
-            assert b[0] == 'x'
-            assert b[1] == '\x00'
-            assert b[2] == 'y'
+            assert b[0:2] == b'x\x00y'
         a.free()
 
     def test_truncate(self):
@@ -941,7 +913,7 @@ class AppTestFfi:
         a = _rawffi.Array('b')(1)
         a[0] = -5
         assert a[0] == -5
-        a[0] = 123L
+        a[0] = 123
         assert a[0] == 123
         a[0] = 0x97817182ab128111111111111171817d042
         assert a[0] == 0x42
@@ -954,7 +926,7 @@ class AppTestFfi:
         a.free()
 
         a = _rawffi.Array('B')(1)
-        a[0] = 123L
+        a[0] = 123
         assert a[0] == 123
         a[0] = 0x18329b1718b97d89b7198db817d042
         assert a[0] == 0x42
@@ -967,7 +939,7 @@ class AppTestFfi:
         a.free()
 
         a = _rawffi.Array('h')(1)
-        a[0] = 123L
+        a[0] = 123
         assert a[0] == 123
         a[0] = 0x9112cbc91bd91db19aaaaaaaaaaaaaa8170d42
         assert a[0] == 0x0d42
@@ -980,7 +952,7 @@ class AppTestFfi:
         a.free()
 
         a = _rawffi.Array('H')(1)
-        a[0] = 123L
+        a[0] = 123
         assert a[0] == 123
         a[0] = 0xeeeeeeeeeeeeeeeeeeeeeeeeeeeee817d042
         assert a[0] == 0xd042
@@ -990,7 +962,7 @@ class AppTestFfi:
 
         maxptr = (256 ** struct.calcsize("P")) - 1
         a = _rawffi.Array('P')(1)
-        a[0] = 123L
+        a[0] = 123
         assert a[0] == 123
         a[0] = 0xeeeeeeeeeeeeeeeeeeeeeeeeeeeee817d042
         assert a[0] == 0xeeeeeeeeeeeeeeeeeeeeeeeeeeeee817d042 & maxptr
@@ -1046,7 +1018,7 @@ class AppTestFfi:
         try:
             f()
         except ValueError as e:
-            assert "Procedure called with not enough arguments" in e.message
+            assert "Procedure called with not enough arguments" in str(e)
         else:
             assert 0, "Did not raise"
 
@@ -1057,7 +1029,7 @@ class AppTestFfi:
         try:
             f(arg)
         except ValueError as e:
-            assert "Procedure called with too many arguments" in e.message
+            assert "Procedure called with too many arguments" in str(e)
         else:
             assert 0, "Did not raise"
         arg.free()
@@ -1067,13 +1039,13 @@ class AppTestFfi:
         X_Y = _rawffi.Structure([('x', 'l'), ('y', 'l')])
         x_y = X_Y()
         lib = _rawffi.CDLL(self.lib_name)
-        print >> sys.stderr, "getting..."
+        print("getting...")
         sum_x_y = lib.ptr('sum_x_y', [(X_Y, 1)], 'l')
         x_y.x = 200
         x_y.y = 220
-        print >> sys.stderr, "calling..."
+        print("calling...")
         res = sum_x_y(x_y)
-        print >> sys.stderr, "done"
+        print("done")
         assert res[0] == 420
         x_y.free()
 
@@ -1148,19 +1120,19 @@ class AppTestFfi:
         import _rawffi
         S = _rawffi.Structure((40, 1))
         s = S(autofree=True)
-        b = buffer(s)
+        b = memoryview(s)
         assert len(b) == 40
-        b[4] = b'X'
+        b[4] = ord(b'X')
         b[:3] = b'ABC'
         assert b[:6] == b'ABC\x00X\x00'
 
         A = _rawffi.Array('c')
         a = A(10, autofree=True)
         a[3] = b'x'
-        b = buffer(a)
+        b = memoryview(a)
         assert len(b) == 10
-        assert b[3] == b'x'
-        b[6] = b'y'
+        assert b[3] == ord(b'x')
+        b[6] = ord(b'y')
         assert a[6] == b'y'
         b[3:5] = b'zt'
         assert a[3] == b'z'
@@ -1168,19 +1140,14 @@ class AppTestFfi:
 
         b = memoryview(a)
         assert len(b) == 10
-        assert b[3] == b'z'
-        b[3] = b'x'
-        assert b[3] == b'x'
+        assert b[3] == ord(b'z')
+        b[3] = ord(b'x')
+        assert b[3] == ord(b'x')
 
     def test_pypy_raw_address(self):
         import _rawffi
         S = _rawffi.Structure((40, 1))
         s = S(autofree=True)
-        addr = buffer(s)._pypy_raw_address()
-        assert type(addr) is int
-        assert buffer(s)._pypy_raw_address() == addr
-        assert buffer(s, 10)._pypy_raw_address() == addr + 10
-
         addr = memoryview(s)._pypy_raw_address()
         assert type(addr) is int
         assert memoryview(s)._pypy_raw_address() == addr
@@ -1242,6 +1209,17 @@ class AppTestFfi:
         assert z == 43
         arg.free()
 
+    def test_char_array_int(self):
+        import _rawffi
+        A = _rawffi.Array('c')
+        a = A(1)
+        a[0] = b'a'
+        assert a[0] == b'a'
+        # also accept int but return bytestring
+        a[0] = 100
+        assert a[0] == b'd'
+        a.free()
+
     def test_cdll_name(self):
         import _rawffi
         lib = _rawffi.CDLL(self.lib_name)
@@ -1298,8 +1276,8 @@ class AppTestAutoFree:
             oldnum = '?'
 
         A = _rawffi.Array('c')
-        a = A(6, 'xxyxx\x00', autofree=True)
-        assert _rawffi.charp2string(a.buffer) == 'xxyxx'
+        a = A(6, b'xxyxx\x00', autofree=True)
+        assert _rawffi.charp2string(a.buffer) == b'xxyxx'
         a = None
         gc.collect()
         if oldnum != '?':
