@@ -5,6 +5,7 @@ from pypy.interpreter.typedef import interp_attrproperty, interp_attrproperty_w
 from pypy.interpreter.typedef import make_weakref_descr
 from pypy.interpreter.gateway import interp2app, unwrap_spec, WrappedDefault
 from pypy.interpreter.error import OperationError, oefmt
+from rpython.rlib.objectmodel import compute_hash
 from rpython.rlib.rarithmetic import intmask
 from rpython.rlib import jit, rutf8
 from rpython.rlib.rstring import StringBuilder
@@ -13,8 +14,8 @@ from rpython.rlib.rstring import StringBuilder
 #
 # Constants and exposed functions
 
-from rpython.rlib.rsre import rsre_core, rsre_utf8
-from rpython.rlib.rsre.rsre_char import CODESIZE, MAXREPEAT, getlower, set_unicode_db
+from rpython.rlib.rsre import rsre_core, rsre_char, rsre_utf8
+from rpython.rlib.rsre.rsre_char import CODESIZE, MAXREPEAT, MAXGROUPS, getlower, set_unicode_db
 
 
 @unwrap_spec(char_ord=int, flags=int)
@@ -32,26 +33,23 @@ set_unicode_db(pypy.objspace.std.unicodeobject.unicodedb)
 # ____________________________________________________________
 #
 
-class UnicodeAsciiMatchContext(rsre_core.StrMatchContext):
-    # we make a subclass just to mark that it originates from a W_UnicodeObject
-    pass
-
 
 def slice_w(space, ctx, start, end, w_default):
     # 'start' and 'end' are byte positions
     if ctx.ZERO <= start <= end:
         if isinstance(ctx, rsre_core.BufMatchContext):
-            return space.newbytes(ctx._buffer.getslice(start, 1, end-start))
-        elif isinstance(ctx, UnicodeAsciiMatchContext):
-            s = ctx._string[start:end]
-            return space.newutf8(s, len(s))
-        elif isinstance(ctx, rsre_core.StrMatchContext):
+            length = ctx._buffer.getlength()
+            start = min(start, length)
+            end = min(end, length)
+            return space.newbytes(ctx._buffer.getslice(start, end, 1,
+                                                        end-start))
+        if isinstance(ctx, rsre_core.StrMatchContext):
             start = ctx._real_pos(start)
             end = ctx._real_pos(end)
             return space.newbytes(ctx._string[start:end])
         elif isinstance(ctx, rsre_utf8.Utf8MatchContext):
             s = ctx._utf8[start:end]
-            lgt = rutf8.codepoints_in_utf8(s)
+            lgt = rutf8.get_utf8_length(s)
             return space.newutf8(s, lgt)
         else:
             # unreachable
@@ -85,8 +83,7 @@ def allgroups_w(space, ctx, fmarks, num_groups, w_default):
 
 
 def import_re(space):
-    w_builtin = space.getbuiltinmodule('__builtin__')
-    w_import = space.getattr(w_builtin, space.newtext("__import__"))
+    w_import = space.getattr(space.builtin, space.newtext("__import__"))
     return space.call_function(w_import, space.newtext("re"))
 
 def matchcontext(space, ctx, pattern):
@@ -105,6 +102,10 @@ def searchcontext(space, ctx, pattern):
 #
 # SRE_Pattern class
 
+FLAG_NAMES = ["re.TEMPLATE", "re.IGNORECASE", "re.LOCALE", "re.MULTILINE",
+              "re.DOTALL", "re.UNICODE", "re.VERBOSE", "re.DEBUG",
+              "re.ASCII"]
+
 class W_SRE_Pattern(W_Root):
     _immutable_fields_ = ["code", "flags", "num_groups", "w_groupindex"]
 
@@ -112,15 +113,86 @@ class W_SRE_Pattern(W_Root):
         space = self.space
         raise oefmt(space.w_TypeError, "cannot copy this pattern object")
 
+    def repr_w(self):
+        space = self.space
+        u = space.utf8_w(space.repr(self.w_pattern))
+        if len(u) > 200:
+            u = u[:200]
+        flag_items = []
+        flags = self.flags
+        if self.is_known_unicode():
+            if ((flags & (rsre_char.SRE_FLAG_LOCALE |
+                          rsre_char.SRE_FLAG_UNICODE |
+                          256))     # rsre_char.SRE_FLAG_ASCII
+                    == rsre_char.SRE_FLAG_UNICODE):
+                flags &= ~rsre_char.SRE_FLAG_UNICODE
+        for i, name in enumerate(FLAG_NAMES):
+            if flags & (1 << i):
+                flags -= (1 << i)
+                flag_items.append(name)
+        if flags != 0:
+            flag_items.append('0x%x' % flags)
+        if len(flag_items) == 0:
+            usep = ''
+            uflags = ''
+        else:
+            usep = ', '
+            uflags = '|'.join(flag_items)
+        return space.newtext('re.compile(%s%s%s)' % (u, usep, uflags))
+
+    def descr_eq(self, space, w_other):
+        if not isinstance(w_other, W_SRE_Pattern):
+            return space.w_NotImplemented
+        other = w_other
+        # Compare the code and the pattern because the same pattern can
+        # produce different codes depending on the locale used to compile the
+        # pattern when the re.LOCALE flag is used. Don't compare groups,
+        # indexgroup nor groupindex: they are derivated from the pattern.
+        return space.newbool(
+            self.flags == other.flags and
+            self.code.pattern == other.code.pattern and
+            space.eq_w(self.w_pattern, other.w_pattern))
+
+    def descr_hash(self, space):
+        from rpython.rlib.rarithmetic import intmask
+        x = 0x345678
+        for c in self.code.pattern:
+            x = intmask((1000003 * x) ^ c)
+        x = intmask((1000003 * x) ^ self.flags)
+        x = intmask((1000003 * x) ^ space.hash_w(self.w_pattern))
+        return space.newint(x)
+
+    def fget_groupindex(self, space):
+        w_groupindex = self.w_groupindex
+        if space.isinstance_w(w_groupindex, space.w_dict):
+            w_groupindex = space.newdictproxy(w_groupindex)
+        return w_groupindex
+
+    def is_known_bytes(self):
+        space = self.space
+        if space.is_none(self.w_pattern):
+            return False
+        return not space.isinstance_w(self.w_pattern, space.w_unicode)
+
+    def is_known_unicode(self):
+        space = self.space
+        if space.is_none(self.w_pattern):
+            return False
+        return space.isinstance_w(self.w_pattern, space.w_unicode)
+
     def make_ctx(self, w_string, pos=0, endpos=sys.maxint):
-        """Make a StrMatchContext, BufMatchContext, UnicodeAsciiMatchContext or
-        a Utf8MatchContext for searching in the given w_string object."""
+        """Make a StrMatchContext, BufMatchContext or a Utf8MatchContext for
+        searching in the given w_string object."""
         space = self.space
         if pos < 0:
             pos = 0
         if endpos < pos:
             endpos = pos
         if space.isinstance_w(w_string, space.w_unicode):
+            if self.is_known_bytes():
+                raise oefmt(space.w_TypeError,
+                            "can't use a bytes pattern on a string-like "
+                            "object")
             w_unicode_obj = space.convert_arg_to_w_unicode(w_string)
             utf8str = w_unicode_obj._utf8
             length = w_unicode_obj._len()
@@ -129,28 +201,33 @@ class W_SRE_Pattern(W_Root):
             elif pos >= length:
                 bytepos = len(utf8str)
             else:
-                bytepos = w_unicode_obj._index_to_byte(pos)
+                index_storage = w_unicode_obj._get_index_storage()
+                bytepos = rutf8.codepoint_position_at_index(utf8str,
+                                index_storage, pos)
             if endpos >= length:
                 endbytepos = len(utf8str)
             else:
-                endbytepos = w_unicode_obj._index_to_byte(endpos)
-            if w_unicode_obj.is_ascii():
-                ctx = UnicodeAsciiMatchContext(
-                        utf8str, bytepos, endbytepos)
-            else:
-                ctx = rsre_utf8.Utf8MatchContext(
-                    utf8str, bytepos, endbytepos)
-                # we store the w_string on the ctx too, for
-                # W_SRE_Match.bytepos_to_charindex()
-                ctx.w_unicode_obj = w_unicode_obj
+                index_storage = w_unicode_obj._get_index_storage()
+                endbytepos = rutf8.codepoint_position_at_index(utf8str,
+                                index_storage, endpos)
+            ctx = rsre_utf8.Utf8MatchContext(
+                utf8str, bytepos, endbytepos, self.flags)
+            # xxx we store the w_string on the ctx too, for
+            # W_SRE_Match.bytepos_to_charindex()
+            ctx.w_unicode_obj = w_unicode_obj
             return ctx
+        elif self.is_known_unicode():
+            raise oefmt(space.w_TypeError,
+                        "can't use a string pattern on a bytes-like "
+                        "object")
         elif space.isinstance_w(w_string, space.w_bytes):
-            str = space.bytes_w(w_string)
-            if pos > len(str):
-                pos = len(str)
-            if endpos > len(str):
-                endpos = len(str)
-            return self._make_str_match_context(str, pos, endpos)
+            string = space.bytes_w(w_string)
+            length = len(string)
+            if pos > length:
+                pos = length
+            if endpos > length:
+                endpos = length
+            return rsre_core.StrMatchContext(string, pos, endpos, self.flags)
         else:
             buf = space.readbuf_w(w_string)
             size = buf.getlength()
@@ -159,23 +236,19 @@ class W_SRE_Pattern(W_Root):
                 pos = size
             if endpos > size:
                 endpos = size
-            return rsre_core.BufMatchContext(buf,
-                                             pos, endpos)
+            return rsre_core.BufMatchContext(buf, pos, endpos, self.flags)
 
     def fresh_copy(self, ctx):
         if isinstance(ctx, rsre_utf8.Utf8MatchContext):
             result = rsre_utf8.Utf8MatchContext(
-                ctx._utf8, ctx.match_start, ctx.end)
+                ctx._utf8, ctx.match_start, ctx.end, ctx.flags)
             result.w_unicode_obj = ctx.w_unicode_obj
-        elif isinstance(ctx, UnicodeAsciiMatchContext):
-            result = UnicodeAsciiMatchContext(
-                ctx._string, ctx.match_start, ctx.end)
         elif isinstance(ctx, rsre_core.StrMatchContext):
             result = self._make_str_match_context(
                 ctx._string, ctx.match_start, ctx.end)
         elif isinstance(ctx, rsre_core.BufMatchContext):
             result = rsre_core.BufMatchContext(
-                ctx._buffer, ctx.match_start, ctx.end)
+                ctx._buffer, ctx.match_start, ctx.end, ctx.flags)
         else:
             raise AssertionError("bad ctx type")
         result.match_end = ctx.match_end
@@ -184,7 +257,7 @@ class W_SRE_Pattern(W_Root):
     def _make_str_match_context(self, str, pos, endpos):
         # for tests to override
         return rsre_core.StrMatchContext(str,
-                                         pos, endpos)
+                                         pos, endpos, self.flags)
 
     def getmatch(self, ctx, found):
         if found:
@@ -197,6 +270,12 @@ class W_SRE_Pattern(W_Root):
         ctx = self.make_ctx(w_string, pos, endpos)
         return self.getmatch(ctx, matchcontext(self.space, ctx, self.code))
 
+    @unwrap_spec(pos=int, endpos=int)
+    def fullmatch_w(self, w_string, pos=0, endpos=sys.maxint):
+        ctx = self.make_ctx(w_string, pos, endpos)
+        ctx.fullmatch_only = True
+        return self.getmatch(ctx, matchcontext(self.space, ctx, self.code))
+
     @unwrap_spec(pos=int, endpos=int)
     def search_w(self, w_string, pos=0, endpos=sys.maxint):
         ctx = self.make_ctx(w_string, pos, endpos)
@@ -243,6 +322,15 @@ class W_SRE_Pattern(W_Root):
     @unwrap_spec(maxsplit=int)
     def split_w(self, w_string, maxsplit=0):
         space = self.space
+
+        if self.code.pattern[0] != rsre_core.OPCODE_INFO or self.code.pattern[3] == 0:
+            if self.code.pattern[0] == rsre_core.OPCODE_INFO and self.code.pattern[4] == 0:
+                raise oefmt(space.w_ValueError,
+                            "split() requires a non-empty pattern match.")
+            space.warn(
+                space.newtext("split() requires a non-empty pattern match."),
+                space.w_FutureWarning)
+        #
         splitlist = []
         n = 0
         ctx = self.make_ctx(w_string)
@@ -286,7 +374,16 @@ class W_SRE_Pattern(W_Root):
         # w_string are both string or both unicode objects, and if w_ptemplate
         # is a literal
         use_builder = '\x00'   # or 'S'tring or 'U'nicode/UTF8
+        is_buffer = False
         filter_as_string = None
+        if space.isinstance_w(w_string, space.w_unicode):
+            if not self.is_known_unicode():
+                raise oefmt(space.w_TypeError,
+                    "cannot use a bytes pattern on a string-like object")
+        else:
+            if self.is_known_unicode():
+                raise oefmt(space.w_TypeError,
+                    "cannot use a string pattern on a bytes-like object")
         if space.is_true(space.callable(w_ptemplate)):
             w_filter = w_ptemplate
             filter_is_callable = True
@@ -296,22 +393,29 @@ class W_SRE_Pattern(W_Root):
                 literal = '\\' not in filter_as_string
                 if space.isinstance_w(w_string, space.w_unicode) and literal:
                     use_builder = 'U'
+            elif space.isinstance_w(w_ptemplate, space.w_bytes):
+                filter_as_string = space.bytes_w(w_ptemplate)
+                literal = '\\' not in filter_as_string
+                if space.isinstance_w(w_string, space.w_bytes) and literal:
+                    use_builder = 'S'
             else:
-                try:
+                if space.isinstance_w(w_ptemplate, space.w_bytes):
                     filter_as_string = space.bytes_w(w_ptemplate)
-                except OperationError as e:
-                    if e.async(space):
-                        raise
-                    literal = False
                 else:
-                    literal = '\\' not in filter_as_string
-                    if space.isinstance_w(w_string, space.w_bytes) and literal:
-                        use_builder = 'S'
+                    filter_as_string = space.readbuf_w(w_ptemplate).as_str()
+                    is_buffer = True
+                literal = '\\' not in filter_as_string
+                if space.isinstance_w(w_string, space.w_bytes) and literal:
+                    use_builder = 'S'
             if literal:
                 w_filter = w_ptemplate
                 filter_is_callable = False
             else:
                 # not a literal; hand it over to the template compiler
+                # FIX for a CPython 3.5 bug: if w_ptemplate is a buffer
+                # (e.g. a bytearray), convert it to a byte string here.
+                if is_buffer:
+                    w_ptemplate = space.newbytes(filter_as_string)
                 w_re = import_re(space)
                 w_filter = space.call_method(w_re, '_subx',
                                              self, w_ptemplate)
@@ -390,10 +494,9 @@ class W_SRE_Pattern(W_Root):
                 assert not isinstance(ctx, rsre_utf8.Utf8MatchContext)
                 return space.newbytes(result_bytes), n
             elif use_builder == 'U':
-                assert (isinstance(ctx, UnicodeAsciiMatchContext) or
-                        isinstance(ctx, rsre_utf8.Utf8MatchContext))
+                assert isinstance(ctx, rsre_utf8.Utf8MatchContext)
                 return space.newutf8(result_bytes,
-                                     rutf8.codepoints_in_utf8(result_bytes)), n
+                                     rutf8.get_utf8_length(result_bytes)), n
             else:
                 raise AssertionError(use_builder)
         else:
@@ -421,16 +524,13 @@ def _sub_append_slice(ctx, space, use_builder, sublist_w,
         assert strbuilder is not None
         if isinstance(ctx, rsre_core.BufMatchContext):
             assert use_builder == 'S'
-            return strbuilder.append(ctx._buffer.getslice(start, 1, end-start))
-        if isinstance(ctx, UnicodeAsciiMatchContext):
-            assert use_builder == 'U'
-            return strbuilder.append_slice(ctx._string, start, end)
+            return strbuilder.append(ctx._buffer.getslice(start, end, 1, end-start))
         if isinstance(ctx, rsre_core.StrMatchContext):
             assert use_builder == 'S'
             start = ctx._real_pos(start)
             end = ctx._real_pos(end)
             return strbuilder.append_slice(ctx._string, start, end)
-        if isinstance(ctx, rsre_utf8.Utf8MatchContext):
+        elif isinstance(ctx, rsre_utf8.Utf8MatchContext):
             assert use_builder == 'U'
             return strbuilder.append_slice(ctx._utf8, start, end)
         assert 0, "unreachable"
@@ -448,13 +548,17 @@ def SRE_Pattern__new__(space, w_subtype, w_pattern, flags, w_code,
     w_srepat = space.allocate_instance(W_SRE_Pattern, w_subtype)
     srepat = space.interp_w(W_SRE_Pattern, w_srepat)
     srepat.space = space
+    # Type check
+    if not (space.is_none(w_pattern) or
+            space.isinstance_w(w_pattern, space.w_unicode)):
+        space.readbuf_w(w_pattern)
     srepat.w_pattern = w_pattern      # the original uncompiled pattern
     srepat.flags = flags
     # note: we assume that the app-level is caching SRE_Pattern objects,
     # so that we don't need to do it here.  Creating new SRE_Pattern
     # objects all the time would be bad for the JIT, which relies on the
     # identity of the CompiledPattern() object.
-    srepat.code = rsre_core.CompiledPattern(code, flags)
+    srepat.code = rsre_core.CompiledPattern(code)
     srepat.num_groups = groups
     srepat.w_groupindex = w_groupindex
     srepat.w_indexgroup = w_indexgroup
@@ -462,14 +566,18 @@ def SRE_Pattern__new__(space, w_subtype, w_pattern, flags, w_code,
 
 
 W_SRE_Pattern.typedef = TypeDef(
-    'SRE_Pattern',
+    '_sre.SRE_Pattern',
     __new__      = interp2app(SRE_Pattern__new__),
     __copy__     = interp2app(W_SRE_Pattern.cannot_copy_w),
     __deepcopy__ = interp2app(W_SRE_Pattern.cannot_copy_w),
+    __repr__     = interp2app(W_SRE_Pattern.repr_w),
     __weakref__  = make_weakref_descr(W_SRE_Pattern),
+    __eq__       = interp2app(W_SRE_Pattern.descr_eq),
+    __hash__     = interp2app(W_SRE_Pattern.descr_hash),
     findall      = interp2app(W_SRE_Pattern.findall_w),
     finditer     = interp2app(W_SRE_Pattern.finditer_w),
     match        = interp2app(W_SRE_Pattern.match_w),
+    fullmatch    = interp2app(W_SRE_Pattern.fullmatch_w),
     scanner      = interp2app(W_SRE_Pattern.finditer_w),    # reuse finditer()
     search       = interp2app(W_SRE_Pattern.search_w),
     split        = interp2app(W_SRE_Pattern.split_w),
@@ -477,7 +585,7 @@ W_SRE_Pattern.typedef = TypeDef(
     subn         = interp2app(W_SRE_Pattern.subn_w),
     flags        = interp_attrproperty('flags', W_SRE_Pattern,
         wrapfn="newint"),
-    groupindex   = interp_attrproperty_w('w_groupindex', W_SRE_Pattern),
+    groupindex   = GetSetProperty(W_SRE_Pattern.fget_groupindex),
     groups       = interp_attrproperty('num_groups', W_SRE_Pattern,
         wrapfn="newint"),
     pattern      = interp_attrproperty_w('w_pattern', W_SRE_Pattern),
@@ -496,10 +604,25 @@ class W_SRE_Match(W_Root):
         self.srepat = srepat
         self.ctx = ctx
 
+    def repr_w(self):
+        space = self.space
+        ctx = self.ctx
+        start, end = ctx.match_start, ctx.match_end
+        w_s = slice_w(space, ctx, start, end, space.w_None)
+        u = space.utf8_w(space.repr(w_s))
+        if len(u) > 50:
+            u = u[:50]
+        return space.newtext('<_sre.SRE_Match object; span=(%d, %d), match=%s>' %
+                          (start, end, u))
+
     def cannot_copy_w(self):
         space = self.space
         raise oefmt(space.w_TypeError, "cannot copy this match object")
 
+    def descr_getitem(self, space, w_index):
+        start, end = self.do_span(w_index)
+        return slice_w(space, self.ctx, start, end, space.w_None)
+
     @jit.look_inside_iff(lambda self, args_w: jit.isconstant(len(args_w)))
     def group_w(self, args_w):
         space = self.space
@@ -577,7 +700,9 @@ class W_SRE_Match(W_Root):
         # handling.
         ctx = self.ctx
         if isinstance(ctx, rsre_utf8.Utf8MatchContext):
-            return ctx.w_unicode_obj._byte_to_index(bytepos)
+            index_storage = ctx.w_unicode_obj._get_index_storage()
+            return rutf8.codepoint_index_at_byte_position(
+                ctx.w_unicode_obj._utf8, index_storage, bytepos)
         else:
             return bytepos
 
@@ -592,7 +717,7 @@ class W_SRE_Match(W_Root):
         # character indexes (for utf8)
         space = self.space
         try:
-            groupnum = space.int_w(w_arg)
+            groupnum = space.getindex_w(w_arg, space.w_OverflowError)
         except OperationError as e:
             if not e.match(space, space.w_TypeError) and \
                     not e.match(space, space.w_OverflowError):
@@ -659,20 +784,22 @@ class W_SRE_Match(W_Root):
         ctx = self.ctx
         if isinstance(ctx, rsre_core.BufMatchContext):
             return space.newbytes(ctx._buffer.as_str())
-        elif isinstance(ctx, UnicodeAsciiMatchContext):
-            return space.newutf8(ctx._string, len(ctx._string))
         elif isinstance(ctx, rsre_core.StrMatchContext):
             return space.newbytes(ctx._string)
         elif isinstance(ctx, rsre_utf8.Utf8MatchContext):
-            return ctx.w_unicode_obj
+            lgt = rutf8.get_utf8_length(ctx._utf8)
+            return space.newutf8(ctx._utf8, lgt)
         else:
             raise SystemError
 
 
 W_SRE_Match.typedef = TypeDef(
-    'SRE_Match',
+    '_sre.SRE_Match',
     __copy__     = interp2app(W_SRE_Match.cannot_copy_w),
     __deepcopy__ = interp2app(W_SRE_Match.cannot_copy_w),
+    __repr__     = interp2app(W_SRE_Match.repr_w),
+    __getitem__  = interp2app(W_SRE_Match.descr_getitem),
+    #
     group        = interp2app(W_SRE_Match.group_w),
     groups       = interp2app(W_SRE_Match.groups_w),
     groupdict    = interp2app(W_SRE_Match.groupdict_w),
@@ -754,9 +881,9 @@ class W_SRE_Scanner(W_Root):
             return None
 
 W_SRE_Scanner.typedef = TypeDef(
-    'SRE_Scanner',
+    '_sre.SRE_Scanner',
     __iter__ = interp2app(W_SRE_Scanner.iter_w),
-    next     = interp2app(W_SRE_Scanner.next_w),
+    __next__ = interp2app(W_SRE_Scanner.next_w),
     match    = interp2app(W_SRE_Scanner.match_w),
     search   = interp2app(W_SRE_Scanner.search_w),
     pattern  = interp_attrproperty_w('srepat', W_SRE_Scanner),
