@@ -2,16 +2,17 @@ from __future__ import with_statement
 from pypy.interpreter.baseobjspace import W_Root, BufferInterfaceNotFound
 from pypy.interpreter.gateway import interp2app, unwrap_spec
 from pypy.interpreter.typedef import TypeDef, GetSetProperty
-from pypy.interpreter.error import OperationError, oefmt, wrap_windowserror
+from pypy.interpreter.error import OperationError, oefmt, wrap_oserror
 from rpython.rtyper.lltypesystem import rffi, lltype
 from rpython.rlib import rwinreg, rwin32
 from rpython.rlib.rarithmetic import r_uint, intmask
 
 def raiseWindowsError(space, errcode, context):
-    message = rwin32.FormatError(errcode)
-    raise OperationError(space.w_WindowsError,
-                         space.newtuple([space.newint(errcode),
-                                         space.newtext(message)]))
+    message = rwin32.FormatErrorW(errcode)
+    w_errcode = space.newint(errcode)
+    w_t = space.newtuple([w_errcode, space.newtext(*message),
+                                        space.w_None, w_errcode])
+    raise OperationError(space.w_WindowsError, w_t)
 
 class W_HKEY(W_Root):
     def __init__(self, space, hkey):
@@ -25,7 +26,7 @@ class W_HKEY(W_Root):
     def as_int(self):
         return rffi.cast(rffi.SIZE_T, self.hkey)
 
-    def descr_nonzero(self, space):
+    def descr_bool(self, space):
         return space.newbool(self.as_int() != 0)
 
     def descr_handle_get(self, space):
@@ -70,7 +71,7 @@ def new_HKEY(space, w_subtype, key):
 descr_HKEY_new = interp2app(new_HKEY)
 
 W_HKEY.typedef = TypeDef(
-    "_winreg.HKEYType",
+    "winreg.HKEYType",
     __doc__ = """\
 PyHKEY Object - A Python object, representing a win32 registry key.
 
@@ -89,13 +90,13 @@ Properties:
 handle - The integer Win32 handle.
 
 Operations:
-__nonzero__ - Handles with an open object return true, otherwise false.
+__bool__ - Handles with an open object return true, otherwise false.
 __int__ - Converting a handle to an integer returns the Win32 handle.
 __cmp__ - Handle objects are compared using the handle value.""",
     __new__ = descr_HKEY_new,
     __repr__ = interp2app(W_HKEY.descr_repr),
     __int__ = interp2app(W_HKEY.descr_int),
-    __nonzero__ = interp2app(W_HKEY.descr_nonzero),
+    __bool__ = interp2app(W_HKEY.descr_bool),
     __enter__ = interp2app(W_HKEY.descr__enter__),
     __exit__ = interp2app(W_HKEY.descr__exit__),
     handle = GetSetProperty(W_HKEY.descr_handle_get),
@@ -110,8 +111,8 @@ def hkey_w(w_hkey, space):
     elif isinstance(w_hkey, W_HKEY):
         return w_hkey.hkey
     elif space.isinstance_w(w_hkey, space.w_int):
-        return rffi.cast(rwinreg.HKEY, space.int_w(w_hkey))
-    elif space.isinstance_w(w_hkey, space.w_long):
+        if space.is_true(space.lt(w_hkey, space.newint(0))):
+            return rffi.cast(rwinreg.HKEY, space.int_w(w_hkey))
         return rffi.cast(rwinreg.HKEY, space.uint_w(w_hkey))
     else:
         raise oefmt(space.w_TypeError, "The object is not a PyHKEY object")
@@ -149,7 +150,7 @@ If you don't know whether a FlushKey() call is required, it probably isn't."""
         if ret != 0:
             raiseWindowsError(space, ret, 'RegFlushKey')
 
-@unwrap_spec(subkey="text", filename="text")
+@unwrap_spec(subkey="unicode", filename="unicode")
 def LoadKey(space, w_hkey, subkey, filename):
     """LoadKey(key, sub_key, file_name) - Creates a subkey under the specified key
 and stores registration information from a specified file into that subkey.
@@ -168,12 +169,17 @@ If key is a handle returned by ConnectRegistry(), then the path specified
 in fileName is relative to the remote computer.
 
 The docs imply key must be in the HKEY_USER or HKEY_LOCAL_MACHINE tree"""
+    # XXX should filename use space.fsencode_w?
     hkey = hkey_w(w_hkey, space)
-    ret = rwinreg.RegLoadKeyA(hkey, subkey, filename)
-    if ret != 0:
-        raiseWindowsError(space, ret, 'RegLoadKey')
+    with rffi.scoped_unicode2wcharp(subkey) as wide_subkey:
+        c_subkey = rffi.cast(rffi.CCHARP, wide_subkey)
+        with rffi.scoped_unicode2wcharp(filename) as wide_filename:
+            c_filename = rffi.cast(rffi.CCHARP, wide_filename)
+            ret = rwinreg.RegLoadKeyW(hkey, c_subkey, c_filename)
+            if ret != 0:
+                raiseWindowsError(space, ret, 'RegLoadKey')
 
-@unwrap_spec(filename="text")
+@unwrap_spec(filename="unicode")
 def SaveKey(space, w_hkey, filename):
     """SaveKey(key, file_name) - Saves the specified key, and all its subkeys to the specified file.
 
@@ -188,11 +194,13 @@ file_name is relative to the remote computer.
 The caller of this method must possess the SeBackupPrivilege security privilege.
 This function passes NULL for security_attributes to the API."""
     hkey = hkey_w(w_hkey, space)
-    ret = rwinreg.RegSaveKeyA(hkey, filename, None)
-    if ret != 0:
-        raiseWindowsError(space, ret, 'RegSaveKey')
+    with rffi.scoped_unicode2wcharp(filename) as wide_filename:
+        c_filename = rffi.cast(rffi.CCHARP, wide_filename)
+        ret = rwinreg.RegSaveKeyW(hkey, c_filename, None)
+        if ret != 0:
+            raiseWindowsError(space, ret, 'RegSaveKey')
 
-@unwrap_spec(typ=int, value="text")
+@unwrap_spec(typ=int, value="unicode")
 def SetValue(space, w_hkey, w_subkey, typ, value):
     """SetValue(key, sub_key, type, value) - Associates a value with a specified key.
 
@@ -212,16 +220,16 @@ the configuration registry.  This helps the registry perform efficiently.
 The key identified by the key parameter must have been opened with
 KEY_SET_VALUE access."""
     if typ != rwinreg.REG_SZ:
-        raise oefmt(space.w_ValueError, "Type must be _winreg.REG_SZ")
+        raise oefmt(space.w_ValueError, "Type must be winreg.REG_SZ")
     hkey = hkey_w(w_hkey, space)
-    if space.is_w(w_subkey, space.w_None):
-        subkey = None
-    else:
-        subkey = space.text_w(w_subkey)
-    with rffi.scoped_str2charp(value) as dataptr:
-        ret = rwinreg.RegSetValueA(hkey, subkey, rwinreg.REG_SZ, dataptr, len(value))
-        if ret != 0:
-            raiseWindowsError(space, ret, 'RegSetValue')
+    with rffi.scoped_unicode2wcharp(space.utf8_w(w_subkey).decode('utf8')) as subkey:
+        c_subkey = rffi.cast(rffi.CCHARP, subkey)
+        with rffi.scoped_unicode2wcharp(value) as dataptr:
+            c_dataptr = rffi.cast(rffi.CCHARP, dataptr)
+            ret = rwinreg.RegSetValueW(hkey, c_subkey, rwinreg.REG_SZ,
+                                            c_dataptr, len(value))
+            if ret != 0:
+                raiseWindowsError(space, ret, 'RegSetValue')
 
 def QueryValue(space, w_hkey, w_subkey):
     """string = QueryValue(key, sub_key) - retrieves the unnamed value for a key.
@@ -238,36 +246,41 @@ But the underlying API call doesn't return the type, Lame Lame Lame, DONT USE TH
     if space.is_w(w_subkey, space.w_None):
         subkey = None
     else:
-        subkey = space.text_w(w_subkey)
-    with lltype.scoped_alloc(rwin32.PLONG.TO, 1) as bufsize_p:
-        ret = rwinreg.RegQueryValueA(hkey, subkey, None, bufsize_p)
-        bufSize = intmask(bufsize_p[0])
-        if ret == rwinreg.ERROR_MORE_DATA:
-            bufSize = 256
-        elif ret != 0:
-            raiseWindowsError(space, ret, 'RegQueryValue')
+        subkey = space.utf8_w(w_subkey).decode('utf8')
+    with rffi.scoped_unicode2wcharp(subkey) as wide_subkey:
+        c_subkey = rffi.cast(rffi.CCHARP, wide_subkey)
+        with lltype.scoped_alloc(rwin32.PLONG.TO, 1) as bufsize_p:
+            ret = rwinreg.RegQueryValueW(hkey, c_subkey, None, bufsize_p)
+            bufSize = intmask(bufsize_p[0])
+            if ret == rwinreg.ERROR_MORE_DATA:
+                bufSize = 256
+            elif ret != 0:
+                raiseWindowsError(space, ret, 'RegQueryValue')
 
-        while True:
-            with lltype.scoped_alloc(rffi.CCHARP.TO, bufSize) as buf:
-                ret = rwinreg.RegQueryValueA(hkey, subkey, buf, bufsize_p)
-                if ret == rwinreg.ERROR_MORE_DATA:
-                    # Resize and retry
-                    bufSize *= 2
-                    bufsize_p[0] = bufSize
-                    continue
+            while True:
+                with lltype.scoped_alloc(rffi.CCHARP.TO, bufSize) as buf:
+                    ret = rwinreg.RegQueryValueW(hkey, c_subkey, buf, bufsize_p)
+                    if ret == rwinreg.ERROR_MORE_DATA:
+                        print 'bufSize was %d, too small' % bufSize
+                        # Resize and retry
+                        bufSize *= 2
+                        bufsize_p[0] = bufSize
+                        continue
 
-                if ret != 0:
-                    raiseWindowsError(space, ret, 'RegQueryValue')
-                length = intmask(bufsize_p[0] - 1)
-                return space.newtext(rffi.charp2strn(buf, length))
+                    if ret != 0:
+                        raiseWindowsError(space, ret, 'RegQueryValue')
+                    length = intmask(bufsize_p[0] - 1) / 2
+                    wide_buf = rffi.cast(rffi.CWCHARP, buf)
+                    return space.newtext(rffi.wcharp2unicoden(wide_buf, length))
 
 def convert_to_regdata(space, w_value, typ):
+    '''
+    returns CCHARP, int
+    '''
     buf = None
 
     if typ == rwinreg.REG_DWORD:
-        if space.is_none(w_value) or (
-                space.isinstance_w(w_value, space.w_int) or
-                space.isinstance_w(w_value, space.w_long)):
+        if space.is_none(w_value) or space.isinstance_w(w_value, space.w_int):
             if space.is_none(w_value):
                 value = r_uint(0)
             else:
@@ -283,11 +296,9 @@ def convert_to_regdata(space, w_value, typ):
             buf = lltype.malloc(rffi.CCHARP.TO, buflen, flavor='raw')
             buf[0] = '\0'
         else:
-            if space.isinstance_w(w_value, space.w_unicode):
-                w_value = space.call_method(w_value, 'encode',
-                                            space.newtext('mbcs'))
-            buf = rffi.str2charp(space.text_w(w_value))
-            buflen = space.len_w(w_value) + 1
+            buf = rffi.unicode2wcharp(space.utf8_w(w_value).decode('utf8'))
+            buf = rffi.cast(rffi.CCHARP, buf)
+            buflen = (space.len_w(w_value) * 2) + 1
 
     elif typ == rwinreg.REG_MULTI_SZ:
         if space.is_w(w_value, space.w_None):
@@ -303,28 +314,29 @@ def convert_to_regdata(space, w_value, typ):
             while True:
                 try:
                     w_item = space.next(w_iter)
-                    if space.isinstance_w(w_item, space.w_unicode):
-                        w_item = space.call_method(w_item, 'encode',
-                                                   space.newtext('mbcs'))
-                    item = space.bytes_w(w_item)
+                    item = space.utf8_w(w_item).decode('utf8')
                     strings.append(item)
-                    buflen += len(item) + 1
+                    buflen += 2 * (len(item) + 1)
                 except OperationError as e:
                     if not e.match(space, space.w_StopIteration):
                         raise       # re-raise other app-level exceptions
                     break
-            buflen += 1
+            buflen += 2
             buf = lltype.malloc(rffi.CCHARP.TO, buflen, flavor='raw')
 
             # Now copy data
             buflen = 0
             for string in strings:
-                for i in range(len(string)):
-                    buf[buflen + i] = string[i]
-                buflen += len(string) + 1
+                with rffi.scoped_unicode2wcharp(string) as wchr:
+                    c_str = rffi.cast(rffi.CCHARP, wchr)
+                    for i in range(len(string) * 2):
+                        buf[buflen + i] = c_str[i]
+                buflen += (len(string) + 1) * 2
                 buf[buflen - 1] = '\0'
-            buflen += 1
+                buf[buflen - 2] = '\0'
+            buflen += 2
             buf[buflen - 1] = '\0'
+            buf[buflen - 2] = '\0'
 
     else: # REG_BINARY and ALL unknown data types.
         if space.is_w(w_value, space.w_None):
@@ -333,7 +345,7 @@ def convert_to_regdata(space, w_value, typ):
             buf[0] = '\0'
         else:
             try:
-                value = w_value.readbuf_w(space)
+                value = w_value.buffer_w(space, space.BUF_SIMPLE)
             except BufferInterfaceNotFound:
                 raise oefmt(space.w_TypeError,
                             "Objects of type '%T' can not be used as binary "
@@ -358,29 +370,32 @@ def convert_from_regdata(space, buf, buflen, typ):
 
     elif typ == rwinreg.REG_SZ or typ == rwinreg.REG_EXPAND_SZ:
         if not buflen:
-            s = ""
+            s = u""
         else:
             # may or may not have a trailing NULL in the buffer.
-            buf = rffi.cast(rffi.CCHARP, buf)
+            buf = rffi.cast(rffi.CWCHARP, buf)
+            buflen /= 2
             if buf[buflen - 1] == '\x00':
                 buflen -= 1
-            s = rffi.charp2strn(buf, buflen)
-        w_s = space.newbytes(s)
-        return space.call_method(w_s, 'decode', space.newtext('mbcs'))
+            s = rffi.wcharp2unicoden(buf, buflen)
+        w_s = space.newtext(s)
+        return w_s
 
     elif typ == rwinreg.REG_MULTI_SZ:
         if not buflen:
             return space.newlist([])
+        buf = rffi.cast(rffi.CWCHARP, buf)
+        buflen /= 2
         i = 0
         l = []
         while i < buflen and buf[i]:
             s = []
-            while i < buflen and buf[i] != '\0':
+            while i < buflen and buf[i] != u'\0':
                 s.append(buf[i])
                 i += 1
             if len(s) == 0:
                 break
-            s = ''.join(s)
+            s = u''.join(s)
             l.append(space.newtext(s))
             i += 1
         return space.newlist(l)
@@ -388,7 +403,7 @@ def convert_from_regdata(space, buf, buflen, typ):
     else: # REG_BINARY and all other types
         return space.newbytes(rffi.charpsize2str(buf, buflen))
 
-@unwrap_spec(value_name="text", typ=int)
+@unwrap_spec(value_name="unicode", typ=int)
 def SetValueEx(space, w_hkey, value_name, w_reserved, typ, w_value):
     """SetValueEx(key, value_name, reserved, type, value) - Stores data in the value field of an open registry key.
 
@@ -423,7 +438,9 @@ the configuration registry.  This helps the registry perform efficiently."""
     hkey = hkey_w(w_hkey, space)
     buf, buflen = convert_to_regdata(space, w_value, typ)
     try:
-        ret = rwinreg.RegSetValueExA(hkey, value_name, 0, typ, buf, buflen)
+        with rffi.scoped_unicode2wcharp(value_name) as wide_vn:
+            c_vn = rffi.cast(rffi.CCHARP, wide_vn)
+            ret = rwinreg.RegSetValueExW(hkey, c_vn, 0, typ, buf, buflen)
     finally:
         lltype.free(buf, flavor='raw')
     if ret != 0:
@@ -438,38 +455,40 @@ value_name is a string indicating the value to query"""
     if space.is_w(w_subkey, space.w_None):
         subkey = None
     else:
-        subkey = space.text_w(w_subkey)
+        subkey = space.utf8_w(w_subkey).decode('utf8')
     null_dword = lltype.nullptr(rwin32.LPDWORD.TO)
-    with lltype.scoped_alloc(rwin32.LPDWORD.TO, 1) as retDataSize:
-        ret = rwinreg.RegQueryValueExA(hkey, subkey, null_dword, null_dword,
-                                      None, retDataSize)
-        bufSize = intmask(retDataSize[0])
-        if ret == rwinreg.ERROR_MORE_DATA:
-            bufSize = 256
-        elif ret != 0:
-            raiseWindowsError(space, ret, 'RegQueryValueEx')
+    with rffi.scoped_unicode2wcharp(subkey) as wide_subkey:
+        c_subkey = rffi.cast(rffi.CCHARP, wide_subkey)
+        with lltype.scoped_alloc(rwin32.LPDWORD.TO, 1) as retDataSize:
+            ret = rwinreg.RegQueryValueExW(hkey, c_subkey, null_dword, null_dword,
+                                          None, retDataSize)
+            bufSize = intmask(retDataSize[0])
+            if ret == rwinreg.ERROR_MORE_DATA:
+                bufSize = 256
+            elif ret != 0:
+                raiseWindowsError(space, ret, 'RegQueryValueEx')
 
-        while True:
-            with lltype.scoped_alloc(rffi.CCHARP.TO, bufSize) as databuf:
-                with lltype.scoped_alloc(rwin32.LPDWORD.TO, 1) as retType:
+            while True:
+                with lltype.scoped_alloc(rffi.CCHARP.TO, bufSize) as databuf:
+                    with lltype.scoped_alloc(rwin32.LPDWORD.TO, 1) as retType:
 
-                    ret = rwinreg.RegQueryValueExA(hkey, subkey, null_dword,
-                                                  retType, databuf, retDataSize)
-                    if ret == rwinreg.ERROR_MORE_DATA:
-                        # Resize and retry
-                        bufSize *= 2
-                        retDataSize[0] = rffi.cast(rwin32.DWORD, bufSize)
-                        continue
-                    if ret != 0:
-                        raiseWindowsError(space, ret, 'RegQueryValueEx')
-                    length = intmask(retDataSize[0])
-                    return space.newtuple([
-                        convert_from_regdata(space, databuf,
-                                             length, retType[0]),
-                        space.newint(intmask(retType[0])),
+                        ret = rwinreg.RegQueryValueExW(hkey, c_subkey, null_dword,
+                                                      retType, databuf, retDataSize)
+                        if ret == rwinreg.ERROR_MORE_DATA:
+                            # Resize and retry
+                            bufSize *= 2
+                            retDataSize[0] = rffi.cast(rwin32.DWORD, bufSize)
+                            continue
+                        if ret != 0:
+                            raiseWindowsError(space, ret, 'RegQueryValueEx')
+                        length = intmask(retDataSize[0])
+                        return space.newtuple([
+                            convert_from_regdata(space, databuf,
+                                                 length, retType[0]),
+                            space.newint(intmask(retType[0])),
                         ])
 
-@unwrap_spec(subkey="text")
+@unwrap_spec(subkey="unicode")
 def CreateKey(space, w_hkey, subkey):
     """key = CreateKey(key, sub_key) - Creates or opens the specified key.
 
@@ -483,14 +502,16 @@ If the key already exists, this function opens the existing key
 The return value is the handle of the opened key.
 If the function fails, an exception is raised."""
     hkey = hkey_w(w_hkey, space)
-    with lltype.scoped_alloc(rwinreg.PHKEY.TO, 1) as rethkey:
-        ret = rwinreg.RegCreateKeyA(hkey, subkey, rethkey)
-        if ret != 0:
-            raiseWindowsError(space, ret, 'CreateKey')
-        return W_HKEY(space, rethkey[0])
+    with rffi.scoped_unicode2wcharp(subkey) as wide_subkey:
+        c_subkey = rffi.cast(rffi.CCHARP, wide_subkey)
+        with lltype.scoped_alloc(rwinreg.PHKEY.TO, 1) as rethkey:
+            ret = rwinreg.RegCreateKeyW(hkey, c_subkey, rethkey)
+            if ret != 0:
+                raiseWindowsError(space, ret, 'CreateKey')
+            return W_HKEY(space, rethkey[0])
 
-@unwrap_spec(subkey="text", res=int, sam=rffi.r_uint)
-def CreateKeyEx(space, w_hkey, subkey, res=0, sam=rwinreg.KEY_WRITE):
+@unwrap_spec(sub_key="unicode", reserved=int, access=rffi.r_uint)
+def CreateKeyEx(space, w_key, sub_key, reserved=0, access=rwinreg.KEY_WRITE):
     """key = CreateKey(key, sub_key) - Creates or opens the specified key.
 
 key is an already open key, or one of the predefined HKEY_* constants
@@ -502,18 +523,20 @@ If the key already exists, this function opens the existing key
 
 The return value is the handle of the opened key.
 If the function fails, an exception is raised."""
-    hkey = hkey_w(w_hkey, space)
-    with lltype.scoped_alloc(rwinreg.PHKEY.TO, 1) as rethkey:
-        ret = rwinreg.RegCreateKeyExA(hkey, subkey, res, None, 0,
-                                     sam, None, rethkey,
-                                     lltype.nullptr(rwin32.LPDWORD.TO))
-        if ret != 0:
-            raiseWindowsError(space, ret, 'CreateKeyEx')
-        return W_HKEY(space, rethkey[0])
+    hkey = hkey_w(w_key, space)
+    with rffi.scoped_unicode2wcharp(sub_key) as wide_sub_key:
+        c_subkey = rffi.cast(rffi.CCHARP, wide_sub_key)
+        with lltype.scoped_alloc(rwinreg.PHKEY.TO, 1) as rethkey:
+            ret = rwinreg.RegCreateKeyExW(hkey, c_subkey, reserved, None, 0,
+                                         access, None, rethkey,
+                                         lltype.nullptr(rwin32.LPDWORD.TO))
+            if ret != 0:
+                raiseWindowsError(space, ret, 'CreateKeyEx')
+            return W_HKEY(space, rethkey[0])
 
-@unwrap_spec(subkey="text")
+@unwrap_spec(subkey="unicode")
 def DeleteKey(space, w_hkey, subkey):
-    """DeleteKey(key, sub_key) - Deletes the specified key.
+    """DeleteKey(key, subkey) - Deletes the specified key.
 
 key is an already open key, or any one of the predefined HKEY_* constants.
 sub_key is a string that must be a subkey of the key identified by the key parameter.
@@ -524,23 +547,27 @@ This method can not delete keys with subkeys.
 If the method succeeds, the entire key, including all of its values,
 is removed.  If the method fails, an EnvironmentError exception is raised."""
     hkey = hkey_w(w_hkey, space)
-    ret = rwinreg.RegDeleteKeyA(hkey, subkey)
-    if ret != 0:
-        raiseWindowsError(space, ret, 'RegDeleteKey')
+    with rffi.scoped_unicode2wcharp(subkey) as wide_subkey:
+        c_subkey = rffi.cast(rffi.CCHARP, wide_subkey)
+        ret = rwinreg.RegDeleteKeyW(hkey, c_subkey)
+        if ret != 0:
+            raiseWindowsError(space, ret, 'RegDeleteKey')
 
-@unwrap_spec(subkey="text")
+@unwrap_spec(subkey="unicode")
 def DeleteValue(space, w_hkey, subkey):
     """DeleteValue(key, value) - Removes a named value from a registry key.
 
 key is an already open key, or any one of the predefined HKEY_* constants.
 value is a string that identifies the value to remove."""
     hkey = hkey_w(w_hkey, space)
-    ret = rwinreg.RegDeleteValueA(hkey, subkey)
-    if ret != 0:
-        raiseWindowsError(space, ret, 'RegDeleteValue')
+    with rffi.scoped_unicode2wcharp(subkey) as wide_subkey:
+        c_subkey = rffi.cast(rffi.CCHARP, wide_subkey)
+        ret = rwinreg.RegDeleteValueW(hkey, c_subkey)
+        if ret != 0:
+            raiseWindowsError(space, ret, 'RegDeleteValue')
 
-@unwrap_spec(subkey="text", res=int, sam=rffi.r_uint)
-def OpenKey(space, w_hkey, subkey, res=0, sam=rwinreg.KEY_READ):
+@unwrap_spec(sub_key="unicode", reserved=int, access=rffi.r_uint)
+def OpenKey(space, w_key, sub_key, reserved=0, access=rwinreg.KEY_READ):
     """key = OpenKey(key, sub_key, res = 0, sam = KEY_READ) - Opens the specified key.
 
 key is an already open key, or any one of the predefined HKEY_* constants.
@@ -551,12 +578,14 @@ sam is an integer that specifies an access mask that describes the desired
 
 The result is a new handle to the specified key
 If the function fails, an EnvironmentError exception is raised."""
-    hkey = hkey_w(w_hkey, space)
-    with lltype.scoped_alloc(rwinreg.PHKEY.TO, 1) as rethkey:
-        ret = rwinreg.RegOpenKeyExA(hkey, subkey, res, sam, rethkey)
-        if ret != 0:
-            raiseWindowsError(space, ret, 'RegOpenKeyEx')
-        return W_HKEY(space, rethkey[0])
+    hkey = hkey_w(w_key, space)
+    with rffi.scoped_unicode2wcharp(sub_key) as wide_subkey:
+        c_subkey = rffi.cast(rffi.CCHARP, wide_subkey)
+        with lltype.scoped_alloc(rwinreg.PHKEY.TO, 1) as rethkey:
+            ret = rwinreg.RegOpenKeyExW(hkey, c_subkey, reserved, access, rethkey)
+            if ret != 0:
+                raiseWindowsError(space, ret, 'RegOpenKeyEx')
+            return W_HKEY(space, rethkey[0])
 
 @unwrap_spec(index=int)
 def EnumValue(space, w_hkey, index):
@@ -578,7 +607,7 @@ data_type is an integer that identifies the type of the value data."""
 
     with lltype.scoped_alloc(rwin32.LPDWORD.TO, 1) as retValueSize:
         with lltype.scoped_alloc(rwin32.LPDWORD.TO, 1) as retDataSize:
-            ret = rwinreg.RegQueryInfoKeyA(
+            ret = rwinreg.RegQueryInfoKeyW(
                 hkey, None, null_dword, null_dword,
                 null_dword, null_dword, null_dword,
                 null_dword, retValueSize, retDataSize,
@@ -591,15 +620,16 @@ data_type is an integer that identifies the type of the value data."""
             bufDataSize = intmask(retDataSize[0])
             bufValueSize = intmask(retValueSize[0])
 
-            with lltype.scoped_alloc(rffi.CCHARP.TO,
+            with lltype.scoped_alloc(rffi.CWCHARP.TO,
                                      intmask(retValueSize[0])) as valuebuf:
                 while True:
                     with lltype.scoped_alloc(rffi.CCHARP.TO,
                                              bufDataSize) as databuf:
                         with lltype.scoped_alloc(rwin32.LPDWORD.TO,
                                                  1) as retType:
-                            ret = rwinreg.RegEnumValueA(
-                                hkey, index, valuebuf, retValueSize,
+                            c_valuebuf = rffi.cast(rffi.CCHARP, valuebuf)
+                            ret = rwinreg.RegEnumValueW(
+                                hkey, index, c_valuebuf, retValueSize,
                                 null_dword, retType, databuf, retDataSize)
                             if ret == rwinreg.ERROR_MORE_DATA:
                                 # Resize and retry
@@ -615,7 +645,7 @@ data_type is an integer that identifies the type of the value data."""
 
                             length = intmask(retDataSize[0])
                             return space.newtuple([
-                                space.newtext(rffi.charp2str(valuebuf)),
+                                space.newtext(rffi.wcharp2unicode(valuebuf)),
                                 convert_from_regdata(space, databuf,
                                                      length, retType[0]),
                                 space.newint(intmask(retType[0])),
@@ -643,12 +673,12 @@ raised, indicating no more values are available."""
     with lltype.scoped_alloc(rffi.CCHARP.TO, 257) as buf:
         with lltype.scoped_alloc(rwin32.LPDWORD.TO, 1) as retValueSize:
             retValueSize[0] = r_uint(257) # includes NULL terminator
-            ret = rwinreg.RegEnumKeyExA(hkey, index, buf, retValueSize,
+            ret = rwinreg.RegEnumKeyExW(hkey, index, buf, retValueSize,
                                        null_dword, None, null_dword,
                                        lltype.nullptr(rwin32.PFILETIME.TO))
             if ret != 0:
                 raiseWindowsError(space, ret, 'RegEnumKeyEx')
-            return space.newtext(rffi.charp2str(buf))
+            return space.newtext(rffi.wcharp2unicode(rffi.cast(rffi.CWCHARP, buf)))
 
 def QueryInfoKey(space, w_hkey):
     """tuple = QueryInfoKey(key) - Returns information about a key.
@@ -665,7 +695,7 @@ A long integer that identifies when the key was last modified (if available)
         with lltype.scoped_alloc(rwin32.LPDWORD.TO, 1) as nValues:
             with lltype.scoped_alloc(rwin32.PFILETIME.TO, 1) as ft:
                 null_dword = lltype.nullptr(rwin32.LPDWORD.TO)
-                ret = rwinreg.RegQueryInfoKeyA(
+                ret = rwinreg.RegQueryInfoKeyW(
                     hkey, None, null_dword, null_dword,
                     nSubKeys, null_dword, null_dword,
                     nValues, null_dword, null_dword,
@@ -692,7 +722,7 @@ If the function fails, an EnvironmentError exception is raised."""
     machine = space.text_or_none_w(w_machine)
     hkey = hkey_w(w_hkey, space)
     with lltype.scoped_alloc(rwinreg.PHKEY.TO, 1) as rethkey:
-        ret = rwinreg.RegConnectRegistryA(machine, hkey, rethkey)
+        ret = rwinreg.RegConnectRegistryW(machine, hkey, rethkey)
         if ret != 0:
             raiseWindowsError(space, ret, 'RegConnectRegistry')
         return W_HKEY(space, rethkey[0])
@@ -704,7 +734,7 @@ def ExpandEnvironmentStrings(space, w_source):
         res, res_ulen = rwinreg.ExpandEnvironmentStrings(source, source_ulen)
         return space.newutf8(res, res_ulen)
     except WindowsError as e:
-        raise wrap_windowserror(space, e)
+        raise wrap_oserror(space, e)
 
 def DisableReflectionKey(space, w_key):
     """Disables registry reflection for 32-bit processes running on a 64-bit
@@ -728,8 +758,8 @@ def QueryReflectionKey(space, w_key):
     raise oefmt(space.w_NotImplementedError,
                 "not implemented on this platform")
 
-@unwrap_spec(subkey="text")
-def DeleteKeyEx(space, w_key, subkey):
+@unwrap_spec(sub_key="unicode", reserved=int, access=rffi.r_uint)
+def DeleteKeyEx(space, w_key, sub_key, reserved=0, access=rwinreg.KEY_WOW64_64KEY):
     """DeleteKeyEx(key, sub_key, sam, res) - Deletes the specified key.
 
     key is an already open key, or any one of the predefined HKEY_* constants.
