@@ -164,12 +164,13 @@ def PySequence_ITEM(space, w_obj, i):
         incref(space, py_res)
         keepalive_until_here(w_obj)
         return py_res
+    
     as_sequence = py_obj.c_ob_type.c_tp_as_sequence
-    if as_sequence and as_sequence.c_sq_item:
-        ret = generic_cpy_call(space, as_sequence.c_sq_item, w_obj, i)
-        return make_ref(space, ret)
-    w_ret = space.getitem(w_obj, space.newint(i))
-    return make_ref(space, w_ret)
+    if not as_sequence or not as_sequence.c_sq_item:
+        raise oefmt(space.w_TypeError,
+                    "'%T' object does not support indexing", w_obj)
+    ret = generic_cpy_call(space, as_sequence.c_sq_item, w_obj, i)
+    return make_ref(space, ret)
 
 @cpython_api([PyObject, Py_ssize_t], PyObject, result_is_ll=True)
 def PySequence_GetItem(space, w_obj, i):
@@ -178,11 +179,6 @@ def PySequence_GetItem(space, w_obj, i):
     if i < 0:
         l = PySequence_Length(space, w_obj)
         i += l
-        if i < 0:
-            # Prevent things like 'abc'[-4] from turning into 'abc'[-1]
-            # since this can end up calling space.getitem()
-            raise oefmt(space.w_IndexError,
-                "%T index out of range")
     return PySequence_ITEM(space, w_obj, i)
 
 @cpython_api([PyObject], PyObject)
@@ -339,7 +335,7 @@ class CPyListStrategy(ListStrategy):
         w_other.strategy = self
         w_other.lstorage = self.getstorage_copy(w_list)
 
-    def clone(self, w_list, sizehint=0):
+    def clone(self, w_list):
         storage = self.getstorage_copy(w_list)
         w_clone = W_ListObject.from_storage_and_strategy(self.space, storage,
                                                          self)
