@@ -6,13 +6,13 @@ from rpython.rtyper.lltypesystem import rffi, lltype
 from rpython.rlib.rarithmetic import widen
 from pypy.module.cpyext.api import (
     slot_function, generic_cpy_call, PyObject, Py_ssize_t,
-    Py_TPFLAGS_CHECKTYPES, Py_buffer, Py_bufferP, PyTypeObjectPtr)
+    Py_buffer, Py_bufferP, PyTypeObjectPtr, cts)
 from pypy.module.cpyext.typeobjectdefs import (
     unaryfunc, ternaryfunc, binaryfunc,
     getattrfunc, getattrofunc, setattrofunc, lenfunc, ssizeargfunc, inquiry,
     ssizessizeargfunc, ssizeobjargproc, iternextfunc, initproc, richcmpfunc,
     cmpfunc, hashfunc, descrgetfunc, descrsetfunc, objobjproc, objobjargproc,
-    readbufferproc, getbufferproc, ssizessizeobjargproc)
+    getbufferproc, ssizessizeobjargproc)
 from pypy.module.cpyext.pyobject import make_ref, from_ref, as_pyobj, decref
 from pypy.module.cpyext.pyerrors import PyErr_Occurred
 from pypy.module.cpyext.memoryobject import fill_Py_buffer
@@ -30,7 +30,7 @@ from rpython.flowspace.model import Constant
 from rpython.flowspace.specialcase import register_flow_sc
 from pypy.module.sys.version import CPYTHON_VERSION
 
-PY3 = CPYTHON_VERSION[0] == 3
+PY2 = CPYTHON_VERSION[0] == 2
 
 # XXX: Also defined in object.h
 Py_LT = 0
@@ -62,10 +62,8 @@ class wrap_init(W_PyCWrapperObject):
         func_init = rffi.cast(initproc, func)
         py_args = tuple_from_args_w(space, __args__.arguments_w)
         w_kwargs = w_kwargs_from_args(space, __args__)
-        try:
-            res = generic_cpy_call(space, func_init, w_self, py_args, w_kwargs)
-        finally:
-            decref(space, py_args)
+        res = generic_cpy_call(space, func_init, w_self, py_args, w_kwargs)
+        decref(space, py_args)
         if rffi.cast(lltype.Signed, res) == -1:
             space.fromcache(State).check_and_raise_exception(always=True)
         return None
@@ -96,10 +94,6 @@ class wrap_binaryfunc_l(W_PyCWrapperObject):
         func = self.get_func_to_call()
         func_binary = rffi.cast(binaryfunc, func)
         w_value = __args__.arguments_w[0]
-        ob_type = _get_ob_type(space, w_self)
-        if (not ob_type.c_tp_flags & Py_TPFLAGS_CHECKTYPES and
-            not space.issubtype_w(space.type(w_value), space.type(w_self))):
-            return space.w_NotImplemented
         return generic_cpy_call(space, func_binary, w_self, w_value)
 
 class wrap_binaryfunc_r(W_PyCWrapperObject):
@@ -108,10 +102,6 @@ class wrap_binaryfunc_r(W_PyCWrapperObject):
         func = self.get_func_to_call()
         func_binary = rffi.cast(binaryfunc, func)
         w_value = __args__.arguments_w[0]
-        ob_type = _get_ob_type(space, w_self)
-        if (not ob_type.c_tp_flags & Py_TPFLAGS_CHECKTYPES and
-            not space.issubtype_w(space.type(w_value), space.type(w_self))):
-            return space.w_NotImplemented
         return generic_cpy_call(space, func_binary, w_value, w_self)
 
 class wrap_ternaryfunc(W_PyCWrapperObject):
@@ -138,10 +128,6 @@ class wrap_ternaryfunc_r(W_PyCWrapperObject):
             w_arg1 = __args__.arguments_w[1]
         else:
             w_arg1 = space.w_None
-        ob_type = _get_ob_type(space, w_self)
-        if (not ob_type.c_tp_flags & Py_TPFLAGS_CHECKTYPES and
-            not space.issubtype_w(space.type(w_arg0), space.type(w_self))):
-            return space.w_NotImplemented
         return generic_cpy_call(space, func_ternary, w_arg0, w_self, w_arg1)
 
 class wrap_inquirypred(W_PyCWrapperObject):
@@ -247,10 +233,8 @@ class wrap_call(W_PyCWrapperObject):
         func_target = rffi.cast(ternaryfunc, func)
         py_args = tuple_from_args_w(space, __args__.arguments_w)
         w_kwargs = w_kwargs_from_args(space, __args__)
-        try:
-            ret = generic_cpy_call(space, func_target, w_self, py_args, w_kwargs)
-        finally:
-            decref(space, py_args)
+        ret = generic_cpy_call(space, func_target, w_self, py_args, w_kwargs)
+        decref(space, py_args)
         return ret
 
 class wrap_ssizessizeobjargproc(W_PyCWrapperObject):
@@ -275,7 +259,7 @@ class wrap_lenfunc(W_PyCWrapperObject):
             space.fromcache(State).check_and_raise_exception(always=True)
         return space.newint(res)
 
-class wrap_ssizeargproc(W_PyCWrapperObject):
+class wrap_sq_item(W_PyCWrapperObject):
     def call(self, space, w_self, __args__):
         self.check_args(__args__, 1)
         func = self.get_func_to_call()
@@ -524,6 +508,9 @@ def make_unary_slot(space, typedef, name, attr):
     return slot_func
 
 UNARY_SLOTS = [
+    'tp_as_async.c_am_await',
+    'tp_as_async.c_am_anext',
+    'tp_as_async.c_am_aiter',
     'tp_as_number.c_nb_int',
     'tp_as_number.c_nb_long',
     'tp_as_number.c_nb_float',
@@ -605,6 +592,7 @@ def make_binary_slot_int(space, typedef, name, attr):
 BINARY_SLOTS_INT = [
     'tp_as_sequence.c_sq_item',
     'tp_as_sequence.c_sq_repeat',
+    'tp_as_sequence.c_sq_repeat',
     'tp_as_sequence.c_sq_inplace_repeat',]
 for name in BINARY_SLOTS_INT:
     slot_factory(name)(make_binary_slot_int)
@@ -714,7 +702,7 @@ def make_tp_call(space, typedef, name, attr):
 @slot_factory('tp_iternext')
 def make_tp_iternext(space, typedef, name, attr):
     w_type = space.gettypeobject(typedef)
-    iternext_fn = w_type.lookup('next')
+    iternext_fn = w_type.lookup('__next__')
     if iternext_fn is None:
         return
 
@@ -760,12 +748,46 @@ def make_tp_new(space, typedef, name, attr):
         return space.call_args(space.get(new_fn, w_self), args)
     return slot_tp_new
 
+@slot_function([PyObject, lltype.Ptr(Py_buffer), rffi.INT_real],
+    rffi.INT_real, error=-1)
+def bytes_getbuffer(space, w_str, view, flags):
+    from pypy.module.cpyext.bytesobject import PyBytes_AsString
+    from pypy.module.cpyext.buffer import PyBuffer_FillInfo
+    c_buf = rffi.cast(rffi.VOIDP, PyBytes_AsString(space, w_str))
+    return PyBuffer_FillInfo(space, view, w_str, c_buf,
+                             space.len_w(w_str), 1, flags)
+
+def slot_from_buffer_w(space, typedef):
+    name = 'bf_getbuffer'
+    @slot_function([PyObject, Py_bufferP, rffi.INT_real],
+            rffi.INT_real, error=-1)
+    @func_renamer("cpyext_%s_%s" % (name, typedef.name))
+    def buff_w(space, w_self, c_view, flags):
+        w_obj = w_self
+        if c_view:
+            #like PyObject_GetBuffer
+            flags = widen(flags)
+            buf = space.buffer_w(w_obj, flags)
+            try:
+                c_view.c_buf = rffi.cast(rffi.VOIDP, buf.get_raw_address())
+                c_view.c_obj = make_ref(space, w_obj)
+            except ValueError:
+                s = buf.as_str()
+                w_s = space.newbytes(s)
+                c_view.c_obj = make_ref(space, w_s)
+                c_view.c_buf = rffi.cast(rffi.VOIDP, rffi.str2charp(
+                                        s, track_allocation=False))
+                rffi.setintfield(c_view, 'c_readonly', 1)
+            ret = fill_Py_buffer(space, buf, c_view)
+            return ret
+        return 0
+    return buff_w
+
 @slot_factory('tp_as_buffer.c_bf_getbuffer')
 def make_bf_getbuffer(space, typedef, name, attr):
     w_type = space.gettypeobject(typedef)
-    buff_fn = w_type.lookup('__buffer__')
-    if buff_fn is not None:
-        return slot_from___buffer__(space, typedef, buff_fn)
+    if space.is_w(w_type, space.w_bytes):
+        return bytes_getbuffer
     elif typedef.buffer:
         return slot_from_buffer_w(space, typedef)
     else:
@@ -811,73 +833,16 @@ def make_tp_descr_set(space, typedef, name, attr):
     return slot_tp_descr_set
 
 
-def slot_from___buffer__(space, typedef, buff_fn):
-    name = 'bf_getbuffer'
-    @slot_function([PyObject, Py_bufferP, rffi.INT_real],
-            rffi.INT_real, error=-1)
-    @func_renamer("cpyext_%s_%s" % (name, typedef.name))
-    def buff_w(space, w_self, c_view, flags):
-        args = Arguments(space, [space.newint(flags)])
-        w_obj = space.call_args(space.get(buff_fn, w_self), args)
-        if c_view:
-            #like PyObject_GetBuffer
-            flags = widen(flags)
-            buf = space.buffer_w(w_obj, flags)
-            try:
-                c_view.c_buf = rffi.cast(rffi.VOIDP, buf.get_raw_address())
-                c_view.c_obj = make_ref(space, w_obj)
-                if space.isinstance_w(w_obj, space.w_bytes):
-                    rffi.setintfield(c_view, 'c_readonly', 1)
-            except ValueError:
-                s = buf.as_str()
-                w_s = space.newbytes(s)
-                c_view.c_obj = make_ref(space, w_s)
-                c_view.c_buf = rffi.cast(rffi.VOIDP, rffi.str2charp(
-                                        s, track_allocation=False))
-                rffi.setintfield(c_view, 'c_readonly', 1)
-            ret = fill_Py_buffer(space, buf, c_view)
-            return ret
-        return 0
-    return buff_w
-
-def slot_from_buffer_w(space, typedef):
-    name = 'bf_getbuffer'
-    @slot_function([PyObject, Py_bufferP, rffi.INT_real],
-            rffi.INT_real, error=-1)
-    @func_renamer("cpyext_%s_%s" % (name, typedef.name))
-    def buff_w(space, w_self, c_view, flags):
-        w_obj = w_self
-        if c_view:
-            #like PyObject_GetBuffer
-            flags = widen(flags)
-            buf = space.buffer_w(w_obj, flags)
-            try:
-                c_view.c_buf = rffi.cast(rffi.VOIDP, buf.get_raw_address())
-                c_view.c_obj = make_ref(space, w_obj)
-            except ValueError:
-                s = buf.as_str()
-                w_s = space.newbytes(s)
-                c_view.c_obj = make_ref(space, w_s)
-                c_view.c_buf = rffi.cast(rffi.VOIDP, rffi.str2charp(
-                                        s, track_allocation=False))
-                rffi.setintfield(c_view, 'c_readonly', 1)
-            ret = fill_Py_buffer(space, buf, c_view)
-            return ret
-        return 0
-    return buff_w
-
 def _make_missing_wrapper(name):
     assert name not in globals()
     class missing_wrapper(W_PyCWrapperObject):
         def call(self, space, w_self, __args__):
-            msg = "cpyext: missing slot wrapper %s" %(
-                    name)
-            print msg
+            print "cpyext: missing slot wrapper " + name
             raise NotImplementedError("Slot wrapper " + name)
     missing_wrapper.__name__ = name
     globals()[name] = missing_wrapper
 
-missing_wrappers = ['wrap_delslice', 'wrap_coercefunc']
+missing_wrappers = ['wrap_indexargfunc', 'wrap_del']
 for name in missing_wrappers:
     _make_missing_wrapper(name)
 
@@ -886,8 +851,9 @@ def make_missing_slot(space, typedef, name, attr):
 
 missing_builtin_slots = [
     'tp_print', 'tp_compare', 'tp_getattr', 'tp_setattr', 'tp_setattro',
-    'tp_richcompare', 'tp_del', 'tp_as_buffer.c_bf_getwritebuffer',
-    'tp_as_number.c_nb_nonzero', 'tp_as_number.c_nb_coerce',
+    'tp_finalize',
+    'tp_richcompare', 'tp_del',
+    'tp_as_number.c_nb_bool', 'tp_as_number.c_nb_coerce',
     'tp_as_number.c_nb_inplace_add', 'tp_as_number.c_nb_inplace_subtract',
     'tp_as_number.c_nb_inplace_multiply', 'tp_as_number.c_nb_inplace_divide',
     'tp_as_number.c_nb_inplace_remainder', 'tp_as_number.c_nb_inplace_power',
@@ -896,9 +862,15 @@ missing_builtin_slots = [
     'tp_as_number.c_nb_inplace_or',
     'tp_as_number.c_nb_floor_divide', 'tp_as_number.c_nb_true_divide',
     'tp_as_number.c_nb_inplace_floor_divide', 'tp_as_number.c_nb_inplace_true_divide',
+    'tp_as_number.c_nb_matrix_multiply',
+    'tp_as_number.c_nb_inplace_matrix_multiply',
     'tp_as_sequence.c_sq_slice', 'tp_as_sequence.c_sq_ass_slice',
     'tp_as_sequence.c_sq_contains',
-    'tp_as_buffer.c_bf_getreadbuffer',
+    ]
+if PY2:
+    missing_builtin_slots += [
+        'tp_as_buffer.c_bf_getreadbuffer',
+        'tp_as_buffer.c_bf_getwritebuffer',
     ]
 for name in missing_builtin_slots:
     slot_factory(name)(make_missing_slot)
@@ -940,6 +912,8 @@ def TPSLOT(NAME, SLOT, FUNCTION, WRAPPER, DOC):
 
 ETSLOT = TPSLOT
 
+def AMSLOT(NAME, SLOT, FUNCTION, WRAPPER, DOC):
+    return ETSLOT(NAME, "tp_as_async.c_" + SLOT, FUNCTION, WRAPPER, DOC)
 def SQSLOT(NAME, SLOT, FUNCTION, WRAPPER, DOC):
     return ETSLOT(NAME, "tp_as_sequence.c_" + SLOT, FUNCTION, WRAPPER, DOC)
 def MPSLOT(NAME, SLOT, FUNCTION, WRAPPER, DOC):
@@ -976,196 +950,188 @@ def RBINSLOTNOTINFIX(NAME, SLOT, FUNCTION, DOC):
 # Copy new slotdefs from typeobject.c
 # Remove comments and tabs
 # Done.
-# XXX NOTE that we already tweaked that string manually!
 slotdefs_str = r"""
 static slotdef slotdefs[] = {
-        SQSLOT("__len__", sq_length, slot_sq_length, wrap_lenfunc,
-               "x.__len__() <==> len(x)"),
-        SQSLOT("__add__", sq_concat, slot_sq_concat, wrap_binaryfunc,
-          "x.__add__(y) <==> x+y"),
-        SQSLOT("__mul__", sq_repeat, NULL, wrap_ssizeargproc,
-          "x.__mul__(n) <==> x*n"),
-        SQSLOT("__rmul__", sq_repeat, NULL, wrap_ssizeargproc,
-          "x.__rmul__(n) <==> n*x"),
-        SQSLOT("__getitem__", sq_item, slot_sq_item, wrap_ssizeargproc,
-               "x.__getitem__(y) <==> x[y]"),
-        SQSLOT("__getslice__", sq_slice, slot_sq_slice, wrap_ssizessizeargfunc,
-               "x.__getslice__(i, j) <==> x[i:j]\n\
-               \n\
-               Use of negative indices is not supported."),
-        SQSLOT("__setitem__", sq_ass_item, slot_sq_ass_item, wrap_sq_setitem,
-               "x.__setitem__(i, y) <==> x[i]=y"),
-        SQSLOT("__delitem__", sq_ass_item, slot_sq_ass_item, wrap_sq_delitem,
-               "x.__delitem__(y) <==> del x[y]"),
-        SQSLOT("__setslice__", sq_ass_slice, slot_sq_ass_slice,
-               wrap_ssizessizeobjargproc,
-               "x.__setslice__(i, j, y) <==> x[i:j]=y\n\
-               \n\
-               Use  of negative indices is not supported."),
-        SQSLOT("__delslice__", sq_ass_slice, slot_sq_ass_slice, wrap_delslice,
-               "x.__delslice__(i, j) <==> del x[i:j]\n\
-               \n\
-               Use of negative indices is not supported."),
-        SQSLOT("__contains__", sq_contains, slot_sq_contains, wrap_objobjproc,
-               "x.__contains__(y) <==> y in x"),
-        SQSLOT("__iadd__", sq_inplace_concat, NULL,
-          wrap_binaryfunc, "x.__iadd__(y) <==> x+=y"),
-        SQSLOT("__imul__", sq_inplace_repeat, NULL,
-          wrap_ssizeargproc, "x.__imul__(y) <==> x*=y"),
+    TPSLOT("__getattribute__", tp_getattr, NULL, NULL, ""),
+    TPSLOT("__getattr__", tp_getattr, NULL, NULL, ""),
+    TPSLOT("__setattr__", tp_setattr, NULL, NULL, ""),
+    TPSLOT("__delattr__", tp_setattr, NULL, NULL, ""),
+    TPSLOT("__repr__", tp_repr, slot_tp_repr, wrap_unaryfunc,
+           "__repr__($self, /)\n--\n\nReturn repr(self)."),
+    TPSLOT("__hash__", tp_hash, slot_tp_hash, wrap_hashfunc,
+           "__hash__($self, /)\n--\n\nReturn hash(self)."),
+    FLSLOT("__call__", tp_call, slot_tp_call, (wrapperfunc)wrap_call,
+           "__call__($self, /, *args, **kwargs)\n--\n\nCall self as a function.",
+           PyWrapperFlag_KEYWORDS),
+    TPSLOT("__str__", tp_str, slot_tp_str, wrap_unaryfunc,
+           "__str__($self, /)\n--\n\nReturn str(self)."),
+    TPSLOT("__getattribute__", tp_getattro, slot_tp_getattr_hook,
+           wrap_binaryfunc,
+           "__getattribute__($self, name, /)\n--\n\nReturn getattr(self, name)."),
+    TPSLOT("__getattr__", tp_getattro, slot_tp_getattr_hook, NULL, ""),
+    TPSLOT("__setattr__", tp_setattro, slot_tp_setattro, wrap_setattr,
+           "__setattr__($self, name, value, /)\n--\n\nImplement setattr(self, name, value)."),
+    TPSLOT("__delattr__", tp_setattro, slot_tp_setattro, wrap_delattr,
+           "__delattr__($self, name, /)\n--\n\nImplement delattr(self, name)."),
+    TPSLOT("__lt__", tp_richcompare, slot_tp_richcompare, richcmp_lt,
+           "__lt__($self, value, /)\n--\n\nReturn self<value."),
+    TPSLOT("__le__", tp_richcompare, slot_tp_richcompare, richcmp_le,
+           "__le__($self, value, /)\n--\n\nReturn self<=value."),
+    TPSLOT("__eq__", tp_richcompare, slot_tp_richcompare, richcmp_eq,
+           "__eq__($self, value, /)\n--\n\nReturn self==value."),
+    TPSLOT("__ne__", tp_richcompare, slot_tp_richcompare, richcmp_ne,
+           "__ne__($self, value, /)\n--\n\nReturn self!=value."),
+    TPSLOT("__gt__", tp_richcompare, slot_tp_richcompare, richcmp_gt,
+           "__gt__($self, value, /)\n--\n\nReturn self>value."),
+    TPSLOT("__ge__", tp_richcompare, slot_tp_richcompare, richcmp_ge,
+           "__ge__($self, value, /)\n--\n\nReturn self>=value."),
+    TPSLOT("__iter__", tp_iter, slot_tp_iter, wrap_unaryfunc,
+           "__iter__($self, /)\n--\n\nImplement iter(self)."),
+    TPSLOT("__next__", tp_iternext, slot_tp_iternext, wrap_next,
+           "__next__($self, /)\n--\n\nImplement next(self)."),
+    TPSLOT("__get__", tp_descr_get, slot_tp_descr_get, wrap_descr_get,
+           "__get__($self, instance, owner, /)\n--\n\nReturn an attribute of instance, which is of type owner."),
+    TPSLOT("__set__", tp_descr_set, slot_tp_descr_set, wrap_descr_set,
+           "__set__($self, instance, value, /)\n--\n\nSet an attribute of instance to value."),
+    TPSLOT("__delete__", tp_descr_set, slot_tp_descr_set,
+           wrap_descr_delete,
+           "__delete__($self, instance, /)\n--\n\nDelete an attribute of instance."),
+    FLSLOT("__init__", tp_init, slot_tp_init, (wrapperfunc)wrap_init,
+           "__init__($self, /, *args, **kwargs)\n--\n\n"
+           "Initialize self.  See help(type(self)) for accurate signature.",
+           PyWrapperFlag_KEYWORDS),
+    TPSLOT("__new__", tp_new, slot_tp_new, NULL,
+           "__new__(type, /, *args, **kwargs)\n--\n\n"
+           "Create and return new object.  See help(type) for accurate signature."),
+    TPSLOT("__del__", tp_finalize, slot_tp_finalize, (wrapperfunc)wrap_del, ""),
 
-        MPSLOT("__len__", mp_length, slot_mp_length, wrap_lenfunc,
-               "x.__len__() <==> len(x)"),
-        MPSLOT("__getitem__", mp_subscript, slot_mp_subscript,
-               wrap_binaryfunc,
-               "x.__getitem__(y) <==> x[y]"),
-        MPSLOT("__setitem__", mp_ass_subscript, slot_mp_ass_subscript,
-               wrap_objobjargproc,
-               "x.__setitem__(i, y) <==> x[i]=y"),
-        MPSLOT("__delitem__", mp_ass_subscript, slot_mp_ass_subscript,
-               wrap_delitem,
-               "x.__delitem__(y) <==> del x[y]"),
+    AMSLOT("__await__", am_await, slot_am_await, wrap_unaryfunc,
+           "__await__($self, /)\n--\n\nReturn an iterator to be used in await expression."),
+    AMSLOT("__aiter__", am_aiter, slot_am_aiter, wrap_unaryfunc,
+           "__aiter__($self, /)\n--\n\nReturn an awaitable, that resolves in asynchronous iterator."),
+    AMSLOT("__anext__", am_anext, slot_am_anext, wrap_unaryfunc,
+           "__anext__($self, /)\n--\n\nReturn a value or raise StopAsyncIteration."),
 
-        BINSLOT("__add__", nb_add, slot_nb_add,
-                "+"),
-        RBINSLOT("__radd__", nb_add, slot_nb_add,
-                 "+"),
-        BINSLOT("__sub__", nb_subtract, slot_nb_subtract,
-                "-"),
-        RBINSLOT("__rsub__", nb_subtract, slot_nb_subtract,
-                 "-"),
-        BINSLOT("__mul__", nb_multiply, slot_nb_multiply,
-                "*"),
-        RBINSLOT("__rmul__", nb_multiply, slot_nb_multiply,
-                 "*"),
-        BINSLOT("__div__", nb_divide, slot_nb_divide,
-                "/"),
-        RBINSLOT("__rdiv__", nb_divide, slot_nb_divide,
-                 "/"),
-        BINSLOT("__mod__", nb_remainder, slot_nb_remainder,
-                "%"),
-        RBINSLOT("__rmod__", nb_remainder, slot_nb_remainder,
-                 "%"),
-        BINSLOTNOTINFIX("__divmod__", nb_divmod, slot_nb_divmod,
-                "divmod(x, y)"),
-        RBINSLOTNOTINFIX("__rdivmod__", nb_divmod, slot_nb_divmod,
-                 "divmod(y, x)"),
-        NBSLOT("__pow__", nb_power, slot_nb_power, wrap_ternaryfunc,
-               "x.__pow__(y[, z]) <==> pow(x, y[, z])"),
-        NBSLOT("__rpow__", nb_power, slot_nb_power, wrap_ternaryfunc_r,
-               "y.__rpow__(x[, z]) <==> pow(x, y[, z])"),
-        UNSLOT("__neg__", nb_negative, slot_nb_negative, wrap_unaryfunc, "-x"),
-        UNSLOT("__pos__", nb_positive, slot_nb_positive, wrap_unaryfunc, "+x"),
-        UNSLOT("__abs__", nb_absolute, slot_nb_absolute, wrap_unaryfunc,
-               "abs(x)"),
-        UNSLOT("__nonzero__", nb_nonzero, slot_nb_nonzero, wrap_inquirypred,
-               "x != 0"),
-        UNSLOT("__invert__", nb_invert, slot_nb_invert, wrap_unaryfunc, "~x"),
-        BINSLOT("__lshift__", nb_lshift, slot_nb_lshift, "<<"),
-        RBINSLOT("__rlshift__", nb_lshift, slot_nb_lshift, "<<"),
-        BINSLOT("__rshift__", nb_rshift, slot_nb_rshift, ">>"),
-        RBINSLOT("__rrshift__", nb_rshift, slot_nb_rshift, ">>"),
-        BINSLOT("__and__", nb_and, slot_nb_and, "&"),
-        RBINSLOT("__rand__", nb_and, slot_nb_and, "&"),
-        BINSLOT("__xor__", nb_xor, slot_nb_xor, "^"),
-        RBINSLOT("__rxor__", nb_xor, slot_nb_xor, "^"),
-        BINSLOT("__or__", nb_or, slot_nb_or, "|"),
-        RBINSLOT("__ror__", nb_or, slot_nb_or, "|"),
-        NBSLOT("__coerce__", nb_coerce, slot_nb_coerce, wrap_coercefunc,
-               "x.__coerce__(y) <==> coerce(x, y)"),
-        UNSLOT("__int__", nb_int, slot_nb_int, wrap_unaryfunc,
-               "int(x)"),
-        UNSLOT("__long__", nb_long, slot_nb_long, wrap_unaryfunc,
-               "long(x)"),
-        UNSLOT("__float__", nb_float, slot_nb_float, wrap_unaryfunc,
-               "float(x)"),
-        UNSLOT("__oct__", nb_oct, slot_nb_oct, wrap_unaryfunc,
-               "oct(x)"),
-        UNSLOT("__hex__", nb_hex, slot_nb_hex, wrap_unaryfunc,
-               "hex(x)"),
-        NBSLOT("__index__", nb_index, slot_nb_index, wrap_unaryfunc,
-               "x[y:z] <==> x[y.__index__():z.__index__()]"),
-        IBSLOT("__iadd__", nb_inplace_add, slot_nb_inplace_add,
-               wrap_binaryfunc, "+"),
-        IBSLOT("__isub__", nb_inplace_subtract, slot_nb_inplace_subtract,
-               wrap_binaryfunc, "-"),
-        IBSLOT("__imul__", nb_inplace_multiply, slot_nb_inplace_multiply,
-               wrap_binaryfunc, "*"),
-        IBSLOT("__idiv__", nb_inplace_divide, slot_nb_inplace_divide,
-               wrap_binaryfunc, "/"),
-        IBSLOT("__imod__", nb_inplace_remainder, slot_nb_inplace_remainder,
-               wrap_binaryfunc, "%"),
-        IBSLOT("__ipow__", nb_inplace_power, slot_nb_inplace_power,
-               wrap_binaryfunc, "**"),
-        IBSLOT("__ilshift__", nb_inplace_lshift, slot_nb_inplace_lshift,
-               wrap_binaryfunc, "<<"),
-        IBSLOT("__irshift__", nb_inplace_rshift, slot_nb_inplace_rshift,
-               wrap_binaryfunc, ">>"),
-        IBSLOT("__iand__", nb_inplace_and, slot_nb_inplace_and,
-               wrap_binaryfunc, "&"),
-        IBSLOT("__ixor__", nb_inplace_xor, slot_nb_inplace_xor,
-               wrap_binaryfunc, "^"),
-        IBSLOT("__ior__", nb_inplace_or, slot_nb_inplace_or,
-               wrap_binaryfunc, "|"),
-        BINSLOT("__floordiv__", nb_floor_divide, slot_nb_floor_divide, "//"),
-        RBINSLOT("__rfloordiv__", nb_floor_divide, slot_nb_floor_divide, "//"),
-        BINSLOT("__truediv__", nb_true_divide, slot_nb_true_divide, "/"),
-        RBINSLOT("__rtruediv__", nb_true_divide, slot_nb_true_divide, "/"),
-        IBSLOT("__ifloordiv__", nb_inplace_floor_divide,
-               slot_nb_inplace_floor_divide, wrap_binaryfunc, "//"),
-        IBSLOT("__itruediv__", nb_inplace_true_divide,
-               slot_nb_inplace_true_divide, wrap_binaryfunc, "/"),
+    BINSLOT("__add__", nb_add, slot_nb_add,
+           "+"),
+    RBINSLOT("__radd__", nb_add, slot_nb_add,
+           "+"),
+    BINSLOT("__sub__", nb_subtract, slot_nb_subtract,
+           "-"),
+    RBINSLOT("__rsub__", nb_subtract, slot_nb_subtract,
+           "-"),
+    BINSLOT("__mul__", nb_multiply, slot_nb_multiply,
+           "*"),
+    RBINSLOT("__rmul__", nb_multiply, slot_nb_multiply,
+           "*"),
+    BINSLOT("__mod__", nb_remainder, slot_nb_remainder,
+           "%"),
+    RBINSLOT("__rmod__", nb_remainder, slot_nb_remainder,
+           "%"),
+    BINSLOTNOTINFIX("__divmod__", nb_divmod, slot_nb_divmod,
+           "Return divmod(self, value)."),
+    RBINSLOTNOTINFIX("__rdivmod__", nb_divmod, slot_nb_divmod,
+           "Return divmod(value, self)."),
+    NBSLOT("__pow__", nb_power, slot_nb_power, wrap_ternaryfunc,
+           "__pow__($self, value, mod=None, /)\n--\n\nReturn pow(self, value, mod)."),
+    NBSLOT("__rpow__", nb_power, slot_nb_power, wrap_ternaryfunc_r,
+           "__rpow__($self, value, mod=None, /)\n--\n\nReturn pow(value, self, mod)."),
+    UNSLOT("__neg__", nb_negative, slot_nb_negative, wrap_unaryfunc, "-self"),
+    UNSLOT("__pos__", nb_positive, slot_nb_positive, wrap_unaryfunc, "+self"),
+    UNSLOT("__abs__", nb_absolute, slot_nb_absolute, wrap_unaryfunc,
+           "abs(self)"),
+    UNSLOT("__bool__", nb_bool, slot_nb_bool, wrap_inquirypred,
+           "self != 0"),
+    UNSLOT("__invert__", nb_invert, slot_nb_invert, wrap_unaryfunc, "~self"),
+    BINSLOT("__lshift__", nb_lshift, slot_nb_lshift, "<<"),
+    RBINSLOT("__rlshift__", nb_lshift, slot_nb_lshift, "<<"),
+    BINSLOT("__rshift__", nb_rshift, slot_nb_rshift, ">>"),
+    RBINSLOT("__rrshift__", nb_rshift, slot_nb_rshift, ">>"),
+    BINSLOT("__and__", nb_and, slot_nb_and, "&"),
+    RBINSLOT("__rand__", nb_and, slot_nb_and, "&"),
+    BINSLOT("__xor__", nb_xor, slot_nb_xor, "^"),
+    RBINSLOT("__rxor__", nb_xor, slot_nb_xor, "^"),
+    BINSLOT("__or__", nb_or, slot_nb_or, "|"),
+    RBINSLOT("__ror__", nb_or, slot_nb_or, "|"),
+    UNSLOT("__int__", nb_int, slot_nb_int, wrap_unaryfunc,
+           "int(self)"),
+    UNSLOT("__float__", nb_float, slot_nb_float, wrap_unaryfunc,
+           "float(self)"),
+    IBSLOT("__iadd__", nb_inplace_add, slot_nb_inplace_add,
+           wrap_binaryfunc, "+="),
+    IBSLOT("__isub__", nb_inplace_subtract, slot_nb_inplace_subtract,
+           wrap_binaryfunc, "-="),
+    IBSLOT("__imul__", nb_inplace_multiply, slot_nb_inplace_multiply,
+           wrap_binaryfunc, "*="),
+    IBSLOT("__imod__", nb_inplace_remainder, slot_nb_inplace_remainder,
+           wrap_binaryfunc, "%="),
+    IBSLOT("__ipow__", nb_inplace_power, slot_nb_inplace_power,
+           wrap_binaryfunc, "**="),
+    IBSLOT("__ilshift__", nb_inplace_lshift, slot_nb_inplace_lshift,
+           wrap_binaryfunc, "<<="),
+    IBSLOT("__irshift__", nb_inplace_rshift, slot_nb_inplace_rshift,
+           wrap_binaryfunc, ">>="),
+    IBSLOT("__iand__", nb_inplace_and, slot_nb_inplace_and,
+           wrap_binaryfunc, "&="),
+    IBSLOT("__ixor__", nb_inplace_xor, slot_nb_inplace_xor,
+           wrap_binaryfunc, "^="),
+    IBSLOT("__ior__", nb_inplace_or, slot_nb_inplace_or,
+           wrap_binaryfunc, "|="),
+    BINSLOT("__floordiv__", nb_floor_divide, slot_nb_floor_divide, "//"),
+    RBINSLOT("__rfloordiv__", nb_floor_divide, slot_nb_floor_divide, "//"),
+    BINSLOT("__truediv__", nb_true_divide, slot_nb_true_divide, "/"),
+    RBINSLOT("__rtruediv__", nb_true_divide, slot_nb_true_divide, "/"),
+    IBSLOT("__ifloordiv__", nb_inplace_floor_divide,
+           slot_nb_inplace_floor_divide, wrap_binaryfunc, "//="),
+    IBSLOT("__itruediv__", nb_inplace_true_divide,
+           slot_nb_inplace_true_divide, wrap_binaryfunc, "/="),
+    NBSLOT("__index__", nb_index, slot_nb_index, wrap_unaryfunc,
+           "__index__($self, /)\n--\n\n"
+           "Return self converted to an integer, if self is suitable "
+           "for use as an index into a list."),
+    BINSLOT("__matmul__", nb_matrix_multiply, slot_nb_matrix_multiply,
+            "@"),
+    RBINSLOT("__rmatmul__", nb_matrix_multiply, slot_nb_matrix_multiply,
+             "@"),
+    IBSLOT("__imatmul__", nb_inplace_matrix_multiply, slot_nb_inplace_matrix_multiply,
+           wrap_binaryfunc, "@="),
+    MPSLOT("__len__", mp_length, slot_mp_length, wrap_lenfunc,
+           "__len__($self, /)\n--\n\nReturn len(self)."),
+    MPSLOT("__getitem__", mp_subscript, slot_mp_subscript,
+           wrap_binaryfunc,
+           "__getitem__($self, key, /)\n--\n\nReturn self[key]."),
+    MPSLOT("__setitem__", mp_ass_subscript, slot_mp_ass_subscript,
+           wrap_objobjargproc,
+           "__setitem__($self, key, value, /)\n--\n\nSet self[key] to value."),
+    MPSLOT("__delitem__", mp_ass_subscript, slot_mp_ass_subscript,
+           wrap_delitem,
+           "__delitem__($self, key, /)\n--\n\nDelete self[key]."),
 
-        TPSLOT("__str__", tp_str, slot_tp_str, wrap_unaryfunc,
-               "x.__str__() <==> str(x)"),
-        TPSLOT("__str__", tp_print, NULL, NULL, ""),
-        TPSLOT("__repr__", tp_repr, slot_tp_repr, wrap_unaryfunc,
-               "x.__repr__() <==> repr(x)"),
-        TPSLOT("__repr__", tp_print, NULL, NULL, ""),
-        TPSLOT("__cmp__", tp_compare, _PyObject_SlotCompare, wrap_cmpfunc,
-               "x.__cmp__(y) <==> cmp(x,y)"),
-        TPSLOT("__hash__", tp_hash, slot_tp_hash, wrap_hashfunc,
-               "x.__hash__() <==> hash(x)"),
-        FLSLOT("__call__", tp_call, slot_tp_call, (wrapperfunc)wrap_call,
-               "x.__call__(...) <==> x(...)", PyWrapperFlag_KEYWORDS),
-        TPSLOT("__getattribute__", tp_getattro, slot_tp_getattr_hook,
-               wrap_binaryfunc, "x.__getattribute__('name') <==> x.name"),
-        TPSLOT("__getattr__", tp_getattro, slot_tp_getattr, NULL, ""),
-        TPSLOT("__getattr__", tp_getattr, NULL, NULL, ""),
-        TPSLOT("__setattr__", tp_setattro, slot_tp_setattro, wrap_setattr,
-               "x.__setattr__('name', value) <==> x.name = value"),
-        TPSLOT("__setattr__", tp_setattr, NULL, NULL, ""),
-        TPSLOT("__delattr__", tp_setattro, slot_tp_setattro, wrap_delattr,
-               "x.__delattr__('name') <==> del x.name"),
-        TPSLOT("__delattr__", tp_setattr, NULL, NULL, ""),
-        TPSLOT("__lt__", tp_richcompare, slot_tp_richcompare, richcmp_lt,
-               "x.__lt__(y) <==> x<y"),
-        TPSLOT("__le__", tp_richcompare, slot_tp_richcompare, richcmp_le,
-               "x.__le__(y) <==> x<=y"),
-        TPSLOT("__eq__", tp_richcompare, slot_tp_richcompare, richcmp_eq,
-               "x.__eq__(y) <==> x==y"),
-        TPSLOT("__ne__", tp_richcompare, slot_tp_richcompare, richcmp_ne,
-               "x.__ne__(y) <==> x!=y"),
-        TPSLOT("__gt__", tp_richcompare, slot_tp_richcompare, richcmp_gt,
-               "x.__gt__(y) <==> x>y"),
-        TPSLOT("__ge__", tp_richcompare, slot_tp_richcompare, richcmp_ge,
-               "x.__ge__(y) <==> x>=y"),
-        TPSLOT("__iter__", tp_iter, slot_tp_iter, wrap_unaryfunc,
-               "x.__iter__() <==> iter(x)"),
-        TPSLOT("next", tp_iternext, slot_tp_iternext, wrap_next,
-               "x.next() -> the next value, or raise StopIteration"),
-        TPSLOT("__get__", tp_descr_get, slot_tp_descr_get, wrap_descr_get,
-               "descr.__get__(obj[, type]) -> value"),
-        TPSLOT("__set__", tp_descr_set, slot_tp_descr_set, wrap_descr_set,
-               "descr.__set__(obj, value)"),
-        TPSLOT("__delete__", tp_descr_set, slot_tp_descr_set,
-               wrap_descr_delete, "descr.__delete__(obj)"),
-        FLSLOT("__init__", tp_init, slot_tp_init, (wrapperfunc)wrap_init,
-               "x.__init__(...) initializes x; "
-               "see x.__class__.__doc__ for signature",
-               PyWrapperFlag_KEYWORDS),
-        TPSLOT("__new__", tp_new, slot_tp_new, NULL, ""),
-        TPSLOT("__del__", tp_del, slot_tp_del, NULL, ""),
-        {NULL}
+    SQSLOT("__len__", sq_length, slot_sq_length, wrap_lenfunc,
+           "__len__($self, /)\n--\n\nReturn len(self)."),
+
+    SQSLOT("__add__", sq_concat, NULL, wrap_binaryfunc,
+           "__add__($self, value, /)\n--\n\nReturn self+value."),
+    SQSLOT("__mul__", sq_repeat, NULL, wrap_indexargfunc,
+           "__mul__($self, value, /)\n--\n\nReturn self*value.n"),
+    SQSLOT("__rmul__", sq_repeat, NULL, wrap_indexargfunc,
+           "__rmul__($self, value, /)\n--\n\nReturn self*value."),
+    SQSLOT("__getitem__", sq_item, slot_sq_item, wrap_sq_item,
+           "__getitem__($self, key, /)\n--\n\nReturn self[key]."),
+    SQSLOT("__setitem__", sq_ass_item, slot_sq_ass_item, wrap_sq_setitem,
+           "__setitem__($self, key, value, /)\n--\n\nSet self[key] to value."),
+    SQSLOT("__delitem__", sq_ass_item, slot_sq_ass_item, wrap_sq_delitem,
+           "__delitem__($self, key, /)\n--\n\nDelete self[key]."),
+    SQSLOT("__contains__", sq_contains, slot_sq_contains, wrap_objobjproc,
+           "__contains__($self, key, /)\n--\n\nReturn key in self."),
+    SQSLOT("__iadd__", sq_inplace_concat, NULL,
+           wrap_binaryfunc,
+           "__iadd__($self, value, /)\n--\n\nImplement self+=value."),
+    SQSLOT("__imul__", sq_inplace_repeat, NULL,
+           wrap_indexargfunc,
+           "__imul__($self, value, /)\n--\n\nImplement self*=value."),
+
+    {NULL}
 };
 """
 
@@ -1192,7 +1158,7 @@ slotdefs += (
     TPSLOT("__buffer__", "tp_as_buffer.c_bf_getbuffer", None, "wrap_getbuffer", ""),
 )
 
-if not PY3:
+if PY2:
     slotdefs += (
         TPSLOT("__rbuffer__", "tp_as_buffer.c_bf_getreadbuffer", None, "wrap_getreadbuffer", ""),
         TPSLOT("__wbuffer__", "tp_as_buffer.c_bf_getwritebuffer", None, "wrap_getwritebuffer", ""),
