@@ -12,7 +12,7 @@ class TestCodecs(BaseApiTest):
         w_encoded = space.call_method(w_encoder, 'encode', space.wrap(u'späm'))
         w_decoder = PyCodec_IncrementalDecoder(space, utf8, None)
         w_decoded = space.call_method(w_decoder, 'decode', w_encoded)
-        assert space.utf8_w(w_decoded) == u'späm'.encode('utf8')
+        assert space.utf8_w(w_decoded) == u'späm'.encode("utf-8")
         rffi.free_charp(utf8)
 
     def test_encoder_decoder(self, space):
@@ -20,7 +20,7 @@ class TestCodecs(BaseApiTest):
         w_encoder = PyCodec_Encoder(space, utf8)
         w_decoder = PyCodec_Decoder(space, utf8)
         rffi.free_charp(utf8)
-        space.appexec([w_encoder, w_decoder], """(encoder, decoder):
-            assert encoder(u"\u1234") == ('\xe1\x88\xb4', 1)
-            assert decoder("\xe1\x88\xb4") == (u'\u1234', 3)
+        space.appexec([w_encoder, w_decoder], r"""(encoder, decoder):
+            assert encoder(u"\u1234") == (b"\xe1\x88\xb4", 1)
+            assert decoder(b"\xe1\x88\xb4") == (u"\u1234", 3)
         """)
