@@ -5,7 +5,8 @@ from pypy.module.cpyext.test.test_cpyext import AppTestCpythonExtensionBase
 from pypy.module.cpyext.test.test_api import BaseApiTest
 from pypy.module.cpyext.api import generic_cpy_call
 from pypy.module.cpyext.pyobject import make_ref, from_ref, decref, as_pyobj
-from pypy.module.cpyext.typeobject import PyTypeObjectPtr, W_PyCTypeObject
+from pypy.module.cpyext.typeobject import cts, PyTypeObjectPtr, W_PyCTypeObject
+
 
 
 class AppTestTypeObject(AppTestCpythonExtensionBase):
@@ -42,7 +43,7 @@ class AppTestTypeObject(AppTestCpythonExtensionBase):
         assert not "im_func" in dir(module.fooType.copy)
         assert module.fooType.copy.__objclass__ is module.fooType
         assert "copy" in repr(module.fooType.copy)
-        assert repr(module.fooType) == "<type 'foo.foo'>"
+        assert repr(module.fooType) == "<class 'foo.foo'>"
         assert repr(obj2) == "<Foo>"
         assert repr(module.fooType.__call__) == "<slot wrapper '__call__' of 'foo.foo' objects>"
         assert obj2(foo=1, bar=2) == dict(foo=1, bar=2)
@@ -59,14 +60,14 @@ class AppTestTypeObject(AppTestCpythonExtensionBase):
         obj.int_member = 42
         raises(TypeError, "obj.int_member = 'not a number'")
         raises(TypeError, "del obj.int_member")
-        raises(TypeError, "obj.int_member_readonly = 42")
-        exc = raises(TypeError, "del obj.int_member_readonly")
+        raises(AttributeError, "obj.int_member_readonly = 42")
+        exc = raises(AttributeError, "del obj.int_member_readonly")
         assert "readonly" in str(exc.value)
         raises(SystemError, "obj.broken_member")
         raises(SystemError, "obj.broken_member = 42")
         assert module.fooType.broken_member.__doc__ is None
         assert module.fooType.object_member.__doc__ == "A Python object."
-        assert str(type(module.fooType.int_member)) == "<type 'member_descriptor'>"
+        assert str(type(module.fooType.int_member)) == "<class 'member_descriptor'>"
 
     def test_typeobject_object_member(self):
         module = self.import_module(name='foo')
@@ -106,7 +107,7 @@ class AppTestTypeObject(AppTestCpythonExtensionBase):
         raises(TypeError, "obj.char_member = 42")
         #
         import sys
-        bignum = sys.maxint - 42
+        bignum = sys.maxsize - 42
         obj.short_member = -12345;     assert obj.short_member == -12345
         obj.long_member = -bignum;     assert obj.long_member == -bignum
         obj.ushort_member = 45678;     assert obj.ushort_member == 45678
@@ -119,7 +120,7 @@ class AppTestTypeObject(AppTestCpythonExtensionBase):
         obj.double_member = 9.25;      assert obj.double_member == 9.25
         obj.longlong_member = -2**59;  assert obj.longlong_member == -2**59
         obj.ulonglong_member = 2**63;  assert obj.ulonglong_member == 2**63
-        obj.ssizet_member = sys.maxint;assert obj.ssizet_member == sys.maxint
+        obj.ssizet_member = sys.maxsize;assert obj.ssizet_member == sys.maxsize
         #
 
     def test_staticmethod(self):
@@ -199,7 +200,7 @@ class AppTestTypeObject(AppTestCpythonExtensionBase):
         obj2 = module.UnicodeSubtype2()
         obj3 = module.UnicodeSubtype3()
         assert obj3.get_val() == 42
-        assert len(type(obj3).mro()) == 6
+        assert len(type(obj3).mro()) == 5
 
     def test_init(self):
         module = self.import_module(name="foo")
@@ -245,29 +246,6 @@ class AppTestTypeObject(AppTestCpythonExtensionBase):
         module = self.import_module(name='foo')
         raises(TypeError, module.MetaType, 'other', (module.fooType,), {})
 
-    def test_sre(self):
-        import sys
-        for m in ['_sre', 'sre_compile', 'sre_constants', 'sre_parse', 're']:
-            # clear out these modules
-            try:
-                del sys.modules[m]
-            except KeyError:
-                pass
-        module = self.import_module(name='_sre')
-        import re
-        assert re.sre_compile._sre is module
-        s = u"Foo " * 1000 + u"Bar"
-        prog = re.compile(u"Foo.*Bar")
-        assert prog.match(s)
-        m = re.search(u"xyz", u"xyzxyz")
-        assert m
-        m = re.search("xyz", "xyzxyz")
-        assert m
-        assert "groupdict" in dir(m)
-        re._cache.clear()
-        re._cache_repl.clear()
-        del prog, m
-
     def test_init_error(self):
         module = self.import_module("foo")
         raises(ValueError, module.InitErrType)
@@ -283,23 +261,13 @@ class AppTestTypeObject(AppTestCpythonExtensionBase):
         cmpr = module.CmpType()
 
         # should not crash
-        cmpr < 4
-        cmpr <= 4
-        cmpr > 4
-        cmpr >= 4
+        raises(TypeError, "cmpr < 4")
+        raises(TypeError, "cmpr <= 4")
+        raises(TypeError, "cmpr > 4")
+        raises(TypeError, "cmpr >= 4")
 
         assert cmpr.__le__(4) is NotImplemented
 
-    def test_tpcompare(self):
-        module = self.import_module("comparisons")
-        cmpr = module.OldCmpType()
-        assert cmpr < cmpr
-
-    def test_unhashable_when_tpcompare(self):
-        module = self.import_module("comparisons")
-        cmpr = module.OldCmpType()
-        raises(TypeError, hash, cmpr)
-
     def test_hash(self):
         module = self.import_module("comparisons")
         cmpr = module.CmpType()
@@ -309,6 +277,13 @@ class AppTestTypeObject(AppTestCpythonExtensionBase):
         assert d[cmpr] == 72
         assert d[3] == 72
 
+    def test_hash_inheritance(self):
+        foo = self.import_module("foo")
+        assert hash(foo.UnicodeSubtype(u'xyz')) == hash(u'xyz')
+        assert foo.UnicodeSubtype.__hash__ is str.__hash__
+        assert hash(foo.UnicodeSubtype3(u'xyz')) == hash(u'xyz')
+        assert foo.UnicodeSubtype3.__hash__ is str.__hash__
+
     def test_descriptor(self):
         module = self.import_module("foo")
         prop = module.Property()
@@ -391,7 +366,7 @@ class AppTestTypeObject(AppTestCpythonExtensionBase):
             ("setattr", "METH_O",
              '''
                 int ret;
-                PyObject* name = PyString_FromString("mymodule");
+                PyObject* name = PyBytes_FromString("mymodule");
                 PyObject *obj = PyType_Type.tp_alloc(&PyType_Type, 0);
                 PyHeapTypeObject *type = (PyHeapTypeObject*)obj;
                 /* this is issue #2434: logic from pybind11 */
@@ -452,30 +427,6 @@ class AppTestTypeObject(AppTestCpythonExtensionBase):
         assert module.hack_tp_dict(obj, "b") == 2
 
 
-    def test_tp_dict_ready(self):
-        module = self.import_extension('foo', [
-           ("new_obj", "METH_NOARGS",
-            '''
-                PyObject *obj;
-                obj = PyObject_New(PyObject, &Foo_Type);
-                return obj;
-            '''
-            )], prologue='''
-            static PyTypeObject Foo_Type = {
-                PyVarObject_HEAD_INIT(NULL, 0)
-                "foo.foo",
-            };
-            ''', more_init = '''
-                Foo_Type.tp_flags = Py_TPFLAGS_DEFAULT;
-                Foo_Type.tp_dict = PyDict_New();
-                PyDict_SetItemString(Foo_Type.tp_dict, "inserted", Py_True);
-                if (PyType_Ready(&Foo_Type) < 0) INITERROR;
-            ''')
-
-        obj = module.new_obj()
-        assert type(obj).inserted is True
-
-
     def test_tp_descr_get(self):
         module = self.import_extension('foo', [
            ("tp_descr_get", "METH_O",
@@ -485,7 +436,7 @@ class AppTestTypeObject(AppTestCpythonExtensionBase):
                     return Py_False;
                 }
                 return args->ob_type->tp_descr_get(args, NULL,
-                                                   (PyObject *)&PyInt_Type);
+                                                   (PyObject *)&PyLong_Type);
              '''
              )
             ])
@@ -543,6 +494,17 @@ class AppTestTypeObject(AppTestCpythonExtensionBase):
         p = property(lambda: "never used", pset, pdel)
         assert module.tp_descr_set(p) is True
 
+    def test_text_signature(self):
+        import sys
+        module = self.import_module(name='docstrings')
+        assert module.SomeType.__text_signature__ == '()'
+        assert module.SomeType.__doc__ == 'A type with a signature'
+        if '__pypy__' in sys.modules:
+            assert module.HeapType.__text_signature__ == '()'
+        else:  # XXX: bug in CPython?
+            assert module.HeapType.__text_signature__ is None
+        assert module.HeapType.__doc__ == 'A type with a signature'
+
 
 class TestTypes(BaseApiTest):
     def test_type_attributes(self, space, api):
@@ -597,6 +559,9 @@ class TestTypes(BaseApiTest):
         assert w_obj is None
         assert api.PyErr_Occurred() is None
 
+    def test_typeslots(self, space):
+        assert cts.macros['Py_tp_doc'] == 56
+
     def test_subclass_not_PyCTypeObject(self, space, api):
         pyobj = make_ref(space, api.PyLong_Type)
         py_type = rffi.cast(PyTypeObjectPtr, pyobj)
@@ -825,7 +790,8 @@ class AppTestSlots(AppTestCpythonExtensionBase):
         class C:
             def __call__(self, *args):
                 return args
-        assert module.tp_call(type(C()), C(), ('x', 2)) == ('x', 2)
+        ret = module.tp_call(C, C(), ('x', 2))
+        assert ret == ('x', 2)
         class D(type):
             def __call__(self, *args):
                 return "foo! %r" % (args,)
@@ -959,47 +925,10 @@ class AppTestSlots(AppTestCpythonExtensionBase):
         res = "foo" in obj
         assert res is True
 
-    def test_sq_ass_slice(self):
-        module = self.import_extension('foo', [
-           ("new_obj", "METH_NOARGS",
-            '''
-                PyObject *obj;
-                obj = PyObject_New(PyObject, &Foo_Type);
-                return obj;
-            '''
-            )], prologue='''
             #if PY_MAJOR_VERSION > 2
             #define PyInt_Check PyLong_Check
             #define PyInt_AsLong PyLong_AsLong
             #endif
-            static int
-            sq_ass_slice(PyObject *self, Py_ssize_t a, Py_ssize_t b, PyObject *o)
-            {
-                int expected = (a == 10 && b == 20 &&
-                                PyInt_Check(o) && PyInt_AsLong(o) == 42);
-                if (!expected) {
-                    PyErr_SetString(PyExc_ValueError, "test failed");
-                    return -1;
-                }
-                return 0;
-            }
-            PySequenceMethods tp_as_sequence;
-            static PyTypeObject Foo_Type = {
-                PyVarObject_HEAD_INIT(NULL, 0)
-                "foo.foo",
-            };
-            ''', more_init='''
-                Foo_Type.tp_flags = Py_TPFLAGS_DEFAULT;
-                Foo_Type.tp_as_sequence = &tp_as_sequence;
-                tp_as_sequence.sq_ass_slice = sq_ass_slice;
-                if (PyType_Ready(&Foo_Type) < 0) INITERROR;
-            ''')
-        obj = module.new_obj()
-        obj[10:20] = 42
-        raises(ValueError, "obj[10:20] = 43")
-        raises(ValueError, "obj[11:20] = 42")
-        raises(ValueError, "obj[10:21] = 42")
-
     def test_sq_ass_item(self):
         module = self.import_extension('foo', [
            ("new_obj", "METH_NOARGS",
@@ -1073,6 +1002,8 @@ class AppTestSlots(AppTestCpythonExtensionBase):
                      return NULL;
                  }
                  result = type->tp_iternext(obj);
+                 /* In py3, returning NULL from tp_iternext means the iterator
+                  * is exhausted */
                  if (!result && !PyErr_Occurred())
                      result = PyString_FromString("stop!");
                  return result;
@@ -1099,7 +1030,7 @@ class AppTestSlots(AppTestCpythonExtensionBase):
             ("newInt", "METH_VARARGS",
              """
                 IntLikeObject *intObj;
-                int intval;
+                long intval;
 
                 if (!PyArg_ParseTuple(args, "i", &intval))
                     return NULL;
@@ -1125,7 +1056,7 @@ class AppTestSlots(AppTestCpythonExtensionBase):
                 intObj->value = intval;
                 isint = PyNumber_Check((PyObject*)intObj);
                 Py_DECREF((PyObject*)intObj);
-                return PyInt_FromLong(isint);
+                return PyLong_FromLong(isint);
             """),
             ], prologue= """
             typedef struct
@@ -1135,7 +1066,7 @@ class AppTestSlots(AppTestCpythonExtensionBase):
             } IntLikeObject;
 
             static int
-            intlike_nb_nonzero(PyObject *o)
+            intlike_nb_bool(PyObject *o)
             {
                 IntLikeObject *v = (IntLikeObject*)o;
                 if (v->value == -42) {
@@ -1150,7 +1081,7 @@ class AppTestSlots(AppTestCpythonExtensionBase):
             intlike_nb_int(PyObject* o)
             {
                 IntLikeObject *v = (IntLikeObject*)o;
-                return PyInt_FromLong(v->value);
+                return PyLong_FromLong(v->value);
             }
 
             PyTypeObject IntLike_Type = {
@@ -1162,7 +1093,7 @@ class AppTestSlots(AppTestCpythonExtensionBase):
             """, more_init="""
             IntLike_Type.tp_flags |= Py_TPFLAGS_DEFAULT;
             IntLike_Type.tp_as_number = &intlike_as_number;
-            intlike_as_number.nb_nonzero = intlike_nb_nonzero;
+            intlike_as_number.nb_bool = intlike_nb_bool;
             intlike_as_number.nb_int = intlike_nb_int;
             PyType_Ready(&IntLike_Type);
             """)
@@ -1264,11 +1195,11 @@ class AppTestSlots(AppTestCpythonExtensionBase):
             };
             """, more_init="""
                 IntLike_Type.tp_as_number = &intlike_as_number;
-                IntLike_Type.tp_flags |= Py_TPFLAGS_DEFAULT | Py_TPFLAGS_CHECKTYPES;
+                IntLike_Type.tp_flags |= Py_TPFLAGS_DEFAULT;
                 intlike_as_number.nb_add = intlike_nb_add;
                 intlike_as_number.nb_power = intlike_nb_pow;
                 if (PyType_Ready(&IntLike_Type) < 0) INITERROR;
-                IntLike_Type_NoOp.tp_flags |= Py_TPFLAGS_DEFAULT | Py_TPFLAGS_CHECKTYPES;
+                IntLike_Type_NoOp.tp_flags |= Py_TPFLAGS_DEFAULT;
                 if (PyType_Ready(&IntLike_Type_NoOp) < 0) INITERROR;
             """)
         a = module.newInt(1)
@@ -1320,12 +1251,10 @@ class AppTestSlots(AppTestCpythonExtensionBase):
         except TypeError as e:
             import sys
             if '__pypy__' in sys.builtin_module_names:
-                print(str(e))
-                assert 'instance layout conflicts in multiple inheritance' in str(e)
+                assert str(e) == 'instance layout conflicts in multiple inheritance'
 
             else:
-                assert str(e) == ('Error when calling the metaclass bases\n'
-                          '    multiple bases have instance lay-out conflict')
+                assert 'instance lay-out conflict' in str(e)
         else:
             raise AssertionError("did not get TypeError!")
 
@@ -1351,7 +1280,7 @@ class AppTestSlots(AppTestCpythonExtensionBase):
              """),
             ("getCounter", "METH_NOARGS",
              """
-                return PyInt_FromLong(foo_counter);
+                return PyLong_FromLong(foo_counter);
              """)], prologue="""
             typedef struct {
                 PyObject_HEAD
@@ -1389,8 +1318,7 @@ class AppTestSlots(AppTestCpythonExtensionBase):
             """, more_init="""
                 Foo_Type.tp_basicsize = sizeof(FooObject);
                 Foo_Type.tp_dealloc = &dealloc_foo;
-                Foo_Type.tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_CHECKTYPES
-                                    | Py_TPFLAGS_BASETYPE;
+                Foo_Type.tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE;
                 Foo_Type.tp_new = &new_foo;
                 Foo_Type.tp_free = &PyObject_Del;
                 if (PyType_Ready(&Foo_Type) < 0) INITERROR;
@@ -1442,7 +1370,7 @@ class AppTestSlots(AppTestCpythonExtensionBase):
             static PyObject *
             my_tp_call(PyObject *self, PyObject *args, PyObject *kwds)
             {
-                return PyInt_FromLong(42);
+                return PyLong_FromLong(42);
             }
             static PyTypeObject Foo_Type = {
                 PyVarObject_HEAD_INIT(NULL, 0)
@@ -1477,8 +1405,11 @@ class AppTestSlots(AppTestCpythonExtensionBase):
         FooType = module.getMetaClass()
         if not self.runappdirect:
             self._check_type_object(FooType)
-        class X(object):
-            __metaclass__ = FooType
+
+        # 2 vs 3 shenanigans to declare
+        # class X(object, metaclass=FooType): pass
+        X = FooType('X', (object,), {})
+
         X()
 
     def test_multiple_inheritance3(self):
@@ -1550,27 +1481,6 @@ class AppTestSlots(AppTestCpythonExtensionBase):
         bases = module.foo(C)
         assert bases == (A, B)
 
-    def test_multiple_inheritance_old_style_base(self):
-        module = self.import_extension('foo', [
-           ("foo", "METH_O",
-            '''
-                PyTypeObject *tp;
-                tp = (PyTypeObject*)args;
-                Py_INCREF(tp->tp_bases);
-                return tp->tp_bases;
-            '''
-            )])
-        # used to segfault after some iterations
-        for i in range(11):
-            class A(object):
-                pass
-            class B:
-                pass
-            class C(A, B):
-                pass
-            bases = module.foo(C)
-            assert bases == (A, B)
-
     def test_getattr_getattro(self):
         module = self.import_module(name='foo')
         assert module.gettype2.dcba == b'getattro:dcba'
@@ -1714,6 +1624,28 @@ class AppTestSlots(AppTestCpythonExtensionBase):
         assert module.get_number() == 4043
         raises(AttributeError, "del foo.bar")
 
+    def test_tp_doc_issue3055(self):
+        module = self.import_extension('foo', [
+           ("new_obj", "METH_NOARGS",
+            '''
+                PyObject *obj;
+                obj = PyObject_New(PyObject, &Foo_Type);
+                return obj;
+            '''
+            )], prologue='''
+            static PyTypeObject Foo_Type = {
+                PyVarObject_HEAD_INIT(NULL, 0)
+                "foo.foo",
+                sizeof(PyObject),
+            };
+            ''', more_init = '''
+                Foo_Type.tp_flags = Py_TPFLAGS_DEFAULT;
+                Foo_Type.tp_doc = "";
+                if (PyType_Ready(&Foo_Type) < 0) INITERROR;
+            ''')
+        obj = module.new_obj()
+        assert type(obj).__doc__ is None
+
 
 class AppTestHashable(AppTestCpythonExtensionBase):
     def test_unhashable(self):
@@ -1769,22 +1701,19 @@ class AppTestFlags(AppTestCpythonExtensionBase):
                 return PyLong_FromLong(0);
             '''),])
         # copied from object.h
-        Py_TPFLAGS_INT_SUBCLASS       = (1L<<23) # goes away on py3
-        Py_TPFLAGS_LONG_SUBCLASS      = (1L<<24)
-        Py_TPFLAGS_LIST_SUBCLASS      = (1L<<25)
-        Py_TPFLAGS_TUPLE_SUBCLASS     = (1L<<26)
-        Py_TPFLAGS_STRING_SUBCLASS    = (1L<<27) # rename to BYTES on py3
-        Py_TPFLAGS_UNICODE_SUBCLASS   = (1L<<28)
-        Py_TPFLAGS_DICT_SUBCLASS      = (1L<<29)
-        Py_TPFLAGS_BASE_EXC_SUBCLASS  = (1L<<30)
-        Py_TPFLAGS_TYPE_SUBCLASS      = (1L<<31)
-        for t,f in ((long, Py_TPFLAGS_LONG_SUBCLASS),
-                    (int, Py_TPFLAGS_INT_SUBCLASS),
+        Py_TPFLAGS_LONG_SUBCLASS = (1<<24)
+        Py_TPFLAGS_LIST_SUBCLASS = (1<<25)
+        Py_TPFLAGS_TUPLE_SUBCLASS = (1<<26)
+        Py_TPFLAGS_BYTES_SUBCLASS = (1<<27)
+        Py_TPFLAGS_UNICODE_SUBCLASS = (1<<28)
+        Py_TPFLAGS_DICT_SUBCLASS = (1<<29)
+        Py_TPFLAGS_BASE_EXC_SUBCLASS = (1<<30)
+        Py_TPFLAGS_TYPE_SUBCLASS = (1<<31)
+        for t,f in ((int, Py_TPFLAGS_LONG_SUBCLASS),
                     (list, Py_TPFLAGS_LIST_SUBCLASS),
                     (tuple, Py_TPFLAGS_TUPLE_SUBCLASS),
-                    (bytes, Py_TPFLAGS_STRING_SUBCLASS),
-                    (str, Py_TPFLAGS_STRING_SUBCLASS),
-                    (unicode, Py_TPFLAGS_UNICODE_SUBCLASS),
+                    (bytes, Py_TPFLAGS_BYTES_SUBCLASS),
+                    (str, Py_TPFLAGS_UNICODE_SUBCLASS),
                     (dict, Py_TPFLAGS_DICT_SUBCLASS),
                     (Exception, Py_TPFLAGS_BASE_EXC_SUBCLASS),
                     (type, Py_TPFLAGS_TYPE_SUBCLASS),
@@ -1820,44 +1749,3 @@ class AppTestFlags(AppTestCpythonExtensionBase):
         assert module.test_pypy_flags(float, Py_TPPYPYFLAGS_FLOAT_SUBCLASS) == 0
         assert module.test_pypy_flags(MyFloat, Py_TPPYPYFLAGS_FLOAT_SUBCLASS) == 0
 
-    def test_newgetset(self):
-        # Taken from the yara-python project
-        module = self.import_extension('foo', [
-            ('newexc', 'METH_NOARGS',
-             """
-                PyObject *YaraWarningError = PyErr_NewException("foo.YaraWarningError", PyExc_Exception, NULL);
-
-                PyTypeObject *YaraWarningError_type = (PyTypeObject *) YaraWarningError;
-                PyObject* descr = PyDescr_NewGetSet(YaraWarningError_type,
-                                                    YaraWarningError_getsetters);
-                if (PyDict_SetItem(YaraWarningError_type->tp_dict,
-                                   PyDescr_NAME(descr), descr) < 0) {
-                    Py_DECREF(descr);
-                    return NULL;
-                }
-                return YaraWarningError;
-            """),
-            ], prologue="""
-                static PyObject* YaraWarningError_getwarnings(PyObject *self, void* closure)
-                {
-                  PyObject *args = PyObject_GetAttrString(self, "args");
-                  if (!args) {
-                    return NULL;
-                  }
-
-                  PyObject* ret = PyTuple_GetItem(args, 0);
-                  Py_XINCREF(ret);
-                  Py_XDECREF(args);
-                  return ret;
-                }
-
-                static PyGetSetDef YaraWarningError_getsetters[] = {
-                  {"warnings", YaraWarningError_getwarnings, NULL, NULL, NULL},
-                  {NULL}
-                };
-             """)
-        errtype = module.newexc()
-        err = errtype("abc")
-        assert err.warnings == "abc"
-
-
