@@ -11,37 +11,24 @@ from rpython.tool.jitlogparser.parser import (SimpleParser, Function,
 from rpython.tool.jitlogparser.storage import LoopStorage
 
 
-def find_ids_range(code):
+def find_id_linenos(code):
     """
     Parse the given function and return a dictionary mapping "ids" to
-    "line ranges".  Ids are identified by comments with a special syntax::
+    "line number".  Ids are identified by comments with a special syntax::
 
         # "myid" corresponds to the whole line
         print 'foo' # ID: myid
     """
     result = {}
-    start_lineno = code.co.co_firstlineno
+    start_lineno = code.startlineno
     for i, line in enumerate(py.code.Source(code.source)):
         m = re.search('# ID: (\w+)', line)
         if m:
             name = m.group(1)
             lineno = start_lineno+i
-            result[name] = xrange(lineno, lineno+1)
+            result.setdefault(name, lineno)
     return result
 
-def find_ids(code):
-    """
-    Parse the given function and return a dictionary mapping "ids" to
-    "opcodes".
-    """
-    ids = {}
-    ranges = find_ids_range(code)
-    for name, linerange in ranges.iteritems():
-        opcodes = [opcode for opcode in code.opcodes
-                   if opcode.lineno in linerange]
-        ids[name] = opcodes
-    return ids
-
 
 class Log(object):
     def __init__(self, rawtraces):
@@ -107,13 +94,18 @@ class TraceWithIds(Function):
     def compute_ids(self, ids):
         #
         # 1. compute the ids of self, i.e. the outer function
-        id2opcodes = find_ids(self.code)
-        all_my_opcodes = self.get_set_of_opcodes()
-        for id, opcodes in id2opcodes.iteritems():
-            if not opcodes:
-                continue
-            target_opcodes = set(opcodes)
-            if all_my_opcodes.intersection(target_opcodes):
+        id2lineno = find_id_linenos(self.code)
+        all_my_tracecodes = self.get_list_of_tracecodes()
+        for id, lineno in id2lineno.iteritems():
+            seen = set()
+            opcodes = []
+            for tracecode in all_my_tracecodes:
+                if tracecode.lineno == lineno:
+                    opcode = tracecode.getopcode()
+                    if opcode not in seen:
+                        seen.add(opcode)
+                        opcodes.append(opcode)
+            if opcodes:
                 ids[id] = opcodes
         #
         # 2. compute the ids of all the inlined functions
@@ -121,12 +113,11 @@ class TraceWithIds(Function):
             if isinstance(chunk, TraceWithIds) and chunk.code:
                 chunk.compute_ids(ids)
 
-    def get_set_of_opcodes(self):
-        result = set()
+    def get_list_of_tracecodes(self):
+        result = []
         for chunk in self.chunks:
             if isinstance(chunk, TraceForOpcode):
-                opcode = chunk.getopcode()
-                result.add(opcode)
+                result.append(chunk)
         return result
 
     def has_id(self, id):
@@ -144,7 +135,7 @@ class TraceWithIds(Function):
         for chunk in self.flatten_chunks():
             opcode = chunk.getopcode()
             if opcode_name is None or \
-                   (opcode and opcode.__class__.__name__ == opcode_name):
+                   (opcode and opcode.match_name(opcode_name)):
                 for op in self._ops_for_chunk(chunk, include_guard_not_invalidated):
                     yield op
             else:
@@ -172,7 +163,7 @@ class TraceWithIds(Function):
         for chunk in self.flatten_chunks():
             opcode = chunk.getopcode()
             if opcode in target_opcodes and (opcode_name is None or
-                                             opcode.__class__.__name__ == opcode_name):
+                                             opcode.match_name(opcode_name)):
                 for op in self._ops_for_chunk(chunk, include_guard_not_invalidated):
                     if op in loop_ops:
                         yield op
@@ -292,6 +283,12 @@ class OpMatcher(object):
         #
         if line.strip() == 'guard_not_invalidated?':
             return 'guard_not_invalidated', None, [], '...', False
+        if line.strip() == 'dummy_get_utf8?':
+            # this is because unicode used to generate dummy getfield_gc_r
+            # of the _utf8 field.  Nowadays it no longer does.  This line
+            # is equivalent to a comment now.
+            return None
+
         # find the resvar, if any
         if ' = ' in line:
             resvar, _, line = line.partition(' = ')
@@ -326,7 +323,7 @@ class OpMatcher(object):
         # to repeat it every time
         ticker_check = """
             guard_not_invalidated?
-            ticker0 = getfield_raw_i(#, descr=<FieldS pypysig_long_struct_inner.c_value .*>)
+            ticker0 = getfield_raw_i(#, descr=<FieldS pypysig_long_struct.c_value .*>)
             ticker_cond0 = int_lt(ticker0, 0)
             guard_false(ticker_cond0, descr=...)
         """
@@ -335,9 +332,9 @@ class OpMatcher(object):
         # this is the ticker check generated if we have threads
         thread_ticker_check = """
             guard_not_invalidated?
-            ticker0 = getfield_raw_i(#, descr=<FieldS pypysig_long_struct_inner.c_value .*>)
+            ticker0 = getfield_raw_i(#, descr=<FieldS pypysig_long_struct.c_value .*>)
             ticker1 = int_sub(ticker0, #)
-            setfield_raw(#, ticker1, descr=<FieldS pypysig_long_struct_inner.c_value .*>)
+            setfield_raw(#, ticker1, descr=<FieldS pypysig_long_struct.c_value .*>)
             ticker_cond0 = int_lt(ticker1, 0)
             guard_false(ticker_cond0, descr=...)
         """
@@ -345,7 +342,7 @@ class OpMatcher(object):
         #
         # this is the ticker check generated in PyFrame.handle_operation_error
         exc_ticker_check = """
-            ticker2 = getfield_raw_i(#, descr=<FieldS pypysig_long_struct_inner.c_value .*>)
+            ticker2 = getfield_raw_i(#, descr=<FieldS pypysig_long_struct.c_value .*>)
             ticker_cond1 = int_lt(ticker2, 0)
             guard_false(ticker_cond1, descr=...)
         """
