@@ -80,7 +80,9 @@ class TestCall(BaseTestPyPyC):
         assert log.opnames(ops) == []
         #
         assert entry_bridge.match_by_id('call', """
+            dummy_get_utf8?
             p38 = call_r(ConstClass(_ll_1_threadlocalref_get__Ptr_GcStruct_objectLlT_Signed), #, descr=<Callr . i EF=1 OS=5>)
+            p99 = getfield_gc_r(p38, descr=<FieldP pypy.interpreter.executioncontext.ExecutionContext.inst_sys_exc_operror .*>)
             p39 = getfield_gc_r(p38, descr=<FieldP pypy.interpreter.executioncontext.ExecutionContext.inst_topframeref .*>)
             i40 = force_token()
             p41 = getfield_gc_r(p38, descr=<FieldP pypy.interpreter.executioncontext.ExecutionContext.inst_w_tracefunc .*>)
@@ -114,8 +116,6 @@ class TestCall(BaseTestPyPyC):
                     self.a = a
                 def f(self, i):
                     return self.a + i
-            a = A("a") # stop field unboxing
-
             i = 0
             a = A(1)
             while i < n:
@@ -405,7 +405,7 @@ class TestCall(BaseTestPyPyC):
         def main(n):
             i = 1
             while i < n:
-                i += len(xrange(i+1)) - i
+                i += len(range(i+1)) - i
             return i
 
         log = self.run(main, [10000])
@@ -441,8 +441,10 @@ class TestCall(BaseTestPyPyC):
             i22 = getfield_gc_i(p12, descr=<FieldS pypy.objspace.std.intobject.W_IntObject.inst_intval .*>)
             i24 = int_lt(i22, 5000)
             guard_true(i24, descr=...)
+            dummy_get_utf8?
             guard_not_invalidated(descr=...)
             p29 = call_r(ConstClass(_ll_1_threadlocalref_get__Ptr_GcStruct_objectLlT_Signed), #, descr=<Callr . i EF=1 OS=5>)
+            p99 = getfield_gc_r(p29, descr=<FieldP pypy.interpreter.executioncontext.ExecutionContext.inst_sys_exc_operror .*>)
             p30 = getfield_gc_r(p29, descr=<FieldP pypy.interpreter.executioncontext.ExecutionContext.inst_topframeref .*>)
             p31 = force_token()
             p32 = getfield_gc_r(p29, descr=<FieldP pypy.interpreter.executioncontext.ExecutionContext.inst_w_tracefunc .*>)
@@ -450,7 +452,11 @@ class TestCall(BaseTestPyPyC):
             i34 = getfield_gc_i(p29, descr=<FieldU pypy.interpreter.executioncontext.ExecutionContext.inst_profilefunc .*>)
             i35 = int_is_zero(i34)
             guard_true(i35, descr=...)
-            i40 = int_add(i22, 5)
+            p37 = getfield_gc_r(ConstPtr(ptr36), descr=<FieldP pypy.interpreter.nestedscope.Cell.inst_w_value .*>)
+            guard_nonnull_class(p37, ConstClass(W_IntObject), descr=...)
+            i39 = getfield_gc_i(p37, descr=<FieldS pypy.objspace.std.intobject.W_IntObject.inst_intval .*>)
+            i40 = int_add_ovf(i22, i39)
+            guard_no_overflow(descr=...)
             --TICK--
         """)
 
@@ -554,26 +560,6 @@ class TestCall(BaseTestPyPyC):
         assert len(calls) == 0
         assert len([op for op in allops if op.name.startswith('new')]) == 0
 
-    def test_kwargs_update_virtual1(self):
-        log = self.run("""
-        def f(**kwargs):
-            return len(kwargs)
-        def main(stop):
-            i = 0
-            res = 0
-            while i < stop:
-                d = {'a': 1, 'b': 2}
-                # used to force the dict!
-                res += f(c=2, **d) # ID: call
-                i += 1
-            return res
-        """, [1000])
-        loop, = log.loops_by_id('call')
-        ops = loop.ops_by_id('call')
-        assert log.opnames(ops) == ["guard_not_invalidated", "force_token",
-                "int_add_ovf", "guard_no_overflow"]
-
-
     def test_kwargs_non_virtual(self):
         log = self.run("""
         def f(a, b, c):
@@ -648,44 +634,3 @@ class TestCall(BaseTestPyPyC):
                 i += 1
             return 13
         """, [1000])
-
-    def test_starargs_list_copying(self):
-        log = self.run("""
-        class A(object):
-            def __init__(self, *args):
-                self.args = args
-
-        def main(stop):
-            i = 0
-            l = [1, 2, "abc", None]
-            while i < stop:
-                A(1, *l) # ID: call
-                i += 1
-            return 13
-        """, [1000])
-        loop, = log.loops_by_id('call')
-        assert loop.match_by_id('call', '''
-        ...
-        guard_not_invalidated(descr=...)
-        p79 = new_array_clear(i34, descr=...)
-        call_n(ConstClass(ll_arraycopy__arrayPtr_arrayPtr_Signed_Signed_Signed), p36, p79, 0, 0, i34, descr=...)
-        p83 = new_array_clear(i41, descr=...)
-        setarrayitem_gc(p83, 0, ConstPtr(ptr85), descr=...)
-        call_n(ConstClass(ll_arraycopy__arrayPtr_arrayPtr_Signed_Signed_Signed), p79, p83, 0, 1, i34, descr=...)
-        p85 = force_token()
-        ...
-        ''')
-
-    def test_function_escape_without_defaults(self):
-        def main():
-            global f
-            res = 0
-            for i in range(10000):
-                def f():
-                    return i + 1
-                res += f() # ID: call
-
-        log = self.run(main, [])
-        loop, = log.loops_by_id('call')
-        opnames = log.opnames(loop._allops())
-        assert opnames.count("new_array_clear") == 2 # for the closure, but not for defaults
