@@ -1,6 +1,5 @@
 from pypy.interpreter.mixedmodule import MixedModule
-from pypy.interpreter.gateway import unwrap_spec
-from pypy.interpreter.pyparser import pytoken, pygram
+from pypy.interpreter.pyparser import pytoken
 
 
 class Module(MixedModule):
@@ -8,9 +7,9 @@ class Module(MixedModule):
     appleveldefs = {}
     interpleveldefs = {
         "NT_OFFSET" : "space.newint(256)",
-        "ISTERMINAL" : "moduledef.isterminal",
-        "ISNONTERMINAL" : "moduledef.isnonterminal",
-        "ISEOF" : "moduledef.iseof"
+        "ISTERMINAL" : "interp_token.isterminal",
+        "ISNONTERMINAL" : "interp_token.isnonterminal",
+        "ISEOF" : "interp_token.iseof"
         }
 
 
@@ -21,18 +20,12 @@ def _init_tokens():
         tok_name[id] = tok
     Module.interpleveldefs["tok_name"] = "space.wrap(%r)" % (tok_name,)
     Module.interpleveldefs["N_TOKENS"] = "space.wrap(%d)" % len(tok_name)
+    all_names = Module.interpleveldefs.keys()
+    # obscure, but these names are not in CPython's token module
+    # so we remove them from 'token.__all__' otherwise they end up
+    # twice in 'tokenize.__all__'
+    all_names.remove('COMMENT')
+    all_names.remove('NL')
+    Module.interpleveldefs["__all__"] = "space.wrap(%r)" % (all_names,)
 
 _init_tokens()
-
-
-@unwrap_spec(tok=int)
-def isterminal(space, tok):
-    return space.newbool(tok < 256)
-
-@unwrap_spec(tok=int)
-def isnonterminal(space, tok):
-    return space.newbool(tok >= 256)
-
-@unwrap_spec(tok=int)
-def iseof(space, tok):
-    return space.newbool(tok == pygram.tokens.ENDMARKER)
