@@ -1,7 +1,9 @@
 """The builtin bytearray implementation"""
 
+import sys
 from rpython.rlib.objectmodel import (
     import_from_mixin, newlist_hint, resizelist_hint, specialize)
+from rpython.rlib.rarithmetic import intmask
 from rpython.rlib.rstring import StringBuilder, ByteListBuilder
 from rpython.rlib.debug import check_list_of_chars, check_nonneg
 from rpython.rtyper.lltypesystem import rffi
@@ -13,14 +15,16 @@ from rpython.rlib.buffer import (GCBuffer,
                                  get_gc_data_offset_for_list_of_chars)
 from pypy.interpreter.baseobjspace import W_Root
 from pypy.interpreter.error import OperationError, oefmt
+from pypy.objspace.std.bytesobject import makebytesdata_w, newbytesdata_w
 from pypy.interpreter.gateway import WrappedDefault, interp2app, unwrap_spec
 from pypy.interpreter.typedef import TypeDef
 from pypy.interpreter.buffer import SimpleView
 from pypy.objspace.std.sliceobject import W_SliceObject, unwrap_start_stop
-from pypy.objspace.std.stringmethods import StringMethods, _get_buffer
+from pypy.objspace.std.stringmethods import StringMethods
 from pypy.objspace.std.stringmethods import _descr_getslice_slowpath
 from pypy.objspace.std.bytesobject import W_BytesObject
 from pypy.objspace.std.util import get_positive_index
+from pypy.objspace.std.formatting import mod_format, FORMAT_BYTEARRAY
 
 
 class W_BytearrayObject(W_Root):
@@ -50,15 +54,6 @@ class W_BytearrayObject(W_Root):
     def buffer_w(self, space, flags):
         return SimpleView(BytearrayBuffer(self))
 
-    def readbuf_w(self, space):
-        return BytearrayBuffer(self, readonly=True)
-
-    def writebuf_w(self, space):
-        return BytearrayBuffer(self)
-
-    def charbuf_w(self, space):
-        return ''.join(self.getdata())
-
     def bytearray_list_of_chars_w(self, space):
         return self.getdata()
 
@@ -107,8 +102,16 @@ class W_BytearrayObject(W_Root):
         return False
 
     @staticmethod
-    def _op_val(space, w_other, strict=None):
-        # bytearray does not enforce the strict restriction (on strip at least)
+    def _op_val(space, w_other, allow_char=False):
+        # Some functions (contains, find) allow a number to specify a
+        # single char.
+        if allow_char and space.isinstance_w(w_other, space.w_int):
+            return StringMethods._single_char(space, w_other)
+        try:
+            return space.bytes_w(w_other)
+        except OperationError as e:
+            if not e.match(space, space.w_TypeError):
+                raise
         return space.buffer_w(w_other, space.BUF_SIMPLE).as_str()
 
     def _chr(self, char):
@@ -170,12 +173,6 @@ class W_BytearrayObject(W_Root):
     def _join_return_one(self, space, w_obj):
         return False
 
-    def _join_check_item(self, space, w_obj):
-        if (space.isinstance_w(w_obj, space.w_bytes) or
-            space.isinstance_w(w_obj, space.w_bytearray)):
-            return 0
-        return 1
-
     def ord(self, space):
         length = self._len()
         if length != 1:
@@ -204,38 +201,21 @@ class W_BytearrayObject(W_Root):
 
     @staticmethod
     def descr_fromhex(space, w_bytearraytype, w_hexstring):
-        hexstring = space.text_w(w_hexstring)
+        if not space.is_w(space.type(w_hexstring), space.w_unicode):
+            raise oefmt(space.w_TypeError, "must be str, not %T", w_hexstring)
+        hexstring = space.utf8_w(w_hexstring)
         data = _hexstring_to_array(space, hexstring)
         # in CPython bytearray.fromhex is a staticmethod, so
         # we ignore w_type and always return a bytearray
-        return new_bytearray(space, space.w_bytearray, data)
+        w_result = new_bytearray(space, space.w_bytearray, data)
+        if w_bytearraytype is not space.w_bytearray:
+            w_result = space.call_function(w_bytearraytype, w_result)
+        return w_result
 
     @unwrap_spec(encoding='text_or_none', errors='text_or_none')
     def descr_init(self, space, w_source=None, encoding=None, errors=None):
-        if w_source is None:
-            w_source = space.newbytes('')
-        if encoding is not None:
-            from pypy.objspace.std.unicodeobject import encode_object
-            # if w_source is an integer this correctly raises a
-            # TypeError the CPython error message is: "encoding or
-            # errors without a string argument" ours is: "expected
-            # unicode, got int object"
-            w_source = encode_object(space, w_source, encoding, errors)
-
-        # Is it an int?
-        try:
-            count = space.int_w(w_source)
-        except OperationError as e:
-            if not e.match(space, space.w_TypeError):
-                raise
-        else:
-            if count < 0:
-                raise oefmt(space.w_ValueError, "bytearray negative count")
-            self._data = resizable_list_supporting_raw_ptr(['\0'] * count)
-            self._offset = 0
-            return
-
-        data = makebytearraydata_w(space, w_source)
+        assert isinstance(self, W_BytearrayObject)
+        data = [c for c in newbytesdata_w(space, w_source, encoding, errors)]
         self._data = resizable_list_supporting_raw_ptr(data)
         self._offset = 0
         _tweak_for_tests(self)
@@ -283,14 +263,17 @@ class W_BytearrayObject(W_Root):
         return space.newtext(buf.build())
 
     def descr_str(self, space):
-        return space.newtext(''.join(self.getdata()))
+        if space.sys.get_flag('bytes_warning'):
+            space.warn(space.newtext("str() on a bytearray instance"),
+                       space.w_BytesWarning)
+        return self.descr_repr(space)
 
     def descr_eq(self, space, w_other):
         if isinstance(w_other, W_BytearrayObject):
             return space.newbool(self.getdata() == w_other.getdata())
 
         try:
-            buffer = _get_buffer(space, w_other)
+            buffer = space.readbuf_w(w_other)
         except OperationError as e:
             if e.match(space, space.w_TypeError):
                 return space.w_NotImplemented
@@ -310,7 +293,7 @@ class W_BytearrayObject(W_Root):
             return space.newbool(self.getdata() != w_other.getdata())
 
         try:
-            buffer = _get_buffer(space, w_other)
+            buffer = space.readbuf_w(w_other)
         except OperationError as e:
             if e.match(space, space.w_TypeError):
                 return space.w_NotImplemented
@@ -338,7 +321,7 @@ class W_BytearrayObject(W_Root):
             cmp = _memcmp(value, other, min(len(value), len(other)))
         else:
             try:
-                buffer = _get_buffer(space, w_other)
+                buffer = space.readbuf_w(w_other)
             except OperationError as e:
                 if e.match(space, space.w_TypeError):
                     return False, 0, 0
@@ -372,9 +355,6 @@ class W_BytearrayObject(W_Root):
             return space.w_NotImplemented
         return space.newbool(cmp > 0 or (cmp == 0 and self._len() >= other_len))
 
-    def descr_iter(self, space):
-        return space.newseqiter(self)
-
     def descr_inplace_add(self, space, w_other):
         if isinstance(w_other, W_BytearrayObject):
             self._data += w_other.getdata()
@@ -383,7 +363,7 @@ class W_BytearrayObject(W_Root):
         if isinstance(w_other, W_BytesObject):
             self._inplace_add(self._op_val(space, w_other))
         else:
-            self._inplace_add(_get_buffer(space, w_other))
+            self._inplace_add(space.readbuf_w(w_other))
         return self
 
     @specialize.argtype(1)
@@ -405,7 +385,7 @@ class W_BytearrayObject(W_Root):
 
     def descr_setitem(self, space, w_index, w_other):
         if isinstance(w_index, W_SliceObject):
-            sequence2 = makebytearraydata_w(space, w_other)
+            sequence2 = makebytesdata_w(space, w_other)
             oldsize = self._len()
             start, stop, step, slicelength = w_index.indices4(space, oldsize)
             if start == 0 and step == 1 and len(sequence2) <= slicelength:
@@ -418,8 +398,7 @@ class W_BytearrayObject(W_Root):
             _setitem_slice_helper(space, data, start, step,
                                   slicelength, sequence2, empty_elem='\x00')
         else:
-            idx = space.getindex_w(w_index, space.w_IndexError,
-                                   "bytearray index")
+            idx = space.getindex_w(w_index, space.w_IndexError, "bytearray")
             newvalue = space.byte_w(w_other)
             self._data[self._fixindex(space, idx)] = newvalue
 
@@ -432,8 +411,7 @@ class W_BytearrayObject(W_Root):
                 _delitem_slice_helper(space, self._data,
                                       start + self._offset, step, slicelength)
         else:
-            idx = space.getindex_w(w_idx, space.w_IndexError,
-                                   "bytearray index")
+            idx = space.getindex_w(w_idx, space.w_IndexError, "bytearray")
             idx = self._fixindex(space, idx)
             if idx == self._offset:    # fast path for del x[0] or del[-len]
                 self._delete_from_start(1)
@@ -453,7 +431,7 @@ class W_BytearrayObject(W_Root):
         if isinstance(w_other, W_BytearrayObject):
             self._data += w_other.getdata()
         else:
-            self._inplace_add(makebytearraydata_w(space, w_other))
+            self._inplace_add(makebytesdata_w(space, w_other))
 
     def descr_insert(self, space, w_idx, w_other):
         where = space.int_w(w_idx)
@@ -480,14 +458,6 @@ class W_BytearrayObject(W_Root):
                 return
         raise oefmt(space.w_ValueError, "value not found in bytearray")
 
-    _StringMethods_descr_contains = descr_contains
-    def descr_contains(self, space, w_sub):
-        if space.isinstance_w(w_sub, space.w_int):
-            char = space.int_w(w_sub)
-            return _descr_contains_bytearray(self.getdata(), space, char)
-
-        return self._StringMethods_descr_contains(space, w_sub)
-
     def descr_add(self, space, w_other):
         if isinstance(w_other, W_BytearrayObject):
             return self._new(self.getdata() + w_other.getdata())
@@ -496,7 +466,7 @@ class W_BytearrayObject(W_Root):
             return self._add(self._op_val(space, w_other))
 
         try:
-            buffer = _get_buffer(space, w_other)
+            buffer = space.readbuf_w(w_other)
         except OperationError as e:
             if e.match(space, space.w_TypeError):
                 return space.w_NotImplemented
@@ -510,6 +480,30 @@ class W_BytearrayObject(W_Root):
     def descr_reverse(self, space):
         self.getdata().reverse()
 
+    def descr_clear(self, space):
+        self._data = []
+        self._offset = 0
+
+    def descr_copy(self, space):
+        return self._new(self._data[self._offset:])
+
+    def descr_hex(self, space):
+        data = self.getdata()
+        return _array_to_hexstring(space, data, 0, 1, len(data), True)
+
+    def descr_mod(self, space, w_values):
+        return mod_format(space, self, w_values, fmt_type=FORMAT_BYTEARRAY)
+
+    def descr_rmod(self, space, w_value):
+        if not isinstance(w_value, W_BytearrayObject):
+            return space.w_NotImplemented
+        return mod_format(space, w_value, self, fmt_type=FORMAT_BYTEARRAY)
+
+    @staticmethod
+    def _iter_getitem_result(self, space, index):
+        assert isinstance(self, W_BytearrayObject)
+        return self._getitem_result(space, index)
+
     def descr_alloc(self, space):
         return space.newint(len(self._data) + 1)   # includes the _offset part
 
@@ -544,15 +538,6 @@ class W_BytearrayObject(W_Root):
 def _make_data(s):
     return [s[i] for i in range(len(s))]
 
-
-def _descr_contains_bytearray(data, space, char):
-    if not 0 <= char < 256:
-        raise oefmt(space.w_ValueError, "byte must be in range(0, 256)")
-    for c in data:
-        if ord(c) == char:
-            return space.w_True
-    return space.w_False
-
 # ____________________________________________________________
 
 
@@ -562,56 +547,6 @@ def new_bytearray(space, w_bytearraytype, data):
     return w_obj
 
 
-def makebytearraydata_w(space, w_source):
-    # String-like argument
-    try:
-        buf = space.buffer_w(w_source, space.BUF_FULL_RO)
-    except OperationError as e:
-        if not e.match(space, space.w_TypeError):
-            raise
-    else:
-        return list(buf.as_str())
-    return _from_byte_sequence(space, w_source)
-
-def _get_printable_location(w_type):
-    return ('bytearray_from_byte_sequence [w_type=%s]' %
-            w_type.getname(w_type.space))
-
-_byteseq_jitdriver = jit.JitDriver(
-    name='bytearray_from_byte_sequence',
-    greens=['w_type'],
-    reds=['w_iter', 'data'],
-    get_printable_location=_get_printable_location)
-
-def _from_byte_sequence(space, w_source):
-    # Split off in a separate function for the JIT's benefit
-    # and add a jitdriver with the type of w_iter as the green key
-    w_iter = space.iter(w_source)
-    length_hint = space.length_hint(w_source, 0)
-    data = newlist_hint(length_hint)
-    #
-    _from_byte_sequence_loop(space, w_iter, data)
-    #
-    extended = len(data)
-    if extended < length_hint:
-        resizelist_hint(data, extended)
-    return data
-
-def _from_byte_sequence_loop(space, w_iter, data):
-    w_type = space.type(w_iter)
-    while True:
-        _byteseq_jitdriver.jit_merge_point(w_type=w_type,
-                                           w_iter=w_iter,
-                                           data=data)
-        try:
-            w_item = space.next(w_iter)
-        except OperationError as e:
-            if not e.match(space, space.w_StopIteration):
-                raise
-            break
-        data.append(space.byte_w(w_item))
-
-
 def _hex_digit_to_int(d):
     val = ord(d)
     if 47 < val < 58:
@@ -646,6 +581,31 @@ def _hexstring_to_array(space, s):
         i += 2
     return data
 
+HEXDIGITS = "0123456789abcdef"
+PY_SIZE_T_MAX = intmask(2**(rffi.sizeof(rffi.SIZE_T)*8-1)-1)
+
+@specialize.arg(5) # raw access
+def _array_to_hexstring(space, buf, start, step, length, rawaccess=False):
+    hexstring = StringBuilder(length*2)
+
+    if length > PY_SIZE_T_MAX/2:
+        raise OperationError(space.w_MemoryError, space.w_None)
+
+    stepped = 0
+    i = start
+    while stepped < length:
+        if rawaccess:
+            byte = ord(buf[i])
+        else:
+            byte = ord(buf.getitem(i))
+        c = (byte >> 4 & 0xf)
+        hexstring.append(HEXDIGITS[c])
+        c = (byte & 0xf)
+        hexstring.append(HEXDIGITS[c])
+        i += step
+        stepped += 1
+
+    return space.newtext(hexstring.build())
 
 class BytearrayDocstrings:
     """bytearray(iterable_of_ints) -> bytearray
@@ -671,7 +631,9 @@ class BytearrayDocstrings:
     def __alloc__():
         """B.__alloc__() -> int
 
-        Return the number of bytes actually allocated.
+        CPython compatibility: return len(B) + 1.
+        (The allocated size might be bigger, but getting it is
+        involved and may create pointless compatibility troubles.)
         """
 
     def __contains__():
@@ -702,7 +664,7 @@ class BytearrayDocstrings:
         """x.__imul__(y) <==> x*=y"""
 
     def __init__():
-        """x.__init__(...) initializes x; see help(type(x)) for signature"""
+        """Initialize self.  See help(type(self)) for accurate signature."""
 
     def __iter__():
         """x.__iter__() <==> iter(x)"""
@@ -719,6 +681,12 @@ class BytearrayDocstrings:
     def __mul__():
         """x.__mul__(n) <==> x*n"""
 
+    def __mod__():
+        """Return self%value."""
+
+    def __rmod__():
+        """Return value%self."""
+
     def __ne__():
         """x.__ne__(y) <==> x!=y"""
 
@@ -763,6 +731,18 @@ class BytearrayDocstrings:
         done using the specified fill character (default is a space).
         """
 
+    def clear():
+        """B.clear() -> None
+
+        Remove all items from B.
+        """
+
+    def copy():
+        """B.copy() -> bytearray
+
+        Return a copy of B.
+        """
+
     def count():
         """B.count(sub[, start[, end]]) -> int
 
@@ -1069,6 +1049,13 @@ class BytearrayDocstrings:
         of the specified width.  B is never truncated.
         """
 
+    def hex():
+        """B.hex() -> unicode
+
+        Return a string object containing two hexadecimal digits
+        for each byte in the instance B.
+        """
+
 
 W_BytearrayObject.typedef = TypeDef(
     "bytearray", None, None, "read-write",
@@ -1079,6 +1066,8 @@ W_BytearrayObject.typedef = TypeDef(
                             doc=BytearrayDocstrings.__reduce__.__doc__),
     fromhex = interp2app(W_BytearrayObject.descr_fromhex, as_classmethod=True,
                          doc=BytearrayDocstrings.fromhex.__doc__),
+    maketrans = interp2app(W_BytearrayObject.descr_maketrans,
+                           as_classmethod=True),
 
     __repr__ = interp2app(W_BytearrayObject.descr_repr,
                           doc=BytearrayDocstrings.__repr__.__doc__),
@@ -1199,6 +1188,10 @@ W_BytearrayObject.typedef = TypeDef(
                              doc=BytearrayDocstrings.__setitem__.__doc__),
     __delitem__ = interp2app(W_BytearrayObject.descr_delitem,
                              doc=BytearrayDocstrings.__delitem__.__doc__),
+    __mod__ = interp2app(W_BytearrayObject.descr_mod,
+                           doc=BytearrayDocstrings.__mod__.__doc__),
+    __rmod__ = interp2app(W_BytearrayObject.descr_rmod,
+                           doc=BytearrayDocstrings.__rmod__.__doc__),
 
     append = interp2app(W_BytearrayObject.descr_append,
                         doc=BytearrayDocstrings.append.__doc__),
@@ -1212,6 +1205,12 @@ W_BytearrayObject.typedef = TypeDef(
                         doc=BytearrayDocstrings.remove.__doc__),
     reverse = interp2app(W_BytearrayObject.descr_reverse,
                          doc=BytearrayDocstrings.reverse.__doc__),
+    clear = interp2app(W_BytearrayObject.descr_clear,
+                       doc=BytearrayDocstrings.clear.__doc__),
+    copy = interp2app(W_BytearrayObject.descr_copy,
+                         doc=BytearrayDocstrings.copy.__doc__),
+    hex = interp2app(W_BytearrayObject.descr_hex,
+                           doc=BytearrayDocstrings.hex.__doc__),
     __alloc__ = interp2app(W_BytearrayObject.descr_alloc,
                            doc=BytearrayDocstrings.__alloc__.__doc__),
 )
