@@ -20,6 +20,7 @@ UNROLL_CUTOFF = 5
 
 class W_BaseSetObject(W_Root):
     typedef = None
+    exact_class_applevel_name = 'set-or-frozenset'
 
     def __init__(self, space, w_iterable=None):
         """Initialize the set by taking ownership of 'setdata'."""
@@ -167,13 +168,6 @@ class W_BaseSetObject(W_Root):
     def descr_repr(self, space):
         return setrepr(space, space.get_objects_in_repr(), self)
 
-    def descr_cmp(self, space, w_other):
-        if space.is_w(space.type(self), space.type(w_other)):
-            # hack hack until we get the expected result
-            raise oefmt(space.w_TypeError, "cannot compare sets using cmp()")
-        else:
-            return space.w_NotImplemented
-
     def descr_eq(self, space, w_other):
         if isinstance(w_other, W_BaseSetObject):
             return space.newbool(self.equals(w_other))
@@ -253,16 +247,10 @@ class W_BaseSetObject(W_Root):
             return space.w_NotImplemented
         return self.difference(w_other)
 
-    def descr_rsub(self, space, w_other):
-        if not isinstance(w_other, W_BaseSetObject):
-            return space.w_NotImplemented
-        return w_other.difference(self)
-
     def descr_and(self, space, w_other):
         if not isinstance(w_other, W_BaseSetObject):
             return space.w_NotImplemented
         return self.intersect(w_other)
-    descr_rand = descr_and # symmetric
 
     def descr_or(self, space, w_other):
         if not isinstance(w_other, W_BaseSetObject):
@@ -270,13 +258,35 @@ class W_BaseSetObject(W_Root):
         w_copy = self.copy_real()
         w_copy.update(w_other)
         return w_copy
-    descr_ror = descr_or # symmetric
 
     def descr_xor(self, space, w_other):
         if not isinstance(w_other, W_BaseSetObject):
             return space.w_NotImplemented
         return self.symmetric_difference(w_other)
-    descr_rxor = descr_xor # symmetric
+
+    def descr_inplace_sub(self, space, w_other):
+        if not isinstance(w_other, W_BaseSetObject):
+            return space.w_NotImplemented
+        self.difference_update(w_other)
+        return self
+
+    def descr_inplace_and(self, space, w_other):
+        if not isinstance(w_other, W_BaseSetObject):
+            return space.w_NotImplemented
+        self.intersect_update(w_other)
+        return self
+
+    def descr_inplace_or(self, space, w_other):
+        if not isinstance(w_other, W_BaseSetObject):
+            return space.w_NotImplemented
+        self.update(w_other)
+        return self
+
+    def descr_inplace_xor(self, space, w_other):
+        if not isinstance(w_other, W_BaseSetObject):
+            return space.w_NotImplemented
+        self.descr_symmetric_difference_update(space, w_other)
+        return self
 
     def descr_copy(self, space):
         """Return a shallow copy of a set."""
@@ -395,65 +405,6 @@ class W_BaseSetObject(W_Root):
                 return space.w_False
         return space.w_True
 
-    @gateway.unwrap_spec(others_w='args_w')
-    def descr_difference_update(self, space, others_w):
-        """Update the set, removing elements found in others."""
-        for w_other in others_w:
-            if isinstance(w_other, W_BaseSetObject):
-                self.difference_update(w_other)
-            else:
-                w_other_as_set = self._newobj(space, w_other)
-                self.difference_update(w_other_as_set)
-
-
-
-class W_SetObject(W_BaseSetObject):
-
-    #overridden here so the error is reported correctly
-    def __init__(self, space, w_iterable=None):
-        """Initialize the set by taking ownership of 'setdata'."""
-        W_BaseSetObject.__init__(self, space, w_iterable)
-
-    def _newobj(self, space, w_iterable):
-        """Make a new set by taking ownership of 'w_iterable'."""
-        if type(self) is W_SetObject:
-            return W_SetObject(space, w_iterable)
-        w_type = space.type(self)
-        w_obj = space.allocate_instance(W_SetObject, w_type)
-        W_SetObject.__init__(w_obj, space, w_iterable)
-        return w_obj
-
-    @staticmethod
-    def descr_new(space, w_settype, __args__):
-        w_obj = space.allocate_instance(W_SetObject, w_settype)
-        W_SetObject.__init__(w_obj, space)
-        return w_obj
-
-    def descr_inplace_sub(self, space, w_other):
-        if not isinstance(w_other, W_BaseSetObject):
-            return space.w_NotImplemented
-        self.difference_update(w_other)
-        return self
-
-    def descr_inplace_and(self, space, w_other):
-        if not isinstance(w_other, W_BaseSetObject):
-            return space.w_NotImplemented
-        self.intersect_update(w_other)
-        return self
-
-    def descr_inplace_or(self, space, w_other):
-        if not isinstance(w_other, W_BaseSetObject):
-            return space.w_NotImplemented
-        self.update(w_other)
-        return self
-
-    def descr_inplace_xor(self, space, w_other):
-        if not isinstance(w_other, W_BaseSetObject):
-            return space.w_NotImplemented
-        self.descr_symmetric_difference_update(space, w_other)
-        return self
-
-
     def descr_add(self, space, w_other):
         """Add an element to a set.
 
@@ -464,6 +415,16 @@ class W_SetObject(W_BaseSetObject):
         """Remove all elements from this set."""
         self.clear()
 
+    @gateway.unwrap_spec(others_w='args_w')
+    def descr_difference_update(self, space, others_w):
+        """Update the set, removing elements found in others."""
+        for w_other in others_w:
+            if isinstance(w_other, W_BaseSetObject):
+                self.difference_update(w_other)
+            else:
+                w_other_as_set = self._newobj(space, w_other)
+                self.difference_update(w_other_as_set)
+
     def _discard_from_set(self, space, w_item):
         """
         Discard an element from a set, with automatic conversion to
@@ -520,16 +481,30 @@ class W_SetObject(W_BaseSetObject):
     @gateway.unwrap_spec(others_w='args_w')
     def descr_update(self, space, others_w):
         """Update a set with the union of itself and another."""
-        self._descr_update(space, others_w)
-
-    @jit.look_inside_iff(lambda self, space, others_w:
-            jit.loop_unrolling_heuristic(others_w, len(others_w), UNROLL_CUTOFF))
-    def _descr_update(self, space, others_w):
         for w_other in others_w:
             if isinstance(w_other, W_BaseSetObject):
                 self.update(w_other)
             else:
-                _update_from_iterable(space, self, w_other)
+                for w_key in space.listview(w_other):
+                    self.add(w_key)
+
+
+class W_SetObject(W_BaseSetObject):
+
+    #overridden here so the error is reported correctly
+    def __init__(self, space, w_iterable=None):
+        """Initialize the set by taking ownership of 'setdata'."""
+        W_BaseSetObject.__init__(self, space, w_iterable)
+
+    def _newobj(self, space, w_iterable):
+        """Make a new set by taking ownership of 'w_iterable'."""
+        return W_SetObject(space, w_iterable)
+
+    @staticmethod
+    def descr_new(space, w_settype, __args__):
+        w_obj = space.allocate_instance(W_SetObject, w_settype)
+        W_SetObject.__init__(w_obj, space)
+        return w_obj
 
 W_SetObject.typedef = TypeDef("set",
     __doc__ = """set(iterable) --> set object
@@ -537,58 +512,53 @@ W_SetObject.typedef = TypeDef("set",
 Build an unordered collection.""",
     __new__ = gateway.interp2app(W_SetObject.descr_new),
     __init__ = gateway.interp2app(W_SetObject.descr_init),
-    __repr__ = gateway.interp2app(W_SetObject.descr_repr),
+    __repr__ = gateway.interp2app(W_BaseSetObject.descr_repr),
     __hash__ = None,
-    __cmp__ = gateway.interp2app(W_SetObject.descr_cmp),
 
     # comparison operators
-    __eq__ = gateway.interp2app(W_SetObject.descr_eq),
-    __ne__ = gateway.interp2app(W_SetObject.descr_ne),
-    __lt__ = gateway.interp2app(W_SetObject.descr_lt),
-    __le__ = gateway.interp2app(W_SetObject.descr_le),
-    __gt__ = gateway.interp2app(W_SetObject.descr_gt),
-    __ge__ = gateway.interp2app(W_SetObject.descr_ge),
+    __eq__ = gateway.interp2app(W_BaseSetObject.descr_eq),
+    __ne__ = gateway.interp2app(W_BaseSetObject.descr_ne),
+    __lt__ = gateway.interp2app(W_BaseSetObject.descr_lt),
+    __le__ = gateway.interp2app(W_BaseSetObject.descr_le),
+    __gt__ = gateway.interp2app(W_BaseSetObject.descr_gt),
+    __ge__ = gateway.interp2app(W_BaseSetObject.descr_ge),
 
     # non-mutating operators
-    __len__ = gateway.interp2app(W_SetObject.descr_len),
-    __iter__ = gateway.interp2app(W_SetObject.descr_iter),
-    __contains__ = gateway.interp2app(W_SetObject.descr_contains),
-    __sub__ = gateway.interp2app(W_SetObject.descr_sub),
-    __rsub__ = gateway.interp2app(W_SetObject.descr_rsub),
-    __and__ = gateway.interp2app(W_SetObject.descr_and),
-    __rand__ = gateway.interp2app(W_SetObject.descr_rand),
-    __or__ = gateway.interp2app(W_SetObject.descr_or),
-    __ror__ = gateway.interp2app(W_SetObject.descr_ror),
-    __xor__ = gateway.interp2app(W_SetObject.descr_xor),
-    __rxor__ = gateway.interp2app(W_SetObject.descr_rxor),
+    __len__ = gateway.interp2app(W_BaseSetObject.descr_len),
+    __iter__ = gateway.interp2app(W_BaseSetObject.descr_iter),
+    __contains__ = gateway.interp2app(W_BaseSetObject.descr_contains),
+    __sub__ = gateway.interp2app(W_BaseSetObject.descr_sub),
+    __and__ = gateway.interp2app(W_BaseSetObject.descr_and),
+    __or__ = gateway.interp2app(W_BaseSetObject.descr_or),
+    __xor__ = gateway.interp2app(W_BaseSetObject.descr_xor),
 
     # mutating operators
-    __isub__ = gateway.interp2app(W_SetObject.descr_inplace_sub),
-    __iand__ = gateway.interp2app(W_SetObject.descr_inplace_and),
-    __ior__ = gateway.interp2app(W_SetObject.descr_inplace_or),
-    __ixor__ = gateway.interp2app(W_SetObject.descr_inplace_xor),
+    __isub__ = gateway.interp2app(W_BaseSetObject.descr_inplace_sub),
+    __iand__ = gateway.interp2app(W_BaseSetObject.descr_inplace_and),
+    __ior__ = gateway.interp2app(W_BaseSetObject.descr_inplace_or),
+    __ixor__ = gateway.interp2app(W_BaseSetObject.descr_inplace_xor),
 
     # non-mutating methods
-    __reduce__ = gateway.interp2app(W_SetObject.descr_reduce),
-    copy = gateway.interp2app(W_SetObject.descr_copy),
-    difference = gateway.interp2app(W_SetObject.descr_difference),
-    intersection = gateway.interp2app(W_SetObject.descr_intersection),
-    issubset = gateway.interp2app(W_SetObject.descr_issubset),
-    issuperset = gateway.interp2app(W_SetObject.descr_issuperset),
-    symmetric_difference = gateway.interp2app(W_SetObject.descr_symmetric_difference),
-    union = gateway.interp2app(W_SetObject.descr_union),
-    isdisjoint = gateway.interp2app(W_SetObject.descr_isdisjoint),
+    __reduce__ = gateway.interp2app(W_BaseSetObject.descr_reduce),
+    copy = gateway.interp2app(W_BaseSetObject.descr_copy),
+    difference = gateway.interp2app(W_BaseSetObject.descr_difference),
+    intersection = gateway.interp2app(W_BaseSetObject.descr_intersection),
+    issubset = gateway.interp2app(W_BaseSetObject.descr_issubset),
+    issuperset = gateway.interp2app(W_BaseSetObject.descr_issuperset),
+    symmetric_difference = gateway.interp2app(W_BaseSetObject.descr_symmetric_difference),
+    union = gateway.interp2app(W_BaseSetObject.descr_union),
+    isdisjoint = gateway.interp2app(W_BaseSetObject.descr_isdisjoint),
 
     # mutating methods
-    add = gateway.interp2app(W_SetObject.descr_add),
-    clear = gateway.interp2app(W_SetObject.descr_clear),
-    difference_update = gateway.interp2app(W_SetObject.descr_difference_update),
-    discard = gateway.interp2app(W_SetObject.descr_discard),
-    intersection_update = gateway.interp2app(W_SetObject.descr_intersection_update),
-    pop = gateway.interp2app(W_SetObject.descr_pop),
-    remove = gateway.interp2app(W_SetObject.descr_remove),
-    symmetric_difference_update = gateway.interp2app(W_SetObject.descr_symmetric_difference_update),
-    update = gateway.interp2app(W_SetObject.descr_update)
+    add = gateway.interp2app(W_BaseSetObject.descr_add),
+    clear = gateway.interp2app(W_BaseSetObject.descr_clear),
+    difference_update = gateway.interp2app(W_BaseSetObject.descr_difference_update),
+    discard = gateway.interp2app(W_BaseSetObject.descr_discard),
+    intersection_update = gateway.interp2app(W_BaseSetObject.descr_intersection_update),
+    pop = gateway.interp2app(W_BaseSetObject.descr_pop),
+    remove = gateway.interp2app(W_BaseSetObject.descr_remove),
+    symmetric_difference_update = gateway.interp2app(W_BaseSetObject.descr_symmetric_difference_update),
+    update = gateway.interp2app(W_BaseSetObject.descr_update)
     )
 set_typedef = W_SetObject.typedef
 
@@ -620,12 +590,7 @@ class W_FrozensetObject(W_BaseSetObject):
 
     def _newobj(self, space, w_iterable):
         """Make a new frozenset by taking ownership of 'w_iterable'."""
-        if type(self) is W_FrozensetObject:
-            return W_FrozensetObject(space, w_iterable)
-        w_type = space.type(self)
-        w_obj = space.allocate_instance(W_FrozensetObject, w_type)
-        W_FrozensetObject.__init__(w_obj, space, w_iterable)
-        return w_obj
+        return W_FrozensetObject(space, w_iterable)
 
     @staticmethod
     def descr_new2(space, w_frozensettype, w_iterable=None):
@@ -658,52 +623,41 @@ class W_FrozensetObject(W_BaseSetObject):
 
         return space.newint(hash)
 
-    def cpyext_add_frozen(self, w_key):
-        if self.hash != 0:
-            return False
-        self.add(w_key)
-        return True
-
 W_FrozensetObject.typedef = TypeDef("frozenset",
     __doc__ = """frozenset(iterable) --> frozenset object
 
 Build an immutable unordered collection.""",
     __new__ = gateway.interp2app(W_FrozensetObject.descr_new2),
-    __repr__ = gateway.interp2app(W_FrozensetObject.descr_repr),
+    __repr__ = gateway.interp2app(W_BaseSetObject.descr_repr),
     __hash__ = gateway.interp2app(W_FrozensetObject.descr_hash),
-    __cmp__ = gateway.interp2app(W_FrozensetObject.descr_cmp),
 
     # comparison operators
-    __eq__ = gateway.interp2app(W_FrozensetObject.descr_eq),
-    __ne__ = gateway.interp2app(W_FrozensetObject.descr_ne),
-    __lt__ = gateway.interp2app(W_FrozensetObject.descr_lt),
-    __le__ = gateway.interp2app(W_FrozensetObject.descr_le),
-    __gt__ = gateway.interp2app(W_FrozensetObject.descr_gt),
-    __ge__ = gateway.interp2app(W_FrozensetObject.descr_ge),
+    __eq__ = gateway.interp2app(W_BaseSetObject.descr_eq),
+    __ne__ = gateway.interp2app(W_BaseSetObject.descr_ne),
+    __lt__ = gateway.interp2app(W_BaseSetObject.descr_lt),
+    __le__ = gateway.interp2app(W_BaseSetObject.descr_le),
+    __gt__ = gateway.interp2app(W_BaseSetObject.descr_gt),
+    __ge__ = gateway.interp2app(W_BaseSetObject.descr_ge),
 
     # non-mutating operators
-    __len__ = gateway.interp2app(W_FrozensetObject.descr_len),
-    __iter__ = gateway.interp2app(W_FrozensetObject.descr_iter),
-    __contains__ = gateway.interp2app(W_FrozensetObject.descr_contains),
-    __sub__ = gateway.interp2app(W_FrozensetObject.descr_sub),
-    __rsub__ = gateway.interp2app(W_FrozensetObject.descr_rsub),
-    __and__ = gateway.interp2app(W_FrozensetObject.descr_and),
-    __rand__ = gateway.interp2app(W_FrozensetObject.descr_rand),
-    __or__ = gateway.interp2app(W_FrozensetObject.descr_or),
-    __ror__ = gateway.interp2app(W_FrozensetObject.descr_ror),
-    __xor__ = gateway.interp2app(W_FrozensetObject.descr_xor),
-    __rxor__ = gateway.interp2app(W_FrozensetObject.descr_rxor),
+    __len__ = gateway.interp2app(W_BaseSetObject.descr_len),
+    __iter__ = gateway.interp2app(W_BaseSetObject.descr_iter),
+    __contains__ = gateway.interp2app(W_BaseSetObject.descr_contains),
+    __sub__ = gateway.interp2app(W_BaseSetObject.descr_sub),
+    __and__ = gateway.interp2app(W_BaseSetObject.descr_and),
+    __or__ = gateway.interp2app(W_BaseSetObject.descr_or),
+    __xor__ = gateway.interp2app(W_BaseSetObject.descr_xor),
 
     # non-mutating methods
-    __reduce__ = gateway.interp2app(W_FrozensetObject.descr_reduce),
-    copy = gateway.interp2app(W_FrozensetObject.descr_copy),
-    difference = gateway.interp2app(W_FrozensetObject.descr_difference),
-    intersection = gateway.interp2app(W_FrozensetObject.descr_intersection),
-    issubset = gateway.interp2app(W_FrozensetObject.descr_issubset),
-    issuperset = gateway.interp2app(W_FrozensetObject.descr_issuperset),
-    symmetric_difference = gateway.interp2app(W_FrozensetObject.descr_symmetric_difference),
-    union = gateway.interp2app(W_FrozensetObject.descr_union),
-    isdisjoint = gateway.interp2app(W_FrozensetObject.descr_isdisjoint)
+    __reduce__ = gateway.interp2app(W_BaseSetObject.descr_reduce),
+    copy = gateway.interp2app(W_BaseSetObject.descr_copy),
+    difference = gateway.interp2app(W_BaseSetObject.descr_difference),
+    intersection = gateway.interp2app(W_BaseSetObject.descr_intersection),
+    issubset = gateway.interp2app(W_BaseSetObject.descr_issubset),
+    issuperset = gateway.interp2app(W_BaseSetObject.descr_issuperset),
+    symmetric_difference = gateway.interp2app(W_BaseSetObject.descr_symmetric_difference),
+    union = gateway.interp2app(W_BaseSetObject.descr_union),
+    isdisjoint = gateway.interp2app(W_BaseSetObject.descr_isdisjoint)
     )
 frozenset_typedef = W_FrozensetObject.typedef
 
@@ -852,8 +806,6 @@ class EmptySetStrategy(SetStrategy):
         return []
 
     def has_key(self, w_set, w_key):
-        # make sure the key is hashable, issue 3824
-        self.space.hash_w(w_key)
         return False
 
     def equals(self, w_set, w_other):
@@ -1595,10 +1547,23 @@ class W_SetIterObject(W_Root):
             return w_key
         raise OperationError(space.w_StopIteration, space.w_None)
 
+    def descr_reduce(self, space):
+        # copy the iterator state
+        w_set = self.iterimplementation.setimplementation
+        w_clone = W_SetIterObject(space, w_set.iter())
+        # spool until we have the same pos
+        for x in xrange(self.iterimplementation.pos):
+            w_clone.descr_next(space)
+        w_res = space.call_function(space.w_list, w_clone)
+        w_iter = space.builtin.get('iter')
+        return space.newtuple([w_iter, space.newtuple([w_res])])
+
+
 W_SetIterObject.typedef = TypeDef("setiterator",
     __length_hint__ = gateway.interp2app(W_SetIterObject.descr_length_hint),
     __iter__ = gateway.interp2app(W_SetIterObject.descr_iter),
-    next = gateway.interp2app(W_SetIterObject.descr_next)
+    __next__ = gateway.interp2app(W_SetIterObject.descr_next),
+    __reduce__ = gateway.interp2app(W_SetIterObject.descr_reduce),
     )
 setiter_typedef = W_SetIterObject.typedef
 
@@ -1643,12 +1608,10 @@ def set_strategy_and_setdata(space, w_set, w_iterable):
 
     length_hint = space.length_hint(w_iterable, 0)
 
-    if jit.isconstant(length_hint) and length_hint:
+    if jit.isconstant(length_hint):
         return _pick_correct_strategy_unroll(space, w_set, w_iterable)
 
-    w_set.strategy = strategy = space.fromcache(EmptySetStrategy)
-    w_set.sstorage = strategy.get_empty_storage()
-    _update_from_iterable(space, w_set, w_iterable)
+    _create_from_iterable(space, w_set, w_iterable)
 
 
 @jit.unroll_safe
@@ -1696,15 +1659,18 @@ def _pick_correct_strategy_unroll(space, w_set, w_iterable):
 
 
 def get_printable_location(tp, strategy):
-    return "update_set: %s %s" % (tp.iterator_greenkey_printable(), strategy)
+    return "create_set: %s %s" % (tp, strategy)
 
-update_set_driver = jit.JitDriver(name='update_set',
+create_set_driver = jit.JitDriver(name='create_set',
                                   greens=['tp', 'strategy'],
                                   reds='auto',
                                   get_printable_location=get_printable_location)
 
-def _update_from_iterable(space, w_set, w_iterable):
-    tp = space.iterator_greenkey(w_iterable)
+def _create_from_iterable(space, w_set, w_iterable):
+    w_set.strategy = strategy = space.fromcache(EmptySetStrategy)
+    w_set.sstorage = strategy.get_empty_storage()
+
+    tp = space.type(w_iterable)
 
     w_iter = space.iter(w_iterable)
     while True:
@@ -1714,7 +1680,7 @@ def _update_from_iterable(space, w_set, w_iterable):
             if not e.match(space, space.w_StopIteration):
                 raise
             return
-        update_set_driver.jit_merge_point(tp=tp, strategy=w_set.strategy)
+        create_set_driver.jit_merge_point(tp=tp, strategy=w_set.strategy)
         w_set.add(w_item)
 
 
@@ -1745,7 +1711,13 @@ app = gateway.applevel("""
             return '%s(...)' % (s.__class__.__name__,)
         currently_in_repr[s] = 1
         try:
-            return '%s(%s)' % (s.__class__.__name__, [x for x in s])
+            if not s:
+                return '%s()' % (s.__class__.__name__,)
+            listrepr = repr([x for x in s])
+            if type(s) is set:
+                return '{%s}' % (listrepr[1:-1],)
+            else:
+                return '%s({%s})' % (s.__class__.__name__, listrepr[1:-1])
         finally:
             try:
                 del currently_in_repr[s]
