@@ -1,9 +1,9 @@
-# -*- encoding: utf-8 -*-
+# encoding: utf-8
 import sys
 import py
 
 from pypy.objspace.std.dictmultiobject import (W_DictMultiObject,
-    W_DictObject, BytesDictStrategy, ObjectDictStrategy)
+    W_DictObject, BytesDictStrategy, ObjectDictStrategy, UnicodeDictStrategy)
 
 
 class TestW_DictObject(object):
@@ -125,6 +125,7 @@ class TestW_DictObject(object):
         assert self.space.eq_w(space.call_function(get, w("33"), w(44)), w(44))
 
     def test_fromkeys_fastpath(self):
+        py.test.skip("doesn't make sense here")
         space = self.space
         w = space.wrap
 
@@ -147,7 +148,7 @@ class TestW_DictObject(object):
         w = self.space.wrap
         w_d = self.space.newdict()
         w_d.initialize_content([(w(u"a"), w(1)), (w(u"b"), w(2))])
-        assert self.space.listview_utf8(w_d) == ["a", "b"]
+        assert self.space.listview_ascii(w_d) == ["a", "b"]
 
     def test_listview_int_dict(self):
         w = self.space.wrap
@@ -161,17 +162,19 @@ class TestW_DictObject(object):
 
         w_d = self.space.newdict()
         w_d.initialize_content([(w(1), wb("a")), (w(2), wb("b"))])
-        w_l = self.space.call_method(w_d, "keys")
+        w_k = self.space.call_method(w_d, "keys")
+        w_l = self.space.call_function(self.space.w_list, w_k)
         assert sorted(self.space.listview_int(w_l)) == [1,2]
 
-        # make sure that .keys() calls newlist_bytes for string dicts
+        # make sure that list(d.keys()) calls newlist_bytes for byte dicts
         def not_allowed(*args):
             assert False, 'should not be called'
         monkeypatch.setattr(self.space, 'newlist', not_allowed)
         #
         w_d = self.space.newdict()
         w_d.initialize_content([(wb("a"), w(1)), (wb("b"), w(6))])
-        w_l = self.space.call_method(w_d, "keys")
+        w_k = self.space.call_method(w_d, "keys")
+        w_l = self.space.call_function(self.space.w_list, w_k)
         assert sorted(self.space.listview_bytes(w_l)) == ["a", "b"]
 
         #---the rest is for listview_unicode(), which is disabled---
@@ -257,21 +260,16 @@ class AppTest_DictObject:
         raises(KeyError, d.pop, "abc")
         assert len(d) == 2
 
-    def test_has_key(self):
-        d = {1: 2, 3: 4}
-        assert d.has_key(1)
-        assert not d.has_key(33)
-
     def test_items(self):
         d = {1: 2, 3: 4}
-        its = d.items()
+        its = list(d.items())
         its.sort()
         assert its == [(1, 2), (3, 4)]
 
     def test_iteritems(self):
         d = {1: 2, 3: 4}
         dd = d.copy()
-        for k, v in d.iteritems():
+        for k, v in d.items():
             assert v == dd[k]
             del dd[k]
         assert not dd
@@ -279,32 +277,32 @@ class AppTest_DictObject:
     def test_iterkeys(self):
         d = {1: 2, 3: 4}
         dd = d.copy()
-        for k in d.iterkeys():
+        for k in d.keys():
             del dd[k]
         assert not dd
 
     def test_itervalues(self):
         d = {1: 2, 3: 4}
         values = []
-        for k in d.itervalues():
+        for k in d.values():
             values.append(k)
-        assert values == d.values()
+        assert values == list(d.values())
 
     def test_reversed_dict(self):
         import __pypy__
         def kw(**d): return d
         for d in [{}, {1: 2, 3: 4, 5: 6}, {"a": 5, "b": 2, "c": 6}, kw(a=1, b=2)]:
-            assert list(__pypy__.reversed_dict(d)) == d.keys()[::-1]
+            assert list(__pypy__.reversed_dict(d)) == list(d.keys())[::-1]
         raises(TypeError, __pypy__.reversed_dict, 42)
 
     def test_reversed_dict_runtimeerror(self):
         import __pypy__
         d = {1: 2, 3: 4, 5: 6}
         it = __pypy__.reversed_dict(d)
-        key = it.next()
+        key = next(it)
         assert key in [1, 3, 5]   # on CPython, dicts are not ordered
         del d[key]
-        raises(RuntimeError, it.next)
+        raises(RuntimeError, next, it)
 
     def test_dict_popitem_first(self):
         import __pypy__
@@ -359,7 +357,7 @@ class AppTest_DictObject:
 
     def test_keys(self):
         d = {1: 2, 3: 4}
-        kys = d.keys()
+        kys = list(d.keys())
         kys.sort()
         assert kys == [1, 3]
 
@@ -482,9 +480,9 @@ class AppTest_DictObject:
 
     def test_values(self):
         d = {1: 2, 3: 4}
-        vals = d.values()
+        vals = list(d.values())
         vals.sort()
-        assert vals == [2,4]
+        assert vals == [2, 4]
 
     def test_eq(self):
         d1 = {1: 2, 3: 4}
@@ -499,45 +497,13 @@ class AppTest_DictObject:
         bool = d1 != d3
         assert bool == True
 
-    def test_lt(self):
+    def test_richcompare(self):
+        import operator
         d1 = {1: 2, 3: 4}
-        d2 = {1: 2, 3: 4}
-        d3 = {1: 2, 3: 5}
-        d4 = {1: 2}
-        bool = d1 < d2
-        assert bool == False
-        bool = d1 < d3
-        assert bool == True
-        bool = d1 < d4
-        assert bool == False
-
-    def test_lt2(self):
-        assert {'a': 1 } < { 'a': 2 }
-        assert not {'a': 1 } > { 'a': 2 }
-        assert not {'a': 1, 'b': 0 } > { 'a': 2, 'b': 0 }
-        assert {'a': 1, 'b': 0 } < { 'a': 2, 'b': 0 }
-        assert {'a': 1, 'b': 0 } < { 'a': 1, 'b': 2 }
-        assert not {'a': 1, 'b': 0 } < { 'a': 1, 'b': -2 }
-        assert {'a': 1 } < { 'b': 1}
-        assert {'a': 1, 'x': 2 } < { 'b': 1, 'x': 2}
-
-    def test_other_rich_cmp(self):
-        d1 = {1: 2, 3: 4}
-        d2 = {1: 2, 3: 4}
-        d3 = {1: 2, 3: 5}
-        d4 = {1: 2}
-
-        assert d1 <= d2
-        assert d1 <= d3
-        assert not d1 <= d4
-
-        assert not d1 > d2
-        assert not d1 > d3
-        assert d1 > d4
-
-        assert d1 >= d2
-        assert not d1 >= d3
-        assert d1 >= d4
+        d2 = {1: 2, 3: 5}
+        for op in 'lt', 'le', 'gt', 'ge':
+            f = getattr(operator, op)
+            raises(TypeError, f, d1, d2)
 
     def test_str_repr(self):
         assert '{}' == str({})
@@ -742,12 +708,20 @@ class AppTest_DictObject:
     def test_empty_dict(self):
         d = {}
         raises(KeyError, d.popitem)
-        assert d.items() == []
-        assert d.values() == []
-        assert d.keys() == []
+        assert list(d.items()) == []
+        assert list(d.values()) == []
+        assert list(d.keys()) == []
+
+    def test_bytes_keys(self):
+        assert isinstance(list({b'a': 1})[0], bytes)
+
+    def test_interned_keywords(self):
+        skip("no longer works")
+        # At some point in the past, we had kwargsdict automatically
+        # intern every single key we get out of it.  That's a big
+        # pointless waste of time.  So the following test fails now.
+        assert list(dict(abcdef=1))[0] is 'abcdef'
 
-    def test_cmp_with_noncmp(self):
-        assert not {} > object()
 
 class AppTest_DictMultiObject(AppTest_DictObject):
 
@@ -764,7 +738,7 @@ class AppTest_DictMultiObject(AppTest_DictObject):
         a = A()
         s = S("abc")
         setattr(a, s, 42)
-        key = a.__dict__.keys()[0]
+        key = next(iter(a.__dict__.keys()))
         assert key == s
         assert key is not s
         assert type(key) is str
@@ -812,24 +786,13 @@ class AppTest_DictMultiObject(AppTest_DictObject):
 class AppTestDictViews:
     def test_dictview(self):
         d = {1: 2, 3: 4}
-        assert len(d.viewkeys()) == 2
-        assert len(d.viewitems()) == 2
-        assert len(d.viewvalues()) == 2
-
-    def test_constructors_not_callable(self):
-        kt = type({}.viewkeys())
-        raises(TypeError, kt, {})
-        raises(TypeError, kt)
-        it = type({}.viewitems())
-        raises(TypeError, it, {})
-        raises(TypeError, it)
-        vt = type({}.viewvalues())
-        raises(TypeError, vt, {})
-        raises(TypeError, vt)
+        assert len(d.keys()) == 2
+        assert len(d.items()) == 2
+        assert len(d.values()) == 2
 
     def test_dict_keys(self):
         d = {1: 10, "a": "ABC"}
-        keys = d.viewkeys()
+        keys = d.keys()
         assert len(keys) == 2
         assert set(keys) == set([1, "a"])
         assert keys == set([1, "a"])
@@ -838,22 +801,22 @@ class AppTestDictViews:
         assert keys != set([1, "b"])
         assert keys != set([1])
         assert keys != 42
+        assert not keys == 42
         assert 1 in keys
         assert "a" in keys
         assert 10 not in keys
         assert "Z" not in keys
         raises(TypeError, "[] in keys")     # [] is unhashable
         raises(TypeError, keys.__contains__, [])
-        assert d.viewkeys() == d.viewkeys()
+        assert d.keys() == d.keys()
         e = {1: 11, "a": "def"}
-        assert d.viewkeys() == e.viewkeys()
+        assert d.keys() == e.keys()
         del e["a"]
-        assert d.viewkeys() != e.viewkeys()
-        assert not d.viewkeys() == 42
+        assert d.keys() != e.keys()
 
     def test_dict_items(self):
         d = {1: 10, "a": "ABC"}
-        items = d.viewitems()
+        items = d.items()
         assert len(items) == 2
         assert set(items) == set([(1, 10), ("a", "ABC")])
         assert items == set([(1, 10), ("a", "ABC")])
@@ -862,6 +825,7 @@ class AppTestDictViews:
         assert items != set([(1, 10), ("a", "def")])
         assert items != set([(1, 10)])
         assert items != 42
+        assert not items == 42
         assert (1, 10) in items
         assert ("a", "ABC") in items
         assert (1, 11) not in items
@@ -869,14 +833,13 @@ class AppTestDictViews:
         assert () not in items
         assert (1,) not in items
         assert (1, 2, 3) not in items
-        assert ([], []) not in items     # [] is unhashable, but no TypeError
-        assert not items.__contains__(([], []))
-        assert d.viewitems() == d.viewitems()
+        raises(TypeError, "([], []) not in items")     # [] is unhashable
+        raises(TypeError, items.__contains__, ([], []))
+        assert d.items() == d.items()
         e = d.copy()
-        assert d.viewitems() == e.viewitems()
+        assert d.items() == e.items()
         e["a"] = "def"
-        assert d.viewitems() != e.viewitems()
-        assert not d.viewitems() == 42
+        assert d.items() != e.items()
 
     def test_dict_items_contains_with_identity(self):
         class BadEq(object):
@@ -886,17 +849,17 @@ class AppTestDictViews:
                 return 7
         k = BadEq()
         v = BadEq()
-        assert (k, v) in {k: v}.viewitems()
+        assert (k, v) in {k: v}.items()
 
     def test_dict_mixed_keys_items(self):
         d = {(1, 1): 11, (2, 2): 22}
         e = {1: 1, 2: 2}
-        assert d.viewkeys() == e.viewitems()
-        assert d.viewitems() != e.viewkeys()
+        assert d.keys() == e.items()
+        assert d.items() != e.keys()
 
     def test_dict_values(self):
         d = {1: 10, "a": "ABC"}
-        values = d.viewvalues()
+        values = d.values()
         assert set(values) == set([10, "ABC"])
         assert len(values) == 2
         assert not values == 42
@@ -904,140 +867,179 @@ class AppTestDictViews:
     def test_dict_repr(self):
         d = {1: 10, "a": "ABC"}
         assert isinstance(repr(d), str)
-        r = repr(d.viewitems())
+        r = repr(d.items())
         assert isinstance(r, str)
         assert (r == "dict_items([('a', 'ABC'), (1, 10)])" or
                 r == "dict_items([(1, 10), ('a', 'ABC')])")
-        r = repr(d.viewkeys())
+        r = repr(d.keys())
         assert isinstance(r, str)
         assert (r == "dict_keys(['a', 1])" or
                 r == "dict_keys([1, 'a'])")
-        r = repr(d.viewvalues())
+        r = repr(d.values())
         assert isinstance(r, str)
         assert (r == "dict_values(['ABC', 10])" or
                 r == "dict_values([10, 'ABC'])")
+        d = {'日本': '日本国'}
+        assert repr(d.items()) == "dict_items([('日本', '日本国')])"
 
     def test_keys_set_operations(self):
         d1 = {'a': 1, 'b': 2}
         d2 = {'b': 3, 'c': 2}
         d3 = {'d': 4, 'e': 5}
-        assert d1.viewkeys() & d1.viewkeys() == set('ab')
-        assert d1.viewkeys() & d2.viewkeys() == set('b')
-        assert d1.viewkeys() & d3.viewkeys() == set()
-        assert d1.viewkeys() & set(d1.viewkeys()) == set('ab')
-        assert d1.viewkeys() & set(d2.viewkeys()) == set('b')
-        assert d1.viewkeys() & set(d3.viewkeys()) == set()
+        assert d1.keys() & d1.keys() == set('ab')
+        assert d1.keys() & d2.keys() == set('b')
+        assert d1.keys() & d3.keys() == set()
+        assert d1.keys() & set(d1.keys()) == set('ab')
+        assert d1.keys() & set(d2.keys()) == set('b')
+        assert d1.keys() & set(d3.keys()) == set()
 
-        assert d1.viewkeys() | d1.viewkeys() == set('ab')
-        assert d1.viewkeys() | d2.viewkeys() == set('abc')
-        assert d1.viewkeys() | d3.viewkeys() == set('abde')
-        assert d1.viewkeys() | set(d1.viewkeys()) == set('ab')
-        assert d1.viewkeys() | set(d2.viewkeys()) == set('abc')
-        assert d1.viewkeys() | set(d3.viewkeys()) == set('abde')
+        assert d1.keys() | d1.keys() == set('ab')
+        assert d1.keys() | d2.keys() == set('abc')
+        assert d1.keys() | d3.keys() == set('abde')
+        assert d1.keys() | set(d1.keys()) == set('ab')
+        assert d1.keys() | set(d2.keys()) == set('abc')
+        assert d1.keys() | set(d3.keys()) == set('abde')
 
-        assert d1.viewkeys() ^ d1.viewkeys() == set()
-        assert d1.viewkeys() ^ d2.viewkeys() == set('ac')
-        assert d1.viewkeys() ^ d3.viewkeys() == set('abde')
-        assert d1.viewkeys() ^ set(d1.viewkeys()) == set()
-        assert d1.viewkeys() ^ set(d2.viewkeys()) == set('ac')
-        assert d1.viewkeys() ^ set(d3.viewkeys()) == set('abde')
+        assert d1.keys() ^ d1.keys() == set()
+        assert d1.keys() ^ d2.keys() == set('ac')
+        assert d1.keys() ^ d3.keys() == set('abde')
+        assert d1.keys() ^ set(d1.keys()) == set()
+        assert d1.keys() ^ set(d2.keys()) == set('ac')
+        assert d1.keys() ^ set(d3.keys()) == set('abde')
+
+        assert d1.keys() - d1.keys() == set()
+        assert d1.keys() - d2.keys() == set('a')
+        assert d1.keys() - d3.keys() == set('ab')
+        assert d1.keys() - set(d1.keys()) == set()
+        assert d1.keys() - set(d2.keys()) == set('a')
+        assert d1.keys() - set(d3.keys()) == set('ab')
+
+        assert not d1.keys().isdisjoint(d1.keys())
+        assert not d1.keys().isdisjoint(d2.keys())
+        assert not d1.keys().isdisjoint(list(d2.keys()))
+        assert not d1.keys().isdisjoint(set(d2.keys()))
+
+        assert d1.keys().isdisjoint(['x', 'y', 'z'])
+        assert d1.keys().isdisjoint(set(['x', 'y', 'z']))
+        assert d1.keys().isdisjoint(set(['x', 'y']))
+        assert d1.keys().isdisjoint(['x', 'y'])
+        assert d1.keys().isdisjoint({})
+        assert d1.keys().isdisjoint(d3.keys())
+
+        de = {}
+        assert de.keys().isdisjoint(set())
+        assert de.keys().isdisjoint([])
+        assert de.keys().isdisjoint(de.keys())
+        assert de.keys().isdisjoint([1])
 
-        assert d1.viewkeys() - d1.viewkeys() == set()
-        assert d1.viewkeys() - d2.viewkeys() == set('a')
-        assert d1.viewkeys() - d3.viewkeys() == set('ab')
-        assert d1.viewkeys() - set(d1.viewkeys()) == set()
-        assert d1.viewkeys() - set(d2.viewkeys()) == set('a')
-        assert d1.viewkeys() - set(d3.viewkeys()) == set('ab')
 
     def test_items_set_operations(self):
         d1 = {'a': 1, 'b': 2}
         d2 = {'a': 2, 'b': 2}
         d3 = {'d': 4, 'e': 5}
-        assert d1.viewitems() & d1.viewitems() == set([('a', 1), ('b', 2)])
-        assert d1.viewitems() & d2.viewitems() == set([('b', 2)])
-        assert d1.viewitems() & d3.viewitems() == set()
-        assert d1.viewitems() & set(d1.viewitems()) == set([('a', 1), ('b', 2)])
-        assert d1.viewitems() & set(d2.viewitems()) == set([('b', 2)])
-        assert d1.viewitems() & set(d3.viewitems()) == set()
+        assert d1.items() & d1.items() == set([('a', 1), ('b', 2)])
+        assert d1.items() & d2.items() == set([('b', 2)])
+        assert d1.items() & d3.items() == set()
+        assert d1.items() & set(d1.items()) == set([('a', 1), ('b', 2)])
+        assert d1.items() & set(d2.items()) == set([('b', 2)])
+        assert d1.items() & set(d3.items()) == set()
 
-        assert d1.viewitems() | d1.viewitems() == set([('a', 1), ('b', 2)])
-        assert (d1.viewitems() | d2.viewitems() ==
+        assert d1.items() | d1.items() == set([('a', 1), ('b', 2)])
+        assert (d1.items() | d2.items() ==
                 set([('a', 1), ('a', 2), ('b', 2)]))
-        assert (d1.viewitems() | d3.viewitems() ==
+        assert (d1.items() | d3.items() ==
                 set([('a', 1), ('b', 2), ('d', 4), ('e', 5)]))
-        assert (d1.viewitems() | set(d1.viewitems()) ==
+        assert (d1.items() | set(d1.items()) ==
                 set([('a', 1), ('b', 2)]))
-        assert (d1.viewitems() | set(d2.viewitems()) ==
+        assert (d1.items() | set(d2.items()) ==
                 set([('a', 1), ('a', 2), ('b', 2)]))
-        assert (d1.viewitems() | set(d3.viewitems()) ==
+        assert (d1.items() | set(d3.items()) ==
                 set([('a', 1), ('b', 2), ('d', 4), ('e', 5)]))
 
-        assert d1.viewitems() ^ d1.viewitems() == set()
-        assert d1.viewitems() ^ d2.viewitems() == set([('a', 1), ('a', 2)])
-        assert (d1.viewitems() ^ d3.viewitems() ==
+        assert d1.items() ^ d1.items() == set()
+        assert d1.items() ^ d2.items() == set([('a', 1), ('a', 2)])
+        assert (d1.items() ^ d3.items() ==
                 set([('a', 1), ('b', 2), ('d', 4), ('e', 5)]))
 
-        assert d1.viewitems() - d1.viewitems() == set()
-        assert d1.viewitems() - d2.viewitems() == set([('a', 1)])
-        assert d1.viewitems() - d3.viewitems() == set([('a', 1), ('b', 2)])
+        assert d1.items() - d1.items() == set()
+        assert d1.items() - d2.items() == set([('a', 1)])
+        assert d1.items() - d3.items() == set([('a', 1), ('b', 2)])
+
+        assert not d1.items().isdisjoint(d1.items())
+        assert not d1.items().isdisjoint(d2.items())
+        assert not d1.items().isdisjoint(list(d2.items()))
+        assert not d1.items().isdisjoint(set(d2.items()))
+        assert d1.items().isdisjoint(['x', 'y', 'z'])
+        assert d1.items().isdisjoint(set(['x', 'y', 'z']))
+        assert d1.items().isdisjoint(set(['x', 'y']))
+        assert d1.items().isdisjoint({})
+        assert d1.items().isdisjoint(d3.items())
+
+        de = {}
+        assert de.items().isdisjoint(set())
+        assert de.items().isdisjoint([])
+        assert de.items().isdisjoint(de.items())
+        assert de.items().isdisjoint([1])
 
     def test_keys_set_operations_any_type(self):
-        d = {1: u'a', 2: u'b', 3: u'c'}
-        assert d.viewkeys() & set([1]) == set([1])
-        assert d.viewkeys() & {1: u'foo'} == set([1])
-        assert d.viewkeys() & [1, 2] == set([1, 2])
+        """
+        d = {1: 'a', 2: 'b', 3: 'c'}
+        assert d.keys() & {1} == {1}
+        assert d.keys() & {1: 'foo'} == {1}
+        assert d.keys() & [1, 2] == {1, 2}
         #
-        assert set([1]) & d.viewkeys() == set([1])
-        assert {1: u'foo'} & d.viewkeys() == set([1])
-        assert [1, 2] & d.viewkeys() == set([1, 2])
+        assert {1} & d.keys() == {1}
+        assert {1: 'foo'} & d.keys() == {1}
+        assert [1, 2] & d.keys() == {1, 2}
         #
-        assert d.viewkeys() - set([1]) == set([2, 3])
-        assert set([1, 4]) - d.viewkeys() == set([4])
+        assert d.keys() - {1} == {2, 3}
+        assert {1, 4} - d.keys() == {4}
         #
-        assert d.viewkeys() == set([1, 2, 3])
-        # XXX: The following 4 commented out are CPython 2.7 bugs
-        #assert set([1, 2, 3]) == d.viewkeys()
-        assert d.viewkeys() == frozenset(set([1, 2, 3]))
-        #assert frozenset(set([1, 2, 3])) == d.viewkeys()
-        assert not d.viewkeys() != set([1, 2, 3])
-        #assert not set([1, 2, 3]) != d.viewkeys()
-        assert not d.viewkeys() != frozenset(set([1, 2, 3]))
-        #assert not frozenset(set([1, 2, 3])) != d.viewkeys()
+        assert d.keys() == {1, 2, 3}
+        assert {1, 2, 3} == d.keys()
+        assert d.keys() == frozenset({1, 2, 3})
+        assert frozenset({1, 2, 3}) == d.keys()
+        assert not d.keys() != {1, 2, 3}
+        assert not {1, 2, 3} != d.keys()
+        assert not d.keys() != frozenset({1, 2, 3})
+        assert not frozenset({1, 2, 3}) != d.keys()
+        """
 
     def test_items_set_operations_any_type(self):
-        d = {1: u'a', 2: u'b', 3: u'c'}
-        assert d.viewitems() & set([(1, u'a')]) == set([(1, u'a')])
-        assert d.viewitems() & {(1, u'a'): u'foo'} == set([(1, u'a')])
-        assert d.viewitems() & [(1, u'a'), (2, u'b')] == set([(1, u'a'), (2, u'b')])
+        """
+        d = {1: 'a', 2: 'b', 3: 'c'}
+        assert d.items() & {(1, 'a')} == {(1, 'a')}
+        assert d.items() & {(1, 'a'): 'foo'} == {(1, 'a')}
+        assert d.items() & [(1, 'a'), (2, 'b')] == {(1, 'a'), (2, 'b')}
         #
-        assert set([(1, u'a')]) & d.viewitems() == set([(1, u'a')])
-        assert {(1, u'a'): u'foo'} & d.viewitems() == set([(1, u'a')])
-        assert [(1, u'a'), (2, u'b')] & d.viewitems() == set([(1, u'a'), (2, u'b')])
+        assert {(1, 'a')} & d.items() == {(1, 'a')}
+        assert {(1, 'a'): 'foo'} & d.items() == {(1, 'a')}
+        assert [(1, 'a'), (2, 'b')] & d.items() == {(1, 'a'), (2, 'b')}
         #
-        assert d.viewitems() - set([(1, u'a')]) == set([(2, u'b'), (3, u'c')])
-        assert set([(1, u'a'), 4]) - d.viewitems() == set([4])
+        assert d.items() - {(1, 'a')} == {(2, 'b'), (3, 'c')}
+        assert {(1, 'a'), 4} - d.items() == {4}
         #
-        assert d.viewitems() == set([(1, u'a'), (2, u'b'), (3, u'c')])
-        # XXX: The following 4 commented out are CPython 2.7 bugs
-        #assert set([(1, u'a'), (2, u'b'), (3, u'c')]) == d.viewitems()
-        assert d.viewitems() == frozenset(set([(1, u'a'), (2, u'b'), (3, u'c')]))
-        #assert frozenset(set([(1, u'a'), (2, u'b'), (3, u'c')])) == d.viewitems()
-        assert not d.viewitems() != set([(1, u'a'), (2, u'b'), (3, u'c')])
-        #assert not set([(1, u'a'), (2, u'b'), (3, u'c')]) != d.viewitems()
-        assert not d.viewitems() != frozenset(set([(1, u'a'), (2, u'b'), (3, u'c')]))
-        #assert not frozenset(set([(1, u'a'), (2, u'b'), (3, u'c')])) != d.viewitems()
+        assert d.items() == {(1, 'a'), (2, 'b'), (3, 'c')}
+        assert {(1, 'a'), (2, 'b'), (3, 'c')} == d.items()
+        assert d.items() == frozenset({(1, 'a'), (2, 'b'), (3, 'c')})
+        assert frozenset({(1, 'a'), (2, 'b'), (3, 'c')}) == d.items()
+        assert not d.items() != {(1, 'a'), (2, 'b'), (3, 'c')}
+        assert not {(1, 'a'), (2, 'b'), (3, 'c')} != d.items()
+        assert not d.items() != frozenset({(1, 'a'), (2, 'b'), (3, 'c')})
+        assert not frozenset({(1, 'a'), (2, 'b'), (3, 'c')}) != d.items()
+        """
 
     def test_dictviewset_unhashable_values(self):
         class C:
             def __eq__(self, other):
                 return True
         d = {1: C()}
-        assert d.viewitems() <= d.viewitems()
+        assert d.items() <= d.items()
 
     def test_compare_keys_and_items(self):
         d1 = {1: 2}
         d2 = {(1, 2): 'foo'}
-        assert d1.viewitems() == d2.viewkeys()
+        assert d1.items() == d2.keys()
 
     def test_keys_items_contained(self):
         def helper(fn):
@@ -1082,8 +1084,20 @@ class AppTestDictViews:
             assert larger != larger3
             assert not larger == larger3
 
-        helper(lambda x: x.viewkeys())
-        helper(lambda x: x.viewitems())
+        helper(lambda x: x.keys())
+        helper(lambda x: x.items())
+
+    def test_pickle(self):
+        d = {1: 1, 2: 2, 3: 3}
+        it = iter(d)
+        first = next(it)
+        reduced = it.__reduce__()
+        rebuild, args = reduced
+        new = rebuild(*args)
+        items = set(new)
+        assert len(items) == 2
+        items.add(first)
+        assert items == set(d)
 
     def test_contains(self):
         logger = []
@@ -1142,17 +1156,30 @@ class AppTestStrategies(object):
         d = o.__dict__ = {}
         assert "EmptyDictStrategy" in self.get_strategy(d)
         o.a = 1
-        assert "BytesDictStrategy" in self.get_strategy(d)
+        assert "UnicodeDictStrategy" in self.get_strategy(d)
 
     def test_empty_to_unicode(self):
         d = {}
         assert "EmptyDictStrategy" in self.get_strategy(d)
         d[u"a"] = 1
         assert "UnicodeDictStrategy" in self.get_strategy(d)
-        assert d[u"a"] == 1
         assert d["a"] == 1
-        assert d.keys() == [u"a"]
-        assert type(d.keys()[0]) is unicode
+        #assert d[b"a"] == 1 # this works in py2, but not in py3
+        assert list(d.keys()) == ["a"]
+        assert type(list(d.keys())[0]) is str
+
+    def test_setitem_str_nonascii(self):
+        d = {}
+        assert "EmptyDictStrategy" in self.get_strategy(d)
+        d[u"a"] = 1
+        assert "UnicodeDictStrategy" in self.get_strategy(d)
+        exec("a = 1", d)
+        assert d["a"] == 1
+        assert "UnicodeDictStrategy" in self.get_strategy(d)
+        exec("ä = 2", d)
+        assert "UnicodeDictStrategy" in self.get_strategy(d)
+        assert d["a"] == 1
+        assert d["ä"] == 2
 
         d = {}
         d[u"ä"] = 1
@@ -1162,38 +1189,51 @@ class AppTestStrategies(object):
         d = {}
         d[1] = "hi"
         assert "IntDictStrategy" in self.get_strategy(d)
-        assert d[1L] == "hi"
 
     def test_iter_dict_length_change(self):
         d = {1: 2, 3: 4, 5: 6}
-        it = d.iteritems()
+        it = iter(d.items())
         d[7] = 8
         # 'd' is now length 4
-        raises(RuntimeError, it.next)
+        raises(RuntimeError, next, it)
 
     def test_iter_dict_strategy_only_change_1(self):
         d = {1: 2, 3: 4, 5: 6}
-        it = d.iteritems()
+        it = d.items()
         class Foo(object):
             def __eq__(self, other):
                 return False
+            def __hash__(self):
+                return 0
         assert d.get(Foo()) is None    # this changes the strategy of 'd'
         lst = list(it)  # but iterating still works
         assert sorted(lst) == [(1, 2), (3, 4), (5, 6)]
 
     def test_iter_dict_strategy_only_change_2(self):
         d = {1: 2, 3: 4, 5: 6}
-        it = d.iteritems()
+        it = d.items()
         d['foo'] = 'bar'
         del d[1]
+        # on default the strategy changes and thus we get the RuntimeError
+        # (commented below). On py3k, we Int and String strategies don't work
+        # yet, and thus we get the "correct" behavior
+        items = list(it)
+        assert set(items) == set([(3, 4), (5, 6), ('foo', 'bar')])
         # 'd' is still length 3, but its strategy changed.  we are
         # getting a RuntimeError because iterating over the old storage
         # gives us (1, 2), but 1 is not in the dict any longer.
-        raises(RuntimeError, list, it)
+        #raises(RuntimeError, list, it)
+
+    def test_bytes_to_object(self):
+        d = {b'a': 'b'}
+        d[object()] = None
+        assert b'a' in list(d)
 
 
-class FakeWrapper(object):
+class FakeString(str):
+
     hash_count = 0
+
     def unwrap(self, space):
         self.unwrapped = True
         return str(self)
@@ -1202,11 +1242,29 @@ class FakeWrapper(object):
         self.hash_count += 1
         return str.__hash__(self)
 
-class FakeString(FakeWrapper, str):
-    pass
+class FakeUnicode(unicode):
 
-class FakeUnicode(FakeWrapper, unicode):
-    pass
+    hash_count = 0
+
+    def unwrap(self, space):
+        self.unwrapped = True
+        return unicode(self)
+
+    def __hash__(self):
+        self.hash_count += 1
+        return unicode.__hash__(self)
+
+    def hash_w(self):
+        return hash(self)
+
+    def eq_w(self, other):
+        return self == other
+
+    def is_ascii(self):
+        return True
+
+    def unwrapped(self):
+        return True
 
 # the minimal 'space' needed to use a W_DictMultiObject
 class FakeSpace:
@@ -1228,28 +1286,58 @@ class FakeSpace:
         return l
     def newlist_bytes(self, l):
         return l
-    newlist_text = newlist_bytes
+    def newlist_text(self, l):
+        return l
+    def newlist_unicode(self, l):
+        return l
     DictObjectCls = W_DictObject
     def type(self, w_obj):
         if isinstance(w_obj, FakeString):
             return str
+        if isinstance(w_obj, FakeUnicode):
+            return unicode
         return type(w_obj)
+    w_unicode = unicode
+    w_text = unicode
     w_bytes = str
-    w_text = str
 
-    def str_w(self, string):
+    def text_w(self, u):
+        assert isinstance(u, unicode)
+        return FakeUnicode(u).encode('utf8')
+
+    def bytes_w(self, string):
         assert isinstance(string, str)
         return string
-    bytes_w = str_w
-    text_w = str_w
+
+    def utf8_w(self, u):
+        if isinstance(u, unicode):
+            u = u.encode('utf8')
+        assert isinstance(u, str)
+        return u
 
     def int_w(self, integer, allow_conversion=True):
         assert isinstance(integer, int)
         return integer
 
     def wrap(self, obj):
+        if isinstance(obj, str):
+            return FakeUnicode(obj.decode('ascii'))
         return obj
-    newtext = newbytes = wrap
+
+    def newtext(self, string):
+        if isinstance(string, str):
+            return FakeUnicode(string.decode('utf-8'))
+        assert isinstance(string, unicode)
+        return FakeUnicode(string)
+
+    def newutf8(self, obj, lgt):
+        return obj
+
+    def newbytes(self, obj):
+        return obj
+
+    def new_interned_str(self, s):
+        return s.decode('utf-8')
 
     def isinstance_w(self, obj, klass):
         return isinstance(obj, klass)
@@ -1322,13 +1410,18 @@ class TestDictImplementation:
             assert value == d.descr_getitem(self.space, key)
 
 class BaseTestRDictImplementation:
+    FakeString = FakeUnicode
+    _str_devolves = False
 
     def setup_method(self,method):
         self.fakespace = FakeSpace()
-        self.string = self.fakespace.wrap("fish")
-        self.string2 = self.fakespace.wrap("fish2")
+        self.string = self.wrapstrorunicode("fish")
+        self.string2 = self.wrapstrorunicode("fish2")
         self.impl = self.get_impl()
 
+    def wrapstrorunicode(self, obj):
+        return self.fakespace.wrap(obj)
+
     def get_impl(self):
         strategy = self.StrategyClass(self.fakespace)
         storage = strategy.get_empty_storage()
@@ -1356,21 +1449,22 @@ class BaseTestRDictImplementation:
         else:
             assert a == self.string2
             assert b == 2000
-            assert self.impl.getitem_str(self.string) == 1000
+            if not self._str_devolves:
+                result = self.impl.getitem_str(self.string.encode('utf-8'))
+            else:
+                result = self.impl.getitem(self.string)
+            assert result == 1000
         self.check_not_devolved()
 
     def test_setitem(self):
         self.impl.setitem(self.string, 1000)
         assert self.impl.length() == 1
         assert self.impl.getitem(self.string) == 1000
-        assert self.impl.getitem_str(self.string) == 1000
-        self.check_not_devolved()
-
-    def test_setitem_str(self):
-        self.impl.setitem_str(self.fakespace.str_w(self.string), 1000)
-        assert self.impl.length() == 1
-        assert self.impl.getitem(self.string) == 1000
-        assert self.impl.getitem_str(self.string) == 1000
+        if not self._str_devolves:
+            result = self.impl.getitem_str(self.string.encode('utf-8'))
+        else:
+            result = self.impl.getitem(self.string)
+        assert result == 1000
         self.check_not_devolved()
 
     def test_delitem(self):
@@ -1427,7 +1521,7 @@ class BaseTestRDictImplementation:
     def test_devolve(self):
         impl = self.impl
         for x in xrange(100):
-            impl.setitem(self.fakespace.str_w(str(x)), x)
+            impl.setitem(self.fakespace.text_w(unicode(x)), x)
             impl.setitem(x, x)
         assert type(impl.get_strategy()) is ObjectDictStrategy
 
@@ -1437,14 +1531,14 @@ class BaseTestRDictImplementation:
     def test_setdefault_fast(self):
         on_pypy = "__pypy__" in sys.builtin_module_names
         impl = self.impl
-        key = FakeString(self.string)
+        key = self.FakeString(self.string)
         x = impl.setdefault(key, 1)
         assert x == 1
-        if on_pypy:
+        if on_pypy and self.FakeString is FakeString:
             assert key.hash_count == self.setdefault_hash_count
         x = impl.setdefault(key, 2)
         assert x == 1
-        if on_pypy:
+        if on_pypy and self.FakeString is FakeString:
             assert key.hash_count == self.setdefault_hash_count + 1
 
     def test_fallback_evil_key(self):
@@ -1477,12 +1571,12 @@ class BaseTestRDictImplementation:
         assert w_key not in d.w_keys()
         assert F() not in d.w_keys()
 
-class TestBytesDictImplementation(BaseTestRDictImplementation):
-    StrategyClass = BytesDictStrategy
+class TestUnicodeDictImplementation(BaseTestRDictImplementation):
+    StrategyClass = UnicodeDictStrategy
 
     def test_str_shortcut(self):
         self.fill_impl()
-        s = FakeString(self.string)
+        s = self.FakeString(self.string)
         assert self.impl.getitem(s) == 1000
         assert s.unwrapped
 
@@ -1490,6 +1584,28 @@ class TestBytesDictImplementation(BaseTestRDictImplementation):
         self.fill_impl()
         assert self.fakespace.view_as_kwargs(self.impl) == (["fish", "fish2"], [1000, 2000])
 
+    def test_setitem_str(self):
+        self.impl.setitem_str(self.fakespace.text_w(self.string), 1000)
+        assert self.impl.length() == 1
+        assert self.impl.getitem(self.string) == 1000
+        assert self.impl.getitem_str(str(self.string)) == 1000
+        self.check_not_devolved()
+
+    def test_setitem_str(self):
+        self.impl.setitem_str(self.fakespace.text_w(self.string), 1000)
+        assert self.impl.length() == 1
+        assert self.impl.getitem(self.string) == 1000
+        assert self.impl.getitem_str(str(self.string)) == 1000
+        self.check_not_devolved()
+
+class TestBytesDictImplementation(BaseTestRDictImplementation):
+    StrategyClass = BytesDictStrategy
+    FakeString = FakeString
+    _str_devolves = True
+
+    def wrapstrorunicode(self, obj):
+        return self.fakespace.newbytes(obj)
+
 
 class BaseTestDevolvedDictImplementation(BaseTestRDictImplementation):
     def fill_impl(self):
@@ -1499,8 +1615,8 @@ class BaseTestDevolvedDictImplementation(BaseTestRDictImplementation):
     def check_not_devolved(self):
         pass
 
-class TestDevolvedBytesDictImplementation(BaseTestDevolvedDictImplementation):
-    StrategyClass = BytesDictStrategy
+class TestDevolvedUnicodeDictImplementation(BaseTestDevolvedDictImplementation):
+    StrategyClass = UnicodeDictStrategy
 
 
 def test_module_uses_strdict():
