@@ -1,3 +1,4 @@
+import pytest
 from pypy.objspace.std.iterobject import W_SeqIterObject
 from pypy.interpreter.error import OperationError
 
@@ -11,15 +12,15 @@ class TestW_IterObject:
         self.body0(w_iter)
 
     def body0(self, w_iter):
-        raises(OperationError, self.space.next, w_iter)
-        raises(OperationError, self.space.next, w_iter)
+        pytest.raises(OperationError, self.space.next, w_iter)
+        pytest.raises(OperationError, self.space.next, w_iter)
 
     def test_iter(self):
         w = self.space.wrap
         w_tuple = self.space.newtuple([w(5), w(3), w(99)])
         w_iter = W_SeqIterObject(w_tuple)
         self.body3(w_iter)
-        
+
     def test_iter_builtin(self):
         w = self.space.wrap
         w_tuple = self.space.newtuple([w(5), w(3), w(99)])
@@ -30,9 +31,8 @@ class TestW_IterObject:
         w_list = self.space.newlist([])
         w_iter = W_SeqIterObject(w_list)
         self.body0(w_iter)
-        
+
     def test_emptyiter_builtin(self):
         w_list = self.space.newlist([])
         w_iter = self.space.iter(w_list)
         self.body0(w_iter)
-
