@@ -1,9 +1,9 @@
+# -*- coding: utf-8 -*-
 import pytest
+
 from pypy.objspace.std.test.test_dictmultiobject import FakeSpace, W_DictObject
 from pypy.objspace.std.mapdict import *
 
-skip_if_no_int_unboxing = pytest.mark.skipif(not ALLOW_UNBOXING_INTS, reason="int unboxing disabled on 32bit")
-
 class Config:
     class objspace:
         class std:
@@ -14,14 +14,12 @@ space = FakeSpace()
 space.config = Config
 
 class Class(object):
-    def __init__(self, hasdict=True, allow_unboxing=False):
+    def __init__(self, hasdict=True):
         self.hasdict = hasdict
         if hasdict:
             self.terminator = DictTerminator(space, self)
-            self.terminator.devolved_dict_terminator.allow_unboxing = allow_unboxing
         else:
             self.terminator = NoDictTerminator(space, self)
-        self.terminator.allow_unboxing = allow_unboxing
 
     def instantiate(self, sp=None):
         if sp is None:
@@ -37,77 +35,51 @@ class ObjectWithoutDict(ObjectWithoutDict):
     class typedef:
         hasdict = False
 
-    @property
-    def checkstorage(self):
-        return [unerase_item(x) for x in self.storage]
-
-    @checkstorage.setter
-    def checkstorage(self, value):
-        self.storage = [erase_item(x) for x in value]
-
-
 class Object(Object):
     class typedef:
         hasdict = False
 
-    @property
-    def checkstorage(self):
-        return [unerase_item(x) for x in self.storage]
-
-    @checkstorage.setter
-    def checkstorage(self, value):
-        self.storage = [erase_item(x) for x in value]
-
-    def _check_unboxed_storage_consistency(self):
-        curr = self._get_mapdict_map()
-        while not isinstance(curr, UnboxedPlainAttribute):
-            if isinstance(curr, Terminator):
-                return
-            curr = curr.back
-        assert len(unerase_unboxed(self._mapdict_read_storage(curr.storageindex))) == curr.listindex + 1
-
-
 def test_plain_attribute():
     w_cls = "class"
     aa = PlainAttribute("b", DICT,
                         PlainAttribute("a", DICT,
-                                       Terminator(space, w_cls), 0), 0)
+                                       Terminator(space, w_cls)))
     assert aa.space is space
     assert aa.terminator.w_cls is w_cls
     assert aa.get_terminator() is aa.terminator
 
     obj = Object()
-    obj.map, obj.checkstorage = aa, [10, 20]
+    obj.map, obj.storage = aa, [10, 20]
     assert obj.getdictvalue(space, "a") == 10
     assert obj.getdictvalue(space, "b") == 20
     assert obj.getdictvalue(space, "c") is None
 
     obj = Object()
-    obj.map, obj.checkstorage = aa, [30, 40]
+    obj.map, obj.storage = aa, [30, 40]
     obj.setdictvalue(space, "a", 50)
-    assert obj.checkstorage == [50, 40]
+    assert obj.storage == [50, 40]
     assert obj.getdictvalue(space, "a") == 50
     obj.setdictvalue(space, "b", 60)
-    assert obj.checkstorage == [50, 60]
+    assert obj.storage == [50, 60]
     assert obj.getdictvalue(space, "b") == 60
 
-    assert aa.storage_needed() == 2
+    assert aa.length() == 2
 
     assert aa.get_terminator() is aa.back.back
 
 def test_huge_chain():
     current = Terminator(space, "cls")
     for i in range(20000):
-        current = PlainAttribute(str(i), DICT, current, 0)
+        current = PlainAttribute(str(i), DICT, current)
     assert current.find_map_attr("0", DICT).storageindex == 0
 
 
 def test_search():
-    aa = PlainAttribute("b", DICT, PlainAttribute("a", DICT, Terminator(None, None), 0), 0)
+    aa = PlainAttribute("b", DICT, PlainAttribute("a", DICT, Terminator(None, None)))
     assert aa.search(DICT) is aa
     assert aa.search(SLOTS_STARTING_FROM) is None
     assert aa.search(SPECIAL) is None
-    bb = PlainAttribute("C", SPECIAL, PlainAttribute("A", SLOTS_STARTING_FROM, aa, 0), 0)
+    bb = PlainAttribute("C", SPECIAL, PlainAttribute("A", SLOTS_STARTING_FROM, aa))
     assert bb.search(DICT) is aa
     assert bb.search(SLOTS_STARTING_FROM) is bb.back
     assert bb.search(SPECIAL) is bb
@@ -116,7 +88,7 @@ def test_add_attribute():
     cls = Class()
     obj = cls.instantiate()
     obj.setdictvalue(space, "a", 10)
-    assert obj.checkstorage == [10]
+    assert obj.storage == [10]
     assert obj.getdictvalue(space, "a") == 10
     assert obj.getdictvalue(space, "b") is None
     assert obj.getdictvalue(space, "c") is None
@@ -126,7 +98,7 @@ def test_add_attribute():
     assert obj.getdictvalue(space, "c") is None
 
     obj.setdictvalue(space, "b", 30)
-    assert obj.checkstorage == [20, 30]
+    assert obj.storage == [20, 30]
     assert obj.getdictvalue(space, "a") == 20
     assert obj.getdictvalue(space, "b") == 30
     assert obj.getdictvalue(space, "c") is None
@@ -152,7 +124,8 @@ def test_add_attribute_limit():
         for i in range(1000):
             obj.setdictvalue(space, str(i), i)
         # moved to dict (which is the remaining non-slot item)
-        assert len(obj.checkstorage) == 1 + numslots
+        assert len(obj.storage) == 1 + numslots
+        assert isinstance(obj.getdict(space).dstrategy, UnicodeDictStrategy)
 
         for i in range(1000):
             assert obj.getdictvalue(space, str(i)) == i
@@ -164,7 +137,7 @@ def test_add_attribute_limit():
     obj = cls.instantiate()
     for i in range(1000):
         obj.setslotvalue(i, i)
-    assert len(obj.checkstorage) == 1000
+    assert len(obj.storage) == 1000
 
     for i in range(1000):
         assert obj.getslotvalue(i) == i
@@ -322,21 +295,18 @@ def test_attr_immutability(monkeypatch):
     obj.setdictvalue(space, "a", 10)
     obj.setdictvalue(space, "b", 20)
     obj.setdictvalue(space, "b", 30)
-    assert obj.checkstorage == [10, 30]
+    assert obj.storage == [10, 30]
     assert obj.map.ever_mutated == True
     assert obj.map.back.ever_mutated == False
-    assert obj.map.repr().startswith(
-        "<PlainAttribute b DICT 1 "
-        "<PlainAttribute a DICT 0 immutable "
-        "<DictTerminator w_cls=")
 
     indices = []
 
-    def _pure_direct_read(obj):
-        indices.append(0)
-        return unerase_item(obj._mapdict_read_storage(0))
+    def _pure_mapdict_read_storage(obj, storageindex):
+        assert storageindex == 0
+        indices.append(storageindex)
+        return obj._mapdict_read_storage(storageindex)
 
-    obj.map.back._pure_direct_read = _pure_direct_read
+    obj.map._pure_mapdict_read_storage = _pure_mapdict_read_storage
     monkeypatch.setattr(jit, "isconstant", lambda c: True)
 
     assert obj.getdictvalue(space, "a") == 10
@@ -373,12 +343,12 @@ def test_delete():
         obj.setdictvalue(space, "a", 50)
         obj.setdictvalue(space, "b", 60)
         obj.setdictvalue(space, "c", 70)
-        assert obj.checkstorage == [50, 60, 70]
+        assert obj.storage == [50, 60, 70]
         res = obj.deldictvalue(space, dattr)
         assert res
         s = [50, 60, 70]
         del s[i]
-        assert obj.checkstorage == s
+        assert obj.storage == s
 
     obj = c.instantiate()
     obj.setdictvalue(space, "a", 50)
@@ -401,7 +371,7 @@ def test_class():
     c2 = Class()
     obj.setclass(space, c2)
     assert obj.getclass(space) is c2
-    assert obj.checkstorage == [50, 60, 70]
+    assert obj.storage == [50, 60, 70]
 
 def test_special():
     from pypy.module._weakref.interp__weakref import WeakrefLifeline
@@ -417,7 +387,7 @@ def test_special():
     assert obj.getdictvalue(space, "a") == 50
     assert obj.getdictvalue(space, "b") == 60
     assert obj.getdictvalue(space, "c") == 70
-    assert obj.checkstorage == [50, 60, 70, lifeline1]
+    assert obj.storage == [50, 60, 70, lifeline1]
     assert obj.getweakref() is lifeline1
 
     obj2 = c.instantiate()
@@ -425,7 +395,7 @@ def test_special():
     obj2.setdictvalue(space, "b", 160)
     obj2.setdictvalue(space, "c", 170)
     obj2.setweakref(space, lifeline2)
-    assert obj2.checkstorage == [150, 160, 170, lifeline2]
+    assert obj2.storage == [150, 160, 170, lifeline2]
     assert obj2.getweakref() is lifeline2
 
     assert obj2.map is obj.map
@@ -442,6 +412,7 @@ def test_special():
     obj.delweakref()
 
 
+
 def test_slots():
     cls = Class()
     obj = cls.instantiate()
@@ -454,7 +425,7 @@ def test_slots():
     assert obj.getslotvalue(a) == 50
     assert obj.getslotvalue(b) == 60
     assert obj.getslotvalue(c) == 70
-    assert obj.checkstorage == [50, 60, 70]
+    assert obj.storage == [50, 60, 70]
 
     obj.setdictvalue(space, "a", 5)
     obj.setdictvalue(space, "b", 6)
@@ -465,7 +436,7 @@ def test_slots():
     assert obj.getslotvalue(a) == 50
     assert obj.getslotvalue(b) == 60
     assert obj.getslotvalue(c) == 70
-    assert obj.checkstorage == [50, 60, 70, 5, 6, 7]
+    assert obj.storage == [50, 60, 70, 5, 6, 7]
 
     obj2 = cls.instantiate()
     obj2.setslotvalue(a, 501)
@@ -474,13 +445,13 @@ def test_slots():
     obj2.setdictvalue(space, "a", 51)
     obj2.setdictvalue(space, "b", 61)
     obj2.setdictvalue(space, "c", 71)
-    assert obj2.checkstorage == [501, 601, 701, 51, 61, 71]
+    assert obj2.storage == [501, 601, 701, 51, 61, 71]
     assert obj.map is obj2.map
 
     assert obj2.getslotvalue(b) == 601
     assert obj2.delslotvalue(b)
     assert obj2.getslotvalue(b) is None
-    assert obj2.checkstorage == [501, 701, 51, 61, 71]
+    assert obj2.storage == [501, 701, 51, 61, 71]
     assert not obj2.delslotvalue(b)
 
 
@@ -493,7 +464,7 @@ def test_slots_no_dict():
     obj.setslotvalue(b, 60)
     assert obj.getslotvalue(a) == 50
     assert obj.getslotvalue(b) == 60
-    assert obj.checkstorage == [50, 60]
+    assert obj.storage == [50, 60]
     assert not obj.setdictvalue(space, "a", 70)
     assert obj.getdict(space) is None
     assert obj.getdictvalue(space, "a") is None
@@ -521,7 +492,7 @@ def test_materialize_r_dict():
     obj.setdictvalue(space, "a", 5)
     obj.setdictvalue(space, "b", 6)
     obj.setdictvalue(space, "c", 7)
-    assert obj.checkstorage == [50, 60, 70, 5, 6, 7]
+    assert obj.storage == [50, 60, 70, 5, 6, 7]
 
     class FakeDict(W_DictObject):
         def __init__(self, d):
@@ -538,291 +509,29 @@ def test_materialize_r_dict():
     assert flag
     materialize_r_dict(space, obj, d)
     assert d == {"a": 5, "b": 6, "c": 7}
-    assert obj.checkstorage == [50, 60, 70, w_d]
+    assert obj.storage == [50, 60, 70, w_d]
 
 
-# ___________________________________________________________
-# unboxed tests
-
-def test_unboxed_compute_indices():
-    w_cls = "class"
-    aa = UnboxedPlainAttribute("b", DICT,
-                        PlainAttribute("a", DICT,
-                                       Terminator(space, w_cls), 0), 0,
-                        int)
-    assert aa.storageindex == 1
-    assert aa.firstunwrapped
-    assert aa.listindex == 0
-    
-    c = UnboxedPlainAttribute("c", DICT, aa, 0, int)
-    assert c.storageindex == 1
-    assert c.listindex == 1
-    assert not c.firstunwrapped
-
-def test_unboxed_storage_needed():
-    w_cls = "class"
-    bb = UnboxedPlainAttribute("c", DICT,
-             Terminator(space, w_cls), 0,
-         int)
-    assert bb.storage_needed() == 1
-    aa = UnboxedPlainAttribute("b", DICT,
-            PlainAttribute("a", DICT,
-               UnboxedPlainAttribute("c", DICT,
-                   Terminator(space, w_cls), 0,
-               int), 0), 0,
-         int)
-    assert aa.storage_needed() == 2
-
-def unboxed_write_int(val1, val2):
-    cls = Class(allow_unboxing=True)
-    w_obj = cls.instantiate(space)
-    w_obj.setdictvalue(space, "a", val1)
-    w_obj.getdictvalue(space, "a") == val1
-    assert isinstance(w_obj.map, UnboxedPlainAttribute)
-
-    w_obj.setdictvalue(space, "b", val2)
-    w_obj.getdictvalue(space, "b") == val2
-    w_obj.getdictvalue(space, "a") == val1
-    assert isinstance(w_obj.map, UnboxedPlainAttribute)
-    assert isinstance(w_obj.map.back, UnboxedPlainAttribute)
-    assert unerase_unboxed(w_obj.storage[0]) == [val1, val2]
-
-def unboxed_write_float(val1, val2):
-    cls = Class(allow_unboxing=True)
-    w_obj = cls.instantiate(space)
-    w_obj.setdictvalue(space, "a", val1)
-    w_obj.getdictvalue(space, "a") == val1
-    assert isinstance(w_obj.map, UnboxedPlainAttribute)
-
-    w_obj.setdictvalue(space, "b", val2)
-    w_obj.getdictvalue(space, "b") == val2
-    w_obj.getdictvalue(space, "a") == val1
-    assert isinstance(w_obj.map, UnboxedPlainAttribute)
-    assert isinstance(w_obj.map.back, UnboxedPlainAttribute)
-    assert unerase_unboxed(w_obj.storage[0]) == [
-            float2longlong(val1), float2longlong(val2)]
-
-try:
-    from hypothesis import given, strategies
-except ImportError:
-    @skip_if_no_int_unboxing
-    def test_unboxed_write_int():
-        unboxed_write_int(15, 20)
-    def test_unboxed_write_float():
-        unboxed_write_float(12.434, -1e17)
-else:
-    @skip_if_no_int_unboxing
-    @given(strategies.integers(-sys.maxint-1, sys.maxint),
-           strategies.integers(-sys.maxint-1, sys.maxint))
-    def test_unboxed_write_int(val1, val2):
-        unboxed_write_int(val1, val2)
-
-    @given(strategies.floats(), strategies.floats())
-    def test_unboxed_write_float(val1, val2):
-        unboxed_write_float(val1, val2)
-
-
-@skip_if_no_int_unboxing
-def test_unboxed_write_mixed():
-    cls = Class(allow_unboxing=True)
-    w_obj = cls.instantiate(space)
-    w_obj.setdictvalue(space, "a", None)
-    w_obj.setdictvalue(space, "b", 15)
-    w_obj.setdictvalue(space, "c", 20.1)
-    w_obj.setdictvalue(space, "d", None)
-    w_obj.getdictvalue(space, "a") is None
-    w_obj.getdictvalue(space, "b") == 15
-    w_obj.getdictvalue(space, "c") == 20.1
-    w_obj.setdictvalue(space, "d", None)
-
-@skip_if_no_int_unboxing
-def test_no_int_unboxing(monkeypatch):
-    from pypy.objspace.std import mapdict
-    monkeypatch.setattr(mapdict, "ALLOW_UNBOXING_INTS", False)
-    cls = Class(allow_unboxing=True)
-    w_obj = cls.instantiate(space)
-    w_obj.setdictvalue(space, "a", 15)
-    assert type(w_obj.map) is PlainAttribute
-    w_obj.setdictvalue(space, "b", 15.0)
-    assert type(w_obj.map) is UnboxedPlainAttribute
-
-def test_unboxed_type_change():
-    cls = Class(allow_unboxing=True)
-    w_obj = cls.instantiate(space)
-    w_obj.setdictvalue(space, "b", 15.12)
-    w_obj.setdictvalue(space, "b", "woopsie")
-    assert w_obj.getdictvalue(space, "b") == "woopsie"
-    assert type(w_obj.map) is PlainAttribute
-    assert w_obj.map.terminator.allow_unboxing == False
-
-    w_obj = cls.instantiate(space)
-    w_obj.setdictvalue(space, "b", 15.12)
-    # next time we won't unbox
-    assert type(w_obj.map) is PlainAttribute
-
-def test_unboxed_type_change_other_object():
-    cls = Class(allow_unboxing=True)
-    w_obj1 = cls.instantiate(space)
-    w_obj1.setdictvalue(space, "b", 15.12)
-    w_obj2 = cls.instantiate(space)
-    w_obj2.setdictvalue(space, "b", 16.12)
-    assert w_obj1.map is w_obj2.map
-    assert type(w_obj1.map) is UnboxedPlainAttribute
-
-    # type change
-    w_obj1.setdictvalue(space, "b", "woopsie")
-    assert w_obj1.getdictvalue(space, "b") == "woopsie"
-    assert type(w_obj1.map) is PlainAttribute
-    assert w_obj1.map.terminator.allow_unboxing == False
-
-    # w_obj2 is unaffected so far
-    assert type(w_obj2.map) is UnboxedPlainAttribute
-    assert w_obj2.getdictvalue(space, "b") == 16.12
-    # now it's switched
-    assert type(w_obj2.map) is PlainAttribute
-    # but the value stays of course
-    assert w_obj2.getdictvalue(space, "b") == 16.12
-
-def test_unboxed_mixed_two_different_instances():
-    cls = Class(allow_unboxing=True)
-    w_obj1 = cls.instantiate(space)
-    w_obj1.setdictvalue(space, "b", 15.12)
-
-    w_obj2 = cls.instantiate(space)
-    w_obj2.setdictvalue(space, "b", "abc")
-
-    assert w_obj2.map.terminator.allow_unboxing == False
-
-def test_unboxed_attr_immutability(monkeypatch):
-    cls = Class(allow_unboxing=True)
-    obj = cls.instantiate()
-    obj.setdictvalue(space, "a", 10.12)
-    obj.setdictvalue(space, "b", 20.12)
-    obj.setdictvalue(space, "b", 30.12)
-    assert obj.map.ever_mutated == True
-    assert obj.map.back.ever_mutated == False
-
-    indices = []
-
-    def _pure_unboxed_read(obj):
-        indices.append(0)
-        return float2longlong(10.12)
-
-    obj.map.back._pure_unboxed_read = _pure_unboxed_read
-    monkeypatch.setattr(jit, "isconstant", lambda c: True)
-
-    assert obj.getdictvalue(space, "a") == 10.12
-    assert obj.getdictvalue(space, "b") == 30.12
-    assert obj.getdictvalue(space, "a") == 10.12
-    assert indices == [0, 0]
-
-    obj2 = cls.instantiate()
-    obj2.setdictvalue(space, "a", 15.12)
-    obj2.setdictvalue(space, "b", 25.12)
-    assert obj2.map is obj.map
-    assert obj2.map.ever_mutated == True
-    assert obj2.map.back.ever_mutated == False
-
-    # mutating obj2 changes the map
-    obj2.setdictvalue(space, "a", 50.12)
-    assert obj2.map.back.ever_mutated == True
-    assert obj2.map is obj.map
-
-
-def test_unboxed_bug():
-    cls = Class(allow_unboxing=True)
-    w_obj = cls.instantiate(space)
-    w_obj.setdictvalue(space, "flags", 0.0)
-    w_obj.setdictvalue(space, "open", [])
-    w_obj.setdictvalue(space, "groups", 1.0)
-    w_obj.setdictvalue(space, "groupdict", {})
-    w_obj.setdictvalue(space, "lookbehind", 0.0)
-
-    assert w_obj.getdictvalue(space, "flags") == 0.0
-    assert w_obj.getdictvalue(space, "open") == []
-    assert w_obj.getdictvalue(space, "groups") == 1.0
-    assert w_obj.getdictvalue(space, "groupdict") == {}
-    assert w_obj.getdictvalue(space, "lookbehind") == 0.0
-
-
-def test_unboxed_reorder_add_bug():
-    cls = Class(allow_unboxing=True)
-    obj = cls.instantiate()
-    obj.setdictvalue(space, "a", 10.0)
-    obj.setdictvalue(space, "b", 20.0)
-    obj.setdictvalue(space, "c", 20.0)
-
-    obj2 = cls.instantiate()
-    obj2.setdictvalue(space, "b", 30.0)
-    obj2.setdictvalue(space, "c", 40.0)
-    obj2.setdictvalue(space, "a", 23.0)
-
-    assert obj.map is obj2.map
-
-def test_unboxed_reorder_add_bug2():
-    cls = Class(allow_unboxing=True)
-    obj = cls.instantiate()
-    obj.setdictvalue(space, "a", 10.0)
-    obj.setdictvalue(space, "b", "20")
-    obj.setdictvalue(space, "c", "20")
-
-    obj2 = cls.instantiate()
-    obj2.setdictvalue(space, "b", "30")
-    obj2.setdictvalue(space, "c", "40")
-    obj2.setdictvalue(space, "a", 23.0)
-
-    assert obj.map is obj2.map
-
-def test_unbox_reorder_bug3():
-    from pypy.objspace.std.mapdict import _make_storage_mixin_size_n
-    from pypy.objspace.std.objectobject import W_ObjectObject
-    class objectcls(W_ObjectObject):
-        objectmodel.import_from_mixin(BaseUserClassMapdict)
-        objectmodel.import_from_mixin(MapdictDictSupport)
-        objectmodel.import_from_mixin(_make_storage_mixin_size_n(5))
-    cls = Class(allow_unboxing=True)
-    obj = objectcls()
-    obj.user_setup(space, cls)
-    obj.setdictvalue(space, "_frame", "frame") # plain 0
-    obj.setdictvalue(space, "_is_started", 0.0) # unboxed 1 0
-    obj.setdictvalue(space, "func", "func") # plain 2
-    obj.setdictvalue(space, "alive", "alive") # plain 3
-    obj.setdictvalue(space, "blocked", "blocked") # plain 4
-    obj.setdictvalue(space, "_task_id", 1.0) # unboxed 1 1
-    obj.setdictvalue(space, "label", "label") # plain 5
-
-    obj2 = objectcls()
-    obj2.user_setup(space, cls)
-    obj2.setdictvalue(space, "_frame", "frame2") # plain 0
-    obj2.setdictvalue(space, "_is_started", 5.0) # unboxed 1 0
-    obj2.setdictvalue(space, "func", "func2") # plain 2
-    obj2.setdictvalue(space, "alive", "alive2") # plain 3
-    obj2.setdictvalue(space, "blocked", "blocked2") # plain 4
-    obj2.setdictvalue(space, "label", "label2") # plain 5
-    obj2.setdictvalue(space, "_task_id", 6.0) # reorder
-    assert obj2.getdictvalue(space, "blocked") == "blocked2"
-
-
-def test_unboxed_insert_different_orders_perm():
-    from itertools import permutations
-    cls = Class(allow_unboxing=True)
-    seen_maps = {}
-    for preexisting in ['', 'x', 'xy']:
-        for i, attributes in enumerate(permutations("abcdef")):
-            obj = cls.instantiate()
-            for i, attr in enumerate(preexisting):
-                obj.setdictvalue(space, attr, str(i*1000))
-            key = preexisting
-            for j, attr in enumerate(attributes):
-                obj.setdictvalue(space, attr, i*10.0+j)
-                obj._check_unboxed_storage_consistency()
-                key = "".join(sorted(key+attr))
-                if key in seen_maps:
-                    assert obj.map is seen_maps[key]
-                else:
-                    seen_maps[key] = obj.map
-
-    print len(seen_maps)
+def test_size_prediction():
+    for i in range(10):
+        c = Class()
+        assert c.terminator.size_estimate() == 0
+        for j in range(1000):
+            obj = c.instantiate()
+            for a in "abcdefghij"[:i]:
+                obj.setdictvalue(space, a, 50)
+        assert c.terminator.size_estimate() == i
+    for i in range(1, 10):
+        c = Class()
+        assert c.terminator.size_estimate() == 0
+        for j in range(1000):
+            obj = c.instantiate()
+            for a in "abcdefghij"[:i]:
+                obj.setdictvalue(space, a, 50)
+            obj = c.instantiate()
+            for a in "klmnopqars":
+                obj.setdictvalue(space, a, 50)
+        assert c.terminator.size_estimate() in [(i + 10) // 2, (i + 11) // 2]
 
 # ___________________________________________________________
 # dict tests
@@ -938,12 +647,12 @@ def test_specialized_class():
         obj = objectcls()
         obj.user_setup(space, cls)
         obj.setdictvalue(space, "a", w1)
-        assert unerase_item(obj._value0) is w1
+        assert obj._value0 is w1
         assert obj.getdictvalue(space, "a") is w1
         assert obj.getdictvalue(space, "b") is None
         assert obj.getdictvalue(space, "c") is None
         obj.setdictvalue(space, "a", w2)
-        assert unerase_item(obj._value0) is w2
+        assert obj._value0 is w2
         assert obj.getdictvalue(space, "a") == w2
         assert obj.getdictvalue(space, "b") is None
         assert obj.getdictvalue(space, "c") is None
@@ -961,7 +670,7 @@ def test_specialized_class():
 
         res = obj.deldictvalue(space, "a")
         assert res
-        assert unerase_item(obj._value0) is w4
+        assert obj._value0 is w4
         assert obj.getdictvalue(space, "a") is None
         assert obj.getdictvalue(space, "b") is w4
         assert obj.getdictvalue(space, "c") is None
@@ -974,45 +683,10 @@ def test_specialized_class():
         assert obj2.getdictvalue(space, "b") is w6
         assert obj2.map is abmap
 
-
-def test_specialized_class_overflow():
-    from pypy.objspace.std.mapdict import _make_storage_mixin_size_n
-    from pypy.objspace.std.objectobject import W_ObjectObject
-    classes = [_make_storage_mixin_size_n(i) for i in range(2, 10)]
-    w1 = W_Root()
-    w2 = W_Root()
-    w3 = W_Root()
-    w4 = W_Root()
-    w5 = W_Root()
-    w6 = W_Root()
-    objs = [w1, w2, 4, w3, w4, w5, w6, 6, 12.6]
-    class objectcls(W_ObjectObject):
-        objectmodel.import_from_mixin(BaseUserClassMapdict)
-        objectmodel.import_from_mixin(MapdictDictSupport)
-        objectmodel.import_from_mixin(_make_storage_mixin_size_n(5))
-    cls = Class()
-    obj = objectcls()
-    obj.user_setup(space, cls)
-    for i in range(20):
-        obj.setdictvalue(space, str(i), objs[i % len(objs)])
-    for i in range(20):
-        assert obj.getdictvalue(space, str(i)) is objs[i % len(objs)]
-    for i in range(20):
-        obj.setdictvalue(space, str(i), objs[(i + 1) % len(objs)])
-    for i in range(20):
-        assert obj.getdictvalue(space, str(i)) is objs[(i + 1) % len(objs)]
-    assert obj._has_storage_list()
-    for i in range(20):
-        assert obj.deldictvalue(space, str(i))
-        for j in range(i + 1):
-            assert obj.getdictvalue(space, str(j)) is None
-        for j in range(i + 1, 20):
-            assert obj.getdictvalue(space, str(j)) is objs[(j + 1) % len(objs)]
-
- 
 # ___________________________________________________________
 # integration tests
 
+# XXX write more
 
 class AppTestWithMapDict(object):
 
@@ -1225,97 +899,19 @@ class AppTestWithMapDict(object):
             pass
         x = X(); x.a = 10; x.b = 20; x.c = 30
         d = x.__dict__
-        assert list(__pypy__.reversed_dict(d)) == d.keys()[::-1]
+        assert list(__pypy__.reversed_dict(d)) == list(d.keys())[::-1]
 
-    def test_bug_materialize_huge_dict(self):
-        import __pypy__
-        d = __pypy__.newdict("instance")
-        for i in range(100):
-            d[str(i)] = i
-        assert len(d) == 100
-
-        for key in d:
-            assert d[key] == int(key)
-
-    def test_bug_iter_checks_map_is_wrong(self):
-        # obvious in hindsight, but this test shows that checking that the map
-        # stays the same during a.__dict__ iterations is too strict now
-        class A(object):
+    def test_nonascii_argname(self):
+        """
+        class X:
             pass
+        x = X()
+        x.日本 = 3
+        assert x.日本 == 3
+        assert x.__dict__ == {'日本': 3}
+        """
 
-        # an instance with unboxed storage
-        a = A()
-        a.x = "a"
-        a.y = 1
-        a.z = "b"
-
-        a1 = A()
-        a1.x = "a"
-        a1.y = 1
-        a1.z = "b"
-        a1.y = None # mark the terminator as allow_unboxing = False
-
-        d = a.__dict__
-        # reading a.y during iteration changes the map! now that the iterators
-        # store all the attrs anyway, just remove the check
-        res = list(d.iteritems())
-        assert res == [('x', 'a'), ('y', 1), ('z', 'b')]
-
-    def test_copy(self):
-        class A(object):
-            pass
-
-        # an instance with unboxed storage
-        a = A()
-        a.x = "a"
-        a.y = 1
-        a.z = "b"
-        assert a.__dict__.copy() == {"x": "a", "y": 1, "z": "b"}
-
-    def test_dict_equality(self):
-        class A(object):
-            pass
-
-        a1 = A()
-        a1.x = "a"
-        a1.y = 1
-        a1.z = "b"
-        a2 = A()
-        a2.x = "a"
-        a2.y = 1
-        a2.z = "b"
-        assert a1.__dict__ == a2.__dict__
-
-        class WeirdCompare(object):
-            def __eq__(self, other):
-                a1.a = None
-                a2.a = None
-                return True
-
-        a1 = A()
-        a1.x = WeirdCompare()
-        a1.a = 'f'
-        a2 = A()
-        a2.x = WeirdCompare()
-        a2.a = 'f'
-        assert a1.__dict__ == a2.__dict__
-
-        class WeirdCompare(object):
-            def __eq__(self, other):
-                import weakref
-                weakref.ref(a1) # changes the map
-                return True
-
-        a1 = A()
-        a1.x = WeirdCompare()
-        a1.a = 'f'
-        a2 = A()
-        a2.x = WeirdCompare()
-        a2.a = 'f'
-        assert a1.__dict__ == a2.__dict__
-
-
-
+@pytest.mark.skipif('config.option.runappdirect')
 class AppTestWithMapDictAndCounters(object):
     spaceconfig = {"objspace.std.withmethodcachecounter": True}
 
@@ -1323,8 +919,8 @@ class AppTestWithMapDictAndCounters(object):
         from pypy.interpreter import gateway
         #
         def check(space, w_func, name):
-            w_code = space.getattr(w_func, space.wrap('func_code'))
-            nameindex = map(space.str_w, w_code.co_names_w).index(name)
+            w_code = space.getattr(w_func, space.wrap('__code__'))
+            nameindex = map(space.text_w, w_code.co_names_w).index(name)
             entry = w_code._mapdict_caches[nameindex]
             entry.failure_counter = 0
             entry.success_counter = 0
@@ -1344,13 +940,6 @@ class AppTestWithMapDictAndCounters(object):
         check.unwrap_spec = [gateway.ObjSpace, gateway.W_Root, 'text']
         cls.w_check = cls.space.wrap(gateway.interp2app(check))
 
-        def is_immutable(space, w_obj, name):
-            map = w_obj._get_mapdict_map()
-            attr = map.find_map_attr(name, DICT)
-            return space.newbool(not attr.ever_mutated)
-        is_immutable.unwrap_spec = [gateway.ObjSpace, gateway.W_Root, 'text']
-        cls.w_is_immutable = cls.space.wrap(gateway.interp2app(is_immutable))
-
     def test_simple(self):
         class A(object):
             pass
@@ -1474,9 +1063,11 @@ class AppTestWithMapDictAndCounters(object):
         assert res == (0, 1, 0)
 
     def test_custom_metaclass(self):
-        class A(object):
-            class __metaclass__(type):
-                pass
+        """
+        class metaclass(type):
+            pass
+        class A(metaclass=metaclass):
+            pass
         a = A()
         a.x = 42
         def f():
@@ -1488,8 +1079,10 @@ class AppTestWithMapDictAndCounters(object):
         assert res == (0, 1, 0)
         res = self.check(f, 'x')
         assert res == (0, 1, 0)
+        """
 
     def test_old_style_base(self):
+        skip('py3k no longer has old style classes')
         class B:
             pass
         class C(object):
@@ -1515,10 +1108,11 @@ class AppTestWithMapDictAndCounters(object):
         class C(object):
             def m(*args):
                 return args
-        C.sm = staticmethod(C.m.im_func)
-        C.cm = classmethod(C.m.im_func)
+        C.sm = staticmethod(C.m)
+        C.cm = classmethod(C.m)
 
-        exec """if 1:
+        d = {'C': C}
+        exec("""if 1:
 
             def f():
                 c = C()
@@ -1537,7 +1131,10 @@ class AppTestWithMapDictAndCounters(object):
                 res = c.cm(1)
                 assert res == (C, 1)
                 return 42
-        """
+        """, d)
+        f = d['f']
+        g = d['g']
+        h = d['h']
         res = self.check(f, 'm')
         assert res == (1, 0, 0)
         res = self.check(f, 'm')
@@ -1567,7 +1164,8 @@ class AppTestWithMapDictAndCounters(object):
             def m(*args):
                 return args
 
-        exec """if 1:
+        d = {'C': C}
+        exec("""if 1:
 
             def f():
                 c = C()
@@ -1578,7 +1176,8 @@ class AppTestWithMapDictAndCounters(object):
                 assert res == (c, 1)
                 return 42
 
-        """
+        """, d)
+        f = d['f']
         res = self.check(f, 'm')
         assert res == (1, 1, 1)
         res = self.check(f, 'm')
@@ -1643,8 +1242,8 @@ class AppTestWithMapDictAndCounters(object):
         a.z = 6
         a.y = 7
         assert list(a.__dict__) == ['x', 'z', 'y']
-        assert a.__dict__.values() == [5, 6, 7]
-        assert list(a.__dict__.iteritems()) == [('x', 5), ('z', 6), ('y', 7)]
+        assert list(a.__dict__.values()) == [5, 6, 7]
+        assert list(a.__dict__.items()) == [('x', 5), ('z', 6), ('y', 7)]
 
     def test_bug_method_change(self):
         class A(object):
@@ -1714,130 +1313,8 @@ class AppTestWithMapDictAndCounters(object):
         assert res1 == "mymethod"
         assert res2 == "foobar"
 
-    def test_load_attr_bug_class_name_turns_into_descriptor(self):
-        class WillTurnIntoDescr(object):
-            pass
-
-        class Obj(object):
-            f = WillTurnIntoDescr()
-        o = Obj()
-        o.f = 12
-
-        def readf(o):
-            return o.f
-
-        # this used to fill the cache
-        assert readf(o) == 12
-
-        # make WillTurnIntoDescr a descriptor
-        WillTurnIntoDescr.__get__ = lambda *args: 15
-        WillTurnIntoDescr.__set__ = lambda *args: None
-        assert readf(o) == 15 # used to return 12
-
-    def test_store_attr_simple(self):
-        class A(object):
-            pass
-        a = A()
-        a.x = 0
-        def f():
-            a.x = 12
-            return 42
-        res = self.check(f, 'x')
-        assert res == (1, 0, 0)
-        res = self.check(f, 'x')
-        assert res == (0, 1, 0)
-        assert a.x == 12
-
-    def test_store_attr_simple_shared_with_load(self):
-        class A(object):
-            pass
-        a = A()
-        a.x = 0
-        def f():
-            a.x = a.x + 1
-            return 42
-        res = self.check(f, 'x')
-        assert res == (1, 1, 0) # miss for the first read, hit for the write
-        res = self.check(f, 'x')
-        assert res == (0, 2, 0)
-        assert a.x == 2
-
-    def test_store_attr_slots(self):
-        class A(object):
-            __slots__ = ['x']
-        a = A()
-        a.x = 42
-        def f():
-            a.x = 12
-            return 42
-        #
-        res = self.check(f, 'x')
-        assert res == (1, 0, 0)
-        res = self.check(f, 'x')
-        assert res == (0, 1, 0)
-        res = self.check(f, 'x')
-        assert res == (0, 1, 0)
-        res = self.check(f, 'x')
-        assert res == (0, 1, 0)
-        assert a.x == 12
-
-    def test_store_attr_load_attr_interaction_bug(self):
-        class A(object):
-            def __setattr__(self, name, value):
-                raise TypeError
-
-        a = A()
-        object.__setattr__(a, "buggyattr", 12)
-        with raises(TypeError):
-            a.buggyattr *= 2 # load_attr reads the cache, store_attr reuses it, bug
-
-        class A(object):
-            def __getattribute__(self, name):
-                raise TypeError
-
-        a = A()
-        a.buggyattr = 12 # initialize, doesn't fill cache
-        a.buggyattr = 12 # fill cache
-        with raises(TypeError):
-            a.buggyattr # store_attr makes a cache entry, load_attr reuses it (but shouldn't)
-
-    def test_store_attr_immutability_bug(self):
-        class A(object):
-            pass
-        a = A()
-        a.x = 12
-        assert self.is_immutable(a, 'x')
-        a.x = 13
-        assert not self.is_immutable(a, 'x')
-
-    def test_store_cache_init(self):
-        class A(object):
-            pass
-        def f():
-            a = A()
-            a.attrinita = 10
-            a.attrinitb = 12
-            return a.attrinitb + 30
-        res = self.check(f, 'attrinitb')
-        assert res == (1, 1, 0)
-        res = self.check(f, 'attrinitb')
-        assert res == (0, 2, 0)
-
-    def test_store_cache_unboxing_problem(self):
-        class A(object):
-            pass
-        def f():
-            a = A()
-            a.attrinita = 10.12
-            a = A()
-            a.attrinita = 10.12
-            a = A()
-            a.attrinita = 12
-            return a.attrinita + 30
-        res = self.check(f, 'attrinita')
-        assert res == (2, 2, 0)
-
 
+@pytest.mark.skipif('config.option.runappdirect')
 class AppTestGlobalCaching(AppTestWithMapDict):
     spaceconfig = {"objspace.std.withmethodcachecounter": True}
 
@@ -1857,9 +1334,10 @@ class AppTestGlobalCaching(AppTestWithMapDict):
             l = [A(), B(), C()] * 10
             __pypy__.reset_method_cache_counter()
             # 'exec' to make sure that a.f() is compiled with CALL_METHOD
-            exec """for i, a in enumerate(l):
+            d = {'l': l}
+            exec("""for i, a in enumerate(l):
                         assert a.f() == 42 + i % 3
-            """ in locals()
+            """, d)
             cache_counter = __pypy__.mapdict_cache_counter("f")
             if cache_counter == (27, 3):
                 break
@@ -1919,20 +1397,3 @@ class TestMapDictImplementationUsingnewdict(BaseTestRDictImplementation):
     def test_setdefault_fast(self):
         # mapdict can't pass this, which is fine
         pass
-
-def test_correct_method_sharing(space):
-    from pypy.objspace.std import bytesobject, objectobject, unicodeobject
-    # see long comment in typedef.py, _getusercls
-
-    # first check that random subclasses share code
-    w_bytessub = space.call_function(space.w_type, space.newtext("bytessub"), space.newtuple([space.w_bytes]), space.newdict())
-    w_unicodesub = space.call_function(space.w_type, space.newtext("unicodesub"), space.newtuple([space.w_unicode]), space.newdict())
-    w_inst1 = space.allocate_instance(bytesobject.W_BytesObject, w_bytessub)
-    w_inst2 = space.allocate_instance(unicodeobject.W_UnicodeObject, w_unicodesub)
-    assert w_inst1.getclass.im_func is w_inst2.getclass.im_func
-
-    # but object doesn't!
-    w_objectsub = space.call_function(space.w_type, space.newtext("objectsub"), space.newtuple([space.w_object]), space.newdict())
-    w_inst2 = space.allocate_instance(objectobject.W_ObjectObject, w_objectsub)
-    assert w_inst1.getclass.im_func is not w_inst2.getclass.im_func
-
