@@ -1,3 +1,4 @@
+# encoding: utf-8
 import py
 from pypy.interpreter.baseobjspace import W_Root
 from pypy.interpreter.gateway import interp2app
@@ -58,33 +59,6 @@ class TestTypeObject:
             space.warn = prev_warn
         assert len(warnings) == 2
 
-    def test_setattr_if_not_from_object(self):
-        space = self.space
-        w_A, w_B = space.unpackiterable(space.appexec([], """():
-            class A(object):
-                pass
-            class B(object):
-                def __setattr__(self, obj, name, value):
-                    pass
-            return A, B
-        """))
-        assert w_A.uses_object_setattr is False
-        w_res = w_A.setattr_if_not_from_object()
-        assert w_res is None
-        assert w_A.uses_object_setattr is True
-
-        assert w_B.uses_object_setattr is False
-        w_res = w_B.setattr_if_not_from_object()
-        assert w_res is not None
-        assert w_B.uses_object_setattr is False
-
-        # check invalidation
-        space.setattr(w_A, space.newtext("__setattr__"), space.newint(1))
-        assert w_A.uses_object_setattr is False
-        w_res = w_A.setattr_if_not_from_object()
-        assert w_res is not None
-        assert w_A.uses_object_setattr is False
-
 
 class AppTestTypeObject:
     def test_abstract_methods(self):
@@ -107,9 +81,8 @@ class AppTestTypeObject:
     def test_call_type(self):
         assert type(42) is int
         C = type('C', (object,), {'x': lambda: 42})
-        unbound_meth = C.x
-        raises(TypeError, unbound_meth)
-        assert unbound_meth.im_func() == 42
+        func = C.x
+        assert func() == 42
         raises(TypeError, type)
         raises(TypeError, type, 'test', (object,))
         raises(TypeError, type, 'test', (object,), {}, 42)
@@ -118,28 +91,29 @@ class AppTestTypeObject:
         raises(TypeError, type, 'test', (object,), 42)
 
     def test_call_type_subclass(self):
+        """
         class A(type):
             pass
 
-        assert A("hello") is str
-
         # Make sure type(x) doesn't call x.__class__.__init__
         class T(type):
             counter = 0
             def __init__(self, *args):
                 T.counter += 1
-        class C:
-            __metaclass__ = T
+        class C(metaclass=T):
+            pass
         assert T.counter == 1
         a = C()
         assert T.counter == 1
         assert type(a) is C
         assert T.counter == 1
+        """
 
     def test_bases(self):
+        """
         assert int.__bases__ == (object,)
-        class X:
-            __metaclass__ = type
+        class X(metaclass=type):
+            pass
         assert X.__bases__ ==  (object,)
         class Y(X): pass
         assert Y.__bases__ ==  (X,)
@@ -149,6 +123,7 @@ class AppTestTypeObject:
         Z.__bases__ = (X,)
         #print Z.__bases__
         assert Z.__bases__ == (X,)
+        """
 
     def test_mutable_bases(self):
         # from CPython's test_descr
@@ -238,30 +213,8 @@ class AppTestTypeObject:
         else:
             assert 0, "shouldn't be able to create inheritance cycles"
 
-        # let's throw a classic class into the mix:
-        class Classic:
-            def meth2(self):
-                return 3
-
-        D.__bases__ = (C, Classic)
-
-        assert d.meth2() == 3
-        assert e.meth2() == 3
-        try:
-            d.a
-        except AttributeError:
-            pass
-        else:
-            assert 0, "attribute should have vanished"
-
-        try:
-            D.__bases__ = (Classic,)
-        except TypeError:
-            pass
-        else:
-            assert 0, "new-style class must have a new-style base"
-
     def test_mutable_bases_with_failing_mro(self):
+        """
         class WorkOnce(type):
             def __new__(self, name, bases, ns):
                 self.flag = 0
@@ -293,11 +246,11 @@ class AppTestTypeObject:
         class E(D):
             pass
 
-        class F(D):
-            __metaclass__ = WorkOnce
+        class F(D, metaclass=WorkOnce):
+            pass
 
-        class G(D):
-            __metaclass__ = WorkAlways
+        class G(D, metaclass=WorkAlways):
+            pass
 
         # Immediate subclasses have their mro's adjusted in alphabetical
         # order, so E's will get adjusted before adjusting F's fails.  We
@@ -313,6 +266,38 @@ class AppTestTypeObject:
             assert E.__mro__ == E_mro_before
         else:
             assert 0, "exception not propagated"
+            """
+
+    def test_mutable_bases_with_failing_mro_2(self): """
+        class E(Exception):
+            pass
+        class M(type):
+            def mro(cls):
+                if cls.__name__ == 'Sub' and A.__bases__ == (Base1,):
+                    A.__bases__ = (Base2,)
+                    raise E
+                return type.mro(cls)
+
+        class Base0:
+            pass
+        class Base1:
+            pass
+        class Base2:
+            pass
+        class A(Base0, metaclass=M):
+            pass
+        class Sub(A):
+            pass
+
+        try:
+            A.__bases__ = (Base1,)
+        except E:
+            assert A.__bases__ == (Base2,)
+            assert A.__mro__ == (A, Base2, object)
+            assert Sub.__mro__ == (Sub, A, Base2, object)
+        else:
+            assert 0
+        """
 
     def test_mutable_bases_catch_mro_conflict(self):
         class A(object):
@@ -439,6 +424,17 @@ class AppTestTypeObject:
         class D(B, C):    # assert does not raise TypeError
             pass
 
+    def test_method_qualname(self):
+        assert dict.copy.__qualname__ == 'dict.copy'
+
+    def test_staticmethod_qualname(self):
+        assert dict.__new__.__qualname__ == 'dict.__new__'
+        class A:
+            @staticmethod
+            def stat():
+                pass
+        assert A.stat.__qualname__.endswith('A.stat')
+
     def test_builtin_add(self):
         x = 5
         assert x.__add__(6) == 11
@@ -471,76 +467,43 @@ class AppTestTypeObject:
             raise AssertionError("this multiple inheritance should fail")
 
     def test_outer_metaclass(self):
+        """
         class OuterMetaClass(type):
             pass
 
-        class HasOuterMetaclass(object):
-            __metaclass__ = OuterMetaClass
-
-        assert type(HasOuterMetaclass) == OuterMetaClass
-        assert type(HasOuterMetaclass) == HasOuterMetaclass.__metaclass__
-
-    def test_inner_metaclass(self):
-        class HasInnerMetaclass(object):
-            class __metaclass__(type):
-                pass
-
-        assert type(HasInnerMetaclass) == HasInnerMetaclass.__metaclass__
-
-    def test_implicit_metaclass(self):
-        class __metaclass__(type):
+        class HasOuterMetaclass(metaclass=OuterMetaClass):
             pass
 
-        g = {'__metaclass__': __metaclass__}
-        exec "class HasImplicitMetaclass: pass\n" in g
-
-        HasImplicitMetaclass = g['HasImplicitMetaclass']
-        assert type(HasImplicitMetaclass) == __metaclass__
+        assert type(HasOuterMetaclass) == OuterMetaClass
+        """
 
     def test_mro(self):
+        """
         class A_mro(object):
             a = 1
 
-        class B_mro(A_mro):
+        class mymeta(type):
+            def mro(self, ignore=False):
+                assert ignore or self.__mro__ is None
+                return [self, object]
+
+        class B_mro(A_mro, metaclass=mymeta):
             b = 1
-            class __metaclass__(type):
-                def mro(self):
-                    return [self, object]
 
         assert B_mro.__bases__ == (A_mro,)
         assert B_mro.__mro__ == (B_mro, object)
-        assert B_mro.mro() == [B_mro, object]
+        assert B_mro.mro(ignore=True) == [B_mro, object]
         assert B_mro.b == 1
         assert B_mro().b == 1
         assert getattr(B_mro, 'a', None) == None
         assert getattr(B_mro(), 'a', None) == None
         # also check what the built-in mro() method would return for 'B_mro'
         assert type.mro(B_mro) == [B_mro, A_mro, object]
-
-    def test_mro_mutation_interaction_bug(self):
-        class Base(object):
-            value = 1
-
-        class WeirdClass(object):
-            class __metaclass__(type):
-                def mro(cls):
-                    return (cls, Base, object)
-
-        assert Base.value == 1
-        assert WeirdClass.value == 1
-
-        Base.value = 2
-        assert Base.value == 2
-        assert WeirdClass.value == 2
-
-        Base.value = 3
-        assert Base.value == 3
-        assert WeirdClass.value == 3
-
-        assert len(Base.__subclasses__()) == 0
+        """
 
     def test_abstract_mro(self):
-        class A1:    # old-style class
+        """
+        class A1:    # in py3k is a new-style class
             pass
         class B1(A1):
             pass
@@ -548,10 +511,11 @@ class AppTestTypeObject:
             pass
         class D1(B1, C1):
             pass
-        class E1(D1, object):
-            __metaclass__ = type
-        # old-style MRO in the classical part of the parent hierarchy
-        assert E1.__mro__ == (E1, D1, B1, A1, C1, object)
+        class E1(D1, object, metaclass=type):
+            pass
+        # new-style MRO, contrarily to python2
+        assert E1.__mro__ == (E1, D1, B1, C1, A1, object)
+        """
 
     def test_nodoc(self):
         class NoDoc(object):
@@ -574,38 +538,42 @@ class AppTestTypeObject:
 
         assert ImplicitDoc.__doc__ == 'foo'
 
-    def test_immutabledoc(self):
-        class ImmutableDoc(object):
-            "foo"
+    def test_set_doc(self):
+        class X:
+            "elephant"
+        X.__doc__ = "banana"
+        assert X.__doc__ == "banana"
+        raises(TypeError, lambda:
+               type(list).__dict__["__doc__"].__set__(list, "blah"))
+        raises((AttributeError, TypeError), lambda:
+               type(X).__dict__["__doc__"].__delete__(X))
+        assert X.__doc__ == "banana"
 
-        try:
-            ImmutableDoc.__doc__ = "bar"
-        except TypeError:
-            pass
-        except AttributeError:
-            # XXX - Python raises TypeError for several descriptors,
-            #       we always raise AttributeError.
-            pass
-        else:
-            raise AssertionError('__doc__ should not be writable')
+    def test_text_signature(self):
+        assert object.__text_signature__ == '()'
 
-        assert ImmutableDoc.__doc__ == 'foo'
+        class A:
+            pass
+        assert A.__text_signature__ is None
 
     def test_metaclass_conflict(self):
+        """
         class T1(type):
             pass
         class T2(type):
             pass
-        class D1:
-            __metaclass__ = T1
-        class D2:
-            __metaclass__ = T2
+        class D1(metaclass=T1):
+            pass
+        class D2(metaclass=T2):
+            pass
         def conflict():
             class C(D1,D2):
                 pass
         raises(TypeError, conflict)
+        """
 
     def test_metaclass_choice(self):
+        """
         events = []
 
         class T1(type):
@@ -613,8 +581,8 @@ class AppTestTypeObject:
                 events.append(args)
                 return type.__new__(*args)
 
-        class D1:
-            __metaclass__ = T1
+        class D1(metaclass=T1):
+            pass
 
         class C(D1):
             pass
@@ -629,6 +597,7 @@ class AppTestTypeObject:
         assert type(D1) is T1
         assert type(C) is T1
         assert type(G) is T1
+        """
 
     def test_descr_typecheck(self):
         raises(TypeError,type.__dict__['__name__'].__get__,1)
@@ -703,7 +672,7 @@ class AppTestTypeObject:
             __slots__ = "abc"
 
         class B(object):
-            __slots__ = u"abc"
+            __slots__ = "abc"
 
         a = A()
         a.abc = "awesome"
@@ -712,6 +681,11 @@ class AppTestTypeObject:
         b.abc = "awesomer"
         assert b.abc == "awesomer"
 
+    def test_bad_slots(self):
+        raises(TypeError, type, 'A', (), {'__slots__': b'x'})
+        raises(TypeError, type, 'A', (), {'__slots__': 42})
+        raises(TypeError, type, 'A', (), {'__slots__': '2_x'})
+
     def test_base_attr(self):
         # check the '__base__'
         class A(object):
@@ -748,20 +722,38 @@ class AppTestTypeObject:
         globals()['__name__'] = 'a'
         class A(object):
             pass
-        assert repr(A) == "<class 'a.A'>"
+        assert repr(A) == "<class 'a.test_repr.<locals>.A'>"
         A.__module__ = 123
         assert repr(A) == "<class 'A'>"
-        assert repr(type(type)) == "<type 'type'>"
-        assert repr(complex) == "<type 'complex'>"
-        assert repr(property) == "<type 'property'>"
-        assert repr(TypeError) == "<type 'exceptions.TypeError'>"
+        assert repr(type(type)) == "<class 'type'>"
+        assert repr(complex) == "<class 'complex'>"
+        assert repr(property) == "<class 'property'>"
+        assert repr(TypeError) == "<class 'TypeError'>"
 
     def test_repr_issue1292(self):
         d = {'object': object}    # no __name__
-        exec "class A(object): pass\n" in d
-        assert d['A'].__module__ == '__builtin__'    # obscure, follows CPython
+        exec("class A(object): pass\n", d)
+        assert d['A'].__module__ == 'builtins'    # obscure, follows CPython
         assert repr(d['A']) == "<class 'A'>"
 
+    def test_repr_nonascii(self):
+        assert repr(type('日本', (), {})) == "<class '%s.日本'>" % __name__
+
+    def test_name_nonascii(self):
+        assert type('日本', (), {}).__name__ == '日本'
+
+    def test_errors_nonascii(self):
+        # Check some arbitrary error messages
+        Japan = type('日本', (), {})
+        obj = Japan()
+        for f in hex, int, len, next, open, set, 'foo'.startswith:
+            try:
+                f(obj)
+            except TypeError as e:
+                assert '日本' in str(e)
+            else:
+                assert False, 'Expected TypeError'
+
     def test_invalid_mro(self):
         class A(object):
             pass
@@ -770,7 +762,27 @@ class AppTestTypeObject:
             pass
         raises(TypeError, "class D(A, C): pass")
 
+    def test_dir(self):
+        class A(object):
+            a_var = None
+            def a_meth(self):
+                pass
+
+        class C(A):
+            c_var = None
+            def c_meth(self):
+                pass
+
+        C_items = dir(C)
+        assert C_items != C.__dir__(C)  # as in cpython
+
+        assert 'a_var' in C_items
+        assert 'c_var' in C_items
+        assert 'a_meth' in C_items
+        assert 'c_meth' in C_items
+
     def test_data_descriptor_without_get(self):
+        """
         class Descr(object):
             def __init__(self, name):
                 self.name = name
@@ -778,22 +790,24 @@ class AppTestTypeObject:
                 pass
         class Meta(type):
             pass
-        class X(object):
-            __metaclass__ = Meta
+        class X(object, metaclass=Meta):
+            pass
         X.a = 42
         Meta.a = Descr("a")
         assert X.a == 42
+        """
 
     def test_user_defined_mro_cls_access(self):
+        """
         d = []
         class T(type):
             def mro(cls):
                 d.append(cls.__dict__)
                 return type.mro(cls)
-        class C:
-            __metaclass__ = T
+        class C(metaclass=T):
+            pass
         assert d
-        assert sorted(d[0].keys()) == ['__dict__','__doc__','__metaclass__','__module__', '__weakref__']
+        assert sorted(d[0].keys()) == ['__dict__', '__doc__', '__module__', '__weakref__']
         d = []
         class T(type):
             def mro(cls):
@@ -802,18 +816,13 @@ class AppTestTypeObject:
                 except AttributeError:
                     d.append('miss')
                 return type.mro(cls)
-        class C:
+        class C(metaclass=T):
             def x(cls):
                 return 1
             x = classmethod(x)
-            __metaclass__ = T
         assert d == ['miss']
         assert C.x() == 1
-
-    def test_only_classic_bases_fails(self):
-        class C:
-            pass
-        raises(TypeError, type, 'D', (C,), {})
+        """
 
     def test_set___class__(self):
         raises(TypeError, "1 .__class__ = int")
@@ -938,25 +947,49 @@ class AppTestTypeObject:
         Abc.__name__ = 'Def'
         assert Abc.__name__ == 'Def'
         raises(TypeError, "Abc.__name__ = 42")
-        raises(TypeError, "Abc.__name__ = u'A'")
-        try:
-            Abc.__name__ = 'G\x00hi'
-        except ValueError as e:
-            assert str(e) == "type name must not contain null characters"
-        else:
-            assert False
+        raises(TypeError, "Abc.__name__ = b'A'")
+        for v, err in [('G\x00hi', "type name must not contain null characters"),
+                       ('A\udcdcB', "surrogates not allowed")]:
+            try:
+                Abc.__name__ = v
+            except ValueError as e:
+                assert err in str(e)
+            else:
+                assert False
+            assert Abc.__name__ == 'Def'
+
+    def test_qualname(self):
+        A = type('A', (), {'__qualname__': 'B.C'})
+        assert A.__name__ == 'A'
+        assert A.__qualname__ == 'B.C'
+        raises(TypeError, type, 'A', (), {'__qualname__': b'B'})
+        assert A.__qualname__ == 'B.C'
+
+        A.__qualname__ = 'D.E'
+        assert A.__name__ == 'A'
+        assert A.__qualname__ == 'D.E'
+
+        C = type('C', (), {})
+        C.__name__ = 'A'
+        assert C.__name__ == 'A'
+        assert C.__qualname__ == 'C'
+
+        e = raises(TypeError, type, 'D', (), {'__qualname__': 42})
+        assert str(e.value) == "type __qualname__ must be a str, not int"
+
+        for v in (42, b'abc'):
+            try:
+                C.__qualname__ = v
+            except TypeError as e:
+                assert 'can only assign string' in str(e)
+            else:
+                assert False
 
     def test_compare(self):
         class A(object):
             pass
         class B(A):
             pass
-        A.__eq__
-        A.__ne__
-        assert A.__eq__(A)
-        assert not A.__eq__(B)
-        assert A.__ne__(B)
-        assert not A.__ne__(A)
         assert A == A
         assert A != B
         assert not A == B
@@ -1007,12 +1040,12 @@ class AppTestTypeObject:
 
     def test_module(self):
         def f(): pass
-        assert object.__module__ == '__builtin__'
-        assert int.__module__ == '__builtin__'
-        assert type.__module__ == '__builtin__'
-        assert type(f).__module__ == '__builtin__'
+        assert object.__module__ == 'builtins'
+        assert int.__module__ == 'builtins'
+        assert type.__module__ == 'builtins'
+        assert type(f).__module__ == 'builtins'
         d = {'__name__': 'yay'}
-        exec """class A(object):\n  pass\n""" in d
+        exec("""class A(object):\n  pass\n""", d)
         A = d['A']
         assert A.__module__ == 'yay'
 
@@ -1113,41 +1146,146 @@ class AppTestTypeObject:
     def test_we_already_got_one_2(self):
         class A(object):
             __slots__ = ()
-        class B:
+        class B(object):
             pass
         class C(A, B):     # "best base" is A
             __slots__ = ("__dict__",)
         class D(A, B):     # "best base" is A
             __slots__ = ("__weakref__",)
-        class C(B, A):     # "best base" is A
-            __slots__ = ("__dict__",)
-        class D(B, A):     # "best base" is A
-            __slots__ = ("__weakref__",)
+
+    def test_slot_shadows_class_variable(self):
+        try:
+            class X:
+                __slots__ = ["foo"]
+                foo = None
+        except ValueError as e:
+            assert str(e) == "'foo' in __slots__ conflicts with class variable"
+        else:
+            assert False, "ValueError expected"
+
+    def test_metaclass_calc(self):
+        """
+        # issue1294232: correct metaclass calculation
+        new_calls = []  # to check the order of __new__ calls
+        class AMeta(type):
+            @staticmethod
+            def __new__(mcls, name, bases, ns):
+                new_calls.append('AMeta')
+                return super().__new__(mcls, name, bases, ns)
+            @classmethod
+            def __prepare__(mcls, name, bases):
+                return {}
+
+        class BMeta(AMeta):
+            @staticmethod
+            def __new__(mcls, name, bases, ns):
+                new_calls.append('BMeta')
+                return super().__new__(mcls, name, bases, ns)
+            @classmethod
+            def __prepare__(mcls, name, bases):
+                ns = super().__prepare__(name, bases)
+                ns['BMeta_was_here'] = True
+                return ns
+
+        class A(metaclass=AMeta):
+            pass
+        assert ['AMeta'] == new_calls
+        new_calls[:] = []
+
+        class B(metaclass=BMeta):
+            pass
+        # BMeta.__new__ calls AMeta.__new__ with super:
+        assert ['BMeta', 'AMeta'] == new_calls
+        new_calls[:] = []
+
+        class C(A, B):
+            pass
+        # The most derived metaclass is BMeta:
+        assert ['BMeta', 'AMeta'] == new_calls
+        new_calls[:] = []
+        # BMeta.__prepare__ should've been called:
+        assert 'BMeta_was_here' in C.__dict__
+
+        # The order of the bases shouldn't matter:
+        class C2(B, A):
+            pass
+        assert ['BMeta', 'AMeta'] == new_calls
+        new_calls[:] = []
+        assert 'BMeta_was_here' in C2.__dict__
+
+        # Check correct metaclass calculation when a metaclass is declared:
+        class D(C, metaclass=type):
+            pass
+        assert ['BMeta', 'AMeta'] == new_calls
+        new_calls[:] = []
+        assert 'BMeta_was_here' in D.__dict__
+
+        class E(C, metaclass=AMeta):
+            pass
+        assert ['BMeta', 'AMeta'] == new_calls
+        new_calls[:] = []
+        assert 'BMeta_was_here' in E.__dict__
+
+        # Special case: the given metaclass isn't a class,
+        # so there is no metaclass calculation.
+        marker = object()
+        def func(*args, **kwargs):
+            return marker
+        class X(metaclass=func):
+            pass
+        class Y(object, metaclass=func):
+            pass
+        class Z(D, metaclass=func):
+            pass
+        assert marker is X
+        assert marker is Y
+        assert marker is Z
+        """
+
+    def test_prepare(self):
+        """
+        classdict = type.__prepare__()
+        assert type(classdict) is dict
+        assert classdict == {}
+        assert type.__prepare__(3) == {}
+        assert type.__prepare__(3, 4) == {}
+        assert type.__prepare__(3, package='sqlalchemy') == {}
+        class M(type):
+            @classmethod
+            def __prepare__(cls, *args, **kwds):
+                d = super().__prepare__(*args, **kwds)
+                d['hello'] = 42
+                return d
+        class C(metaclass=M):
+            foo = hello
+        assert C.foo == 42
+        """
 
     def test_crash_mro_without_object_1(self):
+        """
         class X(type):
             def mro(self):
                 return [self]
-        class C:
-            __metaclass__ = X
+        class C(metaclass=X):
+            pass
         e = raises(TypeError, C)     # the lookup of '__new__' fails
         assert str(e.value) == "cannot create 'C' instances"
+        """
 
     def test_crash_mro_without_object_2(self):
+        """
         class X(type):
             def mro(self):
                 return [self, int]
-        class C(int):
-            __metaclass__ = X
+        class C(int, metaclass=X):
+            pass
         C()    # the lookup of '__new__' succeeds in 'int',
                # but the lookup of '__init__' fails
+        """
 
     def test_instancecheck(self):
         assert int.__instancecheck__(42) is True
         assert int.__instancecheck__(42.0) is False
-        class Foo:
-            __class__ = int
-        assert int.__instancecheck__(Foo()) is False
         class Bar(object):
             __class__ = int
         assert int.__instancecheck__(Bar()) is True
@@ -1155,9 +1293,6 @@ class AppTestTypeObject:
     def test_subclasscheck(self):
         assert int.__subclasscheck__(bool) is True
         assert int.__subclasscheck__(float) is False
-        class Foo:
-            __class__ = int
-        assert int.__subclasscheck__(Foo) is False
         class Bar(object):
             __class__ = int
         assert int.__subclasscheck__(Bar) is False
@@ -1166,12 +1301,71 @@ class AppTestTypeObject:
         assert int.__subclasscheck__(AbstractClass()) is True
 
     def test_bad_args(self):
-        import UserDict
+        import collections
         raises(TypeError, type, 'A', (), dict={})
         raises(TypeError, type, 'A', [], {})
-        raises(TypeError, type, 'A', (), UserDict.UserDict())
+        raises(TypeError, type, 'A', (), collections.UserDict())
         raises(ValueError, type, 'A\x00B', (), {})
-        raises(TypeError, type, u'A', (), {})
+        raises(TypeError, type, b'A', (), {})
+
+    def test_incomplete_extend(self): """
+        # Extending an unitialized type with type.__mro__ is None must
+        # throw a reasonable TypeError exception, instead of failing
+        # with a segfault.
+        class M(type):
+            def mro(cls):
+                if cls.__mro__ is None and cls.__name__ != 'X':
+                    try:
+                        class X(cls):
+                            pass
+                    except TypeError:
+                        found.append(1)
+                return type.mro(cls)
+        found = []
+        class A(metaclass=M):
+            pass
+        assert found == [1]
+        """
+
+    def test_incomplete_extend_2(self): """
+        # Same as test_incomplete_extend, with multiple inheritance
+        class M(type):
+            def mro(cls):
+                if cls.__mro__ is None and cls.__name__ == 'Second':
+                    try:
+                        class X(First, cls):
+                            pass
+                    except TypeError:
+                        found.append(1)
+                return type.mro(cls)
+        found = []
+        class Base(metaclass=M):
+            pass
+        class First(Base):
+            pass
+        class Second(Base):
+            pass
+        assert found == [1]
+        """
+
+    def test_incomplete_extend_3(self): """
+        # this case "works", but gives a slightly strange error message
+        # on both CPython and PyPy
+        class M(type):
+            def mro(cls):
+                if cls.__mro__ is None and cls.__name__ == 'A':
+                    try:
+                        Base.__new__(cls)
+                    except TypeError:
+                        found.append(1)
+                return type.mro(cls)
+        found = []
+        class Base(metaclass=M):
+            pass
+        class A(Base):
+            pass
+        assert found == [1]
+        """
 
 
 class AppTestWithMethodCacheCounter:
@@ -1179,7 +1373,7 @@ class AppTestWithMethodCacheCounter:
 
     def test_module_from_handbuilt_type(self):
         d = {'tuple': tuple, '__name__': 'foomod'}
-        exec """class foo(tuple): pass""" in d
+        exec("""class foo(tuple): pass""", d)
         t = d['foo']
         t.__module__ = 'barmod'
         # this last line used to crash; see ab926f846f39
@@ -1189,6 +1383,7 @@ class AppTestWithMethodCacheCounter:
 class AppTestGetattributeShortcut:
 
     def test_reset_logic(self):
+        """
         class X(object):
             pass
 
@@ -1209,8 +1404,8 @@ class AppTestGetattributeShortcut:
         class M(type):
             pass
 
-        class X(object):
-            __metaclass__ = M
+        class X(metaclass=M):
+            pass
 
         class Y(X):
             pass
@@ -1225,6 +1420,7 @@ class AppTestGetattributeShortcut:
         X.__getattribute__ = ga2
 
         assert y.x == 'GA2'
+        """
 
 class TestNewShortcut:
     spaceconfig = {"objspace.std.newshortcut": True}
@@ -1244,7 +1440,7 @@ class TestNewShortcut:
 
         assert w_A.w_new_function is None
         assert w_B.w_new_function is None
-        assert w_M.w_new_function is None                
+        assert w_M.w_new_function is None
 
         _, w_object_newdescr = space.lookup_in_type_where(space.w_object,
                                                           '__new__')
@@ -1298,28 +1494,11 @@ class AppTestNewShortcut:
 
         assert b == 1
 
-    def test_slots_with_method_in_class(self):
-        # this works in cpython...
-        class A(object):
-            __slots__ = ["f"]
-            def f(self, x):
-                return x + 1
-        a = A()
-        assert a.f(1) == 2
-
     def test_eq_returns_notimplemented(self):
         assert type.__eq__(int, 42) is NotImplemented
         assert type.__ne__(dict, 42) is NotImplemented
-        assert type.__eq__(int, int) is True
-        assert type.__eq__(int, dict) is False
-
-    def test_cmp_on_types(self):
-        class X(type):
-            def __cmp__(self, other):
-                return -1
-        class Y:
-            __metaclass__ = X
-        assert (Y < Y) is True
+        assert type.__eq__(int, int) == True
+        assert type.__eq__(int, dict) is NotImplemented
 
 
 class AppTestComparesByIdentity:
@@ -1340,10 +1519,6 @@ class AppTestComparesByIdentity:
             def __eq__(self, other):
                 return True
 
-        class CustomCmp (object):
-            def __cmp__(self, other):
-                return 0
-
         class CustomHash(object):
             def __hash__(self):
                 return 0
@@ -1351,17 +1526,11 @@ class AppTestComparesByIdentity:
         class TypeSubclass(type):
             pass
 
-        class TypeSubclassCustomCmp(type):
-            def __cmp__(self, other):
-                return 0
-
         assert self.compares_by_identity(Plain)
         assert not self.compares_by_identity(CustomEq)
-        assert not self.compares_by_identity(CustomCmp)
         assert not self.compares_by_identity(CustomHash)
         assert self.compares_by_identity(type)
         assert self.compares_by_identity(TypeSubclass)
-        assert not self.compares_by_identity(TypeSubclassCustomCmp)
 
     def test_modify_class(self):
         class X(object):
@@ -1373,6 +1542,10 @@ class AppTestComparesByIdentity:
         del X.__eq__
         assert self.compares_by_identity(X)
 
+    def test_duplicate_slot_name(self):
+        class X:   # does not raise
+            __slots__ = 'a', 'a'
+
     def test_descriptor_objclass(self):
         class X(object):
             pass
@@ -1380,9 +1553,83 @@ class AppTestComparesByIdentity:
         assert X.__dict__['__weakref__'].__objclass__ is X
         assert object.__dict__['__class__'].__objclass__ is object
         assert int.__dict__['imag'].__objclass__ is int
-        assert file.closed.__objclass__ is file
         assert type.__dict__['__name__'].__objclass__ is type
         assert type.__dict__['__doc__'].__objclass__ is type
         #
         assert type.__dict__['__name__'].__name__ == '__name__'
         assert type.__dict__['__doc__'].__name__ == '__doc__'
+
+    def test_type_construct_unicode_surrogate_issue(self):
+        raises(ValueError, type, 'A\udcdcb', (), {})
+
+    def test_set_name(self):
+        class Descriptor:
+            def __set_name__(self, owner, name):
+                self.owner = owner
+                self.name = name
+
+        class X:
+            a = Descriptor()
+        assert X.a.owner is X
+        assert X.a.name == "a"
+
+    def test_set_name_error(self):
+        class Descriptor:
+            __set_name__ = None
+        def make_class():
+            class A:
+                d = Descriptor()
+        excinfo = raises(RuntimeError, make_class)
+        assert isinstance(excinfo.value.__cause__, TypeError)
+        assert str(excinfo.value) == "Error calling __set_name__ on 'Descriptor' instance 'd' in 'A'"
+        print(excinfo.value)
+
+    def test_type_init_accepts_kwargs(self):
+        type.__init__(type, "a", (object, ), {}, a=1)
+
+    def test_init_subclass_classmethod(self):
+        assert isinstance(object.__dict__['__init_subclass__'], classmethod)
+        class A(object):
+            subclasses = []
+
+            def __init_subclass__(cls):
+                cls.subclass.append(cls)
+        assert isinstance(A.__dict__['__init_subclass__'], classmethod)
+
+    def test_init_subclass(self):
+        class PluginBase(object):
+            subclasses = []
+
+            def __init_subclass__(cls):
+                cls.subclasses.append(cls)
+
+        class B(PluginBase):
+            pass
+
+        class C(PluginBase):
+            pass
+
+        assert PluginBase.subclasses == [B, C]
+
+
+        class X(object):
+            subclasses = []
+
+            def __init_subclass__(cls, **kwargs):
+                cls.kwargs = kwargs
+
+        exec("""if 1:
+        class Y(X, a=1, b=2):
+            pass
+
+        assert Y.kwargs == dict(a=1, b=2)
+        """)
+
+    def test_onearg_type_only_for_type(self):
+        class Meta(type):
+            pass
+
+        info = raises(TypeError, Meta, 5)
+        assert "takes exactly 3 arguments (1 given)" in str(info.value)
+        info = raises(TypeError, Meta, 5, 7)
+        assert "takes exactly 3 arguments (1 given)" in str(info.value)
