@@ -129,11 +129,7 @@ class PureOperation(HLOperation):
                 msg = "%s%r always raises %s: %s" % (
                     self.opname, tuple(args), type(e), e)
                 raise FlowingError(msg)
-            return self._pure_result(result)
-
-    def _pure_result(self, result):
-                # indented for history preservation
-
+            else:
                 # don't try to constant-fold operations giving a 'long'
                 # result.  The result is probably meant to be sent to
                 # an intmask(), but the 'long' constant confuses the
@@ -152,45 +148,6 @@ class PureOperation(HLOperation):
                         # store operation with variable result instead
                         pass
 
-class PureOperation1(PureOperation):
-    pure = True
-
-    def constfold(self):
-        if len(self.args) != 1:
-            return PureOperation.constfold(self)
-        w_arg0, = self.args
-        if not w_arg0.foldable():
-            return
-        arg0 = w_arg0.value
-        # argument is constant: call the operator now
-        try:
-            result = self.pyfunc(arg0)
-        except Exception as e:
-            from rpython.flowspace.flowcontext import FlowingError
-            msg = "%s%r always raises %s: %s" % (
-                self.opname, (arg0, ), type(e), e)
-            raise FlowingError(msg)
-        return self._pure_result(result)
-
-class PureOperation2(PureOperation):
-    pure = True
-
-    def constfold(self):
-        w_arg0, w_arg1 = self.args
-        if not w_arg0.foldable() or not w_arg1.foldable():
-            return
-        arg0 = w_arg0.value
-        arg1 = w_arg1.value
-        # argument is constant: call the operator now
-        try:
-            result = self.pyfunc(arg0, arg1)
-        except Exception as e:
-            from rpython.flowspace.flowcontext import FlowingError
-            msg = "%s%r always raises %s: %s" % (
-                self.opname, (arg0, arg1), type(e), e)
-            raise FlowingError(msg)
-        return self._pure_result(result)
-
 class OverflowingOperation(PureOperation):
     can_overflow = True
 
@@ -312,12 +269,7 @@ def add_operator(name, arity, dispatch=None, pyfunc=None, pure=False, ovf=False)
         assert pure
         base_cls = OverflowingOperation
     elif pure:
-        if arity == 1:
-            base_cls = PureOperation1
-        elif arity == 2:
-            base_cls = PureOperation2
-        else:
-            base_cls = PureOperation
+        base_cls = PureOperation
     else:
         base_cls = HLOperation
     bases.append(base_cls)
@@ -469,7 +421,6 @@ op.is_true = op.nonzero = op.bool  # for llinterp
 add_operator('abs', 1, dispatch=1, pyfunc=abs, pure=True, ovf=True)
 add_operator('hex', 1, dispatch=1, pyfunc=hex, pure=True)
 add_operator('oct', 1, dispatch=1, pyfunc=oct, pure=True)
-add_operator('bin', 1, dispatch=1, pyfunc=bin, pure=True)
 add_operator('ord', 1, dispatch=1, pyfunc=ord, pure=True)
 add_operator('invert', 1, dispatch=1, pure=True)
 add_operator('add', 2, dispatch=2, pure=True, ovf=True)
