@@ -4,10 +4,9 @@ from rpython.jit.metainterp.history import Const, TargetToken, JitCellToken
 from rpython.jit.metainterp.optimizeopt.shortpreamble import ShortBoxes,\
      ShortPreambleBuilder, ExtendedShortPreambleBuilder, PreambleOp
 from rpython.jit.metainterp.optimizeopt import info, intutils
-from rpython.jit.metainterp.optimizeopt.intutils import MININT, MAXINT
 from rpython.jit.metainterp.optimize import InvalidLoop, SpeculativeError
 from rpython.jit.metainterp.optimizeopt.optimizer import Optimizer,\
-     Optimization, LoopInfo, BasicLoopInfo
+     Optimization, LoopInfo, MININT, MAXINT, BasicLoopInfo
 from rpython.jit.metainterp.optimizeopt.vstring import StrPtrInfo
 from rpython.jit.metainterp.optimizeopt.virtualstate import (
     VirtualStateConstructor, VirtualStatesCantMatch)
@@ -91,9 +90,16 @@ class UnrollOptimizer(Optimizer):
             if preamble_info.is_nonnull():
                 self.make_nonnull(op)
         elif isinstance(preamble_info, intutils.IntBound):
-            loop_info = preamble_info.widen()
-            intbound = self.getintbound(op)
-            intbound.intersect(loop_info)
+            fix_lo = preamble_info.has_lower and preamble_info.lower >= MININT/2
+            fix_up = preamble_info.has_upper and preamble_info.upper <= MAXINT/2
+            if fix_lo or fix_up:
+                intbound = self.getintbound(op)
+                if fix_lo:
+                    intbound.has_lower = True
+                    intbound.lower = preamble_info.lower
+                if fix_up:
+                    intbound.has_upper = True
+                    intbound.upper = preamble_info.upper
         elif isinstance(preamble_info, info.FloatConstInfo):
             op.set_forwarded(preamble_info._const)
 
@@ -302,22 +308,6 @@ class OptUnroll(Optimization):
 
 
     def jump_to_existing_trace(self, jump_op, label_op, runtime_boxes, force_boxes=False):
-        # there is a big conceptual problem here: it's not safe at all to catch
-        # InvalidLoop in the callers of _jump_to_existing_trace and then
-        # continue trying to jump to some other label, because inlining the
-        # short preamble could have worked partly, leaving some unwanted new
-        # ops at the end of the trace. Here's at least a stopgap to stop
-        # terrible things from happening: we *must not* move any of those bogus
-        # guards earlier into the trace. see
-        # test_unroll_shortpreamble_mutates_bug in test_loop, and issue #3598
-
-        # leaving the bogus operations at the end of the trace is not great,
-        # but should be safe: at worst, they just always do a bit of stuff and
-        # then fail
-        with self.optimizer.cant_replace_guards():
-            return self._jump_to_existing_trace(jump_op, label_op, runtime_boxes, force_boxes)
-
-    def _jump_to_existing_trace(self, jump_op, label_op, runtime_boxes, force_boxes=False):
         jitcelltoken = jump_op.getdescr()
         assert isinstance(jitcelltoken, JitCellToken)
         virtual_state = self.get_virtual_state(jump_op.getarglist())
@@ -412,15 +402,10 @@ class OptUnroll(Optimization):
                 mapping[sop] = op
                 i += 1
                 self.optimizer.send_extra_operation(op)
-            # force all of them except the virtuals. this can also add more
-            # arguments from the preamble
-            while 1:
-                num_short_jump_args = len(short_jump_args)
-                for arg in (args_no_virtuals +
-                            self._map_args(mapping, short_jump_args)):
-                    self.optimizer.force_box(get_box_replacement(arg))
-                if len(short_jump_args) == num_short_jump_args:
-                    break
+            # force all of them except the virtuals
+            for arg in (args_no_virtuals +
+                        self._map_args(mapping, short_jump_args)):
+                self.optimizer.force_box(get_box_replacement(arg))
             self.optimizer.flush()
             # done unless "short" has grown again
             if i == len(short) - 1:
