@@ -55,38 +55,14 @@ class CompiledPattern(object):
 
     def __init__(self, pattern, flags):
         self.pattern = pattern
-        if not consts.V37:      # 'flags' is ignored in >=3.7 mode
-            self.flags = flags
+        self.flags = flags
         # check we don't get the old value of MAXREPEAT
-        # during the untranslated tests.
+        # during the untranslated tests. 
         # On python3, MAXCODE can appear in patterns. It will be 65535
         # when CODESIZE is 2
         if not we_are_translated() and rsre_char.CODESIZE != 2:
             assert 65535 not in pattern
 
-    def lowa(self, char_ord):
-        """Pre-3.7: uses getlower(flags).
-           Post-3.7: this is always getlower_ascii().
-        """
-        if not consts.V37:
-            return rsre_char.getlower(char_ord, self.flags)
-        else:
-            return rsre_char.getlower_ascii(char_ord)
-
-    def char_loc_ignore(self, index, char_ord):
-        assert consts.V37
-        pattern = self.pat(index)
-        return (char_ord == pattern or
-                rsre_char.getlower_locale(char_ord) == pattern or
-                rsre_char.getupper_locale(char_ord) == pattern)
-
-    def charset_loc_ignore(self, ctx, ppos, char_ord):
-        lo = rsre_char.getlower_locale(char_ord)
-        if rsre_char.check_charset(ctx, self, ppos, lo):
-            return True
-        up = rsre_char.getupper_locale(char_ord)
-        return up != lo and rsre_char.check_charset(ctx, self, ppos, up)
-
     def pat(self, index):
         jit.promote(self)
         check_nonneg(index)
@@ -98,10 +74,6 @@ class CompiledPattern(object):
         assert result >= 0
         return result
 
-MODE_ANY = '\x00'         # an empty match is fine
-MODE_NONEMPTY = '\x01'    # must have a non-empty match
-MODE_FULL = '\x02'        # must match the whole string
-
 class AbstractMatchContext(object):
     """Abstract base class"""
     _immutable_fields_ = ['end']
@@ -109,7 +81,7 @@ class AbstractMatchContext(object):
     match_end = 0
     match_marks = None
     match_marks_flat = None
-    match_mode = MODE_ANY
+    fullmatch_only = False
 
     def __init__(self, match_start, end):
         # 'match_start' and 'end' must be known to be non-negative
@@ -119,30 +91,25 @@ class AbstractMatchContext(object):
         self.match_start = match_start
         self.end = end
 
-    def reset(self, start, must_advance=False):
+    def reset(self, start):
         self.match_start = start
         self.match_marks = None
         self.match_marks_flat = None
-        #
-        assert MODE_ANY == chr(False)
-        assert MODE_NONEMPTY == chr(True)
-        self.match_mode = chr(must_advance)
-
-    @not_rpython
-    def _fullmatch_only(self, x=None):
-        raise Exception("'ctx.fullmatch_only' was replaced with"
-                        " 'ctx.match_mode'")
-    fullmatch_only = property(_fullmatch_only, _fullmatch_only)
 
     @not_rpython
     def str(self, index):
         """Must be overridden in a concrete subclass.
-        The @not_rpython is used to generate a translation-time crash
+        The tag ^^^ here is used to generate a translation-time crash
         if there is a call to str() that is indirect.  All calls must
         be direct for performance reasons; you need to specialize the
         caller with @specializectx."""
         raise NotImplementedError
 
+    @not_rpython
+    def lowstr(self, index, flags):
+        """Similar to str()."""
+        raise NotImplementedError
+
     # The following methods are provided to be overriden in
     # Utf8MatchContext.  The non-utf8 implementation is provided
     # by the FixedMatchContext abstract subclass, in order to use
@@ -269,6 +236,10 @@ class BufMatchContext(FixedMatchContext):
         check_nonneg(index)
         return ord(self._buffer.getitem(index))
 
+    def lowstr(self, index, flags):
+        c = self.str(index)
+        return rsre_char.getlower(c, flags)
+
     def fresh_copy(self, start):
         return BufMatchContext(self._buffer, start,
                                self.end)
@@ -290,6 +261,10 @@ class StrMatchContext(FixedMatchContext):
         check_nonneg(index)
         return ord(self._string[index])
 
+    def lowstr(self, index, flags):
+        c = self.str(index)
+        return rsre_char.getlower(c, flags)
+
     def fresh_copy(self, start):
         return StrMatchContext(self._string, start,
                                self.end)
@@ -314,6 +289,10 @@ class UnicodeMatchContext(FixedMatchContext):
         check_nonneg(index)
         return ord(self._unicodestr[index])
 
+    def lowstr(self, index, flags):
+        c = self.str(index)
+        return rsre_char.getlower(c, flags)
+
     def fresh_copy(self, start):
         return UnicodeMatchContext(self._unicodestr, start,
                                    self.end)
@@ -331,9 +310,6 @@ class Mark(object):
         self.position = position
         self.prev = prev      # chained list
 
-    def __repr__(self):
-        return "Mark(%r, %r, %r)" % (self.gid, self.position, self.prev)
-
 def find_mark(mark, gid):
     while mark is not None:
         if mark.gid == gid:
@@ -546,10 +522,6 @@ class MaxUntilMatchResult(AbstractUntilMatchResult):
             match_more = False
 
 class MinUntilMatchResult(AbstractUntilMatchResult):
-    install_jitdriver('MinUntil',
-                      greens=['ppos', 'tailppos', 'resume', 'pattern'],
-                      reds=['ptr', 'marks', 'self', 'ctx'],
-                      debugprint=(3, 0, 2))
 
     def find_first_result(self, ctx, pattern):
         return self.search_next(ctx, pattern, resume=False)
@@ -558,19 +530,16 @@ class MinUntilMatchResult(AbstractUntilMatchResult):
         return self.search_next(ctx, pattern, resume=True)
 
     def search_next(self, ctx, pattern, resume):
+        # XXX missing jit support here
         ppos = self.ppos
+        min = pattern.pat(ppos+1)
+        max = pattern.pat(ppos+2)
         ptr = self.cur_ptr
         marks = self.cur_marks
-        tailppos = self.tailppos
         while True:
-            ctx.jitdriver_MinUntil.jit_merge_point(
-                ppos=ppos, tailppos=tailppos, resume=resume,
-                ptr=ptr, marks=marks, self=self, ctx=ctx,
-                pattern=pattern)
             # try to match 'tail' if we have enough 'item'
-            min = pattern.pat(ppos+1)
             if not resume and self.num_pending >= min:
-                result = sre_match(ctx, pattern, tailppos, ptr, marks)
+                result = sre_match(ctx, pattern, self.tailppos, ptr, marks)
                 if result is not None:
                     self.subresult = result
                     self.cur_ptr = ptr
@@ -578,7 +547,6 @@ class MinUntilMatchResult(AbstractUntilMatchResult):
                     return self
             resume = False
 
-            max = pattern.pat(ppos+2)
             if max == rsre_char.MAXREPEAT or self.num_pending < max:
                 # try to match one more 'item'
                 enum = sre_match(ctx, pattern, ppos + 3, ptr, marks)
@@ -608,46 +576,6 @@ class MinUntilMatchResult(AbstractUntilMatchResult):
             ptr = ctx.match_end
             marks = ctx.match_marks
 
-install_jitdriver_spec('MaxUntilPossessive',
-                       greens=['ppos', 'pattern'],
-                       reds=['ptr', 'matches_done', 'marks', 'ctx'],
-                       debugprint=(1, 0))
-
-@specializectx
-def find_repetition_end_possessive(ctx, pattern, ppos, ptr, marks):
-    matches_done = 0
-    enum = None
-    while True:
-        ctx.jitdriver_MaxUntilPossessive.jit_merge_point(
-            ppos=ppos,
-            ptr=ptr, marks=marks, ctx=ctx, matches_done=matches_done,
-            pattern=pattern)
-        maxmatch = pattern.pat(ppos+2)
-        if maxmatch == rsre_char.MAXREPEAT or matches_done < maxmatch:
-            # try to match one more 'item'
-            enum = sre_match(ctx, pattern, ppos + 3, ptr, marks)
-        else:
-            enum = None    # 'max' reached, no more matches
-        minmatch = pattern.pat(ppos+1)
-        if enum is not None:
-            matches_done += 1
-            # matched one more 'item'.
-            last_match_zero_length = (ctx.match_end == ptr)
-            ptr = ctx.match_end
-            marks = ctx.match_marks
-            if last_match_zero_length and matches_done >= minmatch:
-                # zero-width protection: after an empty match, if there
-                # are enough matches, don't try to match more.  Instead,
-                # fall through to trying to match 'tail'.
-                pass
-            else:
-                continue
-        if matches_done >= minmatch:
-            ctx.match_marks = marks
-            return ptr
-        return -1
-
-
 # ____________________________________________________________
 
 @specializectx
@@ -671,13 +599,9 @@ def sre_match(ctx, pattern, ppos, ptr, marks):
             return
 
         elif op == consts.OPCODE_SUCCESS:
-            mode = ctx.match_mode
-            if mode == MODE_FULL:
+            if ctx.fullmatch_only:
                 if ptr != ctx.end:
                     return     # not a full match
-            elif mode == MODE_NONEMPTY:
-                if ptr == ctx.match_start:
-                    return     # empty match
             ctx.match_end = ptr
             ctx.match_marks = marks
             return MATCHED_OK
@@ -709,10 +633,10 @@ def sre_match(ctx, pattern, ppos, ptr, marks):
                 ptr1 = ctx.prev_n(ptr, pattern.pat(ppos+1), ctx.ZERO)
             except EndOfString:
                 return
-            saved = ctx.match_mode
-            ctx.match_mode = MODE_ANY
+            saved = ctx.fullmatch_only
+            ctx.fullmatch_only = False
             stop = sre_match(ctx, pattern, ppos + 2, ptr1, marks) is None
-            ctx.match_mode = saved
+            ctx.fullmatch_only = saved
             if stop:
                 return
             marks = ctx.match_marks
@@ -727,10 +651,10 @@ def sre_match(ctx, pattern, ppos, ptr, marks):
             except EndOfString:
                 pass
             else:
-                saved = ctx.match_mode
-                ctx.match_mode = MODE_ANY
+                saved = ctx.fullmatch_only
+                ctx.fullmatch_only = False
                 stop = sre_match(ctx, pattern, ppos + 2, ptr1, marks) is not None
-                ctx.match_mode = saved
+                ctx.fullmatch_only = saved
                 if stop:
                     return
             ppos += pattern.pat(ppos)
@@ -775,29 +699,7 @@ def sre_match(ctx, pattern, ppos, ptr, marks):
             startptr, length_bytes = get_group_ref(ctx, marks, pattern.pat(ppos))
             if length_bytes < 0:
                 return     # group was not previously defined
-            ptr = match_repeated_ignore(ctx, ptr, startptr, length_bytes, pattern)
-            if ptr < ctx.ZERO:
-                return     # no match
-            ppos += 1
-
-        elif consts.eq(op, consts.OPCODE37_GROUPREF_UNI_IGNORE):
-            # unicode version of OPCODE_GROUPREF_IGNORE
-            # <GROUPREF> <groupnum>
-            startptr, length_bytes = get_group_ref(ctx, marks, pattern.pat(ppos))
-            if length_bytes < 0:
-                return     # group was not previously defined
-            ptr = match_repeated_uni_ignore(ctx, ptr, startptr, length_bytes)
-            if ptr < ctx.ZERO:
-                return     # no match
-            ppos += 1
-
-        elif consts.eq(op, consts.OPCODE37_GROUPREF_LOC_IGNORE):
-            # locale version of OPCODE_GROUPREF_IGNORE
-            # <GROUPREF> <groupnum>
-            startptr, length_bytes = get_group_ref(ctx, marks, pattern.pat(ppos))
-            if length_bytes < 0:
-                return     # group was not previously defined
-            ptr = match_repeated_loc_ignore(ctx, ptr, startptr, length_bytes)
+            ptr = match_repeated_ignore(ctx, ptr, startptr, length_bytes, pattern.flags)
             if ptr < ctx.ZERO:
                 return     # no match
             ppos += 1
@@ -824,25 +726,7 @@ def sre_match(ctx, pattern, ppos, ptr, marks):
             # match set member (or non_member), ignoring case
             # <IN> <skip> <set>
             if ptr >= ctx.end or not rsre_char.check_charset(ctx, pattern, ppos+1,
-                                                             pattern.lowa(ctx.str(ptr))):
-                return
-            ppos += pattern.pat(ppos)
-            ptr = ctx.next(ptr)
-
-        elif consts.eq(op, consts.OPCODE37_IN_UNI_IGNORE):
-            # match set member (or non_member), ignoring case, unicode mode
-            # <IN> <skip> <set>
-            if ptr >= ctx.end or not rsre_char.check_charset(ctx, pattern, ppos+1,
-                                                             rsre_char.getlower_unicode(ctx.str(ptr))):
-                return
-            ppos += pattern.pat(ppos)
-            ptr = ctx.next(ptr)
-
-        elif consts.eq(op, consts.OPCODE37_IN_LOC_IGNORE):
-            # match set member (or non_member), ignoring case, locale mode
-            # <IN> <skip> <set>
-            if ptr >= ctx.end or not pattern.charset_loc_ignore(ctx, ppos+1,
-                                                                ctx.str(ptr)):
+                                                             ctx.lowstr(ptr, pattern.flags)):
                 return
             ppos += pattern.pat(ppos)
             ptr = ctx.next(ptr)
@@ -868,23 +752,7 @@ def sre_match(ctx, pattern, ppos, ptr, marks):
         elif op == consts.OPCODE_LITERAL_IGNORE:
             # match literal string, ignoring case
             # <LITERAL_IGNORE> <code>
-            if ptr >= ctx.end or pattern.lowa(ctx.str(ptr)) != pattern.pat(ppos):
-                return
-            ppos += 1
-            ptr = ctx.next(ptr)
-
-        elif consts.eq(op, consts.OPCODE37_LITERAL_UNI_IGNORE):
-            # match literal string, ignoring case, unicode mode
-            # <LITERAL_IGNORE> <code>
-            if ptr >= ctx.end or rsre_char.getlower_unicode(ctx.str(ptr)) != pattern.pat(ppos):
-                return
-            ppos += 1
-            ptr = ctx.next(ptr)
-
-        elif consts.eq(op, consts.OPCODE37_LITERAL_LOC_IGNORE):
-            # match literal string, ignoring case, locale mode
-            # <LITERAL_IGNORE> <code>
-            if ptr >= ctx.end or not pattern.char_loc_ignore(ppos, ctx.str(ptr)):
+            if ptr >= ctx.end or ctx.lowstr(ptr, pattern.flags) != pattern.pat(ppos):
                 return
             ppos += 1
             ptr = ctx.next(ptr)
@@ -907,23 +775,7 @@ def sre_match(ctx, pattern, ppos, ptr, marks):
         elif op == consts.OPCODE_NOT_LITERAL_IGNORE:
             # match if it's not a literal string, ignoring case
             # <NOT_LITERAL> <code>
-            if ptr >= ctx.end or pattern.lowa(ctx.str(ptr)) == pattern.pat(ppos):
-                return
-            ppos += 1
-            ptr = ctx.next(ptr)
-
-        elif consts.eq(op, consts.OPCODE37_NOT_LITERAL_UNI_IGNORE):
-            # match if it's not a literal string, ignoring case, unicode mode
-            # <NOT_LITERAL> <code>
-            if ptr >= ctx.end or rsre_char.getlower_unicode(ctx.str(ptr)) == pattern.pat(ppos):
-                return
-            ppos += 1
-            ptr = ctx.next(ptr)
-
-        elif consts.eq(op, consts.OPCODE37_NOT_LITERAL_LOC_IGNORE):
-            # match if it's not a literal string, ignoring case, locale mode
-            # <NOT_LITERAL> <code>
-            if ptr >= ctx.end or pattern.char_loc_ignore(ppos, ctx.str(ptr)):
+            if ptr >= ctx.end or ctx.lowstr(ptr, pattern.flags) == pattern.pat(ppos):
                 return
             ppos += 1
             ptr = ctx.next(ptr)
@@ -1007,57 +859,7 @@ def sre_match(ctx, pattern, ppos, ptr, marks):
             result = MinRepeatOneMatchResult(nextppos, ppos+3, max_count,
                                              ptr, marks)
             return result.find_first_result(ctx, pattern)
-        elif consts.eq(op, consts.OPCODE_POSSESSIVE_REPEAT_ONE):
-            # match repeated sequence (maximizing regexp) without
-            # backtracking
 
-            # this operator only works if the repeated item is
-            # exactly one character wide, and we're not already
-            # collecting backtracking points.  for other cases,
-            # use the MAX_REPEAT operator
-
-            # <POSSESSIVE_REPEAT_ONE> <skip> <1=min> <2=max> item <SUCCESS>
-            # tail
-            start = ptr
-
-            try:
-                minptr = ctx.next_n(start, pattern.pat(ppos+1), ctx.end)
-            except EndOfString:
-                return    # cannot match
-            ptr = find_repetition_end(ctx, pattern, ppos+3, start,
-                                      pattern.pat(ppos+2),
-                                      marks)
-            # when we arrive here, ptr points to the tail of the target
-            # string. match the rest of the pattern.
-            ppos += pattern.pat(ppos)
-        elif consts.eq(op, consts.OPCODE_POSSESSIVE_REPEAT):
-            # create possessive repeat contexts.
-            # <POSSESSIVE_REPEAT> <skip> <1=min> <2=max> pattern
-            # <SUCCESS> tail
-            start = ptr
-            saved = ctx.match_mode
-            ctx.match_mode = MODE_ANY
-            ptr = find_repetition_end_possessive(
-                    ctx, pattern,
-                    ppos, start,
-                    marks)
-            ctx.match_mode = saved
-            if ptr < 0:
-                return None
-            marks = ctx.match_marks
-            ppos += pattern.pat(ppos) + 1 # match tail now
-        elif consts.eq(op, consts.OPCODE_ATOMIC_GROUP):
-            # Atomic Group Sub Pattern
-            # <ATOMIC_GROUP> <skip> pattern <SUCCESS> tail
-
-            saved = ctx.match_mode
-            ctx.match_mode = MODE_ANY
-            match = sre_match(ctx, pattern, ppos + 1, ptr, marks)
-            ctx.match_mode = saved
-            if match is None:
-                return None
-            ptr = ctx.match_end
-            ppos += pattern.pat(ppos) # match tail now
         else:
             raise Error("bad pattern code %d" % op)
 
@@ -1081,36 +883,12 @@ def match_repeated(ctx, ptr, oldptr, length_bytes):
     return True
 
 @specializectx
-def match_repeated_ignore(ctx, ptr, oldptr, length_bytes, pattern):
+def match_repeated_ignore(ctx, ptr, oldptr, length_bytes, flags):
     oldend = ctx.go_forward_by_bytes(oldptr, length_bytes)
     while oldptr < oldend:
         if ptr >= ctx.end:
             return -1
-        if pattern.lowa(ctx.str(ptr)) != pattern.lowa(ctx.str(oldptr)):
-            return -1
-        ptr = ctx.next(ptr)
-        oldptr = ctx.next(oldptr)
-    return ptr
-
-@specializectx
-def match_repeated_uni_ignore(ctx, ptr, oldptr, length_bytes):
-    oldend = ctx.go_forward_by_bytes(oldptr, length_bytes)
-    while oldptr < oldend:
-        if ptr >= ctx.end:
-            return -1
-        if rsre_char.getlower_unicode(ctx.str(ptr)) != rsre_char.getlower_unicode(ctx.str(oldptr)):
-            return -1
-        ptr = ctx.next(ptr)
-        oldptr = ctx.next(oldptr)
-    return ptr
-
-@specializectx
-def match_repeated_loc_ignore(ctx, ptr, oldptr, length_bytes):
-    oldend = ctx.go_forward_by_bytes(oldptr, length_bytes)
-    while oldptr < oldend:
-        if ptr >= ctx.end:
-            return -1
-        if rsre_char.getlower_locale(ctx.str(ptr)) != rsre_char.getlower_locale(ctx.str(oldptr)):
+        if ctx.lowstr(ptr, flags) != ctx.lowstr(oldptr, flags):
             return -1
         ptr = ctx.next(ptr)
         oldptr = ctx.next(oldptr)
@@ -1177,63 +955,54 @@ def match_IN(ctx, pattern, ptr, ppos):
     return rsre_char.check_charset(ctx, pattern, ppos+2, ctx.str(ptr))
 @specializectx
 def match_IN_IGNORE(ctx, pattern, ptr, ppos):
-    return rsre_char.check_charset(ctx, pattern, ppos+2, pattern.lowa(ctx.str(ptr)))
-@specializectx
-def match_IN_UNI_IGNORE(ctx, pattern, ptr, ppos):
-    return rsre_char.check_charset(ctx, pattern, ppos+2, rsre_char.getlower_unicode(ctx.str(ptr)))
-@specializectx
-def match_IN_LOC_IGNORE(ctx, pattern, ptr, ppos):
-    return pattern.charset_loc_ignore(ctx, ppos+2, ctx.str(ptr))
+    return rsre_char.check_charset(ctx, pattern, ppos+2, ctx.lowstr(ptr, pattern.flags))
 @specializectx
 def match_LITERAL(ctx, pattern, ptr, ppos):
     return ctx.str(ptr) == pattern.pat(ppos+1)
 @specializectx
 def match_LITERAL_IGNORE(ctx, pattern, ptr, ppos):
-    return pattern.lowa(ctx.str(ptr)) == pattern.pat(ppos+1)
-@specializectx
-def match_LITERAL_UNI_IGNORE(ctx, pattern, ptr, ppos):
-    return rsre_char.getlower_unicode(ctx.str(ptr)) == pattern.pat(ppos+1)
-@specializectx
-def match_LITERAL_LOC_IGNORE(ctx, pattern, ptr, ppos):
-    return pattern.char_loc_ignore(ppos+1, ctx.str(ptr))
+    return ctx.lowstr(ptr, pattern.flags) == pattern.pat(ppos+1)
 @specializectx
 def match_NOT_LITERAL(ctx, pattern, ptr, ppos):
     return ctx.str(ptr) != pattern.pat(ppos+1)
 @specializectx
 def match_NOT_LITERAL_IGNORE(ctx, pattern, ptr, ppos):
-    return pattern.lowa(ctx.str(ptr)) != pattern.pat(ppos+1)
-@specializectx
-def match_NOT_LITERAL_UNI_IGNORE(ctx, pattern, ptr, ppos):
-    return rsre_char.getlower_unicode(ctx.str(ptr)) != pattern.pat(ppos+1)
-@specializectx
-def match_NOT_LITERAL_LOC_IGNORE(ctx, pattern, ptr, ppos):
-    return not pattern.char_loc_ignore(ppos+1, ctx.str(ptr))
+    return ctx.lowstr(ptr, pattern.flags) != pattern.pat(ppos+1)
 
 def _make_fre(checkerfn):
     if checkerfn == match_ANY_ALL:
         def fre(ctx, pattern, ptr, end, ppos):
             return end
-    elif checkerfn in (match_IN, match_IN_IGNORE, match_IN_UNI_IGNORE):
-        # produces three jitdrivers:
-        #     MatchIn
-        #     MatchInIgnore
-        #     MatchInUniIgnore
-        name = checkerfn.__name__.title().replace('_', '')
-        method_name = "jitdriver_" + name
-        install_jitdriver_spec(name,
+    elif checkerfn == match_IN:
+        install_jitdriver_spec('MatchIn',
                                greens=['ppos', 'pattern'],
                                reds=['ptr', 'end', 'ctx'],
                                debugprint=(1, 0))
         @specializectx
         def fre(ctx, pattern, ptr, end, ppos):
             while True:
-                getattr(ctx, method_name).jit_merge_point(ctx=ctx, ptr=ptr,
+                ctx.jitdriver_MatchIn.jit_merge_point(ctx=ctx, ptr=ptr,
                                                       end=end, ppos=ppos,
                                                       pattern=pattern)
                 if ptr < end and checkerfn(ctx, pattern, ptr, ppos):
                     ptr = ctx.next(ptr)
                 else:
                     return ptr
+    elif checkerfn == match_IN_IGNORE:
+        install_jitdriver_spec('MatchInIgnore',
+                               greens=['ppos', 'pattern'],
+                               reds=['ptr', 'end', 'ctx'],
+                               debugprint=(1, 0))
+        @specializectx
+        def fre(ctx, pattern, ptr, end, ppos):
+            while True:
+                ctx.jitdriver_MatchInIgnore.jit_merge_point(ctx=ctx, ptr=ptr,
+                                                            end=end, ppos=ppos,
+                                                            pattern=pattern)
+                if ptr < end and checkerfn(ctx, pattern, ptr, ppos):
+                    ptr = ctx.next(ptr)
+                else:
+                    return ptr
     else:
         # in the other cases, the fre() function is not JITted at all
         # and is present as a residual call.
@@ -1250,19 +1019,11 @@ unroll_char_checker = [
     (consts.OPCODE_ANY_ALL,            match_ANY_ALL),
     (consts.OPCODE_IN,                 match_IN),
     (consts.OPCODE_IN_IGNORE,          match_IN_IGNORE),
-    (consts.OPCODE37_IN_UNI_IGNORE,           match_IN_UNI_IGNORE),
-    (consts.OPCODE37_IN_LOC_IGNORE,           match_IN_LOC_IGNORE),
     (consts.OPCODE_LITERAL,            match_LITERAL),
     (consts.OPCODE_LITERAL_IGNORE,     match_LITERAL_IGNORE),
-    (consts.OPCODE37_LITERAL_UNI_IGNORE,      match_LITERAL_UNI_IGNORE),
-    (consts.OPCODE37_LITERAL_LOC_IGNORE,      match_LITERAL_LOC_IGNORE),
     (consts.OPCODE_NOT_LITERAL,        match_NOT_LITERAL),
     (consts.OPCODE_NOT_LITERAL_IGNORE, match_NOT_LITERAL_IGNORE),
-    (consts.OPCODE37_NOT_LITERAL_UNI_IGNORE,  match_NOT_LITERAL_UNI_IGNORE),
-    (consts.OPCODE37_NOT_LITERAL_LOC_IGNORE,  match_NOT_LITERAL_LOC_IGNORE),
     ]
-unroll_char_checker = [(_op, _fn) for (_op, _fn) in unroll_char_checker
-                       if _op is not None]   # possibly removes the OPCODE37_*
 unroll_fre_checker = [(_op, _make_fre(_fn))
                       for (_op, _fn) in unroll_char_checker]
 
@@ -1358,8 +1119,7 @@ def match(pattern, string, start=0, end=sys.maxint, fullmatch=False):
     assert isinstance(pattern, CompiledPattern)
     start, end = _adjust(start, end, len(string))
     ctx = StrMatchContext(string, start, end)
-    if fullmatch:
-        ctx.match_mode = MODE_FULL
+    ctx.fullmatch_only = fullmatch
     if match_context(ctx, pattern):
         return ctx
     else:
