@@ -1091,6 +1091,16 @@ class TestInternalFunctions(object):
         assert (rbigint.fromlong(-9**50).ulonglongmask() ==
                 r_ulonglong(-9**50))
 
+    def test_fits_int(self):
+        assert rbigint.fromlong(0).fits_int()
+        assert rbigint.fromlong(42).fits_int()
+        assert rbigint.fromlong(-42).fits_int()
+        assert rbigint.fromlong(sys.maxint).fits_int()
+        assert not rbigint.fromlong(sys.maxint + 1).fits_int()
+        assert rbigint.fromlong(-sys.maxint - 1).fits_int()
+        assert not rbigint.fromlong(-sys.maxint - 2).fits_int()
+        assert not rbigint.fromlong(-73786976294838206459).fits_int()
+
     def test_parse_digit_string(self):
         from rpython.rlib.rbigint import parse_digit_string
         class Parser:
