@@ -9,11 +9,11 @@ import json
 
 BASE_DIR = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
 if sys.platform.startswith('win'):
-    TARGET_NAME = 'pypy-c.exe'
+    TARGET_NAME = r'pypy3-c.exe'
     # PyPy uses bin as of PR https://github.com/pypa/virtualenv/pull/1400
     TARGET_DIR = 'bin'
 else:
-    TARGET_NAME = 'pypy-c'
+    TARGET_NAME = 'pypy3-c'
     TARGET_DIR = 'bin'
 VENV_DIR = 'pypy-venv'
 
