@@ -11,10 +11,16 @@ rootdir = os.path.dirname(os.path.dirname(os.path.abspath(sys.argv[0])))
 os.environ['PYTHONPATH'] = rootdir
 os.environ['PYTEST_PLUGINS'] = ''
 
+if sys.platform == 'win32':
+    pypyopt = "--pypy=pypy/goal/pypy3-c.exe"
+else:
+    pypyopt = "--pypy=pypy/goal/pypy3-c"
+
 popen = subprocess.Popen(
     [sys.executable, "pypy/test_all.py",
-     "--pypy=pypy/goal/pypy-c",
-     "--timeout=1324",   # make it easy to search for
+     pypyopt,
+     "--timeout=3600",
+     "-rs",
      "--duration=10",
      "--resultlog=cpython.log", "lib-python",
      ] + sys.argv[1:],
