@@ -11,8 +11,14 @@ rootdir = os.path.dirname(os.path.dirname(os.path.abspath(sys.argv[0])))
 os.environ['PYTHONPATH'] = rootdir
 os.environ['PYTEST_PLUGINS'] = ''
 
+if sys.platform == 'win32':
+    pypyopt = "--pypy=pypy/goal/pypy3-c.exe"
+else:
+    pypyopt = "--pypy=pypy/goal/pypy3-c"
+
 popen = subprocess.Popen(
-    ["pypy/goal/pypy-c", "pypy/test_all.py",
+    [sys.executable, "pypy/test_all.py",
+     pypyopt,
      "--resultlog=pypyjit_new.log",
      "pypy/module/pypyjit/test_pypy_c",
      ] + sys.argv[1:],
